@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2019 Fidelity National Information 	*
+ * Copyright (c) 2001-2020 Fidelity National Information 	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -13,12 +13,12 @@
 #include "error.h"
 
 LITDEF	err_msg cmerrors[] = {
-	{ "INVPROT", "Invalid protocol specified by remote partner", 0 },
-	{ "REGNTFND", "Region referenced not initialized", 0 },
-	{ "CMINTQUE", "Interlock failure accessing GT.CM server queue", 0 },
-	{ "INVINTMSG", "Invalid interrupt message received.", 0 },
-	{ "CMEXCDASTLM", "Exceeded AST limit. Cannot open database.", 0 },
-	{ "CMSYSSRV", "Error doing system service, status:", 0 },
+	{ "INVPROT", "Invalid protocol specified by remote partner", 0, 0 },
+	{ "REGNTFND", "Region referenced not initialized", 0, 0 },
+	{ "CMINTQUE", "Interlock failure accessing GT.CM server queue", 0, 0 },
+	{ "INVINTMSG", "Invalid interrupt message received.", 0, 0 },
+	{ "CMEXCDASTLM", "Exceeded AST limit. Cannot open database.", 0, 0 },
+	{ "CMSYSSRV", "Error doing system service, status:", 0, 0 },
 };
 
 LITDEF	int CMERR_INVPROT = 150568970;
