@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001 Sanchez Computer Associates, Inc.	*
+ * Copyright (c) 2001-2019 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -11,9 +12,11 @@
 
 #ifndef BIT_SET_DEFINED
 
+#include "gdsroot.h"
+
 /* Declare parms for bit_set.c */
 
-uint4 bit_set (uint4 bit, sm_uc_ptr_t base);
+uint4 bit_set (block_id bit, sm_uc_ptr_t base);
 
 #define BIT_SET_DEFINED
 
