@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2016 Fidelity National Information	*
+ * Copyright (c) 2001-2019 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -19,6 +19,7 @@
 #define MIN_COLLTYPE	0
 #define XFORM	0
 #define XBACK	1
+#define XUTIL	2
 #define	GTM_NULL_TEXT "GTM NULL COLL"
 #define STD_NULL_TEXT "STD NULL COLL"
 
@@ -92,6 +93,7 @@ typedef struct collseq_struct {
 	int			act;
 	int4			(*xform)();
 	int4			(*xback)();
+	int4			(*xutil)();	/* for zatransform -2/2 prev/next functionality */
 	int4			(*version)();
 	int4			(*verify)();
 	int			argtype;
