@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2003-2018 Fidelity National Information	*
+ * Copyright (c) 2003-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -27,7 +27,7 @@
 #include "util.h"
 #include "dse.h"
 #include "stringpool.h"			/* for GET_CURR_TIME_IN_DOLLARH_AND_ZDATE macro */
-#include "op.h"				/* for op_fnzdate and op_horolog prototype */
+#include "op.h"				/* for op_fnzdate and op_zhorolog prototype */
 #include "wcs_recover.h"		/* for wcs_recover prototype */
 #include "wcs_phase2_commit_wait.h"
 #include "sleep_cnt.h"			/* for SIGNAL_WRITERS_TO_STOP/RESUME and WAIT_FOR_WRITERS_TO_STOP macro */
@@ -40,7 +40,6 @@ GBLREF gd_addr		*original_header;
 error_def(ERR_SIZENOTVALID4);
 
 #define	DB_ABS2REL(X)		(uintszofptr_t)((uintszofptr_t)(X) - (uintszofptr_t)csa->nl)
-#define MAX_UTIL_LEN		40
 #define	CLEAN_VERIFY		"verification is clean"
 #define	UNCLEAN_VERIFY		"verification is NOT clean (see operator log for details)"
 #define	RECOVER_DONE		"recovery complete (see operator log for details)"
@@ -129,10 +128,10 @@ void dse_cache(void)
 			{
 				if (UNIX_ONLY(TRUE)VMS_ONLY(!is_mm))
 				{
-					SET_TRACEABLE_VAR(csa->nl->wc_blocked, TRUE);
+					SET_TRACEABLE_VAR(csa->nl->wc_blocked, WC_BLOCK_RECOVER);
 					/* No need to invoke function "wcs_phase2_commit_wait" as "wcs_recover" does that anyways */
 					wcs_recover(reg);
-					assert(FALSE == csa->nl->wc_blocked);	/* wcs_recover() should have cleared this */
+					assert(WC_UNBLOCK == csa->nl->wc_blocked);	/* wcs_recover() should have cleared this */
 				}
 			}
 			assert(20 == STR_LIT_LEN(DSE_DMP_TIME_FMT)); /* if they are not the same, the !20AD below should change */
