@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -47,7 +47,6 @@
 #include "mutex.h"
 #include "db_snapshot.h"
 
-#define MAX_UTIL_LEN    	64
 #define NEXT_EPOCH_TIME_SPACES	"                   " /* 19 spaces, we have 19 character field width to output Next Epoch Time */
 
 GBLREF	sgmnt_addrs	*cs_addrs;
@@ -245,7 +244,8 @@ void dse_dmp_fhead (void)
 				  ! (RDBF_NOSTATS & csd->reservedDBFlags) ? " TRUE" : "FALSE");
 		util_out_print("  LOCK shares DB critical section     !AD", FALSE, 5, csd->lock_crit_with_db ? " TRUE" : "FALSE");
 		util_out_print("  Read Only                      !AD", TRUE, 3, csd->read_only ? " ON" : "OFF");
-		util_out_print("  Recover interrupted                 !AD", TRUE, 5, (csd->recov_interrupted ? " TRUE" : "FALSE"));
+		util_out_print("  Recover interrupted                 !AD", FALSE, 5, (csd->recov_interrupted ? " TRUE" : "FALSE"));
+		util_out_print("  Full Block Write                 !UL", TRUE, csd->write_fullblk);
 	}
 	if (CLI_PRESENT == cli_present("ALL"))
 	{	/* Only dump if -/ALL as if part of above display */
@@ -279,6 +279,8 @@ void dse_dmp_fhead (void)
 		util_out_print("  Free Global Buffers            0x!XL", FALSE, freeque_cnt);
 		util_out_print("  wcs_wtstart pid count   0x!XL", TRUE, cnl->in_wtstart);
 		util_out_print("  Write Cache is Blocked              !AD", FALSE, 5, (cnl->wc_blocked ? " TRUE" : "FALSE"));
+		util_out_print("  Write Cache to be recovered         !AD", FALSE, 5,
+				((WC_BLOCK_RECOVER == cnl->wc_blocked) ? " TRUE" : "FALSE"));
 		util_out_print("  wcs_wtstart intent cnt  0x!XL", TRUE, cnl->intent_wtstart);
 		util_out_print(0, TRUE);
 		util_out_print("  Quick database rundown is active    !AD", TRUE, 5, (csd->mumps_can_bypass ? " TRUE" : "FALSE"));
@@ -334,7 +336,6 @@ void dse_dmp_fhead (void)
 	}
 	if (NEED_TO_DUMP("ENVIRONMENT"))
 	{
-		util_out_print("  Full Block Writes                       !UL", FALSE, csa->do_fullblockwrites);
 		util_out_print("  Full Block Write Len  !12UL", TRUE, csa->fullblockwrite_len);
 	}
 	if (NEED_TO_DUMP("DB_CSH"))
