@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -29,7 +29,6 @@
 #include "filestruct.h"
 #include "jnl.h"
 #include "buddy_list.h"		/* needed for tp.h */
-#include "hashtab_int4.h"	/* needed for tp.h */
 #include "tp.h"
 #include "cli.h"
 #include "util.h"
@@ -48,8 +47,6 @@
 #include "process_deferred_stale.h"
 #include "gvcst_blk_build.h"
 
-#define MAX_UTIL_LEN 80
-
 GBLREF boolean_t	unhandled_stale_timer_pop;
 GBLREF char		*update_array, *update_array_ptr;
 GBLREF gd_region	*gv_cur_region;
@@ -63,20 +60,20 @@ error_def(ERR_DBRDONLY);
 error_def(ERR_DSEBLKRDFAIL);
 error_def(ERR_DSEFAIL);
 
-
 void dse_maps(void)
 {
 	blk_segment		*bs1, *bs_ptr;
-	block_id		blk, bml_blk;
+	block_id		blk, bml_blk, bml_index, blk_index;
+	block_cnt		total_blks, bml_list_size;
 	boolean_t		was_crit;
 	cache_rec_ptr_t		dummy_cr;
 	char			util_buff[MAX_UTIL_LEN];
 	int			util_len;
 	int4			blk_seg_cnt, blk_size;		/* needed for BLK_INIT, BLK_SEG and BLK_FINI macros */
-	sm_uc_ptr_t		bp;
-	int4			blk_index, blks_in_bitmap, bml_index, bml_list_size, bml_size, bplmap, dummy_int, total_blks;
+	int4			blks_in_bitmap, bml_size, bplmap, dummy_int;
 	sgmnt_addrs		*csa;
 	sgmnt_data_ptr_t	csd;
+	sm_uc_ptr_t		bp;
 	srch_blk_status		blkhist;
 	uchar_ptr_t		blk_ptr;
 	unsigned char		*bml_list;
@@ -106,13 +103,13 @@ void dse_maps(void)
 		total_blks = csa->ti->total_blks;
 		assert(ROUND_DOWN2(blk_size, 2 * SIZEOF(int4)) == blk_size);
 		bml_size = BM_SIZE(bplmap);
-		bml_list_size = (total_blks + bplmap - 1) / bplmap * bml_size;
+		bml_list_size = ((total_blks + bplmap - 1) / bplmap) * bml_size;
 		bml_list = (unsigned char *)malloc(bml_list_size);
 		for (blk_index = 0, bml_index = 0;  blk_index < total_blks; blk_index += bplmap, bml_index++)
 			bml_newmap((blk_hdr_ptr_t)(bml_list + bml_index * bml_size), bml_size, csa->ti->curr_tn);
 		if (!was_crit)
 		{
-			grab_crit_encr_cycle_sync(gv_cur_region);
+			grab_crit_encr_cycle_sync(gv_cur_region, WS_59);
 			csa->hold_onto_crit = TRUE;	/* need to do this AFTER grab_crit */
 		}
 		blk = get_dir_root();
@@ -128,7 +125,7 @@ void dse_maps(void)
 			CWS_RESET;
 			CHECK_AND_RESET_UPDATE_ARRAY;	/* reset update_array_ptr to update_array */
 			assert(csa->ti->early_tn == csa->ti->curr_tn);
-			blk_ptr = bml_list + bml_index * bml_size;
+			blk_ptr = bml_list + (bml_index * bml_size);
 			blkhist.blk_num = blk_index;
 			if (!(blkhist.buffaddr = t_qread(blkhist.blk_num, &blkhist.cycle, &blkhist.cr)))
 				rts_error_csa(CSA_ARG(csa) VARLSTCNT(1) ERR_DSEBLKRDFAIL);
@@ -142,7 +139,11 @@ void dse_maps(void)
 		/* Fill in master map */
 		for (blk_index = 0, bml_index = 0;  blk_index < total_blks; blk_index += bplmap, bml_index++)
 		{
-			blks_in_bitmap = (blk_index + bplmap <= total_blks) ? bplmap : total_blks - blk_index;
+			/* (total_blks - blk_index) is used to determine the number of blks in the last lmap of the DB
+			 * so the value should never be larger then BLKS_PER_LMAP and thus fit in a int4
+			 */
+			assert((blk_index + bplmap <= total_blks) || (BLKS_PER_LMAP >= (total_blks - blk_index)));
+			blks_in_bitmap = (blk_index + bplmap <= total_blks) ? bplmap : (int4)(total_blks - blk_index);
 			assert(1 < blks_in_bitmap);	/* the last valid block in the database should never be a bitmap block */
 			if (NO_FREE_SPACE != bml_find_free(0, (bml_list + bml_index * bml_size) + SIZEOF(blk_hdr), blks_in_bitmap))
 				bit_set(blk_index / bplmap, csa->bmm);
@@ -181,7 +182,7 @@ void dse_maps(void)
 	if (CLI_PRESENT == cli_present("MASTER"))
 	{
 		if (!was_crit)
-			grab_crit_encr_cycle_sync(gv_cur_region);
+			grab_crit_encr_cycle_sync(gv_cur_region, WS_60);
 		bml_blk = blk / bplmap * bplmap;
 		if (dba_mm == csd->acc_meth)
 			bp = MM_BASE_ADDR(csa) + (off_t)bml_blk * blk_size;
@@ -207,13 +208,13 @@ void dse_maps(void)
 	}
 	MEMCPY_LIT(util_buff, "!/Block ");
 	util_len = SIZEOF("!/Block ") - 1;
-	util_len += i2hex_nofill(blk, (uchar_ptr_t)&util_buff[util_len], 8);
+	util_len += i2hexl_nofill(blk, (uchar_ptr_t)&util_buff[util_len], MAX_HEX_INT8);
 	memcpy(&util_buff[util_len], " is marked !AD in its local bit map.!/",
 		SIZEOF(" is marked !AD in its local bit map.!/") - 1);
 	util_len += SIZEOF(" is marked !AD in its local bit map.!/") - 1;
 	util_buff[util_len] = 0;
 	if (!was_crit)
-		grab_crit_encr_cycle_sync(gv_cur_region);
+		grab_crit_encr_cycle_sync(gv_cur_region, WS_61);
 	util_out_print(util_buff, TRUE, 4, dse_is_blk_free(blk, &dummy_int, &dummy_cr) ? "free" : "busy");
 	if (!was_crit)
 		rel_crit(gv_cur_region);
