@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2015 Fidelity National Information 	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -11,7 +11,7 @@
  ****************************************************************/
 
 #include "mdef.h"
-
+#include "gtm_string.h"
 #include "compiler.h"
 #include "opcode.h"
 #include "indir_enum.h"
@@ -125,14 +125,14 @@ int f_order(oprtype *a, opctype op)
 		}
 		assert(TRIP_REF == dir_oprptr->oprclass);
 		triptr = dir_oprptr->oprval.tref;
+		sav_dirref = newtriple(OC_GVSAVTARG); /* for explicit direction, $R at this point set by lit: 1st or var: 2nd arg */
+		triptr = newtriple(OC_GVRECTARG);
+		triptr->operand[0] = put_tref(sav_ref);
 		if (OC_LIT == triptr->opcode)
 		{	/* if direction is a literal - pick it up and stop flailing about */
 			if (MV_IS_TRUEINT(&triptr->operand[0].oprval.mlit->v, &intval) && (1 == intval || -1 == intval))
-			{
 				direction = (1 == intval) ? FORWARD : BACKWARD;
-				sav_ref->opcode = OC_NOOP;
-				sav_ref = NULL;
-			} else
+			else
 			{	/* bad direction */
 				if (NULL != oldchain)
 					setcurtchain(oldchain);
@@ -142,9 +142,6 @@ int f_order(oprtype *a, opctype op)
 		} else
 		{
 			direction = TBD;
-			sav_dirref = newtriple(OC_GVSAVTARG);		/* $R reflects direction eval even if we revisit 1st arg */
-			triptr = newtriple(OC_GVRECTARG);
-			triptr->operand[0] = put_tref(sav_ref);
 			switch (object)
 			{
 			case GLOBAL:		/* The direction may have had a side effect, so take copies of subscripts */
