@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2010 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -9,8 +10,8 @@
  *								*
  ****************************************************************/
 
-#ifndef __GET_RET_TARG_H__
-#define __GET_RET_TARG_H__
+#ifndef GET_RET_TARG_H_INCLUDED
+#define GET_RET_TARG_H_INCLUDED
 
 mval *get_ret_targ(stack_frame **retsf);
 
