@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2004-2018 Fidelity National Information	*
+ * Copyright (c) 2004-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -45,7 +45,6 @@
 #include "filestruct.h"
 #include "buddy_list.h"		/* needed for tp.h */
 #include "jnl.h"
-#include "hashtab_int4.h"	/* needed for tp.h */
 #include "tp.h"
 #include "cli.h"
 
@@ -71,7 +70,6 @@ GBLREF	boolean_t	dollar_zquit_anyway;	/* if TRUE compile QUITs to not care wheth
 GBLREF	uint4		gtmDebugLevel; 		/* Debug level (0 = using default sm module so with
 						   a DEBUG build, even level 0 implies basic debugging) */
 GBLREF	boolean_t	gtmSystemMalloc;	/* Use the system's malloc() instead of our own */
-GBLREF	int4		gtm_fullblockwrites;	/* Do full (not partial) database block writes */
 GBLREF	boolean_t	certify_all_blocks;
 GBLREF	uint4		gtm_blkupgrade_flag;	/* controls whether dynamic block upgrade is attempted or not */
 GBLREF	boolean_t	gtm_dbfilext_syslog_disable;	/* control whether db file extension message is logged or not */
@@ -84,6 +82,7 @@ GBLREF	block_id	gtm_tp_allocation_clue;	/* block# hint to start allocation for c
 GBLREF	boolean_t	gtm_stdxkill;		/* Use M Standard exclusive kill instead of historical GTM */
 GBLREF	boolean_t	ztrap_new;		/* Each time $ZTRAP is set it is automatically NEW'd */
 GBLREF	size_t		gtm_max_storalloc;	/* Used for testing: creates an allocation barrier */
+GBLREF	boolean_t	gtm_nofflf;		/* Used to control "write #" behavior ref GTM-9136 */
 
 void	gtm_env_init(void)
 {
@@ -95,7 +94,6 @@ void	gtm_env_init(void)
 	mstr			val, trans;
 	uint4			tdbglvl, tmsock, reservesize, memsize, cachent, trctblsize, trctblbytes;
 	uint4			max_threads, max_procs;
-	int4			temp_gtm_strpllim;
 	DCL_THREADGBL_ACCESS;
 
 	SETUP_THREADGBL_ACCESS;
@@ -219,29 +217,6 @@ void	gtm_env_init(void)
 		gtm_tp_allocation_clue = (block_id)trans_numeric(&val, &is_defined, TRUE);
 		if (!is_defined)
 			gtm_tp_allocation_clue = (block_id)MAXTOTALBLKS_MAX;
-		/* Full Database-block Write mode */
-		val.addr = GTM_FULLBLOCKWRITES;
-		val.len = SIZEOF(GTM_FULLBLOCKWRITES) - 1;
-		gtm_fullblockwrites = (int)logical_truth_value(&val, FALSE, &is_defined);
-		if (!is_defined) /* Variable not defined */
-			gtm_fullblockwrites = DEFAULT_FBW_FLAG;
-		else if (!gtm_fullblockwrites) /* Set to false */
-			gtm_fullblockwrites = FALSE;
-		else /* Set to true, exam for FULL_DATABASE_WRITE */
-		{
-			gtm_fullblockwrites = trans_numeric(&val, &is_defined, TRUE);
-			switch(gtm_fullblockwrites)
-			{
-			case FULL_FILESYSTEM_WRITE:
-			case FULL_DATABASE_WRITE:
-				/* Both these values are valid */
-				break;
-			default:
-				/* Else, assume FULL_FILESYSTEM_WRITE */
-				gtm_fullblockwrites = FULL_FILESYSTEM_WRITE;
-				break;
-			}
-		}
 		/* GDS Block certification */
 		val.addr = GTM_GDSCERT;
 		val.len = SIZEOF(GTM_GDSCERT) - 1;
@@ -281,6 +256,12 @@ void	gtm_env_init(void)
 		val.len = SIZEOF(GTM_MAX_SOCKETS) - 1;
 		if ((tmsock = trans_numeric(&val, &is_defined, TRUE)) && MAX_MAX_N_SOCKET > tmsock) /* Note assignment!! */
 			gtm_max_sockets = tmsock;
+		/* Initialize TCP_KEEPIDLE and by implication SO_KEEPALIVE */
+		val.addr = GTM_SOCKET_KEEPALIVE_IDLE;
+		val.len = SIZEOF(GTM_SOCKET_KEEPALIVE_IDLE) - 1;
+		TREF(gtm_socket_keepalive_idle) = trans_numeric(&val, &is_defined, TRUE);
+		if (0 > TREF(gtm_socket_keepalive_idle))
+			TREF(gtm_socket_keepalive_idle) = 0;
 		/* Initialize storage to allocate and keep in our back pocket in case run out of memory */
 		outOfMemoryMitigateSize = GTM_MEMORY_RESERVE_DEFAULT;
 		val.addr = GTM_MEMORY_RESERVE;
@@ -407,12 +388,13 @@ void	gtm_env_init(void)
 		gtm_mupjnl_parallel = trans_numeric(&val, &is_defined, TRUE);
 		if (!is_defined)
 			gtm_mupjnl_parallel = 1;
-		/* See if $gtm_string_pool_limit is set */
-		val.addr = GTM_STRPLLIM;
-		val.len = SIZEOF(GTM_STRPLLIM) - 1;
-		temp_gtm_strpllim = trans_numeric(&val, &is_defined, TRUE);
-		if (0 < temp_gtm_strpllim)
-			TREF(gtm_strpllim) = temp_gtm_strpllim;
+		/* gtm_nofflf for GTM-9136.  Default is FALSE */
+		val.addr = GTM_NOFFLF;
+		val.len = SIZEOF(GTM_NOFFLF) - 1;
+		ret = logical_truth_value(&val, FALSE, &is_defined);
+		if (is_defined)
+		        gtm_nofflf = ret;
+
 		/* Platform specific initializations */
 		gtm_env_init_sp();
 	}
