@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -108,6 +108,13 @@
 #  define gmaLine " "
 #endif
 
+/* Tail call optimization eliminates the gtm_malloc/gtm_free stack level in pro, but we have to account for it in debug. */
+#if !defined(DEBUG) || defined(PRO_BUILD)
+#define TAIL_CALL_LEVEL 0
+#else
+#define TAIL_CALL_LEVEL 1
+#endif
+
 /* #GTM_THREAD_SAFE : The below macro (MALLOC) is thread-safe because caller ensures serialization with locks */
 #  define MALLOC(size, addr) 										\
 {													\
@@ -202,15 +209,7 @@
 #else
 #  define GetSizeIndex(size) (size2Index[(size - 1) / MINTWO])
 #endif
-/* Note we use unsigned char * instead of caddr_t for all references to caller_id so the caller id
- * is always 4 bytes. On Tru64, caddr_t is 8 bytes which will throw off the size of our
- * storage header in debug mode.
- */
-#ifdef GTM_MALLOC_DEBUG
-#  define CALLERID (smCallerId)
-#else
-#  define CALLERID ((unsigned char *)caller_id())
-#endif
+#define CALLERID ((unsigned char *)caller_id(stack_level))
 /* Define "routines" to enqueue and dequeue storage elements. Use define so we don't
  * have to depend on each implementation's compiler inlining to get efficient code here.
  */
@@ -250,7 +249,7 @@
 			++sEHdr->elemsAllocd;									\
 		}												\
 	} else													\
-		uStor = findStorElem(sizeIndex);								\
+		uStor = findStorElem(sizeIndex, stack_level + 1);	/* Not in tail call position */		\
 	assert(0 == ((unsigned long)uStor & (TwoTable[sizeIndex] - 1)));	/* Verify alignment */		\
 }
 #ifdef INT8_SUPPORTED
@@ -323,7 +322,6 @@ GBLREF	ch_ret_type	(*ht_rhash_ch)();		/* Function pointer to hashtab_rehash_ch *
 GBLREF	ch_ret_type	(*jbxm_dump_ch)();		/* Function pointer to jobexam_dump_ch */
 GBLREF	ch_ret_type	(*stpgc_ch)();			/* Function pointer to stp_gcol_ch */
 /* This var allows us to call ourselves but still have callerid info */
-GBLREF	unsigned char	*smCallerId;			/* Caller of top level malloc/free */
 GBLREF	volatile int4	fast_lock_count;		/* Stop stale/epoch processing while we have our parts exposed */
 OS_PAGE_SIZE_DECLARE
 #define SIZETABLEDIM MAXTWO/MINTWO
@@ -414,14 +412,16 @@ error_def(ERR_MALLOCMAXUNIX);
 
 /* Internal prototypes */
 void gtmSmInit(void);
-storElem *findStorElem(int sizeIndex);
+storElem *findStorElem(int sizeIndex, int stack_level);
 void release_unused_storage(void);
+void *gtm_malloc_main(size_t, int stack_level);
+void gtm_free_main(void *, int stack_level);
 #ifdef DEBUG
 void backfill(unsigned char *ptr, gtm_msize_t len);
 boolean_t backfillChk(unsigned char *ptr, gtm_msize_t len);
 #else
-void *gtm_malloc_dbg(size_t);
-void gtm_free_dbg(void *);
+void *gtm_malloc_dbg(size_t, int stack_level);
+void gtm_free_dbg(void *, int stack_level);
 void raise_gtmmemory_error_dbg(void);
 size_t gtm_bestfitsize_dbg(size_t);
 #endif
@@ -544,7 +544,7 @@ void gtmSmInit(void)	/* Note renamed to gtmSmInit_dbg when included in gtm_mallo
  * before.
  */
 /* #GTM_THREAD_SAFE : The below function (findStorElem) is thread-safe because caller ensures serialization with locks */
-storElem *findStorElem(int sizeIndex)	/* Note renamed to findStorElem_dbg when included in gtm_malloc_dbg.c */
+storElem *findStorElem(int sizeIndex, int stack_level)	/* Note renamed to findStorElem_dbg when included in gtm_malloc_dbg.c */
 {
 	unsigned char	*uStorAlloc;
 	storElem	*uStor, *uStor2, *qHdr;
@@ -638,13 +638,13 @@ storElem *findStorElem(int sizeIndex)	/* Note renamed to findStorElem_dbg when i
 /* Note, if the below declaration changes, corresponding changes in gtmxc_types.h needs to be done. */
 /* Obtain free storage of the given size */
 /* #GTM_THREAD_SAFE : The below function (gtm_malloc) is thread-safe; serialization is ensured with locks */
-void *gtm_malloc(size_t size)	/* Note renamed to gtm_malloc_dbg when included in gtm_malloc_dbg.c */
+void *gtm_malloc_main(size_t size, int stack_level)	/* Note renamed to gtm_malloc_dbg when included in gtm_malloc_dbg.c */
 {
 	unsigned char	*retVal;
 	storElem 	*uStor, *qHdr;
 	storExtHdr	*sEHdr;
-	gtm_msize_t	tSize;
-	int		sizeIndex, i, hdrSize;
+	gtm_msize_t	tSize, hdrSize;
+	int		sizeIndex, i;
 	unsigned char	*trailerMarker;
 	boolean_t	reentered, was_holder;
 	intrpt_state_t	prev_intrpt_state;
@@ -671,8 +671,8 @@ void *gtm_malloc(size_t size)	/* Note renamed to gtm_malloc_dbg when included in
 		if (gtmSmInitialized)
 		{
 			PTHREAD_MUTEX_LOCK_IF_NEEDED(was_holder);	/* get exclusive thread lock in case threads are in use */
-			hdrSize = OFFSETOF(storElem, userStorage);		/* Size of storElem header */
-			NON_GTM64_ONLY(assertpro((size + hdrSize) >= size));	/* Check for wrap in 32 bit platforms */
+			hdrSize = OFFSETOF(storElem, userStorage);	/* Size of storElem header */
+			assertpro((size + hdrSize) >= size);		/* Check for wrap in all platforms */
 			assert((hdrSize + SIZEOF(markerChar)) < MINTWO);
 			fast_lock_count++;
 			++gtmMallocDepth;				/* Nesting depth of memory calls */
@@ -791,26 +791,22 @@ void *gtm_malloc(size_t size)	/* Note renamed to gtm_malloc_dbg when included in
 			PTHREAD_MUTEX_LOCK_IF_NEEDED(was_holder); /* get thread lock in case threads are in use */
 			gtmSmInit();
 			PTHREAD_MUTEX_UNLOCK_IF_NEEDED(was_holder);	/* release exclusive thread lock if needed */
-			/* Reinvoke gtm_malloc now that we are initialized. Note that this one time (the first
-			 * call to malloc), we will not record the proper caller id in the storage header or in
-			 * the traceback table. The caller will show up as gtm_malloc(). However, all subsequent
-			 * calls will be correct.
+			/* Reinvoke gtm_malloc now that we are initialized.
 			 *
 			 * Note, in a pro build, if the call to gtmSmInit() drives gtm_init_env() which discovers
 			 * we should be be doing debug builds, drive the debug flavor of gtm_malloc instead which
 			 * will do its own initialization if it still needs to (see top of gtmSmInit() above).
 			 */
-			PRO_ONLY(if (0 != gtmDebugLevel) return (void *)gtm_malloc_dbg(size));
-			return (void *)gtm_malloc(size);
+#			ifndef DEBUG
+			if (gtmDebugLevel & GDL_SmAllMallocDebug)
+				return (void *)gtm_malloc_dbg(size, stack_level + TAIL_CALL_LEVEL);
+#			endif
+			return (void *)gtm_malloc_main(size, stack_level + TAIL_CALL_LEVEL);
 		}
 #	ifndef DEBUG
 	} else
-	{	/* We have a non-DEBUG module but debugging is turned on so redirect the call to the appropriate module.
-		 * Stash the caller id for later use, as we can't be sure whether the call to gtm_malloc_dbg() will add
-		 * a stack frame, disrupting later caller_id() requests.
-		 */
-		smCallerId = (unsigned char *)caller_id();
-		return (void *)gtm_malloc_dbg(size);
+	{	/* We have a non-DEBUG module but debugging is turned on so redirect the call to the appropriate module. */
+		return (void *)gtm_malloc_dbg(size, stack_level + TAIL_CALL_LEVEL);
 	}
 #	endif
 }
@@ -818,7 +814,7 @@ void *gtm_malloc(size_t size)	/* Note renamed to gtm_malloc_dbg when included in
 /* Note, if the below declaration changes, corresponding changes in gtmxc_types.h needs to be done. */
 /* Release the free storage at the given address */
 /* #GTM_THREAD_SAFE : The below function (gtm_free) is thread-safe; serialization is ensured with locks */
-void gtm_free(void *addr)	/* Note renamed to gtm_free_dbg when included in gtm_malloc_dbg.c */
+void gtm_free_main(void *addr, int stack_level)	/* Note renamed to gtm_free_dbg when included in gtm_malloc_dbg.c */
 {
 	storElem 	*uStor, *buddyElem;
 	storExtHdr	*sEHdr;
@@ -1015,11 +1011,8 @@ void gtm_free(void *addr)	/* Note renamed to gtm_free_dbg when included in gtm_m
 	} else
 	{	/* If not a debug module and debugging is enabled, reroute call to
 		 * the debugging version.
-		 * Stash the caller id for later use, as we can't be sure whether the call to gtm_malloc_dbg() will add
-		 * a stack frame, disrupting later caller_id() requests.
 		 */
-		smCallerId = (unsigned char *)caller_id();
-		gtm_free_dbg(addr);
+		gtm_free_dbg(addr, stack_level);	/* Not in tail call position */
 	}
 #	endif
 	DEFERRED_EXIT_HANDLING_CHECK;
@@ -1465,3 +1458,16 @@ void system_free(void *addr)
 }
 
 #endif
+
+#ifndef GTM_MALLOC_DEBUG
+
+void *gtm_malloc(size_t size)
+{
+	return gtm_malloc_main(size, TAIL_CALL_LEVEL);
+}
+
+void gtm_free(void *addr)
+{
+	gtm_free_main(addr, TAIL_CALL_LEVEL);
+}
+#endif
