@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2010-2019 Fidelity National Information	*
+ * Copyright (c) 2010-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -155,7 +155,7 @@ THREADGBLDEF(gv_some_subsc_null,		boolean_t)			/* TRUE if SOME subscript other t
 THREADGBLDEF(gv_sparekey,			gv_key *)			/* gv_xform_key working memory */
 THREADGBLDEF(gv_sparekey_mval,			mval)				/* gv_xform_key working memory */
 THREADGBLDEF(gv_sparekey_size,			int4)				/* part gv_xform_key working memory mechanism */
-THREADGBLDEF(gv_tporigkey_ptr,			gv_orig_key_array *)		/* copy of gv_currkey at outermost TSTART */
+THREADGBLDEF(gv_tporigkey_ptr,			gv_key_buf *)			/* copy of gv_currkey at outermost TSTART */
 THREADGBLDEF(gv_tporig_extnam_str,		mstr)				/* copy of extnam_str at outermost TSTART */
 THREADGBLDEF(in_gvcst_bmp_mark_free,		boolean_t)			/* May need to skip online rollback cleanup or
 										 * gvcst_redo_root_search on a restart */
@@ -180,6 +180,7 @@ THREADGBLDEF(ready2signal_gvundef,		boolean_t)			/* TRUE if GET operation about
 THREADGBLDEF(redo_rootsrch_ctxt,		redo_root_search_context)	/* context to be saved and restored during
 										 * gvcst_redo_root_search */
 THREADGBLDEF(semwait2long,			volatile boolean_t)		/* Waited too long for a semaphore */
+THREADGBLDEF(skip_DB_exists_check,		boolean_t)			/* skip check for whether the DB file exists */
 THREADGBLDEF(skip_file_corrupt_check,		boolean_t)			/* skip file_corrupt check in grab_crit */
 THREADGBLDEF(tpnotacidtime,			mval)				/* limit for long non-ACID ops in transactions */
 THREADGBLDEF(tp_restart_count,			uint4)				/* tp_restart counter */
@@ -198,6 +199,7 @@ THREADGBLDEF(wcs_recover_done,			boolean_t)			/* TRUE if wcs_recover was ever in
 										 * process. */
 THREADGBLDEF(statsdb_fnerr_reason,		int)				/* Failure code for "gvcst_set_statsdb_fname" */
 THREADGBLDEF(statsdb_memerr,			boolean_t)			/* If true, Failed to write to statsdb (SIG7)" */
+THREADGBLDEF(non_tp_noiso_key_n_value,		key_cum_value)			/* for non-TP recompute block */
 
 /* Local variables */
 THREADGBLDEF(curr_symval_cycle,			unsigned int)			/* When curr_symval is changed, counter is bumped */
@@ -216,7 +218,6 @@ THREADGBLDEF(lvmon_vars_count,			int)				/* Count of lvmon_vars at lvmon_vars_an
 THREADGBLDEF(lv_null_subs,			int)				/* set in gtm_env_init_sp() */
 THREADGBLDEF(max_lcl_coll_xform_bufsiz,		int)				/* max size of local collation buffer,which extends
 										 * from 32K each time the buffer overflows */
-
 /* Replication variables */
 THREADGBLDEF(replgbl,				replgbl_t)			/* set of global variables needed by the source
 										 * server */
@@ -258,8 +259,7 @@ THREADGBLDEF(gtm_environment_init,		boolean_t)			/* indicates GT.M development e
 										 * than a production environment */
 THREADGBLFPTR(gtm_sigusr1_handler,		void, 		(void))		/* SIGUSR1 signal handler function ptr */
 THREADGBLDEF(gtm_linktmpdir,			mstr)				/* Directory to use for relinkctl files */
-THREADGBLDEF(gtm_strpllim,			int4)				/* if non-zero, sets limit on stringpool */
-THREADGBLDEF(gtm_strpllimwarned,		boolean_t)			/* already hit limit on stringpool  */
+THREADGBLDEF(gtm_tmpdir,			mstr)				/* Directory to use for tmp files */
 THREADGBLDEF(gtm_trigger_etrap,			mval)				/* $etrap - for use in triggers */
 THREADGBLDEF(gtm_trctbl_cur,			trctbl_entry *)			/* Current gtm trace table entry */
 THREADGBLDEF(gtm_trctbl_end,			trctbl_entry *)			/* End of gtm trace table (last entry + 1) */
@@ -273,7 +273,11 @@ THREADGBLDEF(gtmsecshr_comkey,			unsigned int)			/* Hashed version key for gtmse
 THREADGBLDEF(gvcst_statsDB_open_ch_active,	boolean_t)			/* Condition handler is active */
 THREADGBLDEF(in_zwrite,				boolean_t)			/* ZWrite is active */
 THREADGBLDEF(is_socketpool,			boolean_t)			/* True when device-to-be-opened is socketpool */
+THREADGBLDEF(gtm_socket_keepalive_idle,		int)				/* Initialized from $gtm_socket_keepalive_idle */
 THREADGBLDEF(in_mupip_integ,			boolean_t)			/* To let DO_DB_HDR_CHECK skip DBFLCORRP */
+THREADGBLDEF(instance_frozen_crit_skipped,	boolean_t)			/* To indicate Instance Freeze is on, CRIT skipped*/
+THREADGBLDEF(integ_cannotskip_crit,		boolean_t)			/* indicates whether a SKIP CRIT is allowed */
+THREADGBLDEF(retry_buffer_returned_null,	boolean_t)			/* If a retried block in integ returned a NULL */
 THREADGBLDEF(lab_lnr,				lnr_tabent **)			/* Passes address from op_rhd_ext to op_extcall etc.
 										 * Points into either lab_proxy or linkage table
 										 */
@@ -494,6 +498,7 @@ THREADGBLDEF(was_open_reg_seen,			boolean_t)	/* TRUE => there is at least one re
 THREADGBLDEF(nontp_jbuf_rsrv,			jbuf_rsrv_struct_t *)	/* Pointer to structure corresponding to reservations
 									 * on the journal buffer for current non-TP transaction.
 									 */
+THREADGBLDEF(source_line,			int4)		/* keep track of line number in M file while compiling */
 /* Debug values */
 #ifdef DEBUG
 THREADGBLDEF(donot_commit,			boolean_t)			/* debug-only - see gdsfhead.h for purpose */
