@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001 Sanchez Computer Associates, Inc.	*
+ * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -8,8 +9,8 @@
  *	the license, please stop and do not read further.	*
  *								*
  ****************************************************************/
-#ifndef __GTM_WAKE_H__
-#define __GTM_WAKE_H__
+#ifndef GTM_WAKE_H_INCLUDED
+#define GTM_WAKE_H_INCLUDED
 
 void gtm_wake(int4 *pidadr,char *prcnam);
 
