@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2019 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -38,6 +38,7 @@
 #include "gtmio.h"
 #include "util.h"
 #include "repl_log.h"
+#include "repl_comm.h"
 
 GBLDEF	int		gtmrecv_listen_sock_fd = FD_INVALID;
 
@@ -57,6 +58,13 @@ int gtmrecv_comm_init(in_port_t port)
 	int			port_buffer_len;
 	int			temp_sock_fd;
 	int			af;
+	char			err_buffer[512];
+	struct sockaddr_in 	local;
+	struct sockaddr         *local_sa_ptr;
+	char			local_ip[SA_MAXLEN];
+	char                    local_port_buffer[NI_MAXSERV];
+	unsigned int		save_errno;
+	GTM_SOCKLEN_TYPE        len;
 
 	if (FD_INVALID != gtmrecv_listen_sock_fd) /* Initialization done already */
 		return (0);
@@ -95,24 +103,34 @@ int gtmrecv_comm_init(in_port_t port)
 		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(7) ERR_REPLCOMM, 0, ERR_TEXT, 2,
 				RTS_ERROR_LITERAL("Error with receiver server listen socket enable reuseaddr"), ERRNO);
 	}
-	if (0 > BIND(gtmrecv_listen_sock_fd, ai_ptr->ai_addr, ai_ptr->ai_addrlen))
+	if ((0 > BIND(gtmrecv_listen_sock_fd, ai_ptr->ai_addr, ai_ptr->ai_addrlen)) || (WBTEST_ENABLED(WBTEST_REPL_INIT_ERR)) )
 	{
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(7) ERR_REPLCOMM, 0, ERR_TEXT, 2,
-				 RTS_ERROR_LITERAL("Could not bind local address"), ERRNO);
+		GTM_WHITE_BOX_TEST(WBTEST_REPL_INIT_ERR, errno, 98);
+		SNPRINTF(err_buffer, 512, "Could not bind local address. Local Port : %hu", port);
+		SEND_SYSMSG_REPLCOMM(LEN_AND_STR(err_buffer));
 		freeaddrinfo(ai_ptr);
 		CLOSEFILE_RESET(gtmrecv_listen_sock_fd, rc);	/* resets "gtmrecv_listen_sock_fd" to FD_INVALID */
+		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(7) ERR_REPLCOMM, 0, ERR_TEXT, 2,
+				RTS_ERROR_STRING(err_buffer), ERRNO);
 		return (-1);
 	}
 
-	if (0 > listen(gtmrecv_listen_sock_fd, 5))
+	if ((0 > listen(gtmrecv_listen_sock_fd, 5)) || (WBTEST_ENABLED(WBTEST_REPL_INIT_ERR2)))
 	{
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(7) ERR_REPLCOMM, 0, ERR_TEXT, 2,
-				 RTS_ERROR_LITERAL("Could not listen"), ERRNO);
+		save_errno = ERRNO;
+		len = SIZEOF(local);
+		if (0 == getsockname(gtmrecv_listen_sock_fd, (struct sockaddr *)&local, (GTM_SOCKLEN_TYPE *)&len))
+			SNPRINTF(err_buffer, 512, "Could not listen. Local port : %hu", ntohs(local.sin_port));
+		else
+			SNPRINTF(err_buffer, 512, "Could not listen. Local port : *UNKNOWN* : %s\n", strerror(errno));
+		GTM_WHITE_BOX_TEST(WBTEST_REPL_INIT_ERR2, save_errno, 98);
+		SEND_SYSMSG_REPLCOMM(LEN_AND_STR(err_buffer));
 		freeaddrinfo(ai_ptr);
 		CLOSEFILE_RESET(gtmrecv_listen_sock_fd, rc);	/* resets "gtmrecv_listen_sock_fd" to FD_INVALID */
+		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(7) ERR_REPLCOMM, 0, ERR_TEXT, 2,
+				 RTS_ERROR_STRING(err_buffer), save_errno);
 		return (-1);
 	}
-
 	freeaddrinfo(ai_ptr);
 	return (0);
 }
