@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2013 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -26,7 +27,6 @@
 #include "copy.h"
 #include "jnl.h"
 #include "buddy_list.h"		/* needed for tp.h */
-#include "hashtab_int4.h"	/* needed for tp.h */
 #include "tp.h"
 #include "gvcst_blk_build.h"
 #include "gtmimagename.h"
@@ -43,9 +43,7 @@ GBLREF	sgmnt_addrs		*cs_addrs;
 GBLREF	sgmnt_data_ptr_t	cs_data;
 GBLREF	boolean_t		write_after_image;
 GBLREF	unsigned int		t_tries;
-#ifdef UNIX
 GBLREF	jnl_gbls_t		jgbl;
-#endif
 
 void gvcst_blk_build(cw_set_element *cse, sm_uc_ptr_t base_addr, trans_num ctn)
 {
@@ -68,8 +66,8 @@ void gvcst_blk_build(cw_set_element *cse, sm_uc_ptr_t base_addr, trans_num ctn)
 	 * which operates outside crit. The exceptions to this are DSE (write_after_image is TRUE) or ONLINE ROLLBACK
 	 * which holds crit for the entire duration
 	 */
-	assert((dba_bg != cs_data->acc_meth) || dollar_tlevel || !cs_addrs->now_crit || write_after_image
-			UNIX_ONLY(|| jgbl.onlnrlbk));
+	assert((dba_bg != cs_data->acc_meth) || dollar_tlevel || !cs_addrs->now_crit || write_after_image || jgbl.onlnrlbk
+		|| ((NULL != cse->recompute_list_head) && (gds_t_write == cse->mode)));
 	assert((dba_mm != cs_data->acc_meth) || dollar_tlevel || cs_addrs->now_crit);
 	assert(cse->mode != gds_t_writemap);
 	array = (blk_segment *)cse->upd_addr;
@@ -123,14 +121,18 @@ void gvcst_blk_build(cw_set_element *cse, sm_uc_ptr_t base_addr, trans_num ctn)
 	 *	d) For acquired blocks, it is possible that some process had read in the uninitialized block from disk
 	 *		outside of crit (due to concurrency issues). Therefore the buffer could contain garbage. So we cannot
 	 *		rely on the buffer contents to determine the block's transaction number.
-	 *	e) For VMS, if a twin is created, we explicitly set its buffer tn to be equal to ctn in phase1.
+	 *	e) If a twin is created, we explicitly set its buffer tn to be equal to ctn in phase1.
 	 *		But since we are not passed the "cr" in this routine, it is not easily possible to check that.
-	 *		Hence in case of VMS, we relax the check so buffertn == ctn is allowed.
+	 *		Hence in case of twinning, we relax the check so buffertn == ctn is allowed.
 	 */
 	DEBUG_ONLY(blktn = ((blk_hdr_ptr_t)base_addr)->tn);
 	assert(!IS_MCODE_RUNNING || !cs_addrs->t_commit_crit || (dba_bg != cs_data->acc_meth) || (n_gds_t_op < cse->mode)
-	       || (cse->mode == gds_t_acquired) || (blktn UNIX_ONLY(<) VMS_ONLY(<=) ctn));
-	assert((ctn < cs_addrs->ti->early_tn) || write_after_image);
+	       || (cse->mode == gds_t_acquired) || ((!cs_data->asyncio && (blktn < ctn)) || (cs_data->asyncio && (blktn <= ctn))));
+	/* With memory instruction reordering (currently possible only on AIX with the POWER architecture) it is possible
+	 * the early_tn we read in the assert below gets executed BEFORE the curr_tn read that happens a few lines above.
+	 * That could then fail this assert (GTM-8523). Account for that with the AIX_ONLY condition below.
+	 */
+	assert((ctn < cs_addrs->ti->early_tn) || write_after_image AIX_ONLY(|| (cs_data->acc_meth == dba_mm)));
 	((blk_hdr_ptr_t)base_addr)->bver = GDSVCURR;
 	((blk_hdr_ptr_t)base_addr)->tn = ctn;
 	((blk_hdr_ptr_t)base_addr)->bsiz = UINTCAST(array->len);
@@ -178,7 +180,7 @@ void gvcst_blk_build(cw_set_element *cse, sm_uc_ptr_t base_addr, trans_num ctn)
 			chain.cw_index = cse->index;
 			chain.next_off = cse->next_off;
 			ptr = base_addr + cse->ins_off;
-			GET_LONGP(ptr, &chain);
+			GET_BLK_IDP(ptr, &chain);
 			cse->index = 0;
 			cse->ins_off = 0;
 			cse->next_off = 0;
@@ -241,7 +243,7 @@ void gvcst_blk_build(cw_set_element *cse, sm_uc_ptr_t base_addr, trans_num ctn)
 						integ_error_found = TRUE;
 						break;
 					}
-					GET_LONGP(&chain, c);
+					GET_BLK_IDP(&chain, c);
 					if (chain.flag)
 					{
 						assert(NULL == input_base_addr);
@@ -253,7 +255,7 @@ void gvcst_blk_build(cw_set_element *cse, sm_uc_ptr_t base_addr, trans_num ctn)
 					break;
 				if (chainptr < ptrtop)
 				{
-					GET_LONGP(&chain, chainptr);
+					GET_BLK_IDP(&chain, chainptr);
 					assert(1 == chain.flag || (skip_block_chain_tail_check && (0 == chain.next_off)));
 					assert(chain.cw_index < sgm_info_ptr->cw_set_depth);
 					offset = chain.next_off;
