@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2019 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -28,7 +28,6 @@
 #include "jnl.h"		/* needed for tp.h */
 #include "gdskill.h"		/* needed for tp.h */
 #include "buddy_list.h"		/* needed for tp.h */
-#include "hashtab_int4.h"	/* needed for tp.h */
 #include "tp.h"			/* needed for T_BEGIN_READ_NONTP_OR_TP macro */
 #include "repl_msg.h"
 #include "gtmsource.h"
@@ -214,11 +213,11 @@ boolean_t gvcst_get2(mval *v, unsigned char *sn_ptr)
 				} else
 				{
 					if (!sn_ptr)
-                                        {
-                                                ENSURE_STP_FREE_SPACE(data_len);
-                                                assert(stringpool.top - stringpool.free >= data_len);
-                                                memcpy(stringpool.free, (sm_uc_ptr_t)rp + rsiz - data_len, data_len);
-                                        } else
+					{
+						ENSURE_STP_FREE_SPACE(data_len);
+						assert(stringpool.top - stringpool.free >= data_len);
+						memcpy(stringpool.free, (sm_uc_ptr_t)rp + rsiz - data_len, data_len);
+					} else
 						memcpy(sn_ptr, (sm_uc_ptr_t)rp + rsiz - data_len, data_len);
 
 					if (!dollar_tlevel)
