@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -33,7 +33,6 @@
 #include "filestruct.h"
 #include "iosp.h"
 #include "jnl.h"
-#include "hashtab_int4.h"	/* needed for tp.h */
 #include "buddy_list.h"		/* needed for tp.h */
 #include "tp.h"
 #include "gtm_stdlib.h"		/* for ATOI */
@@ -74,6 +73,7 @@
 #include "gtm_semutils.h"
 #include "gtm_sem.h"
 #include "is_file_identical.h"
+#include "gvt_inline.h"
 
 #ifdef	GTM_FD_TRACE
 #include "gtm_dbjnl_dupfd_check.h"
@@ -158,7 +158,6 @@ GBLREF	tp_region		*tp_reg_free_list;	/* Ptr to list of tp_regions that are unuse
 GBLREF	tp_region		*tp_reg_list;		/* Ptr to list of tp_regions for this transaction */
 GBLREF	unsigned int		t_tries;
 GBLREF	struct_jrec_tcom	tcom_record;
-GBLREF	boolean_t		tp_in_use;
 GBLREF	uint4			region_open_count;
 GBLREF	sm_uc_ptr_t		reformat_buffer;
 GBLREF	int			reformat_buffer_len;
@@ -305,6 +304,7 @@ void	assert_jrec_member_offsets(void)
 CONDITION_HANDLER(gvcst_init_autoDB_ch)
 {
 	gd_region	*reg;
+	unix_db_info	*udi;
 
 	START_CH(TRUE);
 	if ((SUCCESS == SEVERITY) || (INFO == SEVERITY))
@@ -316,8 +316,11 @@ CONDITION_HANDLER(gvcst_init_autoDB_ch)
 		STATSDBREG_TO_BASEDBREG(db_init_region, reg);
 	else
 		reg = db_init_region;
-	if ((NULL != ftok_sem_reg) && !ftok_sem_release(reg, FALSE, FALSE))	/* release ftok lock on the base region */
+	udi = FILE_INFO(reg);
+	if (udi->grabbed_ftok_sem && !ftok_sem_release(reg, FALSE, FALSE))	/* release ftok lock on the base region */
 		assert(FALSE);
+	if (ftok_sem_reg == reg)
+		ftok_sem_reg = NULL;
 	/* Enable interrupts in case we are here with intrpt_ok_state == INTRPT_IN_GVCST_INIT due to an rts error.
 	 * Normally we would have the new state stored in "prev_intrpt_state" but that is not possible here because
 	 * the corresponding DEFER_INTERRUPTS happened in "gvcst_init" (a different function) so we have an assert
@@ -934,6 +937,7 @@ void gvcst_init(gd_region *reg, gd_addr *addr)
 		csa->mlkctl = NULL;
 		csa->mlkctl_len = 0;
 		csa->mlkhash = NULL;
+		csa->mlkhash_shmid = INVALID_SHMID;
 		csa->gd_ptr = addr ? addr : gd_header;
 		if (csa->gd_ptr)
 			csa->gd_instinfo = csa->gd_ptr->instinfo;
@@ -1166,7 +1170,7 @@ void gvcst_init(gd_region *reg, gd_addr *addr)
 		 */
 		csa->min_total_nontpjnl_rec_size = PINI_RECLEN + MIN_ALIGN_RECLEN;
 	}
-	if (tp_in_use || !IS_GTM_IMAGE)
+	if (!IS_GTM_IMAGE)
 		gvcst_tp_init(reg);	/* Initialize TP structures, else postpone till TP is used (only if GTM) */
 	if (!global_tlvl_info_list)
 	{
