@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -31,7 +31,6 @@
 #include "jnl.h"		/* needed for tp.h */
 #include "gdskill.h"		/* needed for tp.h */
 #include "buddy_list.h"		/* needed for tp.h */
-#include "hashtab_int4.h"	/* needed for tp.h */
 #include "tp.h"			/* needed for T_BEGIN_READ_NONTP_OR_TP macro */
 
 #include "t_end.h"		/* prototypes */
@@ -43,6 +42,7 @@
 #include "gtmimagename.h"
 #include "error.h"
 #include "io.h"
+#include "gvt_inline.h"
 
 GBLREF	gv_key		*gv_currkey, *gv_altkey;
 GBLREF	int4		gv_keysize;
@@ -58,8 +58,8 @@ GBLREF	boolean_t	mupip_jnl_recover;
 # ifdef DEBUG
 GBLREF	boolean_t	is_rcvr_server;
 GBLREF	boolean_t	is_src_server;
-GBLDEF	unsigned char	t_fail_hist_dbg[T_FAIL_HIST_DBG_SIZE];
-GBLDEF	unsigned int	t_tries_dbg;
+GBLREF	unsigned char	t_fail_hist_dbg[T_FAIL_HIST_DBG_SIZE];
+GBLREF	unsigned int	t_tries_dbg;
 # endif
 GBLREF	jnl_gbls_t	jgbl;
 GBLREF	unsigned char	t_fail_hist[CDB_MAX_TRIES];
@@ -137,7 +137,7 @@ static	mstr	global_collation_mstr;
 	}											\
 	if (mu_reorg_process)									\
 	{	/* In case gv_currkey/gv_target are out of sync. */				\
-		rootsrch_ctxt_ptr->gv_currkey = &rootsrch_ctxt_ptr->currkey[0];			\
+		rootsrch_ctxt_ptr->gv_currkey = (gv_key *)&rootsrch_ctxt_ptr->currkey.key;	\
 		MEMCPY_KEY(rootsrch_ctxt_ptr->gv_currkey, gv_currkey);				\
 		SET_GV_CURRKEY_FROM_GVT(reorg_gv_target);					\
 	}											\
@@ -290,8 +290,8 @@ enum cdb_sc gvcst_root_search(boolean_t donot_restart)
 					T_RETRY_AND_CLEANUP(cdb_sc_rmisalign, donot_restart);
 					continue;
 				}
-				GET_LONG(lcl_root, (rp + hdr_len));
-				if (rlen > hdr_len + SIZEOF(block_id))
+				GET_BLK_ID(lcl_root, rp + hdr_len);
+				if (rlen > (hdr_len + SIZEOF(block_id)))
 				{
 					assert(NULL != global_collation_mstr.addr || 0 == global_collation_mstr.len);
 					if (global_collation_mstr.len < rlen - (hdr_len + SIZEOF(block_id)))
@@ -345,7 +345,7 @@ enum cdb_sc gvcst_root_search(boolean_t donot_restart)
 		{
 			assert(NULL != global_collation_mstr.addr);
 			subrec_ptr = get_spec((uchar_ptr_t)global_collation_mstr.addr,
-					      (int)(rlen - (hdr_len + SIZEOF(block_id))), COLL_SPEC);
+						(int)(rlen - (hdr_len + SIZEOF(block_id))), COLL_SPEC);
 			if (subrec_ptr)
 			{
 				gv_target->nct = *(subrec_ptr + COLL_NCT_OFFSET);
