@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -27,16 +27,16 @@
 #include "filestruct.h"
 #include "jnl.h"
 #include "buddy_list.h"		/* needed for tp.h */
-#include "hashtab_int4.h"	/* needed for tp.h and cws_insert.h */
+#include "hashtab_int4.h"
 #include "tp.h"
 #include "gvcst_blk_build.h"
 #include "t_qread.h"
 #include "longset.h"		/* needed for cws_insert.h */
-#include "hashtab.h"		/* needed for cws_insert.h */
 #include "cws_insert.h"
 #include "gvcst_protos.h"	/* for gvcst_search_blk,gvcst_search_tail,gvcst_search prototype */
 #include "min_max.h"
 #include "gvcst_expand_key.h"
+#include "gvt_inline.h"
 
 GBLREF	boolean_t		mu_reorg_process;
 GBLREF	char			gvcst_search_clue;
@@ -100,7 +100,7 @@ enum cdb_sc 	gvcst_search(gv_key *pKey,		/* Key to search for */
 	pTarg = gv_target;
 	assert(NULL != pTarg);
 	assert(pTarg->root);
-	assert(pKey != &pTarg->clue);
+	assert(pKey != (gv_key *)&pTarg->clue);
 	nKeyLen = pKey->end + 1;
 
 	assert(!dollar_tlevel || ((NULL != sgm_info_ptr) && (cs_addrs->sgm_info_ptr == sgm_info_ptr)));
@@ -151,6 +151,7 @@ enum cdb_sc 	gvcst_search(gv_key *pKey,		/* Key to search for */
 				chain1 = *(off_chain *)&leaf_blk_hist->blk_num;
 				if (chain1.flag == 1)
 				{
+					assert((SIZEOF(int) * 8) >= CW_INDEX_MAX_BITS);
 					assert((int)chain1.cw_index < sgm_info_ptr->cw_set_depth);
 					tp_get_cw(sgm_info_ptr->first_cw_set, (int)chain1.cw_index, &cse);
 				} else
@@ -180,6 +181,7 @@ enum cdb_sc 	gvcst_search(gv_key *pKey,		/* Key to search for */
 			chain1 = *(off_chain *)&leaf_blk_hist->blk_num;
 			if (chain1.flag == 1)
 			{
+				assert((SIZEOF(int) * 8) >= CW_INDEX_MAX_BITS);
 				if ((int)chain1.cw_index >= sgm_info_ptr->cw_set_depth)
 				{
 					assert(sgm_info_ptr->tp_csa == cs_addrs);
@@ -339,12 +341,12 @@ enum cdb_sc 	gvcst_search(gv_key *pKey,		/* Key to search for */
 			 * The case (0 == n1) is not expected a lot (relatively) since the application may be able to optimize
 			 *	a number of reads of the same key into one read by using a local-variable to store the value.
 			 */
-			if (0 < (n1 = memcmp(pKey->base, pTarg->clue.base, nKeyLen)))
+			if (0 < (n1 = memcmp(pKey->base, ((gv_key *)&(pTarg->clue))->base, nKeyLen)))
 			{
 				if (memcmp(pKey->base, pTarg->last_rec->base, nKeyLen) <= 0)
 				{
 					SET_GVCST_SEARCH_CLUE(1);
-					status = gvcst_search_tail(pKey, pTargHist->h, &pTarg->clue);
+					status = gvcst_search_tail(pKey, pTargHist->h, (gv_key *)&pTarg->clue);
 					if (cdb_sc_normal == status)
 					{
 						if (NULL == pHist)
@@ -401,7 +403,7 @@ enum cdb_sc 	gvcst_search(gv_key *pKey,		/* Key to search for */
 					return cdb_sc_normal;
 				}
 				leaf_blk_hist = &pTarg->hist.h[0];
-				status = gvcst_expand_prev_key(leaf_blk_hist, &pTarg->clue, gv_altkey);
+				status = gvcst_expand_prev_key(leaf_blk_hist, (gv_key *)&pTarg->clue, gv_altkey);
 				if (cdb_sc_normal == status)
 				{
 					COPY_PREV_KEY_TO_GVT_CLUE(pTarg, TRUE);
@@ -465,10 +467,10 @@ enum cdb_sc 	gvcst_search(gv_key *pKey,		/* Key to search for */
 			assert(CDB_STAGNATE > t_tries);
 			return cdb_sc_rmisalign;
 		}
-		GET_LONG(nBlkId, (pRec + n0 - SIZEOF(block_id)));
+		GET_BLK_ID(nBlkId, pRec + n0 - SIZEOF(block_id));
 		if (is_mm)
 		{
-			PUT_LONG(&chain2, nBlkId);
+			PUT_BLK_ID(&chain2, nBlkId);
 			if ((0 == chain2.flag) && (nBlkId > cs_addrs->total_blks))
 			{	/* private copy should be taken care of by .flag */
 				if (cs_addrs->total_blks < cs_addrs->ti->total_blks)
