@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2013 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -10,6 +11,7 @@
  ****************************************************************/
 
 #include "mdef.h"
+#include "gtm_string.h"
 #include "compiler.h"
 #include "opcode.h"
 #include "toktyp.h"
@@ -169,7 +171,7 @@ int gvn(void)
 	if (shifting)
 	{
 		if (TREF(saw_side_effect) && ((GTM_BOOL != TREF(gtm_fullbool)) || (OLD_SE != TREF(side_effect_handling))))
-		{	/* saw a side effect in a subscript - time to stop shifting */
+		{	/* saw a side effect in a subscript - our reference has been superceded so no targ game on the name */
 			setcurtchain(oldchain);
 			triptr = (TREF(curtchain))->exorder.bl;
 			dqadd(triptr, &tmpchain, exorder);
@@ -177,8 +179,11 @@ int gvn(void)
 		{
 			newtriple(OC_GVSAVTARG);
 			setcurtchain(oldchain);
+			assert(NULL != TREF(expr_start));
+			assert(&tmpchain != tmpchain.exorder.bl);
 			dqadd(TREF(expr_start), &tmpchain, exorder);
 			TREF(expr_start) = tmpchain.exorder.bl;
+			assert(OC_GVSAVTARG == (TREF(expr_start))->opcode);
 			triptr = newtriple(OC_GVRECTARG);
 			triptr->operand[0] = put_tref(TREF(expr_start));
 		}
