@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -48,7 +48,7 @@ GBLREF	int		*ht_sizes;
 
 #define DEBUGHASHTABLE 0
 
-#define CALLERID ((unsigned char *)caller_id())
+#define HASHTAB_CALLERID ((unsigned char *)caller_id(0))
 
 /* For manual calls to raise_gtmmemory_error() */
 #define SET_GTMMEMORY_ERROR_VARS(SIZE, ERROR)		\
@@ -58,7 +58,7 @@ GBLREF	int		*ht_sizes;
 	GBLREF	int		gtmMallocErrorErrno;	\
 							\
 	gtmMallocErrorSize = SIZE;			\
-	gtmMallocErrorCallerid = CALLERID;		\
+	gtmMallocErrorCallerid = HASHTAB_CALLERID;	\
 	gtmMallocErrorErrno = ERROR;			\
 }
 
@@ -762,17 +762,18 @@ sm_uc_ptr_t COPY_HASHTAB_TO_BUFFER(HASH_TABLE *table, sm_uc_ptr_t buffer, int (*
  * The buffer is expected to stay alive for the duration of
  *  the life of the hastable; we don't perform an extra copy
  * @param [in] buffer the written hashtable value; note that the caller is responsible for making the buffer
- * 	is large enough for the hash table, hash table entries, and the values the entries point too
+ * 	is large enough for the hash table, hash table entries, and the values the entries point to
  * @param [in] copy_entry_from_buffer function pointed to returns an int representing bits used in the buffer,
  * 	and takes the current hash table entry along with a pointer to the buffer. It should fill out the hash
  * 	table entry
  */
 HASH_TABLE *ACTIVATE_HASHTAB_IN_BUFFER(sm_uc_ptr_t buffer, int (*copy_entry_from_buffer)(HT_ENT *, sm_uc_ptr_t))
 {
-	int i;
-	HT_ENT *cur, *top;
+	int		i;
+	HT_ENT		*cur, *top;
+	HASH_TABLE	*table;
 
-	HASH_TABLE *table = (HASH_TABLE *)buffer;
+	table = (HASH_TABLE *)buffer;
 	if (TRUE == table->active)
 		return table;
 	buffer += SIZEOF(HASH_TABLE);
