@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2019 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -127,6 +127,7 @@ typedef struct io_desc_struct
 	unsigned int			width;
 	bool				perm;		/* permanent	*/
 	bool				wrap;		/* if FALSE trunc */
+	boolean_t                       fflf;           /* if TRUE "write #" emits FF+LF on disk files, otherwise bare LF */
 	enum io_dev_type		type;
 	enum io_dev_state		state;
 	struct
@@ -530,50 +531,55 @@ MBSTART {											\
 
 #define SET_DOLLARDEVICE_ERRSTR(IOD, ERRSTR)							\
 MBSTART {											\
-	int errlen;										\
-	char *writebuffer;									\
-	errlen = STRLEN(ERRSTR) + 1;	/* +1 for NULL */					\
-	ALLOCATE_DOLLAR_DEVICE_BUFFER_IF_REQUIRED(IOD, errlen, writebuffer);			\
-	memcpy(writebuffer, ERRSTR, errlen);							\
+	char	*WRITEBUFFER;									\
+	int	LEN;										\
+												\
+	LEN = STRLEN(ERRSTR) + 1;	/* +1 for NULL */					\
+	ALLOCATE_DOLLAR_DEVICE_BUFFER_IF_REQUIRED(IOD, LEN, WRITEBUFFER);			\
+	memcpy(WRITEBUFFER, ERRSTR, LEN);							\
 } MBEND
 
-#define SET_DOLLARDEVICE_ONECOMMA_ERRSTR(IOD, ERRSTR)						\
+#define SET_DOLLARDEVICE_ONECOMMA_ERRSTR(IOD, ERRSTR, ERRLEN)					\
 MBSTART {											\
-	char *writebuffer;									\
-	int errlen, prefixlen, len;								\
-	prefixlen = STR_LIT_LEN(ONE_COMMA);							\
-	errlen = STRLEN(ERRSTR) + 1;	/* +1 for NULL */					\
-	len = prefixlen + errlen;								\
-	ALLOCATE_DOLLAR_DEVICE_BUFFER_IF_REQUIRED(IOD, len, writebuffer);			\
-	memcpy(writebuffer, ONE_COMMA, prefixlen);						\
-	memcpy(&writebuffer[prefixlen], ERRSTR, errlen);					\
+	char	*WRITEBUFFER;									\
+	int	LEN;										\
+												\
+	ERRLEN = STRLEN(ERRSTR);								\
+	LEN = STR_LIT_LEN(ONE_COMMA) + ERRLEN + 1;	/* +1 for NULL */			\
+	ALLOCATE_DOLLAR_DEVICE_BUFFER_IF_REQUIRED(IOD, LEN, WRITEBUFFER);			\
+	memcpy(WRITEBUFFER, ONE_COMMA, STR_LIT_LEN(ONE_COMMA));					\
+	WRITEBUFFER += STR_LIT_LEN(ONE_COMMA);							\
+	memcpy(WRITEBUFFER, ERRSTR, ERRLEN + 1);						\
 } MBEND
 
-#define SET_DOLLARDEVICE_ONECOMMA_ERRSTR1_ERRSTR2(IOD, ERRSTR1, ERRSTR2)			\
+#define SET_DOLLARDEVICE_ONECOMMA_ERRSTR1_ERRSTR2(IOD, ERRSTR1, ERRLEN1, ERRSTR2, ERRLEN2)	\
 MBSTART {											\
-	char *writebuffer;									\
-	int errlen1, errlen2, prefixlen, len;							\
-	prefixlen = STR_LIT_LEN(ONE_COMMA);							\
-	errlen1 = STRLEN(ERRSTR1);	/* No NULL char */					\
-	errlen2 = STRLEN(ERRSTR2) + 1;	/* +1 for NULL */					\
-	len = prefixlen + errlen1 + errlen2;							\
-	ALLOCATE_DOLLAR_DEVICE_BUFFER_IF_REQUIRED(IOD, len, writebuffer);			\
-	memcpy(writebuffer, ONE_COMMA, prefixlen);						\
-	memcpy(&writebuffer[prefixlen], ERRSTR1, errlen1);					\
-	memcpy(&writebuffer[prefixlen + errlen1], ERRSTR2, errlen2);				\
+	char	*WRITEBUFFER;									\
+	int	LEN;										\
+												\
+	ERRLEN1 = STRLEN(ERRSTR1);	/* No NULL char */					\
+	ERRLEN2 = STRLEN(ERRSTR2);								\
+	LEN = STR_LIT_LEN(ONE_COMMA) + ERRLEN1 + ERRLEN2 + 1; /* +1 for NULL */			\
+	ALLOCATE_DOLLAR_DEVICE_BUFFER_IF_REQUIRED(IOD, LEN, WRITEBUFFER);			\
+	memcpy(WRITEBUFFER, ONE_COMMA, STR_LIT_LEN(ONE_COMMA));					\
+	WRITEBUFFER += STR_LIT_LEN(ONE_COMMA);							\
+	memcpy(WRITEBUFFER, ERRSTR1, ERRLEN1);							\
+	WRITEBUFFER += ERRLEN1;									\
+	memcpy(WRITEBUFFER, ERRSTR2, ERRLEN2 + 1);						\
 } MBEND
 
 #define SET_DOLLARDEVICE_ONECOMMA_STRERROR(IOD, ERRNO)						\
 MBSTART {											\
-	char *errstring, *writebuffer;								\
-	int errlen, prefixlen, len;								\
-	prefixlen = STR_LIT_LEN(ONE_COMMA);							\
-	errstring = STRERROR(ERRNO);								\
-	errlen = STRLEN(errstring) + 1;	/* +1 for NULL */					\
-	len = prefixlen + errlen;								\
-	ALLOCATE_DOLLAR_DEVICE_BUFFER_IF_REQUIRED(IOD, len, writebuffer);			\
-	memcpy(writebuffer, ONE_COMMA, prefixlen);						\
-	memcpy(&writebuffer[prefixlen], errstring, errlen);					\
+	char	*ERRSTR, *WRITEBUFFER;								\
+	int	ERRLEN, LEN;									\
+												\
+	ERRSTR = STRERROR(ERRNO);								\
+	ERRLEN = STRLEN(ERRSTR);								\
+	LEN = STR_LIT_LEN(ONE_COMMA) + ERRLEN + 1; /* +1 for NULL */				\
+	ALLOCATE_DOLLAR_DEVICE_BUFFER_IF_REQUIRED(IOD, LEN, WRITEBUFFER);			\
+	memcpy(WRITEBUFFER, ONE_COMMA, STR_LIT_LEN(ONE_COMMA));					\
+	WRITEBUFFER += STR_LIT_LEN(ONE_COMMA);							\
+	memcpy(WRITEBUFFER, ERRSTR, ERRLEN + 1);						\
 } MBEND
 
 #define PUT_DOLLAR_DEVICE_INTO_MSTR(IOD, MSTR)							\
@@ -589,4 +595,49 @@ MBSTART {											\
 		MSTR->len = STRLEN(IOD->dollar.devicebuffer);					\
 	}											\
 } MBEND
+
+/* Set prin_*_dev_failure if a read or write failed on the principal device. If it is a recurrence,
+ * issue the NOPRINCIO error.
+ */
+#define ISSUE_NOPRINCIO_IF_NEEDED(IOD, WRITE, SOCNOERR)							\
+MBSTART {												\
+	/* the following literal and the game played with it deal with the suppression of the */	\
+	/* maintenance of $ZSTATUS while in direct mode */						\
+	LITDEF mstr    dm_did_it = {0, LEN_AND_LIT("Direct Mode activity")};				\
+	LITDEF mstr    silent_soc = {0, LEN_AND_LIT("socket with IOERROR disabled")};			\
+	mval dev, zpos, zstatus;									\
+													\
+	if ((IOD) == ((WRITE) ? io_std_device.out : io_std_device.in))					\
+	{												\
+		if ((WRITE) ? prin_out_dev_failure : prin_in_dev_failure)				\
+		{											\
+			util_out_print("", RESET);	/* Reset output buffer */			\
+			op_svget(SV_ZPOS, &zpos);							\
+			if (memcmp("+1^GTM$DMOD", zpos.str.addr, STR_LIT_LEN("+1^GTM$DMOD")))		\
+			{	/* not in base direct mode so do noisy exit */				\
+				dev.str.len = (WRITE) ? io_std_device.out->trans_name->len		\
+					: io_std_device.in->trans_name->len;				\
+				dev.str.addr = (WRITE) ? io_std_device.out->trans_name->dollar_io	\
+					: io_std_device.in->trans_name->dollar_io;			\
+				if (SOCNOERR)								\
+					zstatus.str = silent_soc;	/* suppressing socket errors */	\
+				else									\
+					zstatus.str = dollar_zstatus.str;				\
+				if (0 == zstatus.str.len)						\
+					zstatus.str = dm_did_it;	/* in case there's no zstatus */\
+				send_msg_csa(CSA_ARG(NULL) VARLSTCNT(10) ERR_NOPRINCIO, 8, 		\
+					RTS_ERROR_STRING((WRITE) ? "WRITE to" : "READ from"),		\
+					dev.str.len, dev.str.addr, zpos.str.len, zpos.str.addr,		\
+					zstatus.str.len, zstatus.str.addr);				\
+			} else if (!prin_out_dev_failure)	/* in direct mode so go quietly */	\
+				flush_pio();			/* unless output is still OK */		\
+			stop_image_no_core();								\
+		}											\
+		if (WRITE)										\
+			prin_out_dev_failure = TRUE;							\
+		else											\
+			prin_in_dev_failure = TRUE;							\
+	}												\
+} MBEND
+
 #endif /* IO_H */
