@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2015 Fidelity National Information	*
+ * Copyright (c) 2001-2019 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -179,8 +179,8 @@ IOP_DESC(160, iop_ioerror, IOP_VAR_SIZE, IOP_OPEN_OK | IOP_USE_OK, IOP_SRC_STR),
 IOP_DESC(161, iop_attach, IOP_VAR_SIZE, IOP_OPEN_OK | IOP_USE_OK, IOP_SRC_STR),
 IOP_DESC(162, iop_detach, IOP_VAR_SIZE, IOP_USE_OK, IOP_SRC_STR),
 IOP_DESC(163, iop_zlisten, IOP_VAR_SIZE, IOP_OPEN_OK | IOP_USE_OK, IOP_SRC_STR),
-IOP_DESC(164, iop_ipchset, IOP_VAR_SIZE, IOP_OPEN_OK, IOP_SRC_STR),
-IOP_DESC(165, iop_opchset, IOP_VAR_SIZE, IOP_OPEN_OK, IOP_SRC_STR),
+IOP_DESC(164, iop_ipchset, IOP_VAR_SIZE, IOP_OPEN_OK | IOP_USE_OK, IOP_SRC_STR),
+IOP_DESC(165, iop_opchset, IOP_VAR_SIZE, IOP_OPEN_OK | IOP_USE_OK, IOP_SRC_STR),
 IOP_DESC(166, iop_nodelimiter, 0, IOP_OPEN_OK | IOP_USE_OK, 0),
 IOP_DESC(167, iop_zdelay, 0, IOP_OPEN_OK | IOP_USE_OK, 0),
 IOP_DESC(168, iop_znodelay, 0, IOP_OPEN_OK | IOP_USE_OK, 0),
@@ -201,7 +201,7 @@ IOP_DESC(182, iop_utf16, 0, IOP_OPEN_OK, 0),
 IOP_DESC(183, iop_utf16be, 0, IOP_OPEN_OK, 0),
 IOP_DESC(184, iop_utf16le, 0, IOP_OPEN_OK, 0),
 IOP_DESC(185, iop_pad, SIZEOF(int4), IOP_OPEN_OK, IOP_SRC_INT),
-IOP_DESC(186, iop_chset, IOP_VAR_SIZE, IOP_OPEN_OK, IOP_SRC_STR),
+IOP_DESC(186, iop_chset, IOP_VAR_SIZE, IOP_OPEN_OK | IOP_USE_OK, IOP_SRC_STR),
 IOP_DESC(187, iop_morereadtime, SIZEOF(int4), IOP_OPEN_OK | IOP_USE_OK, IOP_SRC_INT),
 IOP_DESC(188, iop_shell, IOP_VAR_SIZE, IOP_OPEN_OK, IOP_SRC_STR),
 IOP_DESC(189, iop_command, IOP_VAR_SIZE, IOP_OPEN_OK, IOP_SRC_STR),
@@ -225,4 +225,8 @@ IOP_DESC(206, iop_outseek, IOP_VAR_SIZE, IOP_OPEN_OK | IOP_USE_OK, IOP_SRC_STR),
 IOP_DESC(207, iop_inrewind, 0, IOP_OPEN_OK | IOP_USE_OK, 0),
 IOP_DESC(208, iop_outrewind, 0, IOP_OPEN_OK | IOP_USE_OK, 0),
 IOP_DESC(209, iop_buffered, 0, IOP_OPEN_OK | IOP_USE_OK, 0),
-IOP_DESC(210, n_iops, 0, 0, 0)
+IOP_DESC(210, iop_fflf, 0, IOP_OPEN_OK | IOP_USE_OK, 0),
+IOP_DESC(211, iop_nofflf, 0, IOP_OPEN_OK | IOP_USE_OK, 0),
+IOP_DESC(212, iop_hupenable, 0, IOP_USE_OK, 0),
+IOP_DESC(213, iop_nohupenable, 0, IOP_USE_OK, 0),
+IOP_DESC(214, n_iops, 0, 0, 0)
