@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2019 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -51,13 +51,16 @@ boolean_t iosocket_bind(socket_struct *socketptr, int4 msec_timeout, boolean_t u
 	d_socket_struct		*dsocketptr;
 	struct addrinfo		*ai_ptr;
 	char			port_buffer[NI_MAXSERV];
-	int			errcode;
+	int			errcode, keepalive_opt;
 	ABS_TIME		cur_time, end_time;
 	GTM_SOCKLEN_TYPE	addrlen;
 	GTM_SOCKLEN_TYPE	sockbuflen;
 	struct stat		statbuf;
 	mode_t			filemode;
+	static readonly char 	action[] = "BIND";
+	DCL_THREADGBL_ACCESS;
 
+	SETUP_THREADGBL_ACCESS;
 	dsocketptr = socketptr->dev;
 	ai_ptr = (struct addrinfo*)(&socketptr->local.ai);
 	assert(NULL != dsocketptr);
@@ -203,12 +206,11 @@ boolean_t iosocket_bind(socket_struct *socketptr, int4 msec_timeout, boolean_t u
 			{
 				real_errno = errno;
 				errptr = (char *)STRERROR(real_errno);
-				SET_DOLLARDEVICE_ONECOMMA_ERRSTR(dsocketptr->iod, errptr);
+				SET_DOLLARDEVICE_ONECOMMA_ERRSTR(dsocketptr->iod, errptr, errlen);
 				if (ioerror)
 				{
 					SOCKET_FREE(socketptr);
-					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(5) ERR_SOCKINIT, 3, real_errno, STRLEN(errptr),
-						errptr);
+					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(5) ERR_SOCKINIT, 3, real_errno, errlen, errptr);
 				}
 				return FALSE;
 			}
@@ -220,11 +222,11 @@ boolean_t iosocket_bind(socket_struct *socketptr, int4 msec_timeout, boolean_t u
 	{
 		real_errno = errno;
 		errptr = (char *)STRERROR(real_errno);
-		SET_DOLLARDEVICE_ONECOMMA_ERRSTR(dsocketptr->iod, errptr);
+		SET_DOLLARDEVICE_ONECOMMA_ERRSTR(dsocketptr->iod, errptr, errlen);
 		if (ioerror)
 		{
 			SOCKET_FREE(socketptr);
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(5) ERR_GETSOCKNAMERR, 3, real_errno, STRLEN(errptr), errptr);
+			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(5) ERR_GETSOCKNAMERR, 3, real_errno, errlen, errptr);
 		}
 		return FALSE;
 	}
@@ -236,11 +238,11 @@ boolean_t iosocket_bind(socket_struct *socketptr, int4 msec_timeout, boolean_t u
 		{
 			real_errno = errno;
 			TEXT_ADDRINFO(errptr, errcode, real_errno);
-			SET_DOLLARDEVICE_ONECOMMA_ERRSTR(dsocketptr->iod, errptr);
+			SET_DOLLARDEVICE_ONECOMMA_ERRSTR(dsocketptr->iod, errptr, errlen);
 			if (ioerror)
 			{
 				SOCKET_FREE(socketptr);
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_GETNAMEINFO, 0, ERR_TEXT, 2, STRLEN(errptr), errptr);
+				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_GETNAMEINFO, 0, ERR_TEXT, 2, errlen, errptr);
 			}
 			return FALSE;
 		}
@@ -248,6 +250,9 @@ boolean_t iosocket_bind(socket_struct *socketptr, int4 msec_timeout, boolean_t u
 		if (0 == socketptr->local.port)
 			socketptr->local.port = actual_port;
 		assert(socketptr->local.port == actual_port);
+		keepalive_opt = TREF(gtm_socket_keepalive_idle);	/* deviceparameter would give more granular control */
+		if (keepalive_opt && !iosocket_tcp_keepalive(socketptr, keepalive_opt, action))
+			return FALSE;				/* iosocket_tcp_keepalive issues rts_error rather than return */
 	} else
 	{
 		if (socketptr->filemode_mask)
@@ -261,11 +266,11 @@ boolean_t iosocket_bind(socket_struct *socketptr, int4 msec_timeout, boolean_t u
 			{
 				real_errno = errno;
 				errptr = (char *)STRERROR(real_errno);
-				SET_DOLLARDEVICE_ONECOMMA_ERRSTR(dsocketptr->iod, errptr);
+				SET_DOLLARDEVICE_ONECOMMA_ERRSTR(dsocketptr->iod, errptr, errlen);
 				if (ioerror)
 				{
 					SOCKET_FREE(socketptr);
-					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(9) ERR_SOCKINIT, 3, real_errno, STRLEN(errptr),
+					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(9) ERR_SOCKINIT, 3, real_errno, errlen,
 						errptr, ERR_TEXT, 2, RTS_ERROR_LITERAL("setting protection"));
 				}
 				return FALSE;
@@ -278,11 +283,11 @@ boolean_t iosocket_bind(socket_struct *socketptr, int4 msec_timeout, boolean_t u
 			{
 				real_errno = errno;
 				errptr = (char *)STRERROR(real_errno);
-				SET_DOLLARDEVICE_ONECOMMA_ERRSTR(dsocketptr->iod, errptr);
+				SET_DOLLARDEVICE_ONECOMMA_ERRSTR(dsocketptr->iod, errptr, errlen);
 				if (ioerror)
 				{
 					SOCKET_FREE(socketptr);
-					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(9) ERR_SOCKINIT, 3, real_errno, STRLEN(errptr),
+					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(9) ERR_SOCKINIT, 3, real_errno, errlen,
 						errptr, ERR_TEXT, 2, RTS_ERROR_LITERAL("setting ownership"));
 				}
 				return FALSE;
