@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2014 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2019 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -20,13 +21,10 @@
 
 #include "gtm_string.h"
 #include "gtm_iconv.h"
-#ifndef  VMS
 #include "gtm_stat.h"
 #include "gtmio.h"
-#endif
 #include "gtm_stdio.h"
 #include "gtm_unistd.h"
-
 #include "gtm_socket.h"
 #include "gtm_inet.h"
 
@@ -37,12 +35,16 @@
 #include "iosocketdef.h"
 #include "stringpool.h"
 #include "eintr_wrappers.h"
+#ifdef GTM_TLS
+#include "gtm_tls.h"
+#endif
+#include "error.h"
+#include "op.h"
+#include "indir_enum.h"
 
 GBLREF	io_desc		*active_device;
 GBLREF	int		process_exiting;
-#ifndef VMS
 GBLREF	boolean_t	gtm_pipe_child;
-#endif
 
 LITREF unsigned char		io_params_size[];
 
@@ -50,6 +52,7 @@ error_def(ERR_CLOSEFAIL);
 error_def(ERR_FILEOPENFAIL);
 error_def(ERR_SOCKNOTFND);
 error_def(ERR_SYSCALL);
+error_def(ERR_TLSIOERROR);
 
 void iosocket_close_range(d_socket_struct *dsocketptr, int start, int end, boolean_t socket_delete, boolean_t socket_specified);
 
@@ -59,23 +62,25 @@ void iosocket_close(io_desc *iod, mval *pp)
 	unsigned char	ch;
 	int		handle_len;
 	d_socket_struct	*dsocketptr;
-	char		sock_handle[MAX_HANDLE_LEN];
+	char		sock_handle[MAX_HANDLE_LEN], *errp;
 	int4		start, end, index;
 	int		p_offset = 0;
 	boolean_t	socket_destroy = FALSE;
 	boolean_t	socket_delete = FALSE;
+	boolean_t	ch_set;
+	DCL_THREADGBL_ACCESS;
 
+	SETUP_THREADGBL_ACCESS;
 	assert(iod->type == gtmsocket);
 	dsocketptr = (d_socket_struct *)iod->dev_sp;
+	ESTABLISH_GTMIO_CH(&iod->pair, ch_set);
 
 	while (iop_eol != (ch = *(pp->str.addr + p_offset++)))
 	{
 		switch (ch)
 		{
 		case iop_exception:
-			iod->error_handler.len = (int)(*(pp->str.addr + p_offset));
-			iod->error_handler.addr = (char *)(pp->str.addr + p_offset + 1);
-			s2pool(&iod->error_handler);
+			DEF_EXCEPTION(pp, p_offset, iod);
 			break;
 		case iop_socket:
 			handle_len = (int)(*(pp->str.addr + p_offset));
@@ -84,8 +89,8 @@ void iosocket_close(io_desc *iod, mval *pp)
 			socket_specified = TRUE;
 			break;
 		case iop_ipchset:
-#			if defined(KEEP_zOS_EBCDIC) || defined(VMS)
-			if ( (iconv_t)0 != iod->input_conv_cd )
+#			if defined(KEEP_zOS_EBCDIC)
+			if ((iconv_t)0 != iod->input_conv_cd)
 			{
 				ICONV_CLOSE_CD(iod->input_conv_cd);
 			}
@@ -94,9 +99,9 @@ void iosocket_close(io_desc *iod, mval *pp)
 				ICONV_OPEN_CD(iod->input_conv_cd, INSIDE_CH_SET, (char *)(pp->str.addr + p_offset + 1));
 #			endif
 			break;
-                case iop_opchset:
-#			if defined(KEEP_zOS_EBCDIC) || defined(VMS)
-			if ( (iconv_t)0 != iod->output_conv_cd )
+		case iop_opchset:
+#			if defined(KEEP_zOS_EBCDIC)
+			if ((iconv_t)0 != iod->output_conv_cd)
 			{
 				ICONV_CLOSE_CD(iod->output_conv_cd);
 			}
@@ -111,11 +116,9 @@ void iosocket_close(io_desc *iod, mval *pp)
 		case iop_nodestroy:
 			socket_destroy = FALSE;
 			break;
-#		ifndef VMS
 		case iop_delete:
 			socket_delete = TRUE;
 			break;
-#		endif
 		default:
 			break;
 		}
@@ -137,6 +140,12 @@ void iosocket_close(io_desc *iod, mval *pp)
 		end = 0;
 	}
 	iosocket_close_range(dsocketptr, start, end, socket_delete, socket_specified);
+	/* "forget" the UTF-16 CHSET variant if no sockets are connected */
+	if (0 >= dsocketptr->n_socket)
+	{
+		dsocketptr->ichset_utf16_variant = iod->ichset = 0;
+		dsocketptr->ochset_utf16_variant = iod->ochset = 0;
+	}
 	if (!socket_specified)
 	{
 		iod->state = dev_closed;
@@ -146,42 +155,64 @@ void iosocket_close(io_desc *iod, mval *pp)
 			iosocket_destroy(iod);
 		}
 	}
+	REVERT_GTMIO_CH(&iod->pair, ch_set);
 }
 
 void iosocket_close_range(d_socket_struct *dsocketptr, int start, int end, boolean_t socket_delete, boolean_t socket_specified)
 {
 	int4		ii,jj;
 	int		rc, save_fd, save_rc = 0, save_errno;
+	ssize_t		status;
 	socket_struct	*socketptr;
-#	ifndef VMS
 	struct stat	statbuf, fstatbuf;
 	char		*path;
 	int		res;
 	int		null_fd = 0;
-#	endif
 
 	for (ii = start; ii >= end; ii--)
 	{
 		socketptr = dsocketptr->socket[ii];
-#		ifndef VMS
-		/* Don't reap if in a child process creating a new job or pipe device */
-		if ((socket_local == socketptr->protocol) && socketptr->passive && !gtm_pipe_child)
-		{	/* only delete if passive/listening */
-			assertpro(socketptr->local.sa);
-			path = ((struct sockaddr_un *)(socketptr->local.sa))->sun_path;
-			FSTAT_FILE(socketptr->sd, &fstatbuf, res);
-			if (-1 == res)
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_SYSCALL, 5,
-					LEN_AND_LIT("fstat during socket delete"), CALLFROM, errno);
-			STAT_FILE(path, &statbuf, res);
-			if (-1 == res)
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_SYSCALL, 5,
-					LEN_AND_LIT("stat during socket delete"), CALLFROM, errno);
-			if (UNLINK(path) == -1)
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_SYSCALL, 5,
-					LEN_AND_LIT("unlink during socket delete"), CALLFROM, errno);
+		if (!gtm_pipe_child)
+		{	/* Don't reap if in a child process creating a new job or pipe device */
+			if ((socket_local == socketptr->protocol) && socketptr->passive)
+			{	/* only delete if passive/listening */
+				assertpro(socketptr->local.sa);
+				path = ((struct sockaddr_un *)(socketptr->local.sa))->sun_path;
+				FSTAT_FILE(socketptr->sd, &fstatbuf, res);
+				if (-1 == res)
+					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_SYSCALL, 5,
+						LEN_AND_LIT("fstat during socket delete"), CALLFROM, errno);
+				STAT_FILE(path, &statbuf, res);
+				if (-1 == res)
+					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_SYSCALL, 5,
+						LEN_AND_LIT("stat during socket delete"), CALLFROM, errno);
+				if (UNLINK(path) == -1)
+					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_SYSCALL, 5,
+						LEN_AND_LIT("unlink during socket delete"), CALLFROM, errno);
+			}
+			/* below is similar to iosocket_flush but socketptr may not be current socket */
+			if (socketptr->obuffer_timer_set)
+			{
+				cancel_timer((TID)socketptr);
+				socketptr->obuffer_timer_set = FALSE;
+			}
+			status = 1;		/* OK value */
+			if ((0 < socketptr->obuffer_length) && (0 == socketptr->obuffer_errno))
+			{
+				socketptr->obuffer_output_active = TRUE;
+				status = iosocket_output_buffer(socketptr);
+				socketptr->obuffer_output_active = FALSE;
+			}
+			if ((0 < socketptr->obuffer_size) && ((0 >= status) || (0 != socketptr->obuffer_errno)))
+				iosocket_buffer_error(socketptr);	/* pre-existing error or error flushing buffer */
+#			ifdef GTM_TLS
+			if (socketptr->tlsenabled)
+			{
+				gtm_tls_session_close((gtm_tls_socket_t **)&socketptr->tlssocket);
+				socketptr->tlsenabled = FALSE;
+			}
+#			endif
 		}
-#		endif
 		CLOSE(socketptr->sd, rc);
 		if (-1 == rc)
 		{
@@ -189,7 +220,6 @@ void iosocket_close_range(d_socket_struct *dsocketptr, int start, int end, boole
 			save_fd = socketptr->sd;
 			save_errno = errno;
 		}
-#		ifndef VMS
 		else if (!process_exiting && (3 > socketptr->sd))
 		{
 			OPENFILE("/dev/null", O_RDWR, null_fd);
@@ -199,7 +229,6 @@ void iosocket_close_range(d_socket_struct *dsocketptr, int start, int end, boole
 			}
 			assert(socketptr->sd == null_fd);
 		}
-#		endif
 		SOCKET_FREE(socketptr);
 		if (dsocketptr->current_socket >= ii)
 			dsocketptr->current_socket--;
@@ -211,12 +240,10 @@ void iosocket_close_range(d_socket_struct *dsocketptr, int start, int end, boole
 	{
 		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_CLOSEFAIL, 1, save_fd, save_errno);
 	}
-#	ifndef VMS
 	else if (-1 == null_fd)
 	{
 		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_FILEOPENFAIL, 2, LIT_AND_LEN("/dev/null"), save_errno, 0);
 	}
-#	endif
 }
 
 void iosocket_close_one(d_socket_struct *dsocketptr, int index)
