@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2012 Fidelity Information Services, Inc	*
+ * Copyright (c) 2012-2018 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -16,8 +17,6 @@
 #include "gtm_socket.h"
 
 #include "io.h"
-#include "iotcpdef.h"
-#include "iotcproutine.h"
 #include "iosocketdef.h"
 #include "gtmio.h"
 
@@ -30,10 +29,15 @@ void iosocket_destroy (io_desc *ciod)
 	assertpro(ciod->type == gtmsocket);
 	assertpro(ciod->state == dev_closed);
 	dsocketptr = (d_socket_struct *) ciod->dev_sp;
-	assertpro(dsocketptr != NULL);
+	assertpro(NULL != dsocketptr);
 	for (lpp = &io_root_log_name, lp = *lpp; lp; lp = *lpp)
 	{
-		if (lp->iod->pair.in == ciod)
+		if ((NULL == lp->iod) || (n_io_dev_types == lp->iod->type))
+		{
+			assert(NULL == lp->iod);	/* Can be NULL if we are forced to exit during device setup */
+			/* skip it on pro */
+		}
+		else if (lp->iod->pair.in == ciod)
 		{
 			/* The only device that may be "split" is the principal device. Since it is permanently open,
 			 * it will never get here.
@@ -42,9 +46,14 @@ void iosocket_destroy (io_desc *ciod)
 			assert(lp->iod->pair.out == ciod);
 			*lpp = (*lpp)->next;
 			free(lp);
+			continue;	/* lpp already points at next, so skip the dereference below. */
 		}
-		else
-			lpp = &lp->next;
+		lpp = &lp->next;
+	}
+	if (ciod->dollar.devicebuffer)
+	{
+		free(ciod->dollar.devicebuffer);
+		ciod->dollar.devicebuffer = NULL;
 	}
 	free(dsocketptr);
 	free(ciod);
