@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2013 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -26,21 +27,21 @@
 #include "io.h"
 #include "gt_timer.h"
 #include "iosocketdef.h"
+#include "error.h"
 
 error_def(ERR_CURRSOCKOFR);
 error_def(ERR_LISTENPASSBND);
-error_def(ERR_LQLENGTHNA);
 error_def(ERR_NOSOCKETINDEV);
 error_def(ERR_SOCKLISTEN);
 error_def(ERR_TEXT);
 
 #define LISTENING		"LISTENING"
-#define MAX_LISTEN_QUEUE_LENGTH	5
 
 boolean_t iosocket_listen(io_desc *iod, unsigned short len)
 {
 	d_socket_struct	*dsocketptr;
 	socket_struct	*socketptr;
+	boolean_t	result, ch_set;
 
 	assert(gtmsocket == iod->type);
 	dsocketptr = (d_socket_struct *)iod->dev_sp;
@@ -57,7 +58,10 @@ boolean_t iosocket_listen(io_desc *iod, unsigned short len)
 	socketptr = dsocketptr->socket[dsocketptr->current_socket];
 	assert(socketptr && (socketptr->dev == dsocketptr));
 
-	return iosocket_listen_sock(socketptr, len);
+	ESTABLISH_RET_GTMIO_CH(&iod->pair, FALSE, ch_set);
+	result = iosocket_listen_sock(socketptr, len);
+	REVERT_GTMIO_CH(&iod->pair, ch_set);
+	return result;
 }
 
 boolean_t iosocket_listen_sock(socket_struct *socketptr, unsigned short len)
@@ -66,11 +70,6 @@ boolean_t iosocket_listen_sock(socket_struct *socketptr, unsigned short len)
 	int4		errlen;
 	d_socket_struct	*dsocketptr;
 
-	if (MAX_LISTEN_QUEUE_LENGTH < len)
-	{
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(3) ERR_LQLENGTHNA, 1, len);
-		return FALSE;
-	}
 	if (((socketptr->state != socket_bound) && (socketptr->state != socket_listening)) || (TRUE != socketptr->passive))
 	{
 		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_LISTENPASSBND);
@@ -93,14 +92,10 @@ boolean_t iosocket_listen_sock(socket_struct *socketptr, unsigned short len)
 	memcpy(&dsocketptr->iod->dollar.key[len], socketptr->handle, socketptr->handle_len);
 	len += socketptr->handle_len;
 	dsocketptr->iod->dollar.key[len++] = '|';
-	if (socket_local != socketptr->protocol)
-		SPRINTF(&dsocketptr->iod->dollar.key[len], "%d", socketptr->local.port);
-#	ifndef VMS
+	if (socket_local == socketptr->protocol)
+		SNPRINTF(&dsocketptr->iod->dollar.key[len], DD_BUFLEN - len, "%s",
+					((struct sockaddr_un *)(socketptr->local.sa))->sun_path);
 	else
-	{
-		STRNCPY_STR(&dsocketptr->iod->dollar.key[len],
-			((struct sockaddr_un *)(socketptr->local.sa))->sun_path, DD_BUFLEN - len - 1);
-	}
-#	endif
+		SNPRINTF(&dsocketptr->iod->dollar.key[len], DD_BUFLEN - len, "%d", socketptr->local.port);
 	return TRUE;
 }
