@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2012-2018 Fidelity National Information	*
+ * Copyright (c) 2012-2019 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -37,6 +37,7 @@
 #include "error.h"
 #include "op.h"
 #include "indir_enum.h"
+#include "min_max.h"
 
 GBLREF	d_socket_struct		*socket_pool, *newdsocket;
 GBLREF	io_pair			io_std_device;	/* standard device */
@@ -209,7 +210,7 @@ short	iosocket_open(io_log_name *dev, mval *pp, int file_des, mval *mspace, int4
 						rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_DELIMSIZNA);
 					break;
 				case iop_ipchset:
-					UNICODE_ONLY(
+					UTF8_ONLY(
 						if (gtm_utf8_mode)
 						{	/* Only change ipchset if in UTF8 mode */
 							GET_ADDR_AND_LEN(chset_mstr.addr, chset_mstr.len);
@@ -219,7 +220,7 @@ short	iosocket_open(io_log_name *dev, mval *pp, int file_des, mval *mspace, int4
 					);
 					break;
 				case iop_opchset:
-					UNICODE_ONLY(
+					UTF8_ONLY(
 						if (gtm_utf8_mode)
 						{       /* Only change ipchset if in UTF8 mode */
 							GET_ADDR_AND_LEN(chset_mstr.addr, chset_mstr.len);
@@ -229,7 +230,7 @@ short	iosocket_open(io_log_name *dev, mval *pp, int file_des, mval *mspace, int4
 					);
 					break;
 				case iop_chset:
-					UNICODE_ONLY(
+					UTF8_ONLY(
 						if (gtm_utf8_mode)
 						{       /* Only change ipchset/opchset if in UTF8 mode */
 							GET_ADDR_AND_LEN(chset_mstr.addr, chset_mstr.len);
@@ -243,13 +244,13 @@ short	iosocket_open(io_log_name *dev, mval *pp, int file_des, mval *mspace, int4
 			   but are included here because they can be easily used in internal processing.
 			*/
 				case iop_m:
-					UNICODE_ONLY(
+					UTF8_ONLY(
 						temp_ichset = temp_ochset = CHSET_M;
 						ichset_specified = ochset_specified = TRUE;
 					);
 					break;
 				case iop_utf16:
-					UNICODE_ONLY(
+					UTF8_ONLY(
 						if (gtm_utf8_mode)
 						{       /* Only change chset if in UTF8 mode */
 							temp_ichset = temp_ochset = CHSET_UTF16;
@@ -258,7 +259,7 @@ short	iosocket_open(io_log_name *dev, mval *pp, int file_des, mval *mspace, int4
 					);
 					break;
 				case iop_utf16be:
-					UNICODE_ONLY(
+					UTF8_ONLY(
 						if (gtm_utf8_mode)
 						{       /* Only change chset if in UTF8 mode */
 							temp_ichset = temp_ochset = CHSET_UTF16BE;
@@ -267,7 +268,7 @@ short	iosocket_open(io_log_name *dev, mval *pp, int file_des, mval *mspace, int4
 					);
 					break;
 				case iop_utf16le:
-					UNICODE_ONLY(
+					UTF8_ONLY(
 						if (gtm_utf8_mode)
 						{       /* Only change chset if in UTF8 mode */
 							temp_ichset = temp_ochset = CHSET_UTF16LE;
@@ -463,6 +464,9 @@ short	iosocket_open(io_log_name *dev, mval *pp, int file_des, mval *mspace, int4
 				}
 			} else
 				iosocket_handle(sock_handle, &handle_len, TRUE, dsocketptr);
+			assert(MAX_HANDLE_LEN > handle_len);
+			if (MAX_HANDLE_LEN < handle_len)
+				handle_len = MAX_HANDLE_LEN;
 			socketptr->handle_len = handle_len;
 			memcpy(socketptr->handle, sock_handle, handle_len);
 			/* connects newdsocket and socketptr (the new socket) */
@@ -547,21 +551,22 @@ short	iosocket_open(io_log_name *dev, mval *pp, int file_des, mval *mspace, int4
 		len = SIZEOF(ESTABLISHED) - 1;
 		memcpy(&ioptr->dollar.key[0], ESTABLISHED, len);
 		ioptr->dollar.key[len++] = '|';
-		memcpy(&ioptr->dollar.key[len], socketptr->handle, socketptr->handle_len);
-		len += socketptr->handle_len;
+		assert((DD_BUFLEN - len - 1) >= socketptr->handle_len); /* handle could be MAX_HANDLE_LEN */
+		memcpy(&ioptr->dollar.key[len], socketptr->handle, MIN(socketptr->handle_len, DD_BUFLEN - len - 2));
+		len += MIN(socketptr->handle_len, DD_BUFLEN - len - 2); /* -2 for the next delimiter and trailing null */
 		ioptr->dollar.key[len++] = '|';
 		if (socket_tcpip == socketptr->protocol)
-			strncpy(&ioptr->dollar.key[len], socketptr->remote.saddr_ip, DD_BUFLEN - 1 - len);
+			STRNCPY_STR(&ioptr->dollar.key[len], socketptr->remote.saddr_ip, DD_BUFLEN - len - 1);
 		else if (socket_local == socketptr->protocol)
 		{
 			if ((NULL != socketptr->local.sa)
 					&& ('\0' != ((struct sockaddr_un *)(socketptr->local.sa))->sun_path[0]))
-				strncpy(&ioptr->dollar.key[len], ((struct sockaddr_un *)(socketptr->local.sa))->sun_path,
-					DD_BUFLEN - 1 - len);
+				STRNCPY_STR(&ioptr->dollar.key[len], ((struct sockaddr_un *)(socketptr->local.sa))->sun_path,
+					DD_BUFLEN - len - 1);
 			else if ((NULL != socketptr->remote.sa)
 					&& ('\0' != ((struct sockaddr_un *)(socketptr->remote.sa))->sun_path[0]))
-				strncpy(&ioptr->dollar.key[len], ((struct sockaddr_un *)(socketptr->remote.sa))->sun_path,
-					DD_BUFLEN - 1 - len);
+				STRNCPY_STR(&ioptr->dollar.key[len], ((struct sockaddr_un *)(socketptr->remote.sa))->sun_path,
+					DD_BUFLEN - len - 1);
 			/* set default delimiter on principal local sockets to resemble rm device */
 			delimiter_buffer[0] = NATIVE_NL;
 			delimiter_len = 1;
