@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2009 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -9,8 +10,8 @@
  *								*
  ****************************************************************/
 
-#ifndef __IS_FILE_IDENTICAL_H__
-#define __IS_FILE_IDENTICAL_H__
+#ifndef IS_FILE_IDENTICAL_H_INCLUDED
+#define IS_FILE_IDENTICAL_H_INCLUDED
 
 bool 		is_file_identical(char *filename1, char *filename2);
 bool		is_gdid_file_identical(gd_id_ptr_t fid, char *filename, int4 filelen);
@@ -22,6 +23,6 @@ bool 		is_gdid_gdid_identical(gd_id_ptr_t fid_1, gd_id_ptr_t fid_2);
 bool		is_gdid_identical(gd_id_ptr_t fid1, gd_id_ptr_t fid2);
 bool 		is_gdid_stat_identical(gd_id_ptr_t fid, struct stat *stat_buf);
 void		set_gdid_from_stat(gd_id_ptr_t fid, struct stat *stat_buf);
-boolean_t 	filename_to_id(gd_id_ptr_t fid, char *filename);
+uint4	 	filename_to_id(gd_id_ptr_t fid, char *filename);
 #endif
 #endif
