@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2019 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -86,7 +86,7 @@ error_def(ERR_JNLENDIANLITTLE);
  * plus the overhead of storing the journal records.
  */
 #define JNL_BUFFER_MIN		((MAX_LOGI_JNL_REC_SIZE + ROUND_UP(2 * MAX_IO_BLOCK_SIZE, DISK_BLOCK_SIZE)) / DISK_BLOCK_SIZE + 1)
-#define JNL_BUFFER_MAX		32768	/* # of 512-byte blocks = 16Mb journal buffer size */
+#define JNL_BUFFER_MAX		1048576	/* # of 512-byte blocks = 512Mib journal buffer size */
 
 /*	JNL_EXTEND_DEF	allocation size / 10
 #define JNL_EXTEND_DEF_PERC	0.1
@@ -109,7 +109,7 @@ error_def(ERR_JNLENDIANLITTLE);
 									 */
 
 #define	JNL_WRT_END_MODULUS	8
-#define JNL_WRT_END_MASK	((uint4)~(JNL_WRT_END_MODULUS - 1))
+#define JNL_WRT_END_MASK	((gtm_uint64_t)~(JNL_WRT_END_MODULUS - 1))
 
 #define JNL_MIN_ALIGNSIZE	(1 << 12)	/*    4096 disk blocks effectively    2M alignsize */
 #define JNL_DEF_ALIGNSIZE	(1 << 12)	/*    4096 disk blocks effectively    2M alignsize */
@@ -1846,7 +1846,7 @@ MBSTART {											\
 	{											\
 		was_crit = CSA->now_crit;							\
 		if (!was_crit)									\
-			grab_crit(REG);								\
+			grab_crit(REG, WS_3);						\
 		DSKADDR = jbp->dskaddr;								\
 		FREEADDR = jbp->freeaddr;							\
 		RSRV_FREEADDR = jbp->rsrv_freeaddr;						\
@@ -1885,7 +1885,8 @@ void	jnl_write_inctn_rec(sgmnt_addrs *csa);
 void	jnl_write_logical(sgmnt_addrs *csa, jnl_format_buffer *jfb, uint4 com_csum);
 void	jnl_write_ztp_logical(sgmnt_addrs *csa, jnl_format_buffer *jfb, uint4 com_csum, seq_num jnl_seqno);
 void	jnl_write_eof_rec(sgmnt_addrs *csa, struct_jrec_eof *eof_record);
-void	jnl_write_trunc_rec(sgmnt_addrs *csa, uint4 orig_total_blks, uint4 orig_free_blocks, uint4 total_blks_after_trunc);
+void	jnl_write_trunc_rec(sgmnt_addrs *csa, block_id orig_total_blks,
+		block_id orig_free_blocks, block_id total_blks_after_trunc);
 void	jnl_write_reserve(sgmnt_addrs *csa, jbuf_rsrv_struct_t *nontp_jbuf_rsrv,
 					enum jnl_record_type rectype, uint4 reclen, void *param1);
 void	jnl_write_phase2(sgmnt_addrs *csa, jbuf_rsrv_struct_t *jbuf_rsrv_ptr);
