@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -92,7 +92,7 @@ static uint4 jnl_sub_write_attempt(jnl_private_control *jpc, unsigned int *lcnt,
 			jb->image_count = 0;
 			RELEASE_SWAPLOCK(&jb->io_in_prog_latch);
 		}
-		if (!jb->io_in_prog_latch.u.parts.latch_pid)
+		if ((!jb->io_in_prog_latch.u.parts.latch_pid) DEBUG_ONLY(&& !WBTEST_ENABLED(WBTEST_JNLPROCSTUCK_FORCE)))
 		{
 			if (freeze_waiter)
 			{
@@ -128,28 +128,39 @@ static uint4 jnl_sub_write_attempt(jnl_private_control *jpc, unsigned int *lcnt,
 			writer = CURRENT_JNL_IO_WRITER(jb);
 			loop_image_count = jb->image_count;
 			*lcnt = 1;	/* !!! this should be detected and limited by the caller !!! */
+#			ifdef DEBUG
+			if (WBTEST_ENABLED(WBTEST_JNLPROCSTUCK_FORCE))
+				writer = process_id;
+			else
+#			endif
 			break;
 		}
-		if (*lcnt <= JNL_MAX_FLUSH_TRIES)
+		if ((*lcnt <= JNL_MAX_FLUSH_TRIES) DEBUG_ONLY(&& !(WBTEST_ENABLED(WBTEST_JNLPROCSTUCK_FORCE))))
 		{
 			wcs_sleep(*lcnt);
 			break;
 		}
-		if (writer == CURRENT_JNL_IO_WRITER(jb))
-		{
+		if ((writer == CURRENT_JNL_IO_WRITER(jb)) DEBUG_ONLY(|| (WBTEST_ENABLED(WBTEST_JNLPROCSTUCK_FORCE))))
+		{	/* It isn't strictly necessary to hold crit here since we are doing an atomic operation on
+			 * io_in_prog_latch, which won't have any effect if the writer changed. If things are in a bad state,
+			 * though, grabbing crit will call wcs_recover() for us.
+			 * However, a grab_crit() here may result in a deadlock, so just do a grab_crit_immediate() and proceed.
+			 */
 			if (!was_crit)
-				grab_crit(jpc->region);	/* jnl_write_attempt has an assert about have_crit that this relies on */
-			if (FALSE == is_proc_alive(writer, jb->image_count))
-			{	/* no one home, clear the semaphore; */
+				grab_crit_immediate(jpc->region, TRUE, NOT_APPLICABLE);
+			/* If no one home, try to clear the latch. */
+			if (((FALSE == is_proc_alive(writer, jb->image_count))
+				DEBUG_ONLY(&& !(WBTEST_ENABLED(WBTEST_JNLPROCSTUCK_FORCE))))
+				&& COMPSWAP_UNLOCK(&jb->io_in_prog_latch, writer, jb->image_count, LOCK_AVAILABLE, 0))
+			{	/* We cleared the latch, so report it and restart the loop. */
 				BG_TRACE_PRO_ANY(csa, jnl_blocked_writer_lost);
 				jnl_send_oper(jpc, ERR_JNLQIOSALVAGE);
-				COMPSWAP_UNLOCK(&jb->io_in_prog_latch, writer, jb->image_count, LOCK_AVAILABLE, 0);
-				if (!was_crit)
+				if (!was_crit && csa->now_crit)		/* Check now_crit in case grab_crit_immediate() failed */
 					rel_crit(jpc->region);
 				*lcnt = 1;
 				continue;
 			}
-			if (!was_crit)
+			if (!was_crit && csa->now_crit)		/* Check now_crit in case grab_crit_immediate() failed */
 				rel_crit(jpc->region);
 			/* this is the interesting case: a process is stuck */
 			BG_TRACE_PRO_ANY(csa, jnl_blocked_writer_stuck);
@@ -159,8 +170,11 @@ static uint4 jnl_sub_write_attempt(jnl_private_control *jpc, unsigned int *lcnt,
 				continue;
 			}
 			jpc->status = status;
-			jnl_send_oper(jpc, ERR_JNLFLUSH);
 			send_msg_csa(CSA_ARG(csa) VARLSTCNT(3) ERR_JNLPROCSTUCK, 1, writer);
+#			ifdef DEBUG
+			if (WBTEST_ENABLED(WBTEST_JNLPROCSTUCK_FORCE))
+				gtm_white_box_test_case_enabled = FALSE;
+#			endif
 			stuck_cnt++;
 			if (IS_REPL_INST_FROZEN)
 			{	/* The instance wasn't frozen above, but it is now, so most likely we froze it.
@@ -254,7 +268,7 @@ uint4 jnl_write_attempt(jnl_private_control *jpc, uint4 threshold)
 					 * process which was kill -9ed in phase2 of its jnl commit. Not doing this check would
 					 * cause the process in gds_rundown to be indefinitely stuck in "jnl_wait".
 					 */
-					if (grab_crit_immediate(jpc->region, OK_FOR_WCS_RECOVER_TRUE))
+					if (grab_crit_immediate(jpc->region, OK_FOR_WCS_RECOVER_TRUE, NOT_APPLICABLE))
 					{
 						JNL_PHASE2_CLEANUP_IF_POSSIBLE(csa, jb); /* phase2 commits in progress.
 											  * Clean them up if possible.
@@ -283,8 +297,7 @@ uint4 jnl_write_attempt(jnl_private_control *jpc, uint4 threshold)
 			continue;
 		}
 		if ((ERR_JNLCNTRL == status) || (ERR_JNLACCESS == status)
-			|| (csa->now_crit
-				&& (ERR_JNLWRTDEFER != status) && (ERR_JNLWRTNOWWRTR != status) && (ERR_JNLPROCSTUCK != status)))
+			|| (csa->now_crit && (ERR_JNLWRTDEFER != status) && (ERR_JNLWRTNOWWRTR != status)))
 		{	/* If JNLCNTRL or if holding crit and not waiting for some other writer
 			 * better turn off journaling and proceed with database update to avoid a database hang.
 			 */
@@ -293,13 +306,12 @@ uint4 jnl_write_attempt(jnl_private_control *jpc, uint4 threshold)
 			else
 			{
 				assertpro(0 == have_crit(CRIT_HAVE_ANY_REG));
-				grab_crit(jpc->region);	/* jnl_write_attempt has an assert about have_crit that this relies on */
+				grab_crit(jpc->region, WS_4); /*jnl_write_attempt has assert about have_crit that this relies on */
 			}
 			jnlfile_lost = FALSE;
-			assert((gtm_white_box_test_case_enabled
-				&& (WBTEST_JNL_FILE_LOST_DSKADDR == gtm_white_box_test_case_number))
-			       || TREF(gtm_test_fake_enospc) || WBTEST_ENABLED(WBTEST_RECOVER_ENOSPC));
-			if (JNL_ENABLED(csa->hdr))
+			assert(TREF(gtm_test_fake_enospc) || WBTEST_ENABLED(WBTEST_JNL_FILE_LOST_DSKADDR)
+			|| WBTEST_ENABLED(WBTEST_RECOVER_ENOSPC) || (ERR_JNLPROCSTUCK == status));
+			if (JNL_ENABLED(csa->hdr) && (ERR_JNLPROCSTUCK != status))
 			{	/* We ignore the return value of jnl_file_lost() since we always want to report the journal
 				 * error, whatever its error handling method is.  Also, an operator log will be sent by some
 				 * callers (t_end()) only if an error is returned here, and the operator log is wanted in
@@ -331,7 +343,7 @@ uint4 jnl_write_attempt(jnl_private_control *jpc, uint4 threshold)
 			assert(!csa->jnlpool || (csa->jnlpool == jnlpool));
 			WAIT_FOR_REPL_INST_UNFREEZE_SAFE(csa);
 		}
-		if ((ERR_JNLWRTDEFER != status) && (ERR_JNLWRTNOWWRTR != status) && (ERR_JNLPROCSTUCK != status))
+		if ((ERR_JNLWRTDEFER != status) && (ERR_JNLWRTNOWWRTR != status))
 		{	/* If holding crit, then jnl_sub_write_attempt would have invoked jnl_file_lost which would have
 			 * caused the JNL_FILE_SWITCHED check at the beginning of this for loop to succeed and return from
 			 * this function so we should never have gotten here. Assert accordingly. If not holding crit,
