@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2002 Sanchez Computer Associates, Inc.	*
+ * Copyright (c) 2001-2015 Fidelity National Information 	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -13,7 +14,7 @@
 #define JOBEXAM_PROCESS_INCLUDED
 
 void jobexam_process(mval *dump_file_name, mval *dump_file_spec);
-void jobexam_dump(mval *dump_file_name, mval *dump_file_spec);
+void jobexam_dump(mval *dump_file_name, mval *dump_file_spec, char *fatal_file_name_buff);
 
 #endif
 
