@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2019 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -11,22 +11,14 @@
  ****************************************************************/
 
 #include "mdef.h"
+#include <stdarg.h>
 
 #include "gtm_string.h"
 #include "gtm_stdlib.h"
 #include "gtm_inet.h"	/* Required for gtmsource.h */
 #include "gtm_stdio.h"
 #include "gtm_fcntl.h"	/* Needed for AIX's silly open to open64 translations */
-
-#ifdef VMS
-# include <descrip.h>	/* required for gtmsource.h */
-# include <ssdef.h>
-#endif
-#ifdef UNIX
 #include "gtm_signal.h"
-# include <stdarg.h>
-#endif
-
 #include "ast.h"
 #include "gdsroot.h"
 #include "gdsbt.h"
@@ -51,7 +43,6 @@
 #include "outofband.h"
 #include "stack_frame.h"
 #include "stringpool.h"
-#include "hashtab_int4.h"	/* needed for tp.h */
 #include "buddy_list.h"		/* needed for tp.h */
 #include "tp.h"
 #include "tp_frame.h"
@@ -84,17 +75,17 @@
 #include "create_fatal_error_zshow_dmp.h"
 #include "have_crit.h"
 #include "ztimeout_routines.h"
-#ifdef UNIX
 # include "iormdef.h"
 # include "ftok_sems.h"
 # include "gtm_putmsg_list.h"
-#endif
+#include "gvt_inline.h"
 #ifdef GTM_TRIGGER
 # include "gv_trigger.h"
 # include "gtm_trigger.h"
 #endif
 
-GBLREF	boolean_t		created_core, dont_want_core, in_gvcst_incr, run_time;
+GBLREF	boolean_t		created_core, dont_want_core, in_gvcst_incr, prin_dm_io, prin_in_dev_failure, prin_out_dev_failure,
+				run_time;
 GBLREF	boolean_t		ztrap_explicit_null;		/* whether $ZTRAP was explicitly set to NULL in this frame */
 GBLREF	dollar_ecode_type	dollar_ecode;			/* structure containing $ECODE related information */
 GBLREF	dollar_stack_type	dollar_stack;
@@ -133,13 +124,7 @@ GBLREF	boolean_t		donot_INVOKE_MUMTSTART;
 GBLREF	int			tprestart_state;	/* When triggers restart, multiple states possible - see tp_restart.h */
 GBLREF	int4			gtm_trigger_depth;
 #endif
-#ifdef UNIX
 GBLREF	jnl_gbls_t		jgbl;
-#endif
-#ifdef VMS
-GBLREF	boolean_t		tp_restart_fail_sig_used;
-GBLREF	struct chf$signal_array	*tp_restart_fail_sig;
-#endif
 
 error_def(ERR_ASSERT);
 error_def(ERR_CTRAP);
@@ -162,6 +147,7 @@ error_def(ERR_RTSLOC);
 error_def(ERR_SRCLOCUNKNOWN);
 error_def(ERR_STACKCRIT);
 error_def(ERR_STACKOFLOW);
+error_def(ERR_TERMHANGUP);
 error_def(ERR_TLVLZERO);
 error_def(ERR_TPRETRY);
 error_def(ERR_TPRESTNESTERR);
@@ -175,7 +161,6 @@ error_def(ERR_ZTIMEOUT);
 boolean_t clean_mum_tstart(void);
 void setup_error(sgmnt_addrs *csa, int argcnt, ...);
 
-#ifdef UNIX
 /* When we restart generated code after handling an error, verify that we are not in the frame or one created on its
  * behalf that invoked a trigger or spanning node and caused a dynamic TSTART to be done on its behalf. This can happen
  * for example if a trigger is invoked for the first time but gets a compilation or link failure error or if a spanning
@@ -193,9 +178,6 @@ void setup_error(sgmnt_addrs *csa, int argcnt, ...);
 		OP_TROLLBACK(-1);	/* Unroll implicit TP frame */					\
 	}												\
 }
-#else
-#define MUM_TSTART_FRAME_CHECK
-#endif
 
 /* Since a call to SET_ZSTATUS may cause the stringpool expansion, record whether we are using an indirection pool or not
  * in a local variable rather than recompare the stringpool bases.
@@ -244,7 +226,6 @@ boolean_t clean_mum_tstart(void)
 	return (NULL != err_act);
 }
 
-#ifdef UNIX
 /* Routine to setup an error in util_outbuff as if rts_error had put it there. Used when we morph ERR_TPRETRY
  * to ERR_TPRESTNESTERR. Requires a va_list var containing the args so do this in this separate routine.
  */
@@ -256,7 +237,6 @@ void setup_error(sgmnt_addrs *csa, int argcnt, ...)
 	gtm_putmsg_list(csa, argcnt, var);
 	va_end(var);
 }
-#endif
 
 CONDITION_HANDLER(mdb_condition_handler)
 {
@@ -277,11 +257,9 @@ CONDITION_HANDLER(mdb_condition_handler)
 	boolean_t		repeat_error, etrap_handling, reset_mpc;
 	int			level, rc;
 	boolean_t		reserve_sock_dev = FALSE;
-#	ifdef UNIX
 	unix_db_info		*udi;
 	stack_frame		*lcl_error_frame;
 	mv_stent		*mvst;
-#	endif
 
 	START_CH(FALSE);
 	DBGEHND((stderr, "mdb_condition_handler: Entered with SIGNAL=%d frame_pointer=0x"lvaddr"\n", SIGNAL, frame_pointer));
@@ -294,10 +272,8 @@ CONDITION_HANDLER(mdb_condition_handler)
 		 */
 		if (gtmsocket == gtm_err_dev->type)
 			iosocket_destroy(gtm_err_dev);
-#		ifdef UNIX
 		else
 			remove_rms(gtm_err_dev);
-#		endif
 		gtm_err_dev = NULL;
 	}
 	if (repeat_error = (ERR_REPEATERROR == SIGNAL)) /* assignment and comparison */
@@ -334,7 +310,6 @@ CONDITION_HANDLER(mdb_condition_handler)
 	}
 	if ((int)ERR_TPRETRY == SIGNAL)
 	{
-#		ifdef UNIX
 		lcl_error_frame = error_frame;
 		if ((NULL == lcl_error_frame) && dollar_zininterrupt)
 		{	/* We are in a $zininterrupt handler AND have a restart request. See if we were in error processing
@@ -351,9 +326,7 @@ CONDITION_HANDLER(mdb_condition_handler)
 			lcl_error_frame = mvst->mv_st_cont.mvs_zintr.error_frame_save;
 		}
 		if (NULL == lcl_error_frame)
-#		endif
 		{
-			VMS_ONLY(assert(FALSE == tp_restart_fail_sig_used));
 #			ifdef GTM_TRIGGER
 			/* Assert that we never end up invoking the MUM_TSTART macro handler in case of an implicit tstart restart.
 			 * See GBLDEF of skip_INVOKE_RESTART and donot_INVOKE_MUMTSTART in gbldefs.c for more information.
@@ -388,8 +361,8 @@ CONDITION_HANDLER(mdb_condition_handler)
 				 * prevents an assert failure in UNWIND. START_CH would have done a active_ch-- So we need a
 				 * active_ch[1] to get at the desired active_ch. See similar code in tp_restart.c.
 				 */
-				UNIX_ONLY(assert(active_ch[1].dollar_tlevel >= dollar_tlevel);)
-				UNIX_ONLY(DEBUG_ONLY(active_ch[1].dollar_tlevel = dollar_tlevel;))
+				assert(active_ch[1].dollar_tlevel >= dollar_tlevel);
+				DEBUG_ONLY(active_ch[1].dollar_tlevel = dollar_tlevel;)
 				UNWIND(NULL, NULL);
 			}
 			/* "tp_restart" has succeeded so we have unwound back to the return point but check if the
@@ -405,28 +378,12 @@ CONDITION_HANDLER(mdb_condition_handler)
 			}
 			assert(!donot_INVOKE_MUMTSTART);
 #			endif
-#			ifdef UNIX
 			if (ERR_TPRETRY == SIGNAL)		/* (signal value undisturbed) */
-#			elif defined VMS
-			if (!tp_restart_fail_sig_used)		/* If tp_restart ran clean */
-#			endif
 			{
 				/* Set mumps program counter back tstart level 1 */
 				MUM_TSTART;
 			}
-#			ifdef VMS
-			else
-			{	/* Otherwise tp_restart had a signal that we must now deal with -- replace the TPRETRY
-				 * information with that saved from tp_restart.
-				 * Assert that we have room for these arguments - the array malloc is in tp_restart
-				 */
-				assert(TPRESTART_ARG_CNT >= tp_restart_fail_sig->chf$is_sig_args);
-				memcpy(sig, tp_restart_fail_sig, (tp_restart_fail_sig->chf$l_sig_args + 1) * SIZEOF(int));
-				tp_restart_fail_sig_used = FALSE;
-			}
-#			endif
 		}
-#		ifdef UNIX
 		else
 		{
 			if (0 == dollar_tlevel)
@@ -440,7 +397,6 @@ CONDITION_HANDLER(mdb_condition_handler)
 			 */
 			setup_error(gv_target ? gv_target->gd_csa : NULL, VARLSTCNT(1) SIGNAL);
 		}
-#		endif
 	}
 	/* Ensure gv_target and cs_addrs are in sync. If not, make them so. */
 	if (NULL != gv_target)
@@ -492,17 +448,15 @@ CONDITION_HANDLER(mdb_condition_handler)
 	DBG_CHECK_GVTARGET_GVCURRKEY_IN_SYNC(CHECK_CSA_TRUE);
 	if (DUMPABLE)
 	{	/* Certain conditions we don't want to attempt to create the M-level ZSHOW dump.
-		 * 1) Unix: If gtmMallocDepth > 0 indicating memory manager was active and could be reentered.
-		 * 2) Unix: If we have a SIGBUS or SIGSEGV (could be likely to occur again
-		 *    in the local variable code which would cause immediate shutdown with
-		 *    no cleanup).
-		 * 3) VMS: If we got an ACCVIO for the same as reason (2).
-		 * Note that we will bypass checks 2 and 3 if GDL_ZSHOWDumpOnSignal debug flag is on
+		 * 1) If gtmMallocDepth > 0 indicating memory manager was active and could be reentered.
+		 * 2) If we have a SIGBUS or SIGSEGV (could be likely to occur again
+		 *    in the local variable code which would cause immediate shutdown with no cleanup).
+		 * Note that we will bypass check 2 if GDL_ZSHOWDumpOnSignal debug flag is on
 		 */
 		SET_PROCESS_EXITING_TRUE;	/* So zshow doesn't push stuff on stack to "protect" it when
 						 * we potentially already have a stack overflow */
 		CANCEL_TIMERS;			/* No unsafe interruptions now that we are dying */
-		if (!repeat_error UNIX_ONLY(&& (0 == gtmMallocDepth)))
+		if (!repeat_error && (0 == gtmMallocDepth))
 		{
 			src_line_d.addr = src_line;	/* Setup entry point buffer for set_zstatus() */
 			src_line_d.len = 0;
@@ -512,43 +466,18 @@ CONDITION_HANDLER(mdb_condition_handler)
 		create_fatal_error_zshow_dmp(SIGNAL);
 
 		/* If we are about to core/exit on a stack over flow, only do the core part if a debug
-		 * flag requests this behaviour. Otherwise, supress the core and just exit.
-		 * 2006-03-07 se: If a stack overflow occurs on VMS, it has happened that the stack is no
-		 * longer well formed so attempting to unwind it as it does in MUMPS_EXIT causes things
-		 * to really become screwed up. For this reason, this niceness of avoiding a dump on a
-		 * stack overflow on VMS is being disabled. The dump can be controlled wih set proc/dump
-		 * (or not) as desired.
+		 * flag requests this behaviour. Otherwise, supress the core and just exit. (or not) as desired.
 		 * 2008-01-29 (se): Added fatal MEMORY error so we no longer generate a core for it by
 		 * default unless the DumpOnStackOFlow flag is turned on. Since this flag is not a user-exposed
-		 * interface, I'm avoiding renaming it for now. Note the core avoidance applies to both UNIX
-		 * and VMS since stack formation is not at issue in this sort of memory request.
-		 * Finally note that in UNIX, ch_cond_core (called by DRIVECH macro which invoked this condition
+		 * interface, I'm avoiding renaming it for now.
+		 * Finally note that ch_cond_core (called by DRIVECH macro which invoked this condition
 		 * handler has likely already created the core and set the created_core flag which will prevent
 		 * this process from creating another core for the same SIGNAL. We leave this code in here in
-		 * case methods exist in the future for this module to be driven without invoking cond_core_ch
-		 * first.
+		 * case methods exist in the future for this module to be driven without invoking cond_core_ch first.
 		 */
-		if (!(GDL_DumpOnStackOFlow & gtmDebugLevel) &&
-		    VMS_ONLY((int)ERR_VMSMEMORY == SIGNAL)
-		    UNIX_ONLY(((int)ERR_STACKOFLOW == SIGNAL || (int)ERR_STACKOFLOW == arg
-			       || (int)ERR_MEMORY == SIGNAL  || (int)ERR_MEMORY == arg)))
+		if (!(GDL_DumpOnStackOFlow & gtmDebugLevel) && (((int)ERR_STACKOFLOW == SIGNAL) || ((int)ERR_STACKOFLOW == arg)
+			|| ((int)ERR_MEMORY == SIGNAL)  || ((int)ERR_MEMORY == arg)))
 		{
-#			ifdef VMS
-			/* Inside this ifdef, we are definitely here because of ERR_VMSMEMORY. If the conditions
-			 * of the above if change, revisit these assmuptions.
-			 * For VMSMEMORY error, we have to send the message to the operator log and to the
-			 * console ourselves because the MUMP_EXIT method of exiting on a fatal error does
-			 * not preserve the substitution parameters for the message making it useless. After
-			 * sending the message change the status code so we exit with something other than the
-			 * duplicate message.
-			 */
-			assert(ERR_VMSMEMORY == SIGNAL);
-			send_msg(VARLSTCNT(4) ERR_VMSMEMORY, 2, *(int **)(&sig->chf$is_sig_arg1 + 1),	/* BYPASSOK - send_msg */
-				 *(int **)(&sig->chf$is_sig_arg1 + 2));
-			gtm_putmsg(VARLSTCNT(4) ERR_VMSMEMORY, 2, *(int **)(&sig->chf$is_sig_arg1 + 1),	/* BYPASSOK - gtm_putmsg */
-				   *(int **)(&sig->chf$is_sig_arg1 + 2));
-			SIGNAL = ERR_GTMERREXIT;	/* Override reason for "stop" */
-#			endif
 			MUMPS_EXIT;	/* Do a clean exit rather than messy core exit */
 		}
 		gtm_dump();
@@ -597,7 +526,7 @@ CONDITION_HANDLER(mdb_condition_handler)
 						csa = (sgmnt_addrs *)&FILE_INFO(reg_local)->s_addrs;
 						if (csa && csa->now_crit)
 						{
-							assert(!csa->hold_onto_crit UNIX_ONLY(|| jgbl.onlnrlbk));
+							assert(!csa->hold_onto_crit || jgbl.onlnrlbk);
 							if (csa->hold_onto_crit)
 								csa->hold_onto_crit = FALSE; /* Fix it in pro */
 							rel_crit(reg_local);
@@ -606,7 +535,6 @@ CONDITION_HANDLER(mdb_condition_handler)
 				}
 			}
 		}
-#		ifdef UNIX
 		/* Release FTOK lock on the replication instance file if holding it (possible if error in jnlpool_init) */
 		for (local_jnlpool = jnlpool_head; local_jnlpool; local_jnlpool = local_jnlpool->next)
 			if ((NULL != local_jnlpool->jnlpool_dummy_reg) && local_jnlpool->jnlpool_dummy_reg->open)
@@ -620,7 +548,6 @@ CONDITION_HANDLER(mdb_condition_handler)
 					assert(!udi->grabbed_ftok_sem);
 				}
 			}
-#		endif
 		/* Release crit lock on journal pool if holding it */
 		for (local_jnlpool = jnlpool_head; local_jnlpool; local_jnlpool = local_jnlpool->next)
 			if (local_jnlpool->pool_init)
@@ -662,8 +589,8 @@ CONDITION_HANDLER(mdb_condition_handler)
 #	endif
  	err_dev = active_device;
 	active_device = (io_desc *)NULL;
-	dm_action = (frame_pointer->old_frame_pointer->type & SFT_DM)
-		|| (TREF(compile_time) && (frame_pointer->type & SFT_DM));
+	dm_action = (prin_dm_io || (frame_pointer->old_frame_pointer->type & SFT_DM)
+		|| (TREF(compile_time) && (frame_pointer->type & SFT_DM)));
 	/* The errors are said to be transcendental when they occur during compilation/execution
 	 * of the error trap ({z,e}trap, device exception) or $zinterrupt. The errors in other
 	 * indirect code frames (zbreak, zstep, xecute etc.) aren't defined to be trancendental
@@ -672,31 +599,32 @@ CONDITION_HANDLER(mdb_condition_handler)
 	trans_action = proc_act_type || (frame_pointer->type & SFT_ZTRAP) || (frame_pointer->type & SFT_DEV_ACT);
 	src_line_d.addr = src_line;
 	src_line_d.len = 0;
-	flush_pio();
+	if (sighup != outofband)	/* because prin_out_dev_failure is not yet set - it will be below */
+		flush_pio();
 	if ((int)ERR_CTRLY == SIGNAL)
 	{
 		outofband_clear();
 		/* Verify not indirect or that context is unchanged before reset context */
-		assert(NULL != restart_pc);
-		assert((!(SFF_INDCE & frame_pointer->flags)) || (restart_ctxt == frame_pointer->ctxt));
+		assert(NULL != frame_pointer->restart_pc);
+		assert((!(SFF_INDCE & frame_pointer->flags)) || (frame_pointer->restart_ctxt == frame_pointer->ctxt));
 		DBGEHND((stderr, "mdb_condition_handler(1): Resetting frame 0x"lvaddr" mpc/context with restart_pc/ctxt "
-			 "0x"lvaddr"/0x"lvaddr" - frame has type 0x%04lx\n", frame_pointer, restart_pc, restart_ctxt,
-			 frame_pointer->type));
-		frame_pointer->mpc = restart_pc;
-		frame_pointer->ctxt = restart_ctxt;
+			 "0x"lvaddr"/0x"lvaddr" - frame has type 0x%04lx\n", frame_pointer, frame_pointer->restart_pc,
+			 frame_pointer->restart_ctxt, frame_pointer->type));
+		frame_pointer->mpc = frame_pointer->restart_pc;
+		frame_pointer->ctxt = frame_pointer->restart_ctxt;
 		MUM_TSTART;
 	} else  if ((int)ERR_CTRLC == SIGNAL)
 	{
 		outofband_clear();
 		if (!trans_action && !dm_action)
 		{	/* Verify not indirect or that context is unchanged before reset context */
-			assert(NULL != restart_pc);
-			assert((!(SFF_INDCE & frame_pointer->flags)) || (restart_ctxt == frame_pointer->ctxt));
+			assert(NULL != frame_pointer->restart_pc);
+			assert((!(SFF_INDCE & frame_pointer->flags)) || (frame_pointer->restart_ctxt == frame_pointer->ctxt));
 			DBGEHND((stderr, "mdb_condition_handler(2): Resetting frame 0x"lvaddr" mpc/context with restart_pc/ctxt "
-				 "0x"lvaddr"/0x"lvaddr" - frame has type 0x%04lx\n", frame_pointer, restart_pc, restart_ctxt,
-				 frame_pointer->type));
-			frame_pointer->mpc = restart_pc;
-			frame_pointer->ctxt = restart_ctxt;
+				 "0x"lvaddr"/0x"lvaddr" - frame has type 0x%04lx\n", frame_pointer, frame_pointer->restart_pc,
+				 frame_pointer->restart_ctxt, frame_pointer->type));
+			frame_pointer->mpc = frame_pointer->restart_pc;
+			frame_pointer->ctxt = frame_pointer->restart_ctxt;
 			frame_pointer->flags &= SSF_NORET_VIA_MUMTSTART_OFF;	/* Frame enterable now with mpc reset */
 			GTMTRIG_ONLY(
 				DBGTRIGR((stderr, "mdb_condition_handler: disabling SSF_NORET_VIA_MUMTSTART_OFF (1) in frame "
@@ -727,11 +655,14 @@ CONDITION_HANDLER(mdb_condition_handler)
 			dec_err(VARLSTCNT(4) ERR_NOTPRINCIO, 2, io_curr_device.out->trans_name->len,
 				io_curr_device.out->trans_name->dollar_io);
 		MUM_TSTART;
-	} else if ((int)ERR_CTRAP == SIGNAL)
+	} else if (((int)ERR_CTRAP == SIGNAL) || ((int)ERR_TERMHANGUP == SIGNAL))
 	{
-		if (!repeat_error)
-			/* This has already been done if we are re-throwing the error */
+		if (outofband && !repeat_error)
+		{	/* This has already been done if we are re-throwing the error */
 			outofband_clear();
+			if ((int)ERR_TERMHANGUP == SIGNAL)
+				prin_in_dev_failure = prin_out_dev_failure = TRUE;
+		}
 		if (!trans_action && !(frame_pointer->type & SFT_DM))
 		{
 			if (!repeat_error)
@@ -747,13 +678,14 @@ CONDITION_HANDLER(mdb_condition_handler)
 			 */
 			if (!repeat_error && ((0 != (TREF(dollar_ztrap)).str.len) || ztrap_explicit_null))
 			{	/* Verify not indirect or that context is unchanged before reset context */
-				assert(NULL != restart_pc);
-				assert((!(SFF_INDCE & frame_pointer->flags)) || (restart_ctxt == frame_pointer->ctxt));
+				assert(NULL != frame_pointer->restart_pc);
+				assert((!(SFF_INDCE & frame_pointer->flags))
+						|| (frame_pointer->restart_ctxt == frame_pointer->ctxt));
 				DBGEHND((stderr, "mdb_condition_handler(3): Resetting frame 0x"lvaddr" mpc/context with restart_pc/"
-					 "ctxt 0x"lvaddr"/0x"lvaddr" - frame has type 0x%04lx\n", frame_pointer, restart_pc,
-					 restart_ctxt, frame_pointer->type));
-				frame_pointer->mpc = restart_pc;
-				frame_pointer->ctxt = restart_ctxt;
+					 "ctxt 0x"lvaddr"/0x"lvaddr" - frame has type 0x%04lx\n", frame_pointer,
+					 frame_pointer->restart_pc, frame_pointer->restart_ctxt, frame_pointer->type));
+				frame_pointer->mpc = frame_pointer->restart_pc;
+				frame_pointer->ctxt = frame_pointer->restart_ctxt;
 			}
 			err_act = NULL;
 			dollar_ecode.error_last_ecode = SIGNAL;
@@ -855,13 +787,13 @@ CONDITION_HANDLER(mdb_condition_handler)
 		MUM_TSTART;
 	} else  if ((int)ERR_JOBINTRRQST == SIGNAL)
 	{	/* Verify not indirect or that context is unchanged before reset context */
-		assert(NULL != restart_pc);
-		assert((!(SFF_INDCE & frame_pointer->flags)) || (restart_ctxt == frame_pointer->ctxt));
+		assert(NULL != frame_pointer->restart_pc);
+		assert((!(SFF_INDCE & frame_pointer->flags)) || (frame_pointer->restart_ctxt == frame_pointer->ctxt));
 		DBGEHND((stderr, "mdb_condition_handler(4): Resetting frame 0x"lvaddr" mpc/context with restart_pc/ctxt "
-			 "0x"lvaddr"/0x"lvaddr" - frame has type 0x%04lx\n", frame_pointer, restart_pc, restart_ctxt,
-			 frame_pointer->type));
-		frame_pointer->mpc = restart_pc;
-		frame_pointer->ctxt = restart_ctxt;
+			 "0x"lvaddr"/0x"lvaddr" - frame has type 0x%04lx\n", frame_pointer, frame_pointer->restart_pc,
+			 frame_pointer->restart_ctxt, frame_pointer->type));
+		frame_pointer->mpc = frame_pointer->restart_pc;
+		frame_pointer->ctxt = frame_pointer->restart_ctxt;
 		assert(!dollar_zininterrupt);
 		dollar_zininterrupt = TRUE;	/* Note done before outofband is cleared to prevent nesting */
 		outofband_clear();
@@ -884,13 +816,13 @@ CONDITION_HANDLER(mdb_condition_handler)
 	{	/* Verify not indirect or that context is unchanged before reset context */
 		if (((TREF(dollar_ztimeout)).ztimeout_vector.str.len))
 		{
-			assert(NULL != restart_pc);
-			assert((!(SFF_INDCE & frame_pointer->flags)) || (restart_ctxt == frame_pointer->ctxt));
+			assert(NULL != frame_pointer->restart_pc);
+			assert((!(SFF_INDCE & frame_pointer->flags)) || (frame_pointer->restart_ctxt == frame_pointer->ctxt));
 			DBGEHND((stderr, "mdb_condition_handler(5): Resetting frame 0x"lvaddr" mpc/context with restart_pc/ctxt "
-			"0x"lvaddr"/0x"lvaddr" - frame has type 0x%04lx\n", frame_pointer, restart_pc, restart_ctxt,
-			frame_pointer->type));
-			frame_pointer->mpc = restart_pc;
-			frame_pointer->ctxt = restart_ctxt;
+			"0x"lvaddr"/0x"lvaddr" - frame has type 0x%04lx\n", frame_pointer, frame_pointer->restart_pc,
+			frame_pointer->restart_ctxt, frame_pointer->type));
+			frame_pointer->mpc = frame_pointer->restart_pc;
+			frame_pointer->ctxt = frame_pointer->restart_ctxt;
 			assert(!dollar_zininterrupt);
 			proc_act_type = SFT_ZTIMEOUT | SFT_COUNT;/* | SFT_COUNT;*/
 			err_act = &((TREF(dollar_ztimeout)).ztimeout_vector.str);
@@ -912,12 +844,10 @@ CONDITION_HANDLER(mdb_condition_handler)
 	}
 	if ((SUCCESS == SEVERITY) || (INFO == SEVERITY))
 	{
-		/* In VMS skip printing error messages for INFO messages if skip_gtm_putmsg is TRUE.
-		 * this is currently relied upon by GDE for the VIEW "YCHKCOLL" command.
-		 * In UNIX send out messages only in utilities or direct mode, this addresses the GDE
-		 * issue described above for VMS as GDE is never in direct mode
+		 /* Send out messages only in utilities or direct mode, this meets the GDE requirement for the
+		 * VIEW "YCHKCOLL" command as GDE is never in direct mode
 		 */
-		if (UNIX_ONLY(!IS_GTM_IMAGE || dm_action) VMS_ONLY(!TREF(skip_gtm_putmsg)))
+		if (!IS_GTM_IMAGE || dm_action)
 			PRN_ERROR;
 		CONTINUE;
 	}
@@ -1011,7 +941,7 @@ CONDITION_HANDLER(mdb_condition_handler)
 			iosocket_destroy(err_dev);
 			err_dev = NULL;
 		}
-		if (reset_mpc)
+		if (reset_mpc) /* Either dev err or ZTRAP, retry the error line */
 		{	/* Reset the mpc such that
 			 *   (a) If the current frame is a counted frame, the error line is retried after the error is handled,
 			 *   (b) If the current frame is "transcendental" code, set frame to return.
@@ -1036,8 +966,8 @@ CONDITION_HANDLER(mdb_condition_handler)
 					 * pointer to start over. $ETRAP does things somewhat differently in that the current
 					 * frame is always returned from.
 					 */
-					if (SFT_ZINTR & fp->type)
-					{
+					if ((SFT_ZINTR & fp->type) && (SFF_INDCE & fp->flags))
+					{ /* Not modified by a goto or zgoto entry */
 						assert(SFF_INDCE & fp->flags);
 						fp->mpc = fp->ctxt;
 						fp->flags &= SSF_NORET_VIA_MUMTSTART_OFF; /* Frame enterable now with mpc reset */
@@ -1074,7 +1004,6 @@ CONDITION_HANDLER(mdb_condition_handler)
 		{
 			if (err_dev)
 			{
-#				ifdef UNIX
 				/* On z/OS, if opening a fifo which is not read only we need to fix the err_dev type to rm */
 #				ifdef __MVS__
 				if ((dev_open != err_dev->state) && (ff == err_dev->type))
@@ -1098,7 +1027,6 @@ CONDITION_HANDLER(mdb_condition_handler)
 					/* structures pointed to by err_dev were freed so make sure it's not used again */
 					err_dev = NULL;
 				}
-#				endif
 				if (err_dev && (gtmsocket == err_dev->type) && err_dev->newly_created)
 				{
 					assert(err_dev->state != dev_open);
@@ -1132,7 +1060,6 @@ CONDITION_HANDLER(mdb_condition_handler)
 		DBGEHND((stderr, "mdb_condition_handler: Transient or direct mode frame -- bypassing handler dispatch\n"));
 		if (err_dev)
 		{
-#			ifdef UNIX
 			/* Executed from the direct mode so do the rms check and cleanup if necessary. On z/OS, if opening a fifo
 			 * which is not read only we need to fix the type for the err_dev to rm.
 			 */
@@ -1155,7 +1082,6 @@ CONDITION_HANDLER(mdb_condition_handler)
 				remove_rms(err_dev);
 				err_dev = NULL;
 			}
-#			endif
 			if (err_dev && (gtmsocket == err_dev->type) && err_dev->newly_created)
 			{
 				assert(err_dev->state != dev_open);
@@ -1174,21 +1100,16 @@ CONDITION_HANDLER(mdb_condition_handler)
 		 * any other up to the next counted frame out by changing their mpc to pseudo_ret and will rethrow the
 		 * error we have here so there is no need to push this error out here for anything but direct mode.
 		 */
-		UNIX_ONLY(if (dm_action))
+		if (dm_action)
 		{
 			PRN_ERROR;
 			if (TREF(compile_time) && (((int)ERR_LABELMISSING) != SIGNAL))
 				show_source_line(TRUE);
 		}
 	}
-	/* Slight divergence in how we handle otherwise unhandled errors on UNIX and VMS. UNIX now has a strict no-unsolicited
-	 * output from error messages policy unless dealing with a direct mode frame. This has a dependency on a robust
-	 * error_return() routine which VMS does not presently have. Consequently the previous way of doing things still
-	 * exists on VMS while on UNIX, a new cleaner (from an unsolicited console output) viewpoint is in place. Note that
-	 * while it would be possible to extract out the common code between the two below versions, the result is nowhere
-	 * near as clean looking. In this routine, clarity rules the roost.
+	/* GT.M has a strict no-unsolicited output from error messages policy unless dealing with a direct mode frame.
+	 * This has a dependency on a robust error_return().
 	 */
-#	ifdef UNIX
 	if (trans_action || dm_action)
 	{	/* If true transcendental, do trans_code_cleanup(). If our counted frame is
 		 * masquerading as a transcendental frame, run jobinterrupt_process_cleanup().
@@ -1202,42 +1123,6 @@ CONDITION_HANDLER(mdb_condition_handler)
 		MUM_TSTART_FRAME_CHECK;
 		MUM_TSTART;
 	}
-#	elif VMS	/* VMS only */
-	if (!dm_action && !trans_action && (0 != src_line_d.len))
-	{
-		if (MSG_OUTPUT)
-			dec_err(VARLSTCNT(4) ERR_RTSLOC, 2, src_line_d.len, src_line_d.addr);
-	} else
-	{
-		if (trans_action || dm_action)
-		{	/* If true transcendental, do trans_code_cleanup(). If our counted frame is
-			 * masquerading as a transcendental frame, run jobinterrupt_process_cleanup().
-			 */
-			DBGEHND((stderr, "mdb_condition_handler: trans_code_cleanup() or jobinterrupt_process_cleanup being "
-				 "dispatched\n"));
-			if (!(SFT_ZINTR & proc_act_type))
-				trans_code_cleanup();
-			else
-				jobinterrupt_process_cleanup();
-			MUM_TSTART_FRAME_CHECK;
-			MUM_TSTART;
-		} else if (MSG_OUTPUT)
-		{	/* If a message about the location is needed, it should be possible to pull the location
-			 * out of the $STACK array. If it exists, use it instead.
-			 */
-			if ((NULL != dollar_stack.array) && (0 < dollar_stack.index))
-			{	/* Error entry exists */
-				src_line_d = dollar_stack.array[dollar_stack.index - 1].place_str;
-				assert(src_line_d.len);
-				assert(src_line_d.addr);
-				dec_err(VARLSTCNT(4) ERR_RTSLOC, 2, src_line_d.len, src_line_d.addr);
-			} else
-				dec_err(VARLSTCNT(1) ERR_SRCLOCUNKNOWN);
-		}
-	}
-#	else
-#	 error "Unsupported platform"
-#	endif
 	DBGEHND((stderr, "mdb_condition_handler: Condition not handled -- defaulting to process exit\n"));
 	MUMPS_EXIT;
 }
