@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2019 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -302,17 +302,24 @@ typedef UINTPTR_T uintszofptr_t;
 #ifdef __sparc
 #	define  SPARC_ONLY(X) X
 #else
-#define  SPARC_ONLY(X)
+#define SPARC_ONLY(X)
 #endif
-#define BITS_PER_UCHAR  8 /* note, C does not require this to be 8, see <limits.h> for definitions of CHAR_BIT and UCHAR_MAX */
+#define BITS_PER_UCHAR	8 /* note, C does not require this to be 8, see <limits.h> for definitions of CHAR_BIT and UCHAR_MAX */
 
 #define MAXPOSINT4		((int4)0x7fffffff)
-#define	MAX_DIGITS_IN_INT	10	/* maximum number of decimal digits in a  4-byte integer */
-#define	MAX_DIGITS_IN_INT8	20	/* maximum number of decimal digits in an 8-byte integer */
-#define	MAX_HEX_DIGITS_IN_INT	 8 	/* maximum number of hexadecimal digits in a  4-byte integer */
-#define	MAX_HEX_DIGITS_IN_INT8	16	/* maximum number of hexadecimal digits in an 8-byte integer */
+#define MAX_DIGITS_IN_INT	10	/* maximum number of decimal digits in a  4-byte integer */
+#define MAX_DIGITS_IN_INT8	20	/* maximum number of decimal digits in an 8-byte integer */
+#define MAX_HEX_DIGITS_IN_SHORT	 4	/* maximum number if hexadecimal digits in a  2-byte integer */
+#define MAX_HEX_DIGITS_IN_INT	 8	/* maximum number of hexadecimal digits in a  4-byte integer */
+#define MAX_HEX_DIGITS_IN_INT8	16	/* maximum number of hexadecimal digits in an 8-byte integer */
 
-#define MAX_DIGITS_IN_EXP       2       /* maximum number of decimal digits in an exponent */
+/* These are renames of above macros to cut the number of characters in the macro name in order to
+ * make it easier to fit the macro on a cramped line */
+#define MAX_HEX_SHORT		MAX_HEX_DIGITS_IN_SHORT
+#define MAX_HEX_INT		MAX_HEX_DIGITS_IN_INT
+#define MAX_HEX_INT8		MAX_HEX_DIGITS_IN_INT8
+
+#define MAX_DIGITS_IN_EXP	2	/* maximum number of decimal digits in an exponent */
 #define MAX_HOST_NAME_LEN	256
 #define MAX_LONG_IN_DOUBLE	0xFFFFFFFFFFFFF /*Max Fraction part in IEEE double format*/
 #define MAX_INT_IN_BYTE		255
@@ -610,6 +617,9 @@ MBSTART {					/* also requires threaddef DCL and SETUP*/				\
 #	define	MV_FORCE_LEN_DEC(X) ((!((X)->mvtype & MV_UTF_LEN)) 							\
 				     ? (utf8_len_dec(&(X)->str), ((X)->mvtype |= MV_UTF_LEN), (X)->str.char_len)	\
 				     : (X)->str.char_len)
+#	define	MV_FORCE_LEN_SILENT(X) MV_FORCE_STR(X), ((!((X)->mvtype & MV_UTF_LEN)) 					\
+				     ? (utf8_len_silent(&(X)->str), ((X)->mvtype |= MV_UTF_LEN), (X)->str.char_len)	\
+				     : (X)->str.char_len)
 
 /* MV_FORCE_LEN_STRICT() is used to ensure that mval is valid in addition to computing the char_len.
  * Note that the validation is always forced even if MV_UTF_LEN is set since the previously computed
@@ -1130,6 +1140,7 @@ typedef que_head *	que_head_ptr_t;
   typedef qw_num *qw_num_ptr_t;		/* Define 64 bit pointer to qw_num */
   typedef latch_t *latch_ptr_t;		/* Define 64 bit pointer to latch_t */
   typedef ulatch_t *ulatch_ptr_t;	/* Define 64 bit pointer to ulatch_t */
+  typedef volatile gtm_int64_t *vint8_ptr_t; /* Define 64 bit pointer to volatile int8 */
 
   /* Shared memory connotation */
   typedef char_ptr_t sm_c_ptr_t;	/* Define 64 bit pointer to char */
@@ -1143,6 +1154,7 @@ typedef que_head *	que_head_ptr_t;
   typedef gtm_int64_t sm_long_t;	/* Define 64 bit integer type */
   typedef gtm_uint64_t sm_ulong_t;	/* Define 64 bit unsigned integer type */
   typedef global_latch_t *sm_global_latch_ptr_t; /* Define 64 bit pointer to hp_latch */
+  typedef vint8_ptr_t sm_vlong_ptr_t;	/* Define 64 bit pointer to volatile int8*/
 #  ifdef __osf__
 #    pragma pointer_size(restore)
 #  endif
@@ -1162,6 +1174,7 @@ typedef que_head *	que_head_ptr_t;
   typedef qw_num *qw_num_ptr_t;		/* Define 32 bit pointer to qw_num */
   typedef latch_t *latch_ptr_t;		/* Define 32 bit pointer to latch_t */
   typedef ulatch_t *ulatch_ptr_t;	/* Define 32 bit pointer to ulatch_t */
+  typedef volatile gtm_int64_t *vint8_ptr_t; /* Define 32 bit pointer to volatile int8 */
 
   /* Shared memory connotation */
   typedef char_ptr_t sm_c_ptr_t;	/* Define 32 bit pointer to char */
@@ -1175,6 +1188,7 @@ typedef que_head *	que_head_ptr_t;
   typedef INTPTR_T sm_long_t;		/* Define 32 bit integer type */
   typedef UINTPTR_T sm_ulong_t;		/* Define 32 bit unsigned integer type */
   typedef global_latch_t *sm_global_latch_ptr_t; /* Define 32 bit pointer to hp_latch */
+  typedef vint8_ptr_t sm_vlong_ptr_t;	/* Define 32 bit pointer to volatile int8*/
   /* The macro FILL8DCL is used (on a 32 bit system) to provide a filler area of 32 bits and
      the actual 32 bit declared area. Whether the high order word or the low order word of
      the 64 bit area should be filler depends on the endian mode of the machine. This macro
