@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -29,7 +29,6 @@
 #include "gdscc.h"
 #include "gdskill.h"
 #include "buddy_list.h"		/* needed for tp.h */
-#include "hashtab_int4.h"	/* needed for tp.h */
 #include "tp.h"
 #include "gtmimagename.h"
 #include "cmidef.h"		/* for cmmdef.h */
@@ -44,6 +43,7 @@
 #include "mlk_prcblk_delete.h"
 #include "mlk_shrblk_find.h"
 #include "mlk_shrhash_resize.h"
+#include "mlk_rehash.h"
 #include "mlk_lock.h"
 #include "t_retry.h"
 #include "gvusr.h"
@@ -72,17 +72,15 @@ error_def(ERR_GCAFTER);
  *	> 0 - number of times blocked process was woken up
  * ------------------------------------------------------
  */
-uint4 mlk_lock(mlk_pvtblk *p,
-	       UINTPTR_T  auxown,
-	       bool       new)
+gtm_uint64_t mlk_lock(mlk_pvtblk *p, UINTPTR_T auxown, boolean_t new)
 {
+	boolean_t		blocked, was_crit, added;
+	connection_struct	*curr_entry;	/* for GT.CM GNP server */
+	gtm_uint64_t		retval;
+	int			siz, status;
 	mlk_ctldata_ptr_t	ctl;
 	mlk_shrblk_ptr_t	d;
-	int			siz, retval, status;
-	uint4 old_gc;
-	boolean_t		blocked, was_crit, added;
 	sgmnt_addrs		*csa;
-	connection_struct	*curr_entry;	/* for GT.CM GNP server */
 	DCL_THREADGBL_ACCESS;
 
 	SETUP_THREADGBL_ACCESS;
@@ -106,12 +104,15 @@ uint4 mlk_lock(mlk_pvtblk *p,
 		siz = MLK_PVTBLK_SHRSUB_SIZE(p, p->subscript_cnt);
 		assert(siz >= 0);
 		assert(ctl->blkcnt >= 0);
-		if (ctl->gc_needed || ctl->resize_needed || (ctl->subtop - ctl->subfree < siz) || (ctl->blkcnt < p->subscript_cnt))
+		if (ctl->gc_needed || ctl->resize_needed || ctl->rehash_needed
+			|| (ctl->subtop - ctl->subfree < siz) || (ctl->blkcnt < p->subscript_cnt))
 		{
 			REL_LOCK_CRIT(p->pvtctl, was_crit);
 			prepare_for_gc(&p->pvtctl);
 			GRAB_LOCK_CRIT_AND_SYNC(p->pvtctl, was_crit);
 			assert(ctl->lock_gc_in_progress.u.parts.latch_pid == process_id);
+			if (ctl->rehash_needed)
+				mlk_rehash(&p->pvtctl);
 			if (ctl->resize_needed)
 				mlk_shrhash_resize(&p->pvtctl);
 			else if (ctl->gc_needed || (ctl->subtop - ctl->subfree < siz) || (ctl->blkcnt < p->subscript_cnt))
