@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2009 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -15,14 +16,18 @@
 
 #include "mlkdef.h"
 #include "mlk_prcblk_delete.h"
+#include "min_max.h"
 
-void mlk_prcblk_delete(mlk_ctldata_ptr_t ctl,
+void mlk_prcblk_delete(mlk_pvtctl_ptr_t pctl,
 		       mlk_shrblk_ptr_t d,
 		       uint4 pid)
 {
 	mlk_prcblk_ptr_t	pr;
 	ptroff_t		*prpt;
+	float			ls_free;	/* Free space in bottleneck subspace */
+	mlk_ctldata_ptr_t	ctl;
 
+	ctl = pctl->ctl;
 	for (prpt = (ptroff_t *)&d->pending; *prpt; )
 	{
 		pr = (mlk_prcblk_ptr_t)R2A(*prpt);
@@ -34,7 +39,9 @@ void mlk_prcblk_delete(mlk_ctldata_ptr_t ctl,
 				else
 					A2R(*prpt, R2A(pr->next));
 				memset(pr, 0, SIZEOF(*pr));
-				A2R(pr->next, R2A(ctl->prcfree));
+				pr->next = 0;
+				if (ctl->prcfree)
+					A2R(pr->next, R2A(ctl->prcfree));
 				A2R(ctl->prcfree, pr);
 				assert(ctl->prcfree >= 0);
 				ctl->prccnt++;
@@ -43,5 +50,8 @@ void mlk_prcblk_delete(mlk_ctldata_ptr_t ctl,
 		} else
 				prpt = (ptroff_t *) &pr->next;
 	}
+	ls_free = MIN(((float)ctl->blkcnt) / ctl->max_blkcnt, ((float)ctl->prccnt) / ctl->max_prccnt);
+	if (ls_free >= LOCK_SPACE_FULL_SYSLOG_THRESHOLD)
+		ctl->lockspacefull_logged = FALSE; /* Allow syslog writes if enough free space is established. */
 	return;
 }
