@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001 Sanchez Computer Associates, Inc.	*
+ * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -12,6 +13,6 @@
 #ifndef MLK_WAKE_PENDING_INCLUDED
 #define MLK_WAKE_PENDING_INCLUDED
 
-void mlk_wake_pending(mlk_ctldata_ptr_t ctl, mlk_shrblk_ptr_t d, gd_region *reg);
+void mlk_wake_pending(mlk_pvtctl_ptr_t pctl, mlk_shrblk_ptr_t d);
 
 #endif /* MLK_WAKE_PENDING_INCLUDED */
