@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -17,6 +17,8 @@
 
 #include <sys/types.h>
 
+#include "mmrhash.h"
+
 typedef struct	mlk_tp_struct
 {
 	struct mlk_tp_struct	*next;		/* next stacking struct */
@@ -35,6 +37,22 @@ typedef struct				/* One of these nodes is required for each process which is bl
 	short		filler_4byte;
 } mlk_prcblk;
 
+/* Types and macros for computing subscript hash values */
+typedef hash128_state_t				mlk_subhash_state_t;
+typedef gtm_uint8				mlk_subhash_seed_t;
+typedef gtm_uint16				mlk_subhash_res_t;
+typedef uint4					mlk_subhash_val_t;
+#define MLK_SUBHASH_INIT(PVTBLK, STATEVAR)							\
+MBSTART {											\
+	assert(NULL != (PVTBLK)->pvtctl.ctl);							\
+	(PVTBLK)->hash_seed = (PVTBLK)->pvtctl.ctl->hash_seed;					\
+	HASH128_STATE_INIT(STATEVAR, (PVTBLK)->hash_seed);					\
+} MBEND
+#define MLK_SUBHASH_INIT_PVTCTL(PVTCTL, STATEVAR)	HASH128_STATE_INIT(STATEVAR, (PVTCTL)->ctl->hash_seed)
+#define MLK_SUBHASH_INGEST(STATEVAR, DATA, SIZE)	gtmmrhash_128_ingest(&(STATEVAR), (DATA), (SIZE))
+#define MLK_SUBHASH_FINALIZE(STATEVAR, LENGTH, RESVAR)	gtmmrhash_128_result(&(STATEVAR), LENGTH, &(RESVAR))
+#define MLK_SUBHASH_RES_VAL(RES)			((mlk_subhash_val_t)(RES).one)
+
 typedef struct				/* lock node.  The member descriptions below are correct if the entry
 					 * is in the tree.  If the entry is in the free list, then all
 					 * of the members are meaningless, except for rsib, which is a
@@ -54,7 +72,7 @@ typedef struct				/* lock node.  The member descriptions below are correct if th
 					 * during a direct re-access via pointer from a mlk_pvtblk, the
 					 * sequence numbers do not match, then we must assume that the
 					 * lock was stolen from us by LKE or some other abnormal event. */
-	uint4		hash;		/* The hash value associated with this node, based on the shrsubs of this node
+	mlk_subhash_val_t	hash;	/* The hash value associated with this node, based on the shrsubs of this node
 					 * and its parents. Copied from a mlk_pvtblk via MLK_PVTBLK_SUBHASH().
 					 */
 	int4		auxpid;		/* If non-zero auxowner, this is the pid of the client that is holding the lock */
@@ -62,6 +80,7 @@ typedef struct				/* lock node.  The member descriptions below are correct if th
 	unsigned char	auxnode[16];	/* If non-zero auxowner, this is the nodename of the client that is holding the lock */
 } mlk_shrblk;
 
+/* Types and structures for shrblk lookup hash table */
 #if !defined(MLK_SHRHASH_MAP_32) && !defined(MLK_SHRHASH_MAP_64)
 #define	MLK_SHRHASH_MAP_32
 #endif
@@ -84,7 +103,7 @@ typedef struct mlk_shrhash_struct
 						 * if (bucket+N) % nbuckets is associated with this bucket, i.e.,
 						 * part of this bucket's neighborhood.
 						 */
-	uint4			hash;		/* Hash value associated with the shrblk referenced by this hash bucket.
+	mlk_subhash_val_t	hash;		/* Hash value associated with the shrblk referenced by this hash bucket.
 						 * Compare the hash value before comparing the pvtblk value against the
 						 * shrblk/shrsub chain
 						 */
@@ -128,17 +147,19 @@ typedef struct	mlk_ctldata_struct	/* this describes the entire shared lock secti
 	ptroff_t	subbase;		/* relative pointer to the base of the mlk_shrsub area */
 	ptroff_t	subfree;		/* relative pointer to the first free cell in the shrsub area */
 	ptroff_t	subtop;			/* relative pointer to the top of the shrsub area */
+	gtm_uint64_t	wakeups;		/* lock wakeup counter */
 	uint4		max_prccnt;		/* maximum number of entries in the prcfree chain */
 	uint4		max_blkcnt;		/* maximum number of entries in the blkfree chain */
 	uint4		num_blkhash;		/* number of hash buckets */
 	int4		prccnt;			/* number of entries in the prcfree chain */
 	int4		blkcnt;			/* number of entries in the blkfree chain */
-	unsigned int	wakeups;		/* lock wakeup counter */
 	boolean_t	lockspacefull_logged;	/* whether LOCKSPACEFULL has been issued since last being below threshold */
 	boolean_t	gc_needed;		/* whether we've determined that a garbage collection is needed */
 	boolean_t	resize_needed;		/* whether we've determined that a hash table resize is needed */
+	boolean_t	rehash_needed;		/* whether we've determined that a hash table rehash is needed */
 	global_latch_t	lock_gc_in_progress;	/* pid of the process doing the GC, or 0 if none */
-	int		hash_shmid;		/* shared memory id of hash table, or undefined if internal (see blkhash) */
+	mlk_subhash_seed_t	hash_seed;		/* seed value to use to initialize hash */
+	int		hash_shmid;		/* shared memory id of hash table, or INVALID_SHMID if internal. */
 } mlk_ctldata;
 
 /* Define types for shared memory resident structures */
@@ -198,6 +219,7 @@ typedef struct	mlk_pvtblk_struct	/* one of these entries exists for each nref wh
 	uint4			sequence;		/* shrblk sequence for nodptr node (node we want) */
 	uint4			blk_sequence;		/* shrblk sequence for blocked node (node preventing our lock) */
 	mlk_tp			*tp;			/* pointer to saved tp information */
+	mlk_subhash_seed_t	hash_seed;		/* seed used to initialize hash */
 	uint4			nref_length;		/* the length of the nref portion of the 'value' string. */
 	unsigned short		subscript_cnt;		/* the number of subscripts (plus one for the name) in this nref */
 	unsigned		level : 9;		/* incremental lock level */
@@ -249,7 +271,7 @@ typedef struct	mlk_pvtblk_struct	/* one of these entries exists for each nref wh
 
 /* compute the true size of a mlk_pvtblk, excluding any GT.CM id */
 #define MLK_PVTBLK_SIZE(NREF_LEN, SUBCNT) (ROUND_UP(SIZEOF(mlk_pvtblk) - 1 + (NREF_LEN), SIZEOF(uint4))		\
-						+ SIZEOF(uint4) * (SUBCNT))
+						+ SIZEOF(mlk_subhash_val_t) * (SUBCNT))
 
 #define MLK_PVTBLK_ALLOC(NREF_LEN, SUBCNT, AUX_LEN, RET)							\
 MBSTART {													\
@@ -270,24 +292,36 @@ MBSTART {													\
 } MBEND
 
 /* compute the location of the Nth subscript's hash */
-#define MLK_PVTBLK_SUBHASH(PVTBLK, N) (((uint4 *)&(PVTBLK)->value[ROUND_UP((PVTBLK)->nref_length, SIZEOF(uint4))])[N])
+#define MLK_PVTBLK_SUBHASH(PVTBLK, N)												\
+		(((mlk_subhash_val_t *)&(PVTBLK)->value[ROUND_UP((PVTBLK)->nref_length, SIZEOF(mlk_subhash_val_t))])[N])
 
 /* populate hash data from nref data - keep in sync with the versions in mlk_pvtblk_create() and mlk_shrhash_delete() */
 #define MLK_PVTBLK_SUBHASH_GEN(PVTBLK)							\
 MBSTART {										\
 	unsigned char		*cp;							\
 	int			hi;							\
-	hash128_state_t		accstate, tmpstate;					\
-	gtm_uint16		hashres;						\
+	mlk_subhash_state_t	accstate, tmpstate;					\
+	mlk_subhash_res_t	hashres;						\
 											\
-	HASH128_STATE_INIT(accstate, 0);						\
+	MLK_SUBHASH_INIT(PVTBLK, accstate);						\
 	for (cp = (PVTBLK)->value, hi = 0; hi < (PVTBLK)->subscript_cnt; hi++)		\
 	{										\
-		gtmmrhash_128_ingest(&accstate, cp, *cp + 1);				\
+		MLK_SUBHASH_INGEST(accstate, cp, *cp + 1);				\
 		cp += *cp + 1;								\
 		tmpstate = accstate;							\
-		gtmmrhash_128_result(&tmpstate, (cp - (PVTBLK)->value), &hashres);	\
-		MLK_PVTBLK_SUBHASH(PVTBLK, hi) = (uint4)hashres.one;			\
+		MLK_SUBHASH_FINALIZE(tmpstate, (cp - (PVTBLK)->value), hashres);	\
+		MLK_PVTBLK_SUBHASH(PVTBLK, hi) = MLK_SUBHASH_RES_VAL(hashres);		\
+	}										\
+} MBEND
+
+/* ensure that the seed is correct before using pvtblk hashes */
+#define MLK_PVTBLK_SUBHASH_SYNC(PVTBLK)							\
+MBSTART {										\
+	assert(NULL != (PVTBLK)->pvtctl.ctl);						\
+	if ((PVTBLK)->hash_seed != (PVTBLK)->pvtctl.ctl->hash_seed)			\
+	{										\
+		MLK_PVTBLK_SUBHASH_GEN(PVTBLK);						\
+		assert((PVTBLK)->hash_seed == (PVTBLK)->pvtctl.ctl->hash_seed);		\
 	}										\
 } MBEND
 
@@ -328,7 +362,7 @@ typedef struct mlk_stats_struct
 
 #define CHECK_SHRBLKPTR(RPTR, PCTL)											\
 	assert((RPTR == 0)												\
-		|| (((mlk_shrblk_ptr_t)R2A(RPTR) > (PCTL).shrblk) 							\
+		|| (((mlk_shrblk_ptr_t)R2A(RPTR) > (PCTL).shrblk)							\
 			&& ((mlk_shrblk_ptr_t)R2A(RPTR) < ((PCTL).shrblk + (PCTL).ctl->max_blkcnt + 1))))
 
 #define INVALID_LSIB_MARKER	0x1ee4c0de
@@ -340,7 +374,7 @@ typedef struct mlk_stats_struct
 							 */
 
 /* macros to grab and release a critical section (either shared with DB or not) for LOCK operations */
-#define GRAB_LOCK_CRIT_AND_SYNC(PCTL, RET_WAS_CRIT)								\
+#define GRAB_LOCK_CRIT_AND_SYNC(PCTL, RET_WAS_CRIT)						\
 		grab_lock_crit_and_sync(&(PCTL), &(RET_WAS_CRIT))
 #define GRAB_LOCK_CRIT_INTL(PCTL, RET_WAS_CRIT)							\
 		grab_lock_crit_intl(&(PCTL), &(RET_WAS_CRIT))
