@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -32,6 +32,10 @@
 #include "gtmmsg.h"
 #include "get_spec.h"
 #include "mupip_integ.h"
+#include "repl_msg.h"		/* For gtmsource.h */
+#include "filestruct.h"		/* For gtmsource.h */
+#include "gtmsource.h"		/* for jnlpool_addrs_ptr_t */
+#include "anticipatory_freeze.h"
 #ifdef GTM_TRIGGER
 #include <rtnhdr.h>		/* for rtn_tabent in gv_trigger.h */
 #include "gv_trigger.h"
@@ -49,6 +53,76 @@
 #define SPAN_BYTE_MIN	1
 #define SPAN_SUBS_OFF	48
 
+/* These Macros are called if there's an integ error in this block.
+ * Retry this block only if not already being retried.
+ * If the block is being retried, Simply return FALSE, without printing the error
+ * If a retry returns a FALSE (also has errors), print the original error and continue
+ * Simply continue in case the retry passes.
+ */
+#define MU_INT_RETRY_INCR_PLEN(ERR, PATH, RANGE, BOT, HAS_BOT, TOP, HAS_TOP, LEVEL)						\
+{																\
+	if (instance_frozen && !block_being_retried)										\
+	{															\
+		if (NOT_RETRIED == block_already_retried)									\
+		{														\
+			if (debug_mupip)											\
+				util_out_print("!/MUPIP INFO: Retrying DATA block : !UL; Original error : !UL", TRUE, blk, ERR);\
+			if (!mu_int_blk(blk, level, is_root, bot_key, bot_len, top_key, top_len, eb_ok, ondsk_blkver, TRUE))	\
+			{													\
+				block_already_retried = RETRY_FAIL;								\
+				mu_int_err(ERR, PATH, RANGE, BOT, HAS_BOT, TOP, HAS_TOP, LEVEL);				\
+				mu_int_plen++;											\
+			} else													\
+				block_already_retried = RETRY_SUCCESS;								\
+		} else if (RETRY_FAIL == block_already_retried)									\
+		{														\
+			mu_int_err(ERR, PATH, RANGE, BOT, HAS_BOT, TOP, HAS_TOP, LEVEL);					\
+			mu_int_plen++;												\
+		} 														\
+	} else if (block_being_retried) {											\
+		if (debug_mupip)												\
+			util_out_print("!/MUPIP INFO: Failed retry of DATA block : !UL; Retry error : !UL", TRUE, blk, ERR);	\
+		return FALSE;	/* Retry block. don't worry about free_blk_base */						\
+	} else {														\
+		mu_int_err(ERR, PATH, RANGE, BOT, HAS_BOT, TOP, HAS_TOP, LEVEL);						\
+		mu_int_plen++;													\
+	}															\
+}
+#define MU_INT_RETRY_ERR_RETURN(ERR, PATH, RANGE, BOT, HAS_BOT, TOP, HAS_TOP, LEVEL)						\
+{																\
+	if (instance_frozen && !block_being_retried)										\
+	{															\
+		if (NOT_RETRIED == block_already_retried)									\
+		{														\
+			if (debug_mupip)											\
+				util_out_print("!/MUPIP INFO: Retrying DATA block : !UL; Original error : !UL", TRUE, blk, ERR);\
+			if (!mu_int_blk(blk, level, is_root, bot_key, bot_len, top_key, top_len, eb_ok, ondsk_blkver, TRUE))	\
+			{													\
+				mu_int_err(ERR, PATH, RANGE, BOT, HAS_BOT, TOP, HAS_TOP, LEVEL);				\
+				if (NULL != free_blk_base)									\
+					free(free_blk_base);									\
+				return FALSE;											\
+			} else													\
+				block_already_retried = RETRY_SUCCESS;								\
+		} else if (RETRY_FAIL == block_already_retried)									\
+		{														\
+				mu_int_err(ERR, PATH, RANGE, BOT, HAS_BOT, TOP, HAS_TOP, LEVEL);				\
+				if (NULL != free_blk_base)									\
+					free(free_blk_base);									\
+				return FALSE;											\
+		}														\
+	} else if (block_being_retried) {											\
+		if (debug_mupip)												\
+			util_out_print("!/MUPIP INFO: Failed retry of DATA block : !UL; Retry error : !UL", TRUE, blk, ERR);	\
+		return FALSE;	/* Retry block. don't worry about free_blk_base */						\
+	} else {														\
+		mu_int_err(ERR, PATH, RANGE, BOT, HAS_BOT, TOP, HAS_TOP, LEVEL);						\
+		if (NULL != free_blk_base)											\
+			free(free_blk_base);											\
+		return FALSE;													\
+	}															\
+}
+
 GBLDEF unsigned char		muint_temp_buff[MAX_MIDENT_LEN + 1];
 GBLREF unsigned char		*mu_int_locals;
 GBLREF unsigned char		mu_int_root_level;
@@ -56,14 +130,14 @@ GBLREF bool			mu_ctrly_occurred, mu_ctrlc_occurred;
 GBLREF boolean_t		block;
 GBLREF boolean_t		master_dir;
 GBLREF boolean_t		muint_fast;
-GBLREF boolean_t		muint_key;
-GBLREF boolean_t		muint_subsc;
+GBLREF boolean_t		mu_key;
+GBLREF boolean_t		mu_subsc;
 GBLREF boolean_t		tn_reset_this_reg;
 GBLREF int			disp_maxkey_errors;
 GBLREF int			disp_trans_errors;
 GBLREF int			maxkey_errors;
-GBLREF int			muint_end_keyend;
-GBLREF int			muint_start_keyend;
+GBLREF int			mu_end_keyend;
+GBLREF int			mu_start_keyend;
 GBLREF int			mu_int_plen;
 GBLREF int			trans_errors;
 GBLREF int4			mu_int_adj[];
@@ -74,8 +148,8 @@ GBLREF block_id			mu_int_path[];
 GBLREF int4			mu_int_blks_to_upgrd;
 GBLREF global_list		*trees;
 GBLREF global_list		*trees_tail;
-GBLREF gv_key			*muint_end_key;
-GBLREF gv_key			*muint_start_key;
+GBLREF gv_key			*mu_end_key;
+GBLREF gv_key			*mu_start_key;
 GBLREF sgmnt_data		mu_int_data;
 GBLREF trans_num		largest_tn;
 GBLREF span_node_integ		*sndata;
@@ -84,7 +158,9 @@ GBLREF boolean_t		null_coll_type;
 GBLREF unsigned int		rec_num;
 GBLREF block_id			blk_id;
 GBLREF boolean_t		nct_err_type;
+GBLREF boolean_t		debug_mupip;
 GBLREF int			rec_len;
+GBLREF jnlpool_addrs_ptr_t	jnlpool;
 error_def(ERR_DBBADKYNM);
 error_def(ERR_DBBADNSUB);
 error_def(ERR_DBBADPNTR);
@@ -179,26 +255,35 @@ boolean_t mu_int_blk(
 		int bot_len,
 		unsigned char *top_key,
 		int top_len,
-		boolean_t eb_ok)	/* boolean indicating whether an empty block here is ok.  This is true when
-					   the parent is a root with only a star key */
+		boolean_t eb_ok,	/* boolean indicating whether an empty block here is ok.  This is true when
+					 * the parent is a root with only a star key
+					 */
+		enum db_ver ondsk_blkver_saved,	/* When a block is being retried, pass the correct/current DB VER */
+		boolean_t block_being_retried)	/* If this is true, we are retrying a block from buffer */
 {
 	typedef struct
 	{
 		boolean_t	numeric;
 		int		index;
 	} sub_list;
+	enum retry_block
+	{
+		NOT_RETRIED,
+		RETRY_SUCCESS,
+		RETRY_FAIL,
+	} block_already_retried;
 
 	unsigned char	buff[MAX_KEY_SZ + 1], old_buff[MAX_KEY_SZ + 1], temp_buff[MAX_MIDENT_LEN + 1], util_buff[MAX_UTIL_SIZE];
 	unsigned char	blk_levl, *c1, ch, *ctrlbytes, subscript;
 	unsigned short	cc, rec_cmpc;
-	uchar_ptr_t	c0, c2, c_base, blk_base, blk_top, key_base, ptr, rec_base, rec_top, span_key;
+	uchar_ptr_t	c0, c2, c_base, blk_base, blk_base2, blk_top, key_base, ptr, rec_base, rec_top, span_key;
 	unsigned short	temp_ushort;
-	boolean_t	first_key, is_top, pstar, valid_gbl, hasht_global;
+	boolean_t	first_key, is_top, pstar, valid_gbl, hasht_global, instance_frozen;
 	boolean_t	muint_range_done = FALSE, nct_checked = FALSE;
 	int		blk_size, buff_length, b_index, cmcc, comp_length, key_size, len, name_len,
 			num_len, rec_size, s_index, start_index, sub_start_index, hdr_len, idx;
 	int		tmp_cmpc, tmp_numsubs, max_allowed_key_size;
-	block_id	child, root_pointer;
+	block_id	child, root_pointer, blk_lmap;
 	sub_list	mu_sub_list[MAX_GVSUBSCRIPTS + 1];
 	sub_num		check_vals;
 	trans_num	blk_tn;
@@ -207,16 +292,30 @@ boolean_t mu_int_blk(
 	uint4		cnt, span_curr_blk, rval_len, gblsize;
 	unsigned short	numsubs;
 	unsigned int	null_subscript_cnt;
+	DCL_THREADGBL_ACCESS;
 
+	SETUP_THREADGBL_ACCESS;
+	instance_frozen = TREF(instance_frozen_crit_skipped);
+	block_already_retried = FALSE;
 	mu_int_offset[mu_int_plen] = 0;
-	mu_int_path[mu_int_plen++] = blk;  /* Increment mu_int_plen on entry; decrement explicitly or via mu_int_err() on exit. */
-	mu_int_path[mu_int_plen] = 0;
-	if (!bml_busy(blk, mu_int_locals)) /* block already marked busy */
-	{
+	if (!block_being_retried)	/* Not if the block is being retried */
+	{ /* Increment mu_int_plen on entry; decrement explicitly or via mu_int_err() on exit. */
+		mu_int_path[mu_int_plen++] = blk;
+		mu_int_path[mu_int_plen] = 0;
+	}
+	blk_lmap = (blk / mu_int_data.bplmap) * mu_int_data.bplmap;
+	if (!block_being_retried && !bml_busy(blk - blk_lmap, mu_int_locals + ((blk_lmap * BML_BITS_PER_BLK) / BITS_PER_UCHAR)))
+	{	/* block already marked busy */
 		mu_int_err(ERR_DBBDBALLOC, TRUE, TRUE, bot_key, bot_len, top_key, top_len, (unsigned int)(level));
 		return FALSE;
 	}
-	blk_base = mu_int_read(blk, &ondsk_blkver, &free_blk_base);	/* ondsk_blkver set to GDSV4 or GDSV6 (GDSVCURR) */
+	if (block_being_retried)
+	{
+		blk_base = mu_int_read_buffer(blk, &ondsk_blkver, &free_blk_base);	/* Read block from buffer */
+		ondsk_blkver = ondsk_blkver_saved;
+		free_blk_base = NULL;
+	} else
+		blk_base = mu_int_read(blk, &ondsk_blkver, &free_blk_base);	/* ondsk_blkver set to GDSV4 or GDSV6 (GDSVCURR) */
 	if (!blk_base)
 		return FALSE;	/* Only occurs on malloc failure, so don't worry about mu_int_plen. */
 	blk_size = (int)((blk_hdr_ptr_t)blk_base)->bsiz;
@@ -236,17 +335,13 @@ boolean_t mu_int_blk(
 	pstar = (is_root && (SIZEOF(blk_hdr) + SIZEOF(rec_hdr) + SIZEOF(block_id) == blk_size));
 	if (blk_size < (SIZEOF(blk_hdr) + (eb_ok ? 0 : (SIZEOF(rec_hdr) + (level ? SIZEOF(block_id) : MIN_DATA)))))
 	{
-		mu_int_err(ERR_DBBSIZMN, TRUE, TRUE, bot_key, bot_len, top_key, top_len,
+		MU_INT_RETRY_ERR_RETURN(ERR_DBBSIZMN, TRUE, TRUE, bot_key, bot_len, top_key, top_len,
 				(unsigned int)((blk_hdr_ptr_t)blk_base)->levl);
-		free(free_blk_base);
-		return FALSE;
 	}
 	if (blk_size > mu_int_data.blk_size)
 	{
-		mu_int_err(ERR_DBBSIZMX, TRUE, TRUE, bot_key, bot_len, top_key, top_len,
+		MU_INT_RETRY_ERR_RETURN(ERR_DBBSIZMX, TRUE, TRUE, bot_key, bot_len, top_key, top_len,
 			(unsigned int)((blk_hdr_ptr_t)blk_base)->levl);
-		free(free_blk_base);
-		return FALSE;
 	}
 	blk_top = blk_base + blk_size;
 	blk_levl = ((blk_hdr_ptr_t)blk_base)->levl;
@@ -256,33 +351,26 @@ boolean_t mu_int_blk(
 	{
 		if (blk_levl >= MAX_BT_DEPTH)
 		{
-			mu_int_err(ERR_DBRLEVTOOHI, 0, 0, 0, 0, 0, 0, (unsigned int)blk_levl);
-			free(free_blk_base);
-			return FALSE;
+			MU_INT_RETRY_ERR_RETURN(ERR_DBRLEVTOOHI, 0, 0, 0, 0, 0, 0, (unsigned int)blk_levl);
 		}
 		if (blk_levl < 1)
 		{
-			mu_int_err(ERR_DBRLEVLTONE, 0, 0, 0, 0, 0, 0, (unsigned int)blk_levl);
-			free(free_blk_base);
-			return FALSE;
+			MU_INT_RETRY_ERR_RETURN(ERR_DBRLEVLTONE, 0, 0, 0, 0, 0, 0, (unsigned int)blk_levl);
 		}
 		mu_int_root_level = level = blk_levl;
 	} else  if (blk_levl != level)
 	{
-		mu_int_err(ERR_DBINCLVL, TRUE, TRUE, bot_key, bot_len, top_key, top_len, (unsigned int)blk_levl);
-		free(free_blk_base);
-		return FALSE;
+		MU_INT_RETRY_ERR_RETURN(ERR_DBINCLVL, TRUE, TRUE, bot_key, bot_len, top_key, top_len, (unsigned int)blk_levl);
 	}
-	if (!master_dir)
+	if (!master_dir && !block_being_retried)
 		CHECK_ADJACENCY(blk, level, mu_int_adj[level]);
 	blk_tn = ((blk_hdr_ptr_t)blk_base)->tn;
 	if (blk_tn >= mu_int_data.trans_hist.curr_tn)
 	{
 		if (trans_errors < disp_trans_errors)
 		{
-			mu_int_err(ERR_DBTNTOOLG, TRUE, TRUE, bot_key, bot_len, top_key, top_len,
+			MU_INT_RETRY_INCR_PLEN(ERR_DBTNTOOLG, TRUE, TRUE, bot_key, bot_len, top_key, top_len,
 					(unsigned int)blk_levl);
-			mu_int_plen++;	/* continuing, so compensate for mu_int_err decrement */
 			gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(3) ERR_DBTN, 1, &blk_tn);
 			trans_errors++;
 		} else
@@ -300,14 +388,18 @@ boolean_t mu_int_blk(
 		if (muint_fast)
 		{
 			mu_int_plen--;
-			free(free_blk_base);
+			if (NULL != free_blk_base)
+				free(free_blk_base);
 			return FALSE;
 		}
 		if (blk_tn > largest_tn)
 			largest_tn = blk_tn;
 	}
-	mu_int_cum[BLKS][level]++;
-	mu_int_cum[SIZE][level] += blk_size;
+	if (!block_being_retried)
+	{
+		mu_int_cum[BLKS][level]++;
+		mu_int_cum[SIZE][level] += blk_size;
+	}
 	first_key = TRUE;
 	buff_length = 0;
 	comp_length = bot_len;
@@ -321,21 +413,20 @@ boolean_t mu_int_blk(
 		rec_num++;
 		if (mu_ctrly_occurred || mu_ctrlc_occurred)
 			return FALSE;	/* Only happens on termination, so don't worry about mu_int_plen. */
-		mu_int_cum[RECS][level]++;
+		if (!block_being_retried)
+			mu_int_cum[RECS][level]++;
 		GET_USHORT(temp_ushort, &(((rec_hdr_ptr_t)rec_base)->rsiz));
 		rec_size = temp_ushort;
 		mu_int_offset[mu_int_plen - 1] = (uint4)(rec_base - blk_base);
 		if (rec_size <= SIZEOF(rec_hdr))
 		{
-			mu_int_err(ERR_DBRSIZMN, TRUE, TRUE, buff, comp_length, top_key, top_len, (unsigned int)blk_levl);
-			free(free_blk_base);
-			return FALSE;
+			MU_INT_RETRY_ERR_RETURN(ERR_DBRSIZMN, TRUE, TRUE, buff, comp_length, top_key, top_len,
+											(unsigned int)blk_levl);
 		}
 		if ((rec_size > blk_top - rec_base))
 		{
-			mu_int_err(ERR_DBRSIZMX, TRUE, TRUE, buff, comp_length, top_key, top_len, (unsigned int)blk_levl);
-			free(free_blk_base);
-			return FALSE;
+			MU_INT_RETRY_ERR_RETURN(ERR_DBRSIZMX, TRUE, TRUE, buff, comp_length, top_key, top_len,
+											(unsigned int)blk_levl);
 		}
 		rec_top = rec_base + rec_size;
 		rec_cmpc = EVAL_CMPC((rec_hdr_ptr_t)rec_base);
@@ -344,15 +435,13 @@ boolean_t mu_int_blk(
 			is_top = TRUE;
 			if (SIZEOF(rec_hdr) + SIZEOF(block_id) != rec_size)
 			{
-				mu_int_err(ERR_DBLRCINVSZ, TRUE, TRUE, buff, comp_length, top_key, top_len, (unsigned int)blk_levl);
-				free(free_blk_base);
-				return FALSE;
+				MU_INT_RETRY_ERR_RETURN(ERR_DBLRCINVSZ, TRUE, TRUE, buff, comp_length, top_key, top_len,
+											(unsigned int)blk_levl);
 			}
 			if (rec_cmpc)
 			{
-				mu_int_err(ERR_DBSTARCMP, TRUE, TRUE, buff, comp_length, top_key, top_len, (unsigned int)blk_levl);
-				free(free_blk_base);
-				return FALSE;
+				MU_INT_RETRY_ERR_RETURN(ERR_DBSTARCMP, TRUE, TRUE, buff, comp_length, top_key, top_len,
+											(unsigned int)blk_levl);
 			}
 			ptr = rec_base + SIZEOF(rec_hdr);
 		} else
@@ -361,14 +450,12 @@ boolean_t mu_int_blk(
 			{
 				if (rec_cmpc)
 				{
-					mu_int_err(ERR_DBCMPNZRO, TRUE, TRUE, buff,
+					MU_INT_RETRY_ERR_RETURN(ERR_DBCMPNZRO, TRUE, TRUE, buff,
 						comp_length, top_key, top_len, (unsigned int)blk_levl);
-					free(free_blk_base);
-					return FALSE;
 				}
 			} else  if ((rec_cmpc < name_len) && (FALSE == master_dir))
 			{
-				mu_int_err(ERR_DBINVGBL, TRUE, TRUE, buff, comp_length, top_key, top_len,
+				MU_INT_RETRY_INCR_PLEN(ERR_DBINVGBL, TRUE, TRUE, buff, comp_length, top_key, top_len,
 					(unsigned int)blk_levl);
 				if (!level)
 				{	/* since global names are mixed, the numeric subscript check done later below
@@ -378,24 +465,19 @@ boolean_t mu_int_blk(
 					 */
 					mu_sub_list[0].index = NO_SUBSCRIPTS;
 				}
-				mu_int_plen++;	/* continuing, so compensate for mu_int_err decrement */
 			}
 			if (rec_cmpc && (short int)rec_cmpc >= buff_length)
 			{
-				mu_int_err(ERR_DBCOMPTOOLRG, TRUE, TRUE, buff, comp_length, top_key,
+				MU_INT_RETRY_ERR_RETURN(ERR_DBCOMPTOOLRG, TRUE, TRUE, buff, comp_length, top_key,
 					top_len, (unsigned int)blk_levl);
-				free(free_blk_base);
-				return FALSE;
 			}
 			key_base = rec_base + SIZEOF(rec_hdr);
 			for (ptr = key_base;  ;)
 			{
 				if (ptr >= rec_top)
 				{
-					mu_int_err(ERR_DBKEYMX, TRUE, TRUE, buff, comp_length, top_key, top_len,
+					MU_INT_RETRY_ERR_RETURN(ERR_DBKEYMX, TRUE, TRUE, buff, comp_length, top_key, top_len,
 							(unsigned int)blk_levl);
-					free(free_blk_base);
-					return FALSE;
 				}
 				if (KEY_DELIMITER == *ptr++)
 				{
@@ -420,10 +502,8 @@ boolean_t mu_int_blk(
 							}
 							if (0 != memcmp(trees->key, key_base, trees->keysize + 1))
 							{
-								mu_int_err(ERR_DBINVGBL, TRUE, TRUE, bot_key, bot_len,
+								MU_INT_RETRY_ERR_RETURN(ERR_DBINVGBL, TRUE, TRUE, bot_key, bot_len,
 									top_key, top_len, (unsigned int)blk_levl);
-								free(free_blk_base);
-								return FALSE;
 							}
 						}
 					}
@@ -434,38 +514,29 @@ boolean_t mu_int_blk(
 			key_size = (int)(ptr - key_base);
 			if (level && (rec_size - SIZEOF(block_id) - SIZEOF(rec_hdr) != key_size))
 			{
-				mu_int_err(ERR_DBKEYMN, TRUE, TRUE, buff, comp_length, top_key, top_len,
+				MU_INT_RETRY_ERR_RETURN(ERR_DBKEYMN, TRUE, TRUE, buff, comp_length, top_key, top_len,
 						(unsigned int)blk_levl);
-				free(free_blk_base);
-				return FALSE;
 			}
 			if (key_size + rec_cmpc > MAX_KEY_SZ)
 			{	/* We'll allow index keys to be whatever length, so long as they don't exceed MAX_KEY_SZ */
-				mu_int_err(ERR_DBKGTALLW, TRUE, TRUE, buff, comp_length, top_key,
+				MU_INT_RETRY_ERR_RETURN(ERR_DBKGTALLW, TRUE, TRUE, buff, comp_length, top_key,
 						top_len, (unsigned int)blk_levl);
-				free(free_blk_base);
-				return FALSE;
 			}
 			if ((short int)rec_cmpc < buff_length && buff[rec_cmpc] == *key_base)
 			{
-				mu_int_err(ERR_DBCMPBAD, TRUE, TRUE, buff, comp_length, top_key, top_len,
+				MU_INT_RETRY_ERR_RETURN(ERR_DBCMPBAD, TRUE, TRUE, buff, comp_length, top_key, top_len,
 					(unsigned int)blk_levl);
-				free(free_blk_base);
-				return FALSE;
 			}
 			if (memvcmp(buff + rec_cmpc, comp_length - rec_cmpc, key_base, key_size) >= 0)
 			{
 				if (3 == mu_int_offset[mu_int_plen - 1])
 				{
-					mu_int_err(ERR_DBLTSIBL,
+					MU_INT_RETRY_INCR_PLEN(ERR_DBLTSIBL,
 							TRUE, TRUE, buff, comp_length, top_key, top_len, (unsigned int)blk_levl);
-					mu_int_plen++;	/* continuing, so compensate for mu_int_err decrement */
 				} else
 				{
-					mu_int_err(ERR_DBKEYORD, TRUE, TRUE, buff, comp_length, top_key, top_len,
+					MU_INT_RETRY_ERR_RETURN(ERR_DBKEYORD, TRUE, TRUE, buff, comp_length, top_key, top_len,
 						(unsigned int)blk_levl);
-					free(free_blk_base);
-					return FALSE;
 				}
 			}
 			memcpy(old_buff, buff, comp_length);
@@ -506,51 +577,56 @@ boolean_t mu_int_blk(
 				valid_gbl = (valid_gbl && (KEY_DELIMITER == buff[idx]) && (KEY_DELIMITER == buff[idx + 1]));
 				if (!valid_gbl)
 				{
-					mu_int_err(ERR_DBBADKYNM, TRUE, TRUE, buff, buff_length, top_key, top_len,
+					MU_INT_RETRY_INCR_PLEN(ERR_DBBADKYNM, TRUE, TRUE, buff, buff_length, top_key, top_len,
 						(unsigned int)blk_levl);
-					mu_int_plen++;  /* continuing, so compensate for mu_int_err decrement */
 				}
 			}
 			if (!master_dir)
 			{	/* master_directory has no subscripts; block splits don't preserve numeric integrity in index */
-				if (muint_subsc)
+				if (mu_subsc)
 				{
-					if (muint_end_key)
+					if (mu_end_key)
 					{
-						if (memcmp(buff, muint_end_key->base, muint_end_key->end + 1) > 0)
+						if (memcmp(buff, mu_end_key->base, mu_end_key->end + 1) > 0)
 						{
 							if (level)
 								muint_range_done = TRUE;
 							else
 							{
-								mu_int_cum[RECS][level]--;
+								if (!block_being_retried)
+									mu_int_cum[RECS][level]--;
 								mu_int_plen--;
-								free(free_blk_base);
+								if (NULL != free_blk_base)
+									free(free_blk_base);
 								return TRUE;
 							}
 						}
-						if (memcmp(buff, muint_start_key->base, muint_start_key->end + 1) < 0)
+						if (memcmp(buff, mu_start_key->base, mu_start_key->end + 1) < 0)
 						{
-							mu_int_cum[RECS][level]--;
+							if (!block_being_retried)
+								mu_int_cum[RECS][level]--;
 							continue;
 						}
 					} else
 					{
-						if (memcmp(buff, muint_start_key->base, muint_start_key->end + 1) > 0)
+						if (memcmp(buff, mu_start_key->base, mu_start_key->end + 1) > 0)
 						{
 							if (level)
 								muint_range_done = TRUE;
 							else
 							{
-								mu_int_cum[RECS][level]--;
+								if (!block_being_retried)
+									mu_int_cum[RECS][level]--;
 								mu_int_plen--;
-								free(free_blk_base);
+								if (NULL != free_blk_base)
+									free(free_blk_base);
 								return TRUE;
 							}
 						}
-						if (memcmp(buff, muint_start_key->base, muint_start_key->end + 1) < 0)
+						if (memcmp(buff, mu_start_key->base, mu_start_key->end + 1) < 0)
 						{
-							mu_int_cum[RECS][level]--;
+							if (!block_being_retried)
+								mu_int_cum[RECS][level]--;
 							continue;
 						}
 					}
@@ -590,9 +666,8 @@ boolean_t mu_int_blk(
 					}
 					if (MAX_GVSUBSCRIPTS < s_index)
 					{
-						mu_int_err(ERR_DBMAXNRSUBS, TRUE, TRUE, buff,
+						MU_INT_RETRY_INCR_PLEN(ERR_DBMAXNRSUBS, TRUE, TRUE, buff,
 							comp_length, top_key, top_len, (unsigned int)blk_levl);
-						mu_int_plen++;	/* continuing, so compensate for mu_int_err decrement */
 						break;
 					}
 					nct_checked = FALSE;
@@ -610,11 +685,9 @@ boolean_t mu_int_blk(
 									memcpy(&check_vals, &buff[b_index], 1);
 									if (!mu_int_possub[check_vals.one][check_vals.two])
 									{
-										mu_int_err(ERR_DBBADNSUB, TRUE, TRUE,
+										MU_INT_RETRY_ERR_RETURN(ERR_DBBADNSUB, TRUE, TRUE,
 											buff, comp_length, top_key, top_len,
 											(unsigned int)blk_levl);
-										free(free_blk_base);
-										return FALSE;
 									}
 									b_index++;
 								}
@@ -626,29 +699,24 @@ boolean_t mu_int_blk(
 									memcpy(&check_vals, &buff[b_index], 1);
 									if (!mu_int_negsub[check_vals.one][check_vals.two])
 									{
-										mu_int_err(ERR_DBBADNSUB, TRUE, TRUE,
+										MU_INT_RETRY_ERR_RETURN(ERR_DBBADNSUB, TRUE, TRUE,
 											buff, comp_length, top_key, top_len,
 											(unsigned int)blk_levl);
-										free(free_blk_base);
-										return FALSE;
 									}
 									b_index++;
 								}
 								if (STR_SUB_PREFIX != buff[b_index++] || (buff[b_index]))
 								{
-									mu_int_err(ERR_DBBADNSUB, TRUE, TRUE, buff, comp_length,
-											top_key, top_len, (unsigned int)blk_levl);
-									free(free_blk_base);
-									return FALSE;
+									MU_INT_RETRY_ERR_RETURN(ERR_DBBADNSUB, TRUE, TRUE, buff,
+									comp_length, top_key, top_len, (unsigned int)blk_levl);
 								}
 							}
 							if (!nct_checked && trees->nct)
 							{
 								nct_checked = TRUE;
 								nct_err_type = TRUE;
-								mu_int_err(ERR_DBNONUMSUBS, TRUE, TRUE, buff, comp_length, top_key,
-										top_len, (unsigned int)blk_levl);
-								mu_int_plen++;
+								MU_INT_RETRY_INCR_PLEN(ERR_DBNONUMSUBS, TRUE, TRUE, buff,
+									comp_length, top_key, top_len, (unsigned int)blk_levl);
 								nct_err_type = FALSE;
 							}
 						}
@@ -671,18 +739,16 @@ boolean_t mu_int_blk(
 								null_coll_type_err = TRUE;
 								null_coll_type = mu_int_data.std_null_coll;
 								blk_id = blk;
-								mu_int_err(ERR_DBNULCOL, TRUE, TRUE, buff, comp_length, top_key,
-										top_len, (unsigned int)blk_levl);
-								mu_int_plen++;
+								MU_INT_RETRY_INCR_PLEN(ERR_DBNULCOL, TRUE, TRUE, buff, comp_length,
+										top_key, top_len, (unsigned int)blk_levl);
 								null_coll_type_err = FALSE;
 							}
 						}
 					}
 					if (0 < null_subscript_cnt && !mu_int_data.null_subs)
 					{
-						mu_int_err(ERR_NULSUBSC, TRUE, TRUE, buff, comp_length, 0,0,
+						MU_INT_RETRY_INCR_PLEN(ERR_NULSUBSC, TRUE, TRUE, buff, comp_length, 0,0,
 							(unsigned int)blk_levl);
-						 mu_int_plen++;
 					}
 				}
 			}
@@ -692,10 +758,8 @@ boolean_t mu_int_blk(
 				boolean_t is_trigger = (STRCMP(top_key, "#t") == 0 || STRCMP(bot_key, "#t") == 0) ? TRUE : FALSE;
 				if (mu_int_data.max_rec_size < rval_len && !is_trigger)
 				{
-					mu_int_err(ERR_DBDATAMX, TRUE, TRUE, buff, comp_length,
+					MU_INT_RETRY_ERR_RETURN(ERR_DBDATAMX, TRUE, TRUE, buff, comp_length,
 							top_key, top_len, (unsigned int)blk_levl);
-					free(free_blk_base);
-					return FALSE;
 				}
 				span_key = buff + buff_length - SPAN_SUBS_LENGTH - 1;
 				if (is_trigger)
@@ -755,26 +819,22 @@ boolean_t mu_int_blk(
 								}
 								if (sndata->val_len > sndata->span_node_sz)
 								{
-									mu_int_err(ERR_DBDATAMX, TRUE, TRUE,
+									MU_INT_RETRY_ERR_RETURN(ERR_DBDATAMX, TRUE, TRUE,
 										   sndata->span_node_buf,
 										   sndata->key_len, top_key,
 										   top_len,
 										   (unsigned int)blk_levl);
-									free(free_blk_base);
-									return FALSE;
 								}
 							}
 							else { /* ERROR 1: There is discontinuity in the spanning node
 							  	* blocks; adjacent spanning block is missing
 							  	*/
-								mu_int_err(ERR_DBSPANGLOINCMP, TRUE, FALSE,
+								MU_INT_RETRY_INCR_PLEN(ERR_DBSPANGLOINCMP, TRUE, FALSE,
 									   sndata->span_node_buf,
 									   sndata->key_len, top_key,
 								 	   top_len, (unsigned int)blk_levl);
 								sndata->sn_type = SN_NOT;
 								sndata->sn_blk_cnt += sndata->span_blk_cnt;
-								/* continuing, so compensate for mu_int_err decrement */
-								mu_int_plen++;
 								maxkey_errors++;
 							}
 							break;
@@ -787,7 +847,7 @@ boolean_t mu_int_blk(
 							else { /* ERROR 2: Unexpected spanning-node-block occurred in
 							 	* the middle of spanning-node fragment
 							  	*/
-								mu_int_err(ERR_DBSPANCHUNKORD, TRUE, FALSE,
+								MU_INT_RETRY_INCR_PLEN(ERR_DBSPANCHUNKORD, TRUE, FALSE,
 									   sndata->span_node_buf,
 									   sndata->key_len, top_key,
 									   top_len, (unsigned int)blk_levl);
@@ -798,8 +858,6 @@ boolean_t mu_int_blk(
 								sndata->key_len = buff_length;
 								memcpy(sndata->span_node_buf, buff, buff_length);
 								sndata->sn_blk_cnt += sndata->span_blk_cnt;
-								/* continuing, so compensate for mu_int_err decrement */
-								mu_int_plen++;
 								maxkey_errors++;
 							}
 							break;
@@ -811,20 +869,18 @@ boolean_t mu_int_blk(
 					{ /* ERROR 1: There is discontinuity in the spanning node blocks;
 					   * adjacent spanning block is missing
 					   */
-						mu_int_err(ERR_DBSPANGLOINCMP, TRUE, FALSE,
+						MU_INT_RETRY_INCR_PLEN(ERR_DBSPANGLOINCMP, TRUE, FALSE,
 							   sndata->span_node_buf, sndata->key_len, top_key,
 							   top_len, (unsigned int)blk_levl);
 					} else /*INCOMPLETE SPANNING NODE FRAGMENT*/
 					{  /* ERROR 2: Spanning-node-block occurred in the middle
 					    * of non-spanning block
 					    */
-						mu_int_err(ERR_DBSPANCHUNKORD, TRUE, FALSE,
+						MU_INT_RETRY_INCR_PLEN(ERR_DBSPANCHUNKORD, TRUE, FALSE,
 							   sndata->span_node_buf, sndata->key_len, top_key,
 							   top_len, (unsigned int)blk_levl);
 					}
 					sndata->sn_blk_cnt += sndata->span_blk_cnt;
-					/* continuing, so compensate for mu_int_err decrement */
-					mu_int_plen++;
 					maxkey_errors++;
 					sndata->sn_type = SN_NOT;
 				}
@@ -832,9 +888,8 @@ boolean_t mu_int_blk(
 				{
 					if (maxkey_errors < disp_maxkey_errors)
 					{
-						mu_int_err(ERR_DBGTDBMAX, TRUE, FALSE, buff, comp_length, top_key,
+						MU_INT_RETRY_INCR_PLEN(ERR_DBGTDBMAX, TRUE, FALSE, buff, comp_length, top_key,
 							top_len, (unsigned int)blk_levl);
-						mu_int_plen++;	/* continuing, so compensate for mu_int_err decrement */
 						maxkey_errors++;
 					} else
 					{
@@ -846,109 +901,101 @@ boolean_t mu_int_blk(
 		}
 		if (level)
 		{
-			GET_LONG(child, ptr);
+			GET_BLK_ID(child, ptr);
 			if (child < 0)
 			{
-				mu_int_err(ERR_DBPTRNOTPOS, TRUE, TRUE, buff, comp_length, top_key, top_len,
+				MU_INT_RETRY_ERR_RETURN(ERR_DBPTRNOTPOS, TRUE, TRUE, buff, comp_length, top_key, top_len,
 						(unsigned int)blk_levl);
-				free(free_blk_base);
-				return FALSE;
 			}
 			if (child > mu_int_data.trans_hist.total_blks)
 			{
-				mu_int_err(ERR_DBPTRMX, TRUE, TRUE, buff, comp_length, top_key,
+				MU_INT_RETRY_ERR_RETURN(ERR_DBPTRMX, TRUE, TRUE, buff, comp_length, top_key,
 						top_len, (unsigned int)blk_levl);
-				free(free_blk_base);
-				return FALSE;
 			}
 			if (!(child % mu_int_data.bplmap))
 			{
-				mu_int_err(ERR_DBBNPNTR, TRUE, TRUE, buff, comp_length, top_key, top_len,
+				MU_INT_RETRY_ERR_RETURN(ERR_DBBNPNTR, TRUE, TRUE, buff, comp_length, top_key, top_len,
 						(unsigned int)blk_levl);
-				free(free_blk_base);
-				return FALSE;
 			}
 			if (!muint_fast || (level > 1) || master_dir)
 			{
 				if (is_top)
-					mu_int_blk(child, level - 1, FALSE, buff, comp_length, top_key, top_len, pstar);
+					mu_int_blk(child, level - 1, FALSE, buff, comp_length, top_key, top_len, pstar, 0, FALSE);
 				else
-					mu_int_blk(child, level - 1, FALSE, old_buff, comp_length, buff, buff_length, pstar);
+					mu_int_blk(child, level - 1, FALSE, old_buff, comp_length, buff, buff_length, pstar, 0,
+															FALSE);
 			} else
 			{
-				if (!bml_busy(child, mu_int_locals))
+				blk_lmap = (child / mu_int_data.bplmap) * mu_int_data.bplmap;
+				if (!bml_busy(child - blk_lmap,
+						mu_int_locals + ((blk_lmap * BML_BITS_PER_BLK) / BITS_PER_UCHAR)))
 				{
 					mu_int_offset[mu_int_plen]=0;
 					mu_int_path[mu_int_plen++]=child;	/* Increment mu_int_plen */
 					mu_int_err(ERR_DBBDBALLOC, TRUE, TRUE, old_buff, comp_length, buff, buff_length,
 							(unsigned int)((blk_hdr_ptr_t)ptr)->levl);
 					mu_int_plen--;				/* Revert above increment */
-					free(free_blk_base);
+					if (NULL != free_blk_base)
+						free(free_blk_base);
 					return FALSE;
 				}
-				mu_int_cum[BLKS][0]++;
-				if (muint_fast && (1 == level))
+				if (!block_being_retried)
+					mu_int_cum[BLKS][0]++;
+				if (muint_fast && (1 == level) && !block_being_retried)
 					CHECK_ADJACENCY(child, 0, mu_int_adj[0]);
 			}
 		} else
 		{
 			if (master_dir)
 			{
-				for (c0 = c_base = (uchar_ptr_t)rec_base + SIZEOF(rec_hdr), cc = MAX_MIDENT_LEN;
-							*c0 && cc;  c0++, cc--);
+				for (c0 = c_base = ((uchar_ptr_t)rec_base + SIZEOF(rec_hdr)), cc = MAX_MIDENT_LEN;
+							*c0 && cc; c0++, cc--);
 				if (!cc && *c0) /*Key value too long*/
 				{
-					mu_int_err(ERR_DBKEYMX, TRUE, TRUE, buff, comp_length, top_key, top_len,
+					MU_INT_RETRY_ERR_RETURN(ERR_DBKEYMX, TRUE, TRUE, buff, comp_length, top_key, top_len,
 							(unsigned int)blk_levl);
-					free(free_blk_base);
-					return FALSE;
 				}
-				GET_LONG(root_pointer, ((block_id *)(c0 + 2)));
+				GET_BLK_ID(root_pointer, c0 + 2);
 				if (root_pointer > mu_int_data.trans_hist.total_blks || root_pointer < 2)
 				{	/* 0=master map, 1=dir root*/
-					mu_int_err(ERR_DBBADPNTR, TRUE, TRUE, buff, comp_length, top_key,
+					MU_INT_RETRY_INCR_PLEN(ERR_DBBADPNTR, TRUE, TRUE, buff, comp_length, top_key,
 							top_len, (unsigned int)blk_levl);
-					mu_int_plen++;	/* continuing, so compensate for mu_int_err decrement */
 				}
 				c1 = temp_buff;
 				if (rec_cmpc)
 				{
 					if (rec_cmpc > MAX_MIDENT_LEN)
 					{
-						mu_int_err(ERR_DBCMPBAD, TRUE, TRUE, buff, comp_length, top_key, top_len,
-										(unsigned int)blk_levl);
-						free(free_blk_base);
-						return FALSE;
+						MU_INT_RETRY_ERR_RETURN(ERR_DBCMPBAD, TRUE, TRUE, buff, comp_length, top_key,
+										top_len, (unsigned int)blk_levl);
 					}
 					for (c2 = muint_temp_buff, cc = 0;  *c2 && cc < rec_cmpc;  cc++)
 						*c1++ = *c2++;
 				}
-				for (cc = (c1 - temp_buff);  ((c_base < c0) && (cc <= MAX_MIDENT_LEN)); cc++)
+				for (cc = (c1 - temp_buff); ((c_base < c0) && (cc <= MAX_MIDENT_LEN)); cc++)
 					*c1++ = *c_base++;
 				if (cc > MAX_MIDENT_LEN)
 				{
-					mu_int_err(ERR_DBKEYMX, TRUE, TRUE, buff, comp_length, top_key, top_len,
+					MU_INT_RETRY_ERR_RETURN(ERR_DBKEYMX, TRUE, TRUE, buff, comp_length, top_key, top_len,
 							(unsigned int)blk_levl);
-					free(free_blk_base);
-					return FALSE;
 				}
 				*c1 = 0;
 				assert(SIZEOF(muint_temp_buff) == SIZEOF(temp_buff));
 				memcpy(muint_temp_buff, temp_buff, SIZEOF(temp_buff));
-				if (muint_key)
+				if (mu_key)
 				{
-					if (muint_end_key)	/* range */
+					if (mu_end_key)	/* range */
 					{
 						len = (int)(c1 - temp_buff + 1);
-						if ((0 < memcmp(muint_start_key->base, temp_buff, len < muint_start_keyend ?
-									len : muint_start_keyend))
-							|| (0 >  memcmp(muint_end_key->base, temp_buff, len < muint_end_keyend ?
-									len : muint_end_keyend)))
+						if ((0 < memcmp(mu_start_key->base, temp_buff, len < mu_start_keyend ?
+									len : mu_start_keyend))
+							|| (0 >  memcmp(mu_end_key->base, temp_buff, len < mu_end_keyend ?
+									len : mu_end_keyend)))
 								continue;
 					} else
 					{
-						if (((muint_start_keyend - 1) != (c1 - temp_buff))
-							|| (memcmp(muint_start_key->base, temp_buff, muint_start_keyend)))
+						if (((mu_start_keyend - 1) != (c1 - temp_buff))
+							|| (memcmp(mu_start_key->base, temp_buff, mu_start_keyend)))
 								continue;
 					}
 				}
@@ -997,14 +1044,13 @@ boolean_t mu_int_blk(
 		{
 			if ((0 != cmcc) || level)
 			{
-				mu_int_err(ERR_DBKEYGTIND, TRUE, TRUE, buff, comp_length, top_key, top_len,
+				MU_INT_RETRY_ERR_RETURN(ERR_DBKEYGTIND, TRUE, TRUE, buff, comp_length, top_key, top_len,
 					(unsigned int)blk_levl);
-				free(free_blk_base);
-				return FALSE;
 			}
 		}
 	}
 	mu_int_plen--;
-	free(free_blk_base);
+	if (NULL != free_blk_base)
+		free(free_blk_base);
 	return TRUE;
 }
