@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2005-2017 Fidelity National Information	*
+ * Copyright (c) 2005-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -28,7 +28,6 @@
 #include "gdskill.h"
 #include "jnl.h"
 #include "buddy_list.h"		/* needed for tp.h */
-#include "hashtab_int4.h"	/* needed for tp.h */
 #include "tp.h"
 #include "cli.h"
 #include "iosp.h"		/* for SS_NORMAL */
@@ -51,8 +50,8 @@
 #include "targ_alloc.h"
 #include "util.h"		/* for util_out_print prototype */
 #include "wcs_flu.h"
-#include "repl_msg.h"			/* for gtmsource.h */
-#include "gtmsource.h"			/* for jnlpool_addrs_ptr_t */
+#include "repl_msg.h"		/* for gtmsource.h */
+#include "gtmsource.h"		/* for jnlpool_addrs_ptr_t */
 
 #define	REORG_CONTINUE	1
 #define	REORG_BREAK	2
@@ -83,7 +82,7 @@ GBLREF	unsigned char		t_fail_hist[CDB_MAX_TRIES];
 GBLREF 	unsigned int		t_tries;
 GBLREF	cw_set_element		cw_set[];		/* create write set. */
 GBLREF	unsigned char		cw_set_depth;
-GBLREF	unsigned char    	cw_map_depth;
+GBLREF	unsigned char		cw_map_depth;
 GBLREF	uint4			update_trans;
 
 error_def(ERR_BUFFLUFAILED);
@@ -101,7 +100,7 @@ error_def(ERR_REORGCTRLY);
 /* actually want the following to be a static variable in this module, but getting the address of a
  * static variable through the debugger might be tricky on some platforms. hence use a global variable instead.
  */
-GBLDEF	trans_num		mu_reorg_upgrd_dwngrd_start_tn;
+GBLDEF	trans_num	mu_reorg_upgrd_dwngrd_start_tn;
 
 typedef struct
 {
@@ -131,7 +130,7 @@ void	mu_reorg_upgrd_dwngrd(void)
 	gd_region		*reg;
 	int			cycle;
 	int4			blk_seg_cnt, blk_size;	/* needed for BLK_INIT,BLK_SEG and BLK_FINI macros */
-	int4			blocks_left, expected_blks2upgrd, actual_blks2upgrd, total_blks, free_blks;
+	block_id		blocks_left, expected_blks2upgrd, actual_blks2upgrd, total_blks, free_blks;
 	int4			status, status1, mapsize, lcnt, bml_status;
 	reorg_stats_t		reorg_stats;
 	sgmnt_addrs		*csa;
@@ -141,7 +140,7 @@ void	mu_reorg_upgrd_dwngrd(void)
 	srch_blk_status		*blkhist, bmlhist;
 	tp_region		*rptr;
 	trans_num		curr_tn;
-	unsigned char    	save_cw_set_depth;
+	unsigned char		save_cw_set_depth;
 	uint4			lcl_update_trans;
 	unix_db_info		*udi;
 	DCL_THREADGBL_ACCESS;
@@ -230,7 +229,7 @@ void	mu_reorg_upgrd_dwngrd(void)
 		}
 		assert(GDSVCURR == GDSV6); /* so we trip this assert in case GDSVCURR changes without a change to this module */
 		new_db_format = (upgrade ? GDSV6 : GDSV4);
-		grab_crit(reg);
+		grab_crit(reg, WS_8);
 		curr_tn = csd->trans_hist.curr_tn;
 		/* set the desired db format in the file header to the appropriate version, increment transaction number */
 		status1 = desired_db_format_set(reg, new_db_format, reorg_command);
@@ -264,10 +263,10 @@ void	mu_reorg_upgrd_dwngrd(void)
 			|| (!upgrade && ((total_blks - free_blks) == actual_blks2upgrd)))
 		{
 			util_out_print("Region !AD : Blocks to Upgrade counter indicates no action needed for MUPIP REORG !AD",
-				       TRUE, REG_LEN_STR(reg), LEN_AND_STR(command));
+					TRUE, REG_LEN_STR(reg), LEN_AND_STR(command));
 			util_out_print("Region !AD : Total Blocks = [0x!XL] : Free Blocks = [0x!XL] : "
-				       "Blocks to upgrade = [0x!XL]",
-				       TRUE, REG_LEN_STR(reg), total_blks, free_blks, actual_blks2upgrd);
+					"Blocks to upgrade = [0x!XL]",
+					TRUE, REG_LEN_STR(reg), total_blks, free_blks, actual_blks2upgrd);
 			util_out_print("Region !AD : MUPIP REORG !AD finished!/", TRUE, REG_LEN_STR(reg), LEN_AND_STR(command));
 			rel_crit(reg);
 			continue;
@@ -333,7 +332,7 @@ void	mu_reorg_upgrd_dwngrd(void)
 			assert(!csa->now_crit);
 			bml_sm_buff = t_qread(curbmp, (sm_int_ptr_t)&cycle, &cr); /* bring block into the cache outside of crit */
 			reorg_stats.blks_read_from_disk_bmp++;
-			grab_crit_encr_cycle_sync(reg); /* needed so t_qread does not return NULL below */
+			grab_crit_encr_cycle_sync(reg, WS_64); /* needed so t_qread does not return NULL below */
 			if (mu_reorg_upgrd_dwngrd_start_tn != csd->desired_db_format_tn)
 			{	/* csd->desired_db_format changed since reorg started. discontinue the reorg */
 				/* see later comment on "csd->reorg_upgrd_dwngrd_restart_block" for why the assignment
@@ -389,11 +388,15 @@ void	mu_reorg_upgrd_dwngrd(void)
 			 * ------------------------------------------------------------------------
 			 */
 			curblk = (curbmp == start_bmp) ? start_blk : curbmp;
-			mapsize = (curbmp == last_bmp) ? (stop_blk - curbmp) : BLKS_PER_LMAP;
+			/* (stop_blk - curbmp) can be cast because it should never be larger then BLKS_PER_LMAP if used */
+			assert((BLKS_PER_LMAP >= (stop_blk - curbmp)) || (curbmp != last_bmp));
+			mapsize = (curbmp == last_bmp) ? (int4)(stop_blk - curbmp) : BLKS_PER_LMAP;
 			assert(0 != mapsize);
-			assert(mapsize <= BLKS_PER_LMAP);
+			assert(BLKS_PER_LMAP >= mapsize);
 			db_got_to_v5_once = csd->db_got_to_v5_once;
-			for (lcnt = curblk - curbmp; lcnt < mapsize; lcnt++, curblk++)
+			/* (curblk - curbmp) can be cast because it should never be larger then BLKS_PER_LMAP */
+			assert(BLKS_PER_LMAP > (curblk - curbmp));
+			for (lcnt = (int4)(curblk - curbmp); lcnt < mapsize; lcnt++, curblk++)
 			{
 				if (mu_ctrly_occurred || mu_ctrlc_occurred)
 				{
@@ -556,7 +559,7 @@ void	mu_reorg_upgrd_dwngrd(void)
 									FALSE, GDS_WRITE_PLAIN);
 								/* The directory tree status for now is only used to determine
 								 * whether writing the block to snapshot file (see t_end_sysops.c).
- 								 * For reorg upgrade/downgrade process, the block is updated in a
+								 * For reorg upgrade/downgrade process, the block is updated in a
 								 * sequential way without changing the gv_target. In this case, we
 								 * assume the block is in directory tree so as to have it written to
 								 * the snapshot file.
@@ -642,7 +645,7 @@ void	mu_reorg_upgrd_dwngrd(void)
 		}
 	stop_reorg_on_this_reg:
 		/* even though ctrl-c occurred, update file-header fields to store reorg's progress before exiting */
-		grab_crit(reg);
+		grab_crit(reg, WS_9);
 		blocks_left = 0;
 		assert(csd->trans_hist.total_blks >= csd->blks_to_upgrd);
 		actual_blks2upgrd = csd->blks_to_upgrd;
@@ -721,8 +724,8 @@ void	mu_reorg_upgrd_dwngrd(void)
 			status1 = ERR_MUNOFINISH;
 		} else
 			util_out_print("Region !AD : Total Blocks = [0x!XL] : Free Blocks = [0x!XL] : "
-				       "Blocks to upgrade = [0x!XL]",
-				       TRUE, REG_LEN_STR(reg), total_blks, free_blks, actual_blks2upgrd);
+					"Blocks to upgrade = [0x!XL]",
+					TRUE, REG_LEN_STR(reg), total_blks, free_blks, actual_blks2upgrd);
 		/* Issue success or failure message for this region */
 		if (SS_NORMAL == status1)
 		{	/* issue success only if REORG did not encounter any error in its processing */
