@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2013 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -49,8 +50,8 @@ mu_swap_blk.c:
 #include "copy.h"
 #include "muextr.h"
 #include "mu_reorg.h"
-#include "hashtab_int4.h"
 #include "cws_insert.h"
+#include "gvt_inline.h"
 
 /* Include prototypes */
 #include "t_qread.h"
@@ -109,7 +110,7 @@ enum cdb_sc mu_swap_blk(int level, block_id *pdest_blk_id, kill_set *kill_set_pt
 				bn_ptr, bmp_buff, tblk_ptr, rec_base, key_base;
 	boolean_t		gbl_target_was_set, blk_was_free, deleted;
 	gv_namehead		*save_targ;
-	srch_blk_status		bmlhist, destblkhist, *hist_ptr;
+	srch_blk_status		bmlhist, destblkhist, *hist_ptr, *work_blk_stat;
 	unsigned char    	save_cw_set_depth;
 	cw_set_element		*tmpcse;
 	jnl_buffer_ptr_t	jbbp; /* jbbp is non-NULL only if before-image journaling */
@@ -126,12 +127,14 @@ enum cdb_sc mu_swap_blk(int level, block_id *pdest_blk_id, kill_set *kill_set_pt
 	blk_size = cs_data->blk_size;
 	work_parent_ptr = gv_target->hist.h[level+1].buffaddr;
 	work_parent_size = ((blk_hdr_ptr_t)work_parent_ptr)->bsiz;
-	work_blk_ptr = gv_target->hist.h[level].buffaddr;
+	work_blk_stat = &gv_target->hist.h[level];
+	work_blk_ptr = work_blk_stat->buffaddr;
 	work_blk_size = ((blk_hdr_ptr_t)work_blk_ptr)->bsiz;
-	work_blk_id = gv_target->hist.h[level].blk_num;
+	work_blk_id = work_blk_stat->blk_num;
 	if (blk_size < work_blk_size)
 	{
 		assert(t_tries < CDB_STAGNATE);
+		NONTP_TRACE_HIST_MOD(work_blk_stat, t_blkmod_mu_swap_blk);
 		return cdb_sc_blkmod;
 	}
 	cws_reorg_remove_index = 0;
@@ -221,7 +224,7 @@ enum cdb_sc mu_swap_blk(int level, block_id *pdest_blk_id, kill_set *kill_set_pt
 		tblk_ptr = dest_blk_ptr;
 		while ((BSTAR_REC_SIZE == rec_size1) && (0 != nslevel))
 		{
-			GET_LONG(child1, (rec_base + SIZEOF(rec_hdr)));
+			GET_BLK_ID(child1, rec_base + SIZEOF(rec_hdr));
 			if (0 == child1 || child1 > cs_data->trans_hist.total_blks - 1)
 			{
 				assert(t_tries < CDB_STAGNATE);
@@ -253,7 +256,10 @@ enum cdb_sc mu_swap_blk(int level, block_id *pdest_blk_id, kill_set *kill_set_pt
 			if (CDB_STAGNATE <= t_tries)
 				continue;
 			else
+			{
+				NONTP_TRACE_HIST_MOD(work_blk_stat, t_blkmod_mu_swap_blk);
 				return cdb_sc_blkmod;
+			}
 		}
 		memcpy(&((TREF(gv_reorgkey))->base[0]), rec_base + SIZEOF(rec_hdr), key_len_dir);
 		(TREF(gv_reorgkey))->base[key_len_dir] = 0;
@@ -302,11 +308,12 @@ enum cdb_sc mu_swap_blk(int level, block_id *pdest_blk_id, kill_set *kill_set_pt
 		key_base = dir_hist_ptr->h[0].buffaddr + dir_hist_ptr->h[0].curr_rec.offset + SIZEOF(rec_hdr);
 		key_len_dir = get_key_len(dir_hist_ptr->h[0].buffaddr, key_base);
 		/* Get root of GVT for dest_blk_id */
-		GET_LONG(gv_target->root, key_base + key_len_dir);
+		GET_BLK_ID(gv_target->root, key_base + key_len_dir);
 		if ((0 == gv_target->root) || (gv_target->root > (cs_data->trans_hist.total_blks - 1)))
 		{
 			assert(t_tries < CDB_STAGNATE);
 			RESET_GV_TARGET_LCL_AND_CLR_GBL(save_targ, DO_GVT_GVKEY_CHECK);
+			NONTP_TRACE_HIST_MOD(work_blk_stat, t_blkmod_mu_swap_blk);
 			return cdb_sc_blkmod;
 		}
 		/* Assign Global Variable Tree path to find dest_blk_id in dest_hist_ptr */
@@ -319,7 +326,13 @@ enum cdb_sc mu_swap_blk(int level, block_id *pdest_blk_id, kill_set *kill_set_pt
 		if ((cdb_sc_normal != status) || (dest_hist_ptr->h[nslevel].curr_rec.match != ((TREF(gv_reorgkey))->end + 1)))
 		{
 			assert(t_tries < CDB_STAGNATE);
-			return (cdb_sc_normal != status ? status : cdb_sc_blkmod);
+			if (cdb_sc_normal != status)
+				return status;
+			else
+			{
+				NONTP_TRACE_HIST_MOD(work_blk_stat, t_blkmod_mu_swap_blk);
+				return cdb_sc_blkmod;
+			}
 		}
 		for (wlevel = nslevel; wlevel <= dest_blk_level; wlevel++)
 			dest_hist_ptr->h[wlevel].tn = ctn;
@@ -340,6 +353,7 @@ enum cdb_sc mu_swap_blk(int level, block_id *pdest_blk_id, kill_set *kill_set_pt
 		if (!BLK_FINI (bs_ptr,bs1))
 		{
 			assert(t_tries < CDB_STAGNATE);
+			NONTP_TRACE_HIST_MOD(work_blk_stat, t_blkmod_mu_swap_blk);
 			return cdb_sc_blkmod;
 		}
 		assert(gv_target->hist.h[level].blk_num == work_blk_id);
@@ -355,6 +369,7 @@ enum cdb_sc mu_swap_blk(int level, block_id *pdest_blk_id, kill_set *kill_set_pt
 		if (dest_blk_level + 1 != level || dest_parent_size != work_blk_size)
 		{
 			assert(t_tries < CDB_STAGNATE);
+			NONTP_TRACE_HIST_MOD(work_blk_stat, t_blkmod_mu_swap_blk);
 			return cdb_sc_blkmod;
 		}
 		BLK_INIT(bs_ptr, bs1);
@@ -365,17 +380,19 @@ enum cdb_sc mu_swap_blk(int level, block_id *pdest_blk_id, kill_set *kill_set_pt
 		if (work_blk_size < first_offset + rec_size1)
 		{
 			assert(t_tries < CDB_STAGNATE);
+			NONTP_TRACE_HIST_MOD(work_blk_stat, t_blkmod_mu_swap_blk);
 			return cdb_sc_blkmod;
 		}
 		piece_len1 =  first_offset + rec_size1;
 		BLK_SEG(bs_ptr, saved_blk + SIZEOF(blk_hdr), piece_len1 - SIZEOF(block_id) - SIZEOF(blk_hdr));
 		BLK_ADDR(bn_ptr, SIZEOF(block_id), unsigned char);
-		PUT_LONG(bn_ptr, work_blk_id); /* since work_blk_id will now be the child of dest_blk_id */
+		PUT_BLK_ID(bn_ptr, work_blk_id); /* since work_blk_id will now be the child of dest_blk_id */
 		BLK_SEG(bs_ptr, bn_ptr, SIZEOF(block_id));
 		BLK_SEG(bs_ptr, saved_blk + piece_len1, dest_parent_size - piece_len1);
 		if (!BLK_FINI(bs_ptr, bs1))
 		{
 			assert(t_tries < CDB_STAGNATE);
+			NONTP_TRACE_HIST_MOD(work_blk_stat, t_blkmod_mu_swap_blk);
 			return cdb_sc_blkmod;
 		}
 		assert(dest_blk_id == dest_hist_ptr->h[dest_blk_level].blk_num);
@@ -390,6 +407,7 @@ enum cdb_sc mu_swap_blk(int level, block_id *pdest_blk_id, kill_set *kill_set_pt
 		if (!BLK_FINI(bs_ptr, bs1))
 		{
 			assert(t_tries < CDB_STAGNATE);
+			NONTP_TRACE_HIST_MOD(work_blk_stat, t_blkmod_mu_swap_blk);
 			return cdb_sc_blkmod;
 		}
 		if (blk_was_free)
@@ -458,6 +476,7 @@ enum cdb_sc mu_swap_blk(int level, block_id *pdest_blk_id, kill_set *kill_set_pt
 				gv_target->hist.h[level+1].curr_rec.offset == dest_hist_ptr->h[level+1].curr_rec.offset)
 			{
 				assert(t_tries < CDB_STAGNATE);
+				NONTP_TRACE_HIST_MOD(work_blk_stat, t_blkmod_mu_swap_blk);
 				return cdb_sc_blkmod;
 			}
 			if (gv_target->hist.h[level+1].curr_rec.offset < dest_hist_ptr->h[level+1].curr_rec.offset)
@@ -476,26 +495,28 @@ enum cdb_sc mu_swap_blk(int level, block_id *pdest_blk_id, kill_set *kill_set_pt
 				BSTAR_REC_SIZE >= rec_size1 || BSTAR_REC_SIZE > rec_size2)
 			{
 				assert(t_tries < CDB_STAGNATE);
+				NONTP_TRACE_HIST_MOD(work_blk_stat, t_blkmod_mu_swap_blk);
 				return cdb_sc_blkmod;
 			}
 			piece_len1 =  first_offset + rec_size1 - SIZEOF(block_id);
 			piece_len2 =  second_offset + rec_size2 - SIZEOF(block_id);
-			GET_LONG(child1, dest_parent_ptr + piece_len1);
-			GET_LONG(child2, dest_parent_ptr + piece_len2);
+			GET_BLK_ID(child1, dest_parent_ptr + piece_len1);
+			GET_BLK_ID(child2, dest_parent_ptr + piece_len2);
 			BLK_SEG(bs_ptr, dest_parent_ptr + SIZEOF(blk_hdr), piece_len1 - SIZEOF(blk_hdr));
 			BLK_ADDR(bn_ptr, SIZEOF(block_id), unsigned char);
-			PUT_LONG(bn_ptr, child2);
+			PUT_BLK_ID(bn_ptr, child2);
 			BLK_SEG(bs_ptr, bn_ptr, SIZEOF(block_id));
 			BLK_SEG(bs_ptr, dest_parent_ptr + first_offset + rec_size1,
 				second_offset + rec_size2 - SIZEOF(block_id) - first_offset - rec_size1);
 			BLK_ADDR(bn_ptr, SIZEOF(block_id), unsigned char);
-			PUT_LONG(bn_ptr, child1);
+			PUT_BLK_ID(bn_ptr, child1);
 			BLK_SEG(bs_ptr, bn_ptr, SIZEOF(block_id));
 			BLK_SEG(bs_ptr, dest_parent_ptr + second_offset + rec_size2,
 				dest_parent_size - second_offset - rec_size2);
 			if (!BLK_FINI(bs_ptr,bs1))
 			{
 				assert(t_tries < CDB_STAGNATE);
+				NONTP_TRACE_HIST_MOD(work_blk_stat, t_blkmod_mu_swap_blk);
 				return cdb_sc_blkmod;
 			}
 			assert(level == dest_blk_level);
@@ -511,18 +532,20 @@ enum cdb_sc mu_swap_blk(int level, block_id *pdest_blk_id, kill_set *kill_set_pt
 				BSTAR_REC_SIZE > rec_size1)
 			{
 				assert(t_tries < CDB_STAGNATE);
+				NONTP_TRACE_HIST_MOD(work_blk_stat, t_blkmod_mu_swap_blk);
 				return cdb_sc_blkmod;
 			}
 			BLK_SEG (bs_ptr, dest_parent_ptr + SIZEOF(blk_hdr),
 			    dest_hist_ptr->h[dest_blk_level+1].curr_rec.offset + rec_size1 - SIZEOF(blk_hdr) - SIZEOF(block_id));
 			BLK_ADDR(bn_ptr, SIZEOF(block_id), unsigned char);
-			PUT_LONG(bn_ptr, work_blk_id);
+			PUT_BLK_ID(bn_ptr, work_blk_id);
 			BLK_SEG(bs_ptr, bn_ptr, SIZEOF(block_id));
 			BLK_SEG(bs_ptr, dest_parent_ptr + dest_hist_ptr->h[dest_blk_level+1].curr_rec.offset + rec_size1,
 				dest_parent_size - dest_hist_ptr->h[dest_blk_level+1].curr_rec.offset  - rec_size1);
 			if (!BLK_FINI(bs_ptr,bs1))
 			{
 				assert(t_tries < CDB_STAGNATE);
+				NONTP_TRACE_HIST_MOD(work_blk_stat, t_blkmod_mu_swap_blk);
 				return cdb_sc_blkmod;
 			}
 			assert(dest_parent_ptr == dest_hist_ptr->h[dest_blk_level+1].buffaddr);
@@ -537,19 +560,21 @@ enum cdb_sc mu_swap_blk(int level, block_id *pdest_blk_id, kill_set *kill_set_pt
 		if (work_parent_size < rec_size1 +  gv_target->hist.h[level+1].curr_rec.offset || BSTAR_REC_SIZE > rec_size1)
 		{
 			assert(t_tries < CDB_STAGNATE);
+			NONTP_TRACE_HIST_MOD(work_blk_stat, t_blkmod_mu_swap_blk);
 			return cdb_sc_blkmod;
 		}
 		BLK_INIT(bs_ptr, bs1);
 		BLK_SEG(bs_ptr, work_parent_ptr + SIZEOF(blk_hdr),
 			gv_target->hist.h[level+1].curr_rec.offset + rec_size1 - SIZEOF(blk_hdr) - SIZEOF(block_id));
 		BLK_ADDR(bn_ptr, SIZEOF(block_id), unsigned char);
-		PUT_LONG(bn_ptr, dest_blk_id);
+		PUT_BLK_ID(bn_ptr, dest_blk_id);
 		BLK_SEG(bs_ptr, bn_ptr, SIZEOF(block_id));
 		BLK_SEG(bs_ptr, work_parent_ptr + gv_target->hist.h[level+1].curr_rec.offset + rec_size1,
 			work_parent_size - gv_target->hist.h[level+1].curr_rec.offset - rec_size1);
 		if (!BLK_FINI(bs_ptr, bs1))
 		{
 			assert(t_tries < CDB_STAGNATE);
+			NONTP_TRACE_HIST_MOD(work_blk_stat, t_blkmod_mu_swap_blk);
 			return cdb_sc_blkmod;
 		}
 		assert(gv_target->hist.h[level+1].buffaddr == work_parent_ptr);
@@ -568,7 +593,7 @@ enum cdb_sc mu_swap_blk(int level, block_id *pdest_blk_id, kill_set *kill_set_pt
 		bmlhist.blk_num = child1;
 		child1 = dest_blk_id - child1;
 		assert(child1);
-		PUT_LONG(update_array_ptr, child1);
+		PUT_BLK_ID(update_array_ptr, child1);
 		/* Need to put bit maps on the end of the cw set for concurrency checking.
 		 * We want to simulate t_write_map, except we want to update "cw_map_depth" instead of "cw_set_depth".
 		 * Hence the save and restore logic (for "cw_set_depth") below.
@@ -581,7 +606,7 @@ enum cdb_sc mu_swap_blk(int level, block_id *pdest_blk_id, kill_set *kill_set_pt
 		/* t_write_map simulation end */
 		update_array_ptr += SIZEOF(block_id);
 		child1 = 0;
-		PUT_LONG(update_array_ptr, child1);
+		PUT_BLK_ID(update_array_ptr, child1);
 		update_array_ptr += SIZEOF(block_id);
 		assert(1 == cw_set[cw_map_depth - 1].reference_cnt);	/* 1 free block is now becoming BLK_USED in the bitmap */
 		/* working block will be removed */
@@ -594,7 +619,7 @@ enum cdb_sc mu_swap_blk(int level, block_id *pdest_blk_id, kill_set *kill_set_pt
 }
 
 /***************************************************************
-Checks if a key is present in exclude global lists.
+Checks if an input global name is present in exclude global lists.
 	curr_key_ptr = Key pointer
 	key_len = curr_key_ptr length excludeing nulls
 	exclude_glist_ptr = list of globals in -EXCLUDE option
@@ -608,7 +633,8 @@ boolean_t in_exclude_list(unsigned char *curr_key_ptr, int key_len, glist *exclu
 
 	for (gl_ptr = exclude_glist_ptr->next; gl_ptr; gl_ptr = gl_ptr->next)
 	{
-		if (gl_ptr->name.str.len == key_len && 0 == memcmp(gl_ptr->name.str.addr, curr_key_ptr, gl_ptr->name.str.len))
+		if ((GNAME(gl_ptr).len == key_len)
+				&& (0 == memcmp(GNAME(gl_ptr).addr, curr_key_ptr, key_len)))
 			return TRUE;
 	}
 	return FALSE;
