@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001 Sanchez Computer Associates, Inc.	*
+ * Copyright (c) 2001-2019 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -12,6 +13,8 @@
 #ifndef MUCREGINI_INCLUDED
 #define MUCREGINI_INCLUDED
 
-void mucregini(int4 blk_init_size);
+#include "gdsroot.h"
+
+void mucregini(block_id blk_init_size);
 
 #endif /* MUCREGINI_INCLUDED */
