@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -44,7 +44,6 @@
 #include "gdskill.h"
 #include "jnl.h"
 #include "buddy_list.h"		/* needed for tp.h */
-#include "hashtab_int4.h"	/* needed for tp.h */
 #include "tp.h"
 #include "io.h"
 #include "interlock.h"
@@ -627,6 +626,26 @@ void mupip_backup(void)
 				mubclnup(rptr, need_to_del_tempfile);
 				mupip_exit(ERR_FILENAMETOOLONG);
 			}
+			/* GTM-9182 : If there's just enough space for the temp filepath, BUT NOT for the backup filepath, backup
+			 *  succeeds even though the full pathname of backup file exceeds MAX_FN_LEN. This should NOT be allowed.
+			 * This check is not required for backup files having full pathnames
+			 */
+			for (index=0; index < file->len; index++)
+			{
+				if ('/' == file->addr[index])
+					break;
+			}
+			if ((index >= file->len) && (MAX_FN_LEN < (tempdir_full.len + file->len)))
+			{
+				tempdir_trans.addr[tempdir_trans.len] = '/';
+				tempdir_trans.len++;
+				memcpy(&tempdir_trans.addr[tempdir_trans.len], file->addr, file->len);
+				tempdir_trans.len += file->len;
+				gtm_putmsg_csa(CSA_ARG(cs_addrs) VARLSTCNT(4) ERR_FILEPARSE, 2, tempdir_trans.len,
+						tempdir_trans.addr);
+				mubclnup(rptr, need_to_del_tempfile);
+				mupip_exit(ERR_FILENAMETOOLONG);
+			}
 			SNPRINTF(tempfilename + tempdir_full.len, MAX_FN_LEN - tempdir_full.len, "/%s_XXXXXX", tempnam_prefix);
 			MKSTEMP(tempfilename, rptr->backup_fd);
 			if (FD_INVALID == rptr->backup_fd)
@@ -901,7 +920,7 @@ repl_inst_bkup_done1:
 			assert(!cs_addrs->hold_onto_crit); /* DSE/Rollback can never reach here */
 			if (online)
 			{
-				grab_crit(gv_cur_region);
+				grab_crit(gv_cur_region, WS_10);
 				DEBUG_ONLY(reg_count++);
 				INCR_INHIBIT_KILLS(cs_addrs->nl);
 				if (cs_data->kill_in_prog)
@@ -911,7 +930,7 @@ repl_inst_bkup_done1:
 				 * same timestamp in journal records across ALL regions (in wcs_flu below). We will
 				 * release crit as soon as the wcs_flu across all regions is done.
 				 */
-				grab_crit(gv_cur_region);
+				grab_crit(gv_cur_region, WS_65);
 				DEBUG_ONLY(reg_count++);
 			}
 			if (FROZEN_CHILLED(cs_addrs))
@@ -991,7 +1010,7 @@ repl_inst_bkup_done1:
 			if (rptr->not_this_time > keep_going)
 				continue;
 			TP_CHANGE_REG(rptr->reg);
-			grab_crit(gv_cur_region);
+			grab_crit(gv_cur_region, WS_66);
 			/* Now that we have crit, check if this region is actively journaled and if gbl_jrec_time needs to be
 			 * adjusted (to ensure time ordering of journal records within this region's journal file).
 			 * This needs to be done BEFORE writing any journal records for this region. The value of
