@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -28,7 +28,6 @@
 #include "filestruct.h"
 #include "jnl.h"
 #include "buddy_list.h"		/* needed for tp.h */
-#include "hashtab_int4.h"	/* needed for tp.h */
 #include "tp.h"
 #include "mu_int_maps.h"
 #include "util.h"
@@ -50,9 +49,10 @@
 #include "mu_gv_cur_reg_init.h"
 #include "warn_db_sz.h"
 #include "gvcst_protos.h"
+#include "muextr.h"
+#include "mu_getkey.h"
+#include "gvt_inline.h"
 
-#define DUMMY_GLOBAL_VARIABLE		"%D%DUMMY_VARIABLE"
-#define DUMMY_GLOBAL_VARIABLE_LEN	SIZEOF(DUMMY_GLOBAL_VARIABLE)
 #define MAX_UTIL_LEN			96
 #define APPROX_ALL_ERRORS		1000000
 #define DEFAULT_ERR_LIMIT		10
@@ -103,8 +103,6 @@ GBLDEF int			trans_errors = 0;
 GBLDEF boolean_t		block = FALSE;
 GBLDEF boolean_t		muint_fast = FALSE;
 GBLDEF boolean_t		master_dir;
-GBLDEF boolean_t		muint_key = MUINTKEY_FALSE;
-GBLDEF boolean_t		muint_subsc = FALSE;
 GBLDEF boolean_t		mu_int_err_ranges;
 GBLDEF boolean_t		tn_reset_specified;	/* use this to avoid recomputing cli_present("TN_RESET") in the loop */
 GBLDEF boolean_t		tn_reset_this_reg;
@@ -145,8 +143,9 @@ GBLREF sgmnt_addrs		*cs_addrs;
 GBLREF tp_region		*grlist;
 GBLREF bool			region;
 GBLREF boolean_t		debug_mupip;
-GBLREF gv_key			*muint_end_key;
-GBLREF gv_key			*muint_start_key;
+GBLREF gv_key			*mu_end_key;
+GBLREF boolean_t		mu_key;
+GBLREF gv_key			*mu_start_key;
 
 error_def(ERR_CTRLC);
 error_def(ERR_CTRLY);
@@ -185,7 +184,7 @@ void mupip_integ(void)
 	enum cum_type		c_type;
 	enum tot_type		t_type;
 	file_control		*fc;
-	gtm_uint64_t		blocks_free = (gtm_uint64_t)MAXUINT8;
+	gtm_uint64_t		blocks_free = MAXUINT8;
 	gtm_uint64_t		mu_int_tot[TOT_TYPE_MAX][CUM_TYPE_MAX], tot_blks, tot_recs;
 	tp_region		*rptr;
 	sgmnt_addrs		*csa;
@@ -197,7 +196,7 @@ void mupip_integ(void)
 	gd_region		*baseDBreg, *reg;
 	sgmnt_addrs		*baseDBcsa;
 	node_local_ptr_t	baseDBnl;
-	sgmnt_addrs 		*tcsa;
+	sgmnt_addrs		*tcsa;
 	char 			*db_file_name;
 	DCL_THREADGBL_ACCESS;
 
@@ -303,15 +302,15 @@ void mupip_integ(void)
 	online_integ = ((TRUE != cli_negated("ONLINE")) && region); /* Default option for INTEG is -ONLINE */
 	preserve_snapshot = (CLI_PRESENT == cli_present("PRESERVE")); /* Should snapshot file be preserved ? */
 	assert(!online_integ || (region && !tn_reset_specified));
-	assert(MUINTKEY_FALSE == muint_key);
+	assert(MUKEY_FALSE == mu_key);
 	if (CLI_PRESENT == cli_present("SUBSCRIPT"))
 	{
 		keylen = SIZEOF(key_buff);
 		if (0 == cli_get_str("SUBSCRIPT", (char *)key_buff, &keylen))
 			mupip_exit(ERR_MUPCLIERR);
-		if (FALSE == mu_int_getkey(key_buff, keylen))
+		if (FALSE == mu_getkey(key_buff, keylen))
 			mupip_exit(ERR_MUPCLIERR);
-		assert(muint_key);	/* or else "mu_int_getkey" call above would have returned FALSE */
+		assert(mu_key);	/* or else "mu_getkey" call above would have returned FALSE */
 		disp_map_errors = 0;
 	}
 	if (CLI_PRESENT == cli_present("BLOCK"))
@@ -322,7 +321,7 @@ void mupip_integ(void)
 		disp_map_errors = 0;
 		master_dir = FALSE;
 	}
-	muint_all_index_blocks = !(block || muint_key);
+	muint_all_index_blocks = !(block || mu_key);
 	mu_int_master = malloc(MASTER_MAP_SIZE_MAX);
 	tn_reset_specified = (CLI_PRESENT == cli_present("TN_RESET"));
 	mu_outofband_setup();
@@ -481,13 +480,14 @@ void mupip_integ(void)
 			master_dir = FALSE;
 			trees->root = muint_block;
 		}
-		if ((MUINTKEY_NULLSUBS == muint_key) && gv_cur_region->std_null_coll)
+		if ((MUKEY_NULLSUBS == mu_key) && gv_cur_region->std_null_coll)
 		{	/* -SUBSCRIPT was specified AND at least one null-subscript was specified in it.
-			 * muint_start_key and muint_end_key have been constructed assuming gv_cur_region->std_null_coll is FALSE.
-			 * Update muint_start_key and muint_end_key to reflect the current gv_cur_region->std_null_coll value.
+			 * mu_start_key and mu_end_key have been constructed assuming gv_cur_region->std_null_coll is FALSE.
+			 * Update mu_start_key and mu_end_key to reflect the current gv_cur_region->std_null_coll value.
 			 */
-			GTM2STDNULLCOLL(muint_start_key->base, muint_start_key->end);
-			GTM2STDNULLCOLL(muint_end_key->base, muint_end_key->end);
+			GTM2STDNULLCOLL(mu_start_key->base, mu_start_key->end);
+			if (mu_end_key)
+				GTM2STDNULLCOLL(mu_end_key->base, mu_end_key->end);
 		}
 		for (trees->link = 0;  ;  master_dir = FALSE, temp = (char*)trees,  trees = trees->link,  free(temp))
 		{
@@ -545,7 +545,7 @@ void mupip_integ(void)
 			gv_altkey->base[gv_altkey->end] = '\0';
 			if (gv_target->act)
 				act_in_gvt(gv_target);
-			if (mu_int_blk(trees->root, MAX_BT_DEPTH, TRUE, gv_altkey->base, gv_altkey->end, &dummy, 0, 0))
+			if (mu_int_blk(trees->root, MAX_BT_DEPTH, TRUE, gv_altkey->base, gv_altkey->end, &dummy, 0, 0, 0, FALSE))
 			{
 				/* We are done with the INTEG CHECK for the current GVT, but if the spanning node INTEG
 				 * check is not finished, either of the following two are occurred.
@@ -634,10 +634,10 @@ void mupip_integ(void)
 				mu_int_errknt--; /* if this error is not supposed to increment the error count */
 			}
 		}
-		if ((MUINTKEY_NULLSUBS == muint_key) && gv_cur_region->std_null_coll)
-		{	/* muint_start_key and muint_end_key have been modified for this region. Undo that change. */
-			STD2GTMNULLCOLL(muint_start_key->base, muint_start_key->end);
-			STD2GTMNULLCOLL(muint_end_key->base, muint_end_key->end);
+		if ((MUKEY_NULLSUBS == mu_key) && gv_cur_region->std_null_coll)
+		{	/* mu_start_key and mu_end_key have been modified for this region. Undo that change. */
+			STD2GTMNULLCOLL(mu_start_key->base, mu_start_key->end);
+			STD2GTMNULLCOLL(mu_end_key->base, mu_end_key->end);
 		}
 		if (muint_all_index_blocks)
 		{
@@ -714,7 +714,7 @@ void mupip_integ(void)
 			util_out_print("Data  !15@UQ !15@UQ    !8UL.!3ZL  !12UL", TRUE, &mu_int_tot[DATA][BLKS],
 				&mu_int_tot[DATA][RECS], leftpt, rightpt, mu_data_adj);
 		}
-		if ((FALSE == block) && (MUINTKEY_FALSE == muint_key))
+		if ((FALSE == block) && (MUKEY_FALSE == mu_key))
 		{
 			tot_blks = mu_int_data.trans_hist.total_blks
 				- ((mu_int_data.trans_hist.total_blks + mu_int_data.bplmap - 1) / mu_int_data.bplmap);
@@ -778,7 +778,7 @@ void mupip_integ(void)
 			 */
 			if (!gv_cur_region->read_only && !ointeg_this_reg)
 			{
-				if ((gtm_uint64_t)MAXUINT8 != blocks_free)
+				if (MAXUINT8 != blocks_free)
 					csd->trans_hist.free_blocks = blocks_free;
 				if (!mu_int_errknt && muint_all_index_blocks && !muint_fast)
 				{
@@ -903,7 +903,7 @@ void mupip_integ(void)
 		&& (0 != MAXTOTALBLKS(tcsa->hdr)))
 	{
 		if ((NULL != gv_cur_region->dyn.addr) && (NULL != gv_cur_region->dyn.addr->fname))
-                        db_file_name = (char *)gv_cur_region->dyn.addr->fname;
+			db_file_name = (char *)gv_cur_region->dyn.addr->fname;
 		else
 			db_file_name = "";
 		warn_db_sz(db_file_name, 0, tcsa->ti->total_blks, MAXTOTALBLKS(tcsa->hdr));
