@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2019 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -43,6 +43,14 @@ error_def(ERR_SYSCALL);
 	murgbl.extr_buff[extract_len++] = '\\';				\
 }
 
+#define EXTQWX(I)							\
+{									\
+	ptr = &murgbl.extr_buff[extract_len];				\
+	ptr = (char *)i2asclx((uchar_ptr_t)ptr, I);			\
+	extract_len += (int)(ptr - &murgbl.extr_buff[extract_len]);	\
+	murgbl.extr_buff[extract_len++] = '\\';				\
+}
+
 #define	EXT_STRM_SEQNO(SEQ)				\
 {							\
 	seq_num	input_seq, lcl_strm_seqno;		\
@@ -267,8 +275,21 @@ MBSTART {												\
 		/* One might theoretically get an interrupt which unlinks and clears the file between	\
 		 * the if() and DEFER_INTERRUPTS(), so just in case, ignore errors if the filename has	\
 		 * been cleared.									\
+		 *											\
+		 * Also, if the file does not exist (erno == ENOENT), don't take any action		\
+		 * as consensus is we can treat that as a success, someone else having done our		\
+		 * work for us.	(GTM-9193)								\
+		 *											\
+		 * We set up a whitebox test which assigns rc & errno to test this condition.		\
+		 * We check that we aren't substituting our failure for a real failure			\
+		 * (Note the comma operator for assigning both errno & rc)				\
 		 */											\
-		if (-1 == rc)										\
+		if ( (-1 != rc) )									\
+		{											\
+			GTM_WHITE_BOX_TEST(WBTEST_JNLEXT_PREREMOVED, rc, (errno = ENOENT,-1))		\
+		}											\
+													\
+		if ((-1 == rc) && (ENOENT != errno))							\
 		{											\
 			assert(FALSE);									\
 			save_errno = errno;								\
@@ -636,6 +657,9 @@ typedef struct reg_ctl_list_struct
 	int			extr_fn_len_orig[TOT_EXTR_TYPES];	/* fn_len before the region-name suffix was added */
 	boolean_t		last_jext_logical_rec[TOT_EXTR_TYPES];/* Whether corresponding last_jext_rec[] is logical record */
 	dio_buff_t		dio_buff;	/* Used for O_DIRECT IO in case this region has asyncio=TRUE */
+	int			region_index;	/* Per-region index value, aligned to the order the region appears in grlist.
+						 * Used by mupip_freeze()
+						 */
 } reg_ctl_list;
 
 typedef struct redirect_list_struct
@@ -685,6 +709,7 @@ typedef struct
 				lookback_opers;
 	boolean_t		forward,
 				update,
+				corruptdb,
 				rollback,
 				rollback_losttnonly,
 				verify,
@@ -709,6 +734,7 @@ typedef struct
 	redirect_list		*redirect;
 	select_list		*user,
 				*database,
+				*patterns,
 				*global,
 				*process;
 	long_list		*id;
@@ -1133,7 +1159,7 @@ typedef struct {
 		if (multi_proc_in_use)												\
 		{														\
 			mp_hdr = multi_proc_shm_hdr;	/* Note: "mp_hdr" is usable only if "multi_proc_in_use" is TRUE */	\
-			if (!grab_latch(&mp_hdr->multi_proc_latch, MULTI_PROC_LATCH_TIMEOUT_SEC))				\
+			if (!grab_latch(&mp_hdr->multi_proc_latch, MULTI_PROC_LATCH_TIMEOUT_SEC, NOT_APPLICABLE, NULL))		\
 			{													\
 				assert(FALSE);											\
 				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4)							\
@@ -1299,6 +1325,7 @@ void			mur_rem_jctls(reg_ctl_list *rctl);
 boolean_t		mur_report_error(jnl_ctl_list *jctl, enum mur_error code);
 multi_struct 		*mur_token_lookup(token_num token, off_jnl_t rec_time, enum rec_fence_type fence);
 int			gtmrecv_fetchresync(int port, seq_num *resync_seqno, seq_num max_reg_seqno);
+void			gtmrecv_fetchresync_connect(int port);
 void 			mur_tp_resolve_time(jnl_tm_t max_lvrec_time);
 void			mur_show_header(jnl_ctl_list *jctl);
 boolean_t		mur_select_rec(jnl_ctl_list *jctl);
