@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2003-2017 Fidelity National Information	*
+ * Copyright (c) 2003-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -94,7 +94,7 @@ GBLREF	sgmnt_data		*cs_data;
 GBLREF	uint4			process_id;
 GBLREF	void			(*call_on_signal)();
 #ifdef DEBUG
-GBLREF	boolean_t		exiting_on_dev_out_error;
+GBLREF	boolean_t		prin_out_dev_failure;
 #endif
 
 error_def(ERR_FILERENAME);
@@ -436,7 +436,7 @@ boolean_t mur_close_files(void)
 				 */
 				was_crit = csa->now_crit;
 				if (!was_crit)
-					grab_crit(reg);
+					grab_crit(reg, WS_72);
 				assert(JNL_ENABLED(csd));
 				jnl_status = jnl_ensure_open(reg, csa);
 				assert(0 == jnl_status);
