@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2003-2018 Fidelity National Information	*
+ * Copyright (c) 2003-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -262,7 +262,6 @@ void release_all_locks(unix_db_info *udi, gtmsource_local_ptr_t gtmsourcelocal_p
 		assert(FALSE);
 }
 
-
 uint4 mur_open_files(boolean_t retry)
 {
 	boolean_t			interrupted_rollback;
@@ -314,6 +313,11 @@ uint4 mur_open_files(boolean_t retry)
 		0 == memcmp(jnl_file_list, STAR_QUOTE, STR_LIT_LEN(STAR_QUOTE))))
 	{
 		star_specified = TRUE;
+		if (mur_options.corruptdb)
+		{
+			gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_STARFILE, 2, LEN_AND_LIT("CORRUPTDB qualifier"));
+			mupip_exit(ERR_MUPCLIERR);
+		}
 		if (NULL != mur_options.redirect)
 		{
 			gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_STARFILE, 2, LEN_AND_LIT("REDIRECT qualifier"));
@@ -443,7 +447,7 @@ uint4 mur_open_files(boolean_t retry)
 					rctl->standalone = TRUE;
 				}
 			}
-			if (mur_options.update || mur_options.extr[GOOD_TN])
+			if (!TREF(skip_DB_exists_check) && (mur_options.update || mur_options.extr[GOOD_TN]))
 			{
 				gvcst_init(rctl->gd, NULL);
 				TP_CHANGE_REG(rctl->gd);
@@ -544,7 +548,7 @@ uint4 mur_open_files(boolean_t retry)
 				tmpcsa = &FILE_INFO(reg)->s_addrs;
 				tmpcsd = tmpcsa->hdr;
 				GRAB_ACCESS_CONTROL(rl);
-				grab_crit(reg);
+				grab_crit(reg, WS_73);
 				DO_CHILLED_AUTORELEASE(tmpcsa, tmpcsd);
 				if (FROZEN(tmpcsd))
 				{
@@ -646,7 +650,7 @@ uint4 mur_open_files(boolean_t retry)
 			if (!wcs_flu(WCSFLU_NONE))
 			{
 				assert(cs_addrs->nl->wcs_phase2_commit_pidcnt); /* only reason why wcs_flu can fail */
-				SET_TRACEABLE_VAR(cs_addrs->nl->wc_blocked, TRUE);
+				SET_TRACEABLE_VAR(cs_addrs->nl->wc_blocked, WC_BLOCK_RECOVER);
 				BG_TRACE_PRO_ANY(cs_addrs, wc_blocked_onln_rlbk);
 				send_msg_csa(CSA_ARG(cs_addrs) VARLSTCNT(8) ERR_WCBLOCKED, 6, LEN_AND_LIT("wc_blocked_onln_rlbk"),
 					 process_id, &cs_addrs->ti->curr_tn, DB_LEN_STR(gv_cur_region));
@@ -689,7 +693,7 @@ uint4 mur_open_files(boolean_t retry)
 	{
 		if (rctl->db_present)
 		{
-			if (mur_options.update || mur_options.extr[GOOD_TN])
+			if (!TREF(skip_DB_exists_check) && (mur_options.update || mur_options.extr[GOOD_TN]))
 			{	/* NOTE: Only for collation info extract needs database access */
 				DEFER_INTERRUPTS(INTRPT_IN_MUR_OPEN_FILES, prev_intrpt_state);	/* temporarily disable
 												 * MUPIP STOP/signal handling. */
@@ -861,7 +865,8 @@ uint4 mur_open_files(boolean_t retry)
 				/* 1) show 2) extract 3) verify action does not need standalone access.
 				 * In this case csa is NULL
 				 */
-				if (!file_head_read((char *)rctl->gd->dyn.addr->fname, rctl->csd, SGMNT_HDR_LEN))
+				if (!file_head_read((char *)rctl->gd->dyn.addr->fname, rctl->csd, SGMNT_HDR_LEN)
+					&& !TREF(skip_DB_exists_check))
 				{
 					gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_DBFILOPERR, 2, REG_LEN_STR(rctl->gd));
 					return FALSE;
@@ -997,7 +1002,7 @@ uint4 mur_open_files(boolean_t retry)
 					}
 				}
 				/* db list was created from journal file header. So it is not possible */
-				assertpro(rctl != rctl_top);
+				assertpro(TREF(skip_DB_exists_check) || (rctl != rctl_top));
 			}
 			/* Detect and report 1st case of any duplicated files in mupip forward recovery command. */
 			if (mur_options.forward)
@@ -1116,7 +1121,7 @@ uint4 mur_open_files(boolean_t retry)
 	for (rctl = mur_ctl, rctl_top = mur_ctl + murgbl.reg_total; rctl < rctl_top; rctl++)
 	{
 		jctl = rctl->jctl_head;
-		if (rctl->db_present && (mur_options.update || mur_options.extr[GOOD_TN]))
+		if (rctl->db_present && (!TREF(skip_DB_exists_check) && (mur_options.update || mur_options.extr[GOOD_TN])))
 			rctl->csa->miscptr = (void *)rctl; /* needed by gdsfilext_nojnl/jnl_write_pini/mur_pini_addr_reset */
 		if (mur_options.update)
 		{
