@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2013 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2016 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -13,6 +14,9 @@
 #include "gtm_string.h"
 #include "stringpool.h"
 #include "op.h"
+#include <rtnhdr.h>
+#include "stack_frame.h"
+#include "mv_stent.h"
 
 #define PLUS 	1
 #define MINUS 	2
@@ -21,7 +25,8 @@
 #define PAREN 	16
 #define FNERROR 7
 
-GBLREF spdesc stringpool;
+GBLREF spdesc	stringpool;
+GBLREF mv_stent	*mv_chain;
 
 LITREF mval	literal_zero;
 
@@ -32,35 +37,33 @@ void op_fnfnumber(mval *src, mval *fmt, boolean_t use_fract, int fract, mval *ds
 {
 	boolean_t	comma, paren;
 	int 		ct, x, xx, y, z;
-	mval		t_src, *t_src_p;
 	unsigned char	*ch, *cp, *ff, *ff_top, fncode, sign, *t;
 
 	if (!MV_DEFINED(fmt))		/* catch this up front so noundef mode can't cause trouble - so fmt no empty context */
 		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(2) ERR_FNUMARG, 0);
-	t_src_p = &t_src;		/* operate on src in a temp, so conversions are possible without modifying src */
-	*t_src_p = *src;
+	/* if the dst will be different than the src we'll build the new value in the string pool and repoint dst there,
+	 * otherwise, dst will anyway become the same as src, therefore we can safely use dst as a "temporary" copy of src
+	 */
+	*dst = *src;
 	if (use_fract)
-		op_fnj3(t_src_p, 0, fract, t_src_p);
+		op_fnj3(dst, 0, fract, dst);
 	else
 	{
-		MV_FORCE_NUM(t_src_p);
-		MV_FORCE_CANONICAL(t_src_p);	/* if the source operand is not a canonical number, force conversion */
+		MV_FORCE_NUM(dst);
+		MV_FORCE_CANONICAL(dst);	/* if the source operand is not a canonical number, force conversion */
 	}
 	assert (stringpool.free >= stringpool.base);
 	assert (stringpool.free <= stringpool.top);
 	/* assure there is adequate space for two string forms of a number as a local
 	 * version of the src must be operated upon in order to get a canonical number
 	 */
-	ENSURE_STP_FREE_SPACE(MAX_NUM_SIZE * 2);
 	MV_FORCE_STR(fmt);
-	MV_FORCE_STR(t_src_p);
+	MV_FORCE_STR(dst);
 	if (0 == fmt->str.len)
-	{
-		*dst = *t_src_p;
 		return;
-	}
-	ch = (unsigned char *)t_src_p->str.addr;
-	ct = t_src_p->str.len;
+	ENSURE_STP_FREE_SPACE(MAX_NUM_SIZE * 2);
+	ch = (unsigned char *)dst->str.addr;
+	ct = dst->str.len;
 	cp = stringpool.free;
 	fncode = 0;
 	for (ff = (unsigned char *)fmt->str.addr, ff_top = ff + fmt->str.len; ff < ff_top;)
@@ -114,8 +117,8 @@ void op_fnfnumber(mval *src, mval *fmt, boolean_t use_fract, int fract, mval *ds
 		/* Only add '+' if > 0 */
 		if ((0 != (fncode & PLUS)) && (0 == sign))
 		{	/* Need to make into num and check for int 0 in case was preprocessed by op_fnj3() */
-			MV_FORCE_NUM(t_src_p);
-			if ((0 == (t_src_p->mvtype & MV_INT)) || (0 != t_src_p->m[1]))
+			MV_FORCE_NUM(dst);
+			if ((0 == (dst->mvtype & MV_INT)) || (0 != dst->m[1]))
 				sign = '+';
 		}
 		if ((0 != (fncode & MINUS)) && ('-' == sign))
