@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -57,6 +57,7 @@
 #include "interlock.h"
 #include "cache.h"
 #include "hashtab_objcode.h"
+#include "gvt_inline.h"
 
 GBLREF spdesc			stringpool;
 GBLREF int4			cache_hits, cache_fails, max_cache_entries;
@@ -124,7 +125,7 @@ void	op_fnview(int numarg, mval *dst, ...)
 	gd_gblname	*gname;
 	gd_region	*reg, *reg_start, *reg_top, *statsDBreg;
 	gv_key		*gvkey;
-	gv_key		save_currkey[DBKEYALLOC(MAX_KEY_SZ)];
+	gv_key_buf	save_currkey;
 	gv_namehead	temp_gv_target;
 	gvnh_reg_t	*gvnh_reg;
 	gvnh_spanreg_t	*gvspan;
@@ -403,6 +404,8 @@ void	op_fnview(int numarg, mval *dst, ...)
 		case VTK_GRABJNLPOOLPH2:
 		case VTK_GRABJNLQIO:
 		case VTK_RELJNLQIO:
+		case VTK_GRABFSYNC:
+		case VTK_RELFSYNC:
 			reg = parmblk.gv_ptr;
 			if (!reg->open)
 				gv_init_reg(reg, NULL);
@@ -410,7 +413,7 @@ void	op_fnview(int numarg, mval *dst, ...)
 			if (NULL != csa->hdr)
 			{
 				if (VTK_GRABCRIT == vtp->keycode)
-					grab_crit(reg);
+					grab_crit(reg, WS_76);
 				else if (VTK_RELCRIT == vtp->keycode)
 					rel_crit(reg);
 				else if (VTK_GRABLOCK == vtp->keycode)
@@ -424,14 +427,16 @@ void	op_fnview(int numarg, mval *dst, ...)
 					if (jnlpool)
 						rel_lock(jnlpool->jnlpool_dummy_reg);
 				} else if (VTK_GRABJNLPH2 == vtp->keycode)
-					grab_latch(&csa->jnl->jnl_buff->phase2_commit_latch, GRAB_LATCH_INDEFINITE_WAIT);
+					grab_latch(&csa->jnl->jnl_buff->phase2_commit_latch, GRAB_LATCH_INDEFINITE_WAIT,
+						WS_30, csa);
 				else if (VTK_RELJNLPH2 == vtp->keycode)
 					rel_latch(&csa->jnl->jnl_buff->phase2_commit_latch);
 				else if (VTK_GRABJNLPOOLPH2 == vtp->keycode)
 				{
 					assert((NULL != jnlpool) && (NULL != jnlpool->jnlpool_ctl));
 					if (jnlpool && jnlpool->jnlpool_ctl)
-						grab_latch(&jnlpool->jnlpool_ctl->phase2_commit_latch, GRAB_LATCH_INDEFINITE_WAIT);
+						grab_latch(&jnlpool->jnlpool_ctl->phase2_commit_latch, GRAB_LATCH_INDEFINITE_WAIT,
+							WS_31, csa);
 				} else if (VTK_RELJNLPOOLPH2 == vtp->keycode)
 				{
 					assert((NULL != jnlpool) && (NULL != jnlpool->jnlpool_ctl));
@@ -443,6 +448,12 @@ void	op_fnview(int numarg, mval *dst, ...)
 						SHORT_SLEEP(1);
 				} else if (VTK_RELJNLQIO == vtp->keycode)
 					RELEASE_SWAPLOCK(&csa->jnl->jnl_buff->io_in_prog_latch);
+				else if (VTK_GRABFSYNC == vtp->keycode)
+				{
+					while (!GET_SWAPLOCK(&csa->jnl->jnl_buff->fsync_in_prog_latch))
+						SHORT_SLEEP(1);
+				} else if (VTK_RELFSYNC == vtp->keycode)
+					RELEASE_SWAPLOCK(&csa->jnl->jnl_buff->fsync_in_prog_latch);
 			}
 			break;
 #endif
@@ -455,7 +466,7 @@ void	op_fnview(int numarg, mval *dst, ...)
 			if (NULL != csa->hdr)
 			{
 				csa->crit_probe = TRUE;
-				grab_crit(reg);
+				grab_crit(reg, WS_77);
 				csa->crit_probe = FALSE;
 				if (!WBTEST_ENABLED(WBTEST_HOLD_CRIT_ENABLED))
 					rel_crit(reg);
@@ -481,7 +492,7 @@ void	op_fnview(int numarg, mval *dst, ...)
 				n = (NULL != gname) ? gname->act : 0;
 			} else
 				n = 0;
-			gvkey = (gv_key *)save_currkey;
+			gvkey = (gv_key *)&save_currkey.key;
 			key = gvn2gds(arg1, gvkey, n);
 			assert(key > &gvkey->base[0]);
 			assert(gvkey->end == key - &gvkey->base[0] - 1);
@@ -701,12 +712,12 @@ void	op_fnview(int numarg, mval *dst, ...)
 			break;
 		case VTK_YGVN2GDS:
 			n = (NULL != arg2) ? mval2i(arg2) : 0;
-			gvkey = &save_currkey[0];
+			gvkey = (gv_key *)&save_currkey.key;
 			key = gvn2gds(arg1, gvkey, n);
 			if (!n)
 				key = &gvkey->base[0];
 			/* If input has error at some point, copy whatever subscripts (+ gblname) have been successfully parsed */
-			COPY_ARG_TO_STRINGPOOL(dst, key, &gvkey->base[0]);
+			COPY_ARG_TO_STRINGPOOL(dst, key, gvkey->base);
 			break;
 		case VTK_YLCT:
 			n = -1;
