@@ -281,7 +281,7 @@ void	op_fnzsocket(UNIX_ONLY_COMMA(int numarg) mval *dst, ...)
 			assert(creator_passed >= socketptr->howcreated);
 			dst->str.addr = (char *)zsocket_howcreated_names[socketptr->howcreated];
 			dst->str.len = STRLEN(dst->str.addr);
-			UNICODE_ONLY(dst->str.char_len = 0);
+			UTF8_ONLY(dst->str.char_len = 0);
 			s2pool(&dst->str);
 			break;
 		case zsocket_index:
@@ -335,7 +335,7 @@ void	op_fnzsocket(UNIX_ONLY_COMMA(int numarg) mval *dst, ...)
 #			endif
 			} else
 				*dst = literal_null;
-			UNICODE_ONLY(dst->str.char_len = 0);
+			UTF8_ONLY(dst->str.char_len = 0);
 			s2pool(&dst->str);
 			break;
 		case zsocket_localport:
@@ -364,7 +364,7 @@ void	op_fnzsocket(UNIX_ONLY_COMMA(int numarg) mval *dst, ...)
 			{
 				dst->str.addr = socketptr->parenthandle;
 				dst->str.len = STRLEN(socketptr->parenthandle);
-				UNICODE_ONLY(dst->str.char_len = 0);
+				UTF8_ONLY(dst->str.char_len = 0);
 				s2pool(&dst->str);
 			} else
 				*dst = literal_null;
@@ -416,7 +416,7 @@ void	op_fnzsocket(UNIX_ONLY_COMMA(int numarg) mval *dst, ...)
 #			endif
 			} else
 				*dst = literal_null;
-			UNICODE_ONLY(dst->str.char_len = 0);
+			UTF8_ONLY(dst->str.char_len = 0);
 			s2pool(&dst->str);
 			break;
 		case zsocket_remoteport:
@@ -431,14 +431,14 @@ void	op_fnzsocket(UNIX_ONLY_COMMA(int numarg) mval *dst, ...)
 		case zsocket_sockethandle:
 			dst->str.addr = socketptr->handle;
 			dst->str.len = socketptr->handle_len;
-			UNICODE_ONLY(dst->str.char_len = 0);
+			UTF8_ONLY(dst->str.char_len = 0);
 			s2pool(&dst->str);
 			break;
 		case zsocket_state:
 			assert(socket_connect_inprogress >= socketptr->state);
 			dst->str.addr = (char *)zsocket_state_names[socketptr->state];
 			dst->str.len = STRLEN(dst->str.addr);
-			UNICODE_ONLY(dst->str.char_len = 0);
+			UTF8_ONLY(dst->str.char_len = 0);
 			s2pool(&dst->str);
 			break;
 		case zsocket_tls:
