@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2017 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -33,6 +33,7 @@
 #include <rtnhdr.h>
 #include "mv_stent.h"		/* for COPY_SUBS_TO_GVCURRKEY macro */
 #include "gvcst_protos.h"	/* for gvcst_root_search in GV_BIND_NAME_AND_ROOT_SEARCH macro */
+#include "gvt_inline.h"
 
 /*the header files below are for environment translation*/
 #ifdef UNIX
@@ -144,6 +145,6 @@ STATICFNDEF void op_gvextnam_common(int count, int hash_code, mval *val1, va_lis
 						in current key */
 	TREF(gv_last_subsc_null) = is_null; /* if true, it indicates that last subscript in current key is null */
 	if (was_null && (NEVER == reg->null_subs))
-		sgnl_gvnulsubsc();
+		sgnl_gvnulsubsc(NULL);
 	return;
 }
