@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2017 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -33,7 +33,6 @@
 #include "gdscc.h"			/* needed for tp.h */
 #include "gdskill.h"			/* needed for tp.h */
 #include "buddy_list.h"			/* needed for tp.h */
-#include "hashtab_int4.h"		/* needed for tp.h */
 #include "filestruct.h"			/* needed for jnl.h */
 #include "jnl.h"			/* needed for tp.h */
 #include "tp.h"
@@ -59,7 +58,7 @@ void op_gvorder(mval *v)
 	gd_addr			*gd_targ;
 	gd_binding		*gd_map_start, *map, *map_top;
 	gd_region		*save_gv_cur_region;
-	gv_key			save_currkey[DBKEYALLOC(MAX_KEY_SZ)];
+	gv_key_buf		save_currkey;
 	gv_namehead		*gvt, *save_gv_target;
 	gvnh_reg_t		*gvnh_reg;
 	gvnh_spanreg_t		*gvspan;
@@ -155,8 +154,10 @@ void op_gvorder(mval *v)
 		assert(KEY_DELIMITER == gv_currkey->base[gv_currkey->end]);
 		assert(KEY_DELIMITER == gv_currkey->base[gv_currkey->end - 1]);
 		SAVE_REGION_INFO(save_currkey, save_gv_target, save_gv_cur_region, save_sgm_info_ptr, save_jnlpool);
-		if (STR_SUB_ESCAPE == save_currkey->base[gv_currkey->end - 2])
-			save_currkey->base[gv_currkey->end - 2] = KEY_DELIMITER;	/* strip the byte added to get past curr */
+		if (STR_SUB_ESCAPE == *(save_currkey.split.base + gv_currkey->end - 2))
+		{	/* strip the byte added to get past curr */
+			*(save_currkey.split.base + gv_currkey->end - 2) = KEY_DELIMITER;
+		}
 		gd_targ = TREF(gd_targ_addr);
 		gd_map_start = gd_targ->maps;
 		map_top = gd_map_start + gd_targ->n_maps;
