@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2013 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2019 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -41,7 +42,10 @@ GBLREF	bool		gv_replopen_error;
 GBLREF	jnl_gbls_t	jgbl;
 #endif
 
+#define	NONULLSUBS	"ZKILL/ZWITHDRAW failed because"
+
 error_def(ERR_DBPRIVERR);
+error_def(ERR_PCTYRESERVED);
 
 void with_var(void);
 
@@ -51,14 +55,17 @@ void op_gvzwithdraw(void)
 	DCL_THREADGBL_ACCESS;
 
 	SETUP_THREADGBL_ACCESS;
+	/* If specified var name is global ^%Y*, the name is illegal to use in a SET or KILL command, only GETs are allowed */
+	if ((RESERVED_NAMESPACE_LEN <= gv_currkey->end) && (0 == MEMCMP_LIT(gv_currkey->base, RESERVED_NAMESPACE)))
+		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_PCTYRESERVED);
 	if (gv_cur_region->read_only)
 	{
 		assert(cs_addrs == &FILE_INFO(gv_cur_region)->s_addrs);
 		rts_error_csa(CSA_ARG(cs_addrs) VARLSTCNT(4) ERR_DBPRIVERR, 2, DB_LEN_STR(gv_cur_region));
 	}
 	if (TREF(gv_last_subsc_null) && NEVER == gv_cur_region->null_subs)
-		sgnl_gvnulsubsc();
-	if (REG_ACC_METH(gv_cur_region) == dba_bg || REG_ACC_METH(gv_cur_region) == dba_mm)
+		sgnl_gvnulsubsc(NONULLSUBS);
+	if (IS_REG_BG_OR_MM(gv_cur_region))
 	{
 		/* No special code needed for spanning globals here since we are in the region we want to be
 		 * and all we want to do is kill one node in this region (not a subtree underneath) even if
