@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2014 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -49,7 +50,6 @@
 #include "gdscc.h"
 #include "copy.h"
 #include "jnl.h"
-#include "hashtab_int4.h"	/* needed for tp.h */
 #include "buddy_list.h"
 #include "tp.h"
 #include "gtm_string.h"
@@ -68,6 +68,7 @@
 #include "collseq.h"
 #include "alias.h"
 #include "gtmimagename.h"
+#include "gvt_inline.h"
 
 GBLREF sgmnt_addrs	*cs_addrs;
 GBLREF mv_stent		*mv_chain;
@@ -85,6 +86,8 @@ GBLREF lvzwrite_datablk *lvzwrite_block;
 GBLREF lv_val		*active_lv;
 #endif
 
+#define	NONULLSUBS	"MERGE failed because"
+
 error_def(ERR_MAXNRSUBSCRIPTS);
 error_def(ERR_MERGEINCOMPL);
 error_def(ERR_NCTCOLLDIFF);
@@ -150,8 +153,7 @@ void op_merge(void)
 			merge_args = 0;	/* Must reset to zero to reuse the Global */
 			return;
 		}
-		nontp_and_bgormm = ((dba_bg == REG_ACC_METH(gv_cur_region)) || (dba_mm == REG_ACC_METH(gv_cur_region)))
-				&& !dollar_tlevel;
+		nontp_and_bgormm = (!dollar_tlevel && IS_REG_BG_OR_MM(gv_cur_region));
 		if (NULL == TREF(gv_mergekey2))
 		{	/* We need to initialize gvn2 (right hand side). */
 			GVKEY_INIT(TREF(gv_mergekey2), DBKEYSIZE(MAX_KEY_SZ));
@@ -250,7 +252,7 @@ void op_merge(void)
 				assert (0 < delta2);
 				GET_NSUBS_IN_GVKEY(mkey->str.addr + org_glvn2_keysz - 2, delta2, nsubs); /* sets "nsubs" */
 				/* Check if the target node in ^gvn1 exceeds max # of subscripts */
-				if (MAX_GVSUBSCRIPTS <= (gvn1subs + gvn2subs + nsubs))
+				if (MAX_GVSUBSCRIPTS <= (gvn1subs + nsubs))
 					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(3) ERR_MERGEINCOMPL, 0, ERR_MAXNRSUBSCRIPTS);
 				/* Save the new source key for next iteration. Do not use gvname_env_save since that relies
 				 * on gv_currkey holding the key to be saved which is not the case here. Also gvname_env_save
@@ -264,7 +266,9 @@ void op_merge(void)
 				{
 					if (NULL != gvnh_reg2)
 					{	/* ^gvn2 spans multiple regions. Find region where the current key was obtained */
-						map = gv_srch_map(gbl2_gd_addr, (char *)&gv_currkey->base[0], gv_currkey->end - 1);
+						map = gv_srch_map(gbl2_gd_addr,
+									(char *)&gv_currkey->base[0], gv_currkey->end - 1,
+									SKIP_BASEDB_OPEN_FALSE);
 						reg2 = map->reg.addr;
 					}
 				}
@@ -360,7 +364,7 @@ void op_merge(void)
 							 * have become nearly impossible to create a fully standard
 							 * implementation. : From Ed de Moel : 2/1/2
 							 */
-							sgnl_gvnulsubsc();
+							sgnl_gvnulsubsc(NONULLSUBS);
 					}
 				}
 				/* Now put value of ^glvn2 descendant into corresponding descendant under ^glvn1 */
@@ -374,6 +378,7 @@ void op_merge(void)
 			/* Need to protect subsc created from global variable subscripts from stpgcol */
 			PUSH_MV_STENT(MVST_MVAL);
 			subsc = &mv_chain->mv_st_cont.mvs_mval;
+			subsc->mvtype = 0; /* initialize mval in the M-stack in case stp_gcol gets called before it is set below */
 			/* At this time gv_currkey already points to gblp2 */
 			if (1 == dollardata_src || 11 == dollardata_src)
 			{	/* SET lvn1=^gvn2 */
@@ -421,7 +426,7 @@ void op_merge(void)
 				do
 				{
 					LV_SBS_DEPTH(dst_lv, is_base_var, sbs_depth);
-					if (MAX_LVSUBSCRIPTS <= sbs_depth)
+					if ((MAX_LVSUBSCRIPTS - 1) <= sbs_depth)
 						rts_error_csa(CSA_ARG(NULL) VARLSTCNT(3) ERR_MERGEINCOMPL, 0, ERR_MAXNRSUBSCRIPTS);
 					opstr.addr = (char *)buff;
 					opstr.len = MAX_ZWR_KEY_SZ;
@@ -469,6 +474,7 @@ void op_merge(void)
 				lvzwr_init(zwr_patrn_mident, &mglvnp->lclp[IND2]->v);
 				lvzwr_arg(ZWRITE_ASTERISK, 0, 0);
 				lvzwr_var(mglvnp->lclp[IND2], 0);
+				TREF(in_zwrite) = FALSE;
 #				ifdef DEBUG
 				/* assert that destination got all data of the source and its descendants */
 				op_fndata(mglvnp->lclp[IND2], value);
@@ -495,6 +501,7 @@ void op_merge(void)
 				lvzwr_init(zwr_patrn_mident, &mglvnp->lclp[IND2]->v);
 				lvzwr_arg(ZWRITE_ASTERISK, 0, 0);
 				lvzwr_var(mglvnp->lclp[IND2], 0);
+				TREF(in_zwrite) = FALSE;
 				gvname_env_restore(gblp1);	 /* store destination as naked indicator in gv_currkey */
 			}
 		}
