@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2017 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -52,13 +52,13 @@
 #include "copy.h"
 #include "jnl.h"
 #include "buddy_list.h"
-#include "hashtab_int4.h"	/* needed for tp.h */
 #include "tp.h"
 #include "gvname_info.h"
 #include "op_merge.h"
 #include "format_targ_key.h"
 #include "cmidef.h"
 #include "cmmdef.h"
+#include "gvt_inline.h"
 
 GBLREF int		merge_args;
 GBLREF merge_glvn_ptr	mglvnp;
