@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- *	Copyright 2012 Fidelity Information Services, Inc	*
+ *	Copyright 2012, 2014 Fidelity Information Services, Inc	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -39,12 +39,18 @@ void op_stoglvn(uint4 indx, mval *value)
 	DCL_THREADGBL_ACCESS;
 
 	SETUP_THREADGBL_ACCESS;
+	MV_FORCE_DEFINED(value);	/* do what op_sto.s does */
 	oc = (TREF(glvn_pool_ptr))->slot[indx].sav_opcode;
 	if (OC_SAVLVN == oc)
 	{	/* lvn */
 		lv = op_rfrshlvn(indx, OC_PUTINDX);
+		/* Below two lines do what op_sto.s does. Ideally we should be invoking op_sto from here but
+		 * it is not easily possible due to op_sto expecting its arguments in registers which this
+		 * C function cannot ensure. Any changes to op_sto.s might need to be correspondingly made here.
+		 */
 		lv->v = *value;
-	} else if (OC_NOOP != oc)			/* if indirect error blew set up, skip this */
+		lv->v.mvtype &= ~MV_ALIASCONT;	/* Make sure alias container property does not pass */
+	} else if (OC_NOOP != oc)		/* if indirect error blew set up, skip this */
 	{	/* gvn */
 		op_rfrshgvn(indx, oc);
 		op_gvput(value);
