@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2019 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -31,7 +31,6 @@
 #include "stringpool.h"
 #include "svnames.h"
 #include "buddy_list.h"		/* needed for tp.h */
-#include "hashtab_int4.h"	/* needed for tp.h */
 #include "tp.h"
 #include "error.h"
 #include "op.h"
@@ -51,6 +50,7 @@
 #include "get_reference.h"
 #include "dollar_quit.h"
 #include "ztimeout_routines.h"
+#include "mlkdef.h"
 #ifdef UNIX
 #  include "iormdef.h"
 #  ifdef DEBUG
@@ -100,6 +100,8 @@ GBLREF io_pair		*io_std_device;
 GBLREF mstr		dollar_zpin;
 GBLREF mstr		dollar_zpout;
 GBLREF int		process_exiting;
+GBLREF mlk_subhash_val_t	mlk_last_hash;
+
 #ifdef GTM_TRIGGER
 GBLREF	mstr		*dollar_ztname;
 GBLREF	mval		*dollar_ztdata;
@@ -127,6 +129,7 @@ LITREF int4		gtm_release_stamp_len;
 
 void op_svget(int varnum, mval *v)
 {
+	boolean_t	lcl_compile_time;
 	io_log_name	*tl;
 	int 		count;
 	gtm_uint64_t	ucount;
@@ -162,10 +165,10 @@ void op_svget(int varnum, mval *v)
 	switch (varnum)
 	{
 		case SV_HOROLOG:
-			op_horolog(v);
+			op_zhorolog(v, FALSE);
 			break;
 		case SV_ZHOROLOG:
-			op_zhorolog(v);
+			op_zhorolog(v, TRUE);
 			break;
 		case SV_ZUT:
 			op_zut(v);
@@ -621,7 +624,7 @@ void op_svget(int varnum, mval *v)
 			get_dlr_zkey(v);
 			break;
 		case SV_ZSTRPLLIM:
-			count = TREF(gtm_strpllim);
+			count = stringpool.strpllim;
 			MV_FORCE_MVAL(v, count);
 			break;
 		case SV_ZTIMEOUT:
@@ -629,6 +632,9 @@ void op_svget(int varnum, mval *v)
 			if (-1 == count)
 				MV_FORCE_MVAL(v, count);
 			break;
+		case SV_ZMLKHASH:
+			i2usmval(v, mlk_last_hash);
+			break;
 		default:
 			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_INVSVN);
 	}
@@ -639,6 +645,9 @@ void op_svget(int varnum, mval *v)
 	} else if (!(MVTYPE_IS_NUMERIC(v->mvtype)))
 	{
 		assert(MVTYPE_IS_STRING(v->mvtype));
+		lcl_compile_time = TREF(compile_time);
+		TREF(compile_time) = TRUE;
 		s2n(v);
+		TREF(compile_time) = lcl_compile_time;
 	}
 }
