@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2019 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -30,7 +30,6 @@
 #include "min_max.h"		/* needed for MM extend */
 #include "gdsblkops.h"		/* needed for MM extend */
 
-#include "hashtab_int4.h"	/* needed for tp.h */
 #include "tp.h"
 #include "tp_frame.h"
 #include "copy.h"
@@ -66,6 +65,7 @@
 #include "gvcst_protos.h"
 #include "repl_msg.h"			/* for gtmsource.h */
 #include "gtmsource.h"			/* for jnlpool_addrs_ptr_t */
+#include "gvt_inline.h"
 
 #include "db_snapshot.h"
 
@@ -216,7 +216,7 @@ enum cdb_sc	op_tcommit(void)
 
 	SETUP_THREADGBL_ACCESS;
 #	ifdef GTM_TRIGGER
-	DBGTRIGR((stderr, "op_tcommit: Entry from 0x"lvaddr"\n", caller_id()));
+	DBGTRIGR((stderr, "op_tcommit: Entry from 0x"lvaddr"\n", caller_id(0)));
 	skip_invoke_restart = skip_INVOKE_RESTART;	/* note down global value in local variable */
 	skip_INVOKE_RESTART = FALSE;	/* reset global variable to default state as soon as possible */
 #	else
@@ -278,13 +278,14 @@ enum cdb_sc	op_tcommit(void)
 				is_mm = (dba_mm == csa->hdr->acc_meth);
 				csa->tp_hint = 0;	/* will be set to non-zero later if we invoke "bm_getfree" */
 				si->cr_array_index = 0;
-#				ifdef DEBUG
-				if (WBTEST_ENABLED(WBTEST_MM_CONCURRENT_FILE_EXTEND)
-					&& !MEMCMP_LIT(gv_cur_region->rname, "DEFAULT") && !csa->nl->wbox_test_seq_num)
-				{
-					while (0 == csa->nl->wbox_test_seq_num)
-						SHORT_SLEEP(1);	/* wait for signal from mubfilcpy that it has reached sync point */
-				}
+#				ifdef DEBUG /* This code is shared by two WB tests */
+				if (WBTEST_ENABLED(WBTEST_MM_CONCURRENT_FILE_EXTEND) ||
+					(WBTEST_ENABLED(WBTEST_WSSTATS_PAUSE) && (10 == gtm_white_box_test_case_count)))
+					if (!MEMCMP_LIT(gv_cur_region->rname, "DEFAULT") && !csa->nl->wbox_test_seq_num)
+					{
+						while (0 == csa->nl->wbox_test_seq_num)
+						SHORT_SLEEP(1); /* wait for signal from mubfilcpy that it has reached sync point */
+					}
 #				endif
 				if (!is_mm && (si->cr_array_size < (si->num_of_blks + (si->cw_set_depth * 2))))
 				{	/* reallocate a bigger cr_array. We need atmost read-set (si->num_of_blks) +
@@ -336,8 +337,7 @@ enum cdb_sc	op_tcommit(void)
 						 * FALSE.
 						 */
 						SS_INIT_IF_NEEDED(csa, cnl);
-					} else
-						SS_RELEASE_IF_NEEDED(csa, cnl);
+					}
 					read_before_image = ((NULL != jbp) || csa->backup_in_prog || SNAPSHOTS_IN_PROG(csa));
 					/* The following section allocates new blocks required by the transaction it is done
 					 * before going crit in order to reduce the change of having to wait on a read while crit.
