@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2014 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -33,9 +34,15 @@
 #include "gdsbt.h"
 #include "gdsfhead.h"
 #include "alias.h"
+#include "zr_unlink_rtn.h"
 #ifdef GTM_TRIGGER
 # include "gtm_trigger_trc.h"
 #endif
+#include "xfer_enum.h"
+#include "deferred_events.h"
+#include "deferred_events_queue.h"
+#include "ztimeout_routines.h"
+#include "jobinterrupt_event.h"
 
 GBLREF	void			(*unw_prof_frame_ptr)(void);
 GBLREF	stack_frame		*frame_pointer, *zyerr_frame;
@@ -49,7 +56,6 @@ GBLREF	mval			*alias_retarg;
 GBLREF	boolean_t		tp_timeout_deferred;
 GBLREF	dollar_ecode_type	dollar_ecode;
 GBLREF	boolean_t		ztrap_explicit_null;
-GBLREF	mval			dollar_ztrap;
 GBLREF	boolean_t		dollar_zininterrupt;
 GBLREF	boolean_t		dollar_truth;
 
@@ -59,7 +65,11 @@ error_def(ERR_TPQUIT);
 /* This has to be maintained in parallel with unw_retarg(), the unwind with a return argument (extrinisic quit) routine. */
 void op_unwind(void)
 {
-	mv_stent 		*mvc;
+	rhdtyp			*rtnhdr;
+	mv_stent		*mvc;
+	int4			event_type, param_val;
+	void (*set_fn)(int4 param);
+
 	DBGEHND_ONLY(stack_frame *prevfp;)
 	DCL_THREADGBL_ACCESS;
 
@@ -109,13 +119,12 @@ void op_unwind(void)
 	msp = (unsigned char *)frame_pointer + SIZEOF(stack_frame);
 	if (msp > stackbase)
 		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_STACKUNDERFLO);
-#	ifdef GTM_TRIGGER
-	if (SFF_IMPLTSTART_CALLD & frame_pointer->type)
-		DBGTRIGR((stderr, "op_unwind: Unwinding frame 0x"lvaddr" with type %d which has SFF_IMPLTSTART_CALLD turned on\n",
+	if (SSF_NORET_VIA_MUMTSTART & frame_pointer->type)
+		DBGTRIGR((stderr, "op_unwind: Unwinding frame 0x"lvaddr" with type %d which has SSF_NORET_VIA_MUMTSTART enabled\n",
 			  frame_pointer, frame_pointer->type));
-#	endif
 	DRAIN_GLVN_POOL_IF_NEEDED;
 	PARM_ACT_UNSTACK_IF_NEEDED;
+	USHBIN_ONLY(rtnhdr = frame_pointer->rvector);	/* Save rtnhdr for cleanup call below */
 	frame_pointer = frame_pointer->old_frame_pointer;
 	DBGEHND((stderr, "op_unwind: Stack frame 0x"lvaddr" unwound - frame 0x"lvaddr" now current - New msp: 0x"lvaddr"\n",
 		 prevfp, frame_pointer, msp));
@@ -128,11 +137,30 @@ void op_unwind(void)
 			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_STACKUNDERFLO);
 		assert((frame_pointer < frame_pointer->old_frame_pointer) || (NULL == frame_pointer->old_frame_pointer));
 	}
+	USHBIN_ONLY(CLEANUP_COPIED_RECURSIVE_RTN(rtnhdr));
 	/* We just unwound a frame. May have been either a zintrupt frame and/or may have unwound a NEW'd ZTRAP or even cleared
 	 * our error state. If we have a deferred timeout and none of the deferral conditions are anymore in effect, release
 	 * the hounds.
 	 */
-	if (tp_timeout_deferred UNIX_ONLY(&& !dollar_zininterrupt) && ((0 == dollar_ecode.index) || !(ETRAP_IN_EFFECT)))
-		tptimeout_set(0);
+	if ((TREF(save_xfer_root)))
+	{
+		/*If TP timeout or ztimeout , check conditions before popping out */
+		if (((TREF(save_xfer_root))->set_fn == tptimeout_set) || ((TREF(save_xfer_root))->set_fn == ztimeout_set))
+		{
+			if ((tp_timeout_deferred || TREF(ztimeout_deferred))
+				UNIX_ONLY(&& !dollar_zininterrupt) && ((0 == dollar_ecode.index)
+									|| !(ETRAP_IN_EFFECT)))
+			{
+				DBGDFRDEVNT((stderr, "op_unwind1: Calling pop_reset_xfer from op_unwind\n"));
+				POP_XFER_ENTRY(&event_type, &set_fn, &param_val);
+				xfer_set_handlers(event_type, set_fn, param_val, TRUE);
+			}
+		} else if (((TREF(save_xfer_root))->set_fn == jobinterrupt_set) UNIX_ONLY(&& !dollar_zininterrupt))
+		{
+			DBGDFRDEVNT((stderr, "op_unwind2: Calling pop_reset_xfer from op_unwind\n"));
+			POP_XFER_ENTRY(&event_type, &set_fn, &param_val);
+			xfer_set_handlers(event_type, set_fn, param_val, TRUE);
+		}
+	}
 	return;
 }
