@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2019 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -24,9 +24,11 @@ GBLREF unsigned char	*restart_ctxt, *restart_pc;
 GBLREF void             (*tp_timeout_action_ptr)(void);
 GBLREF volatile int4 	ctrap_action_is, outofband;
 GBLREF void		(*ztimeout_action_ptr)(void);
+
 error_def(ERR_CTRAP);
 error_def(ERR_CTRLC);
 error_def(ERR_CTRLY);
+error_def(ERR_TERMHANGUP);
 error_def(ERR_JOBINTRRQST);
 
 void outofband_action(boolean_t lnfetch_or_start)
@@ -37,8 +39,8 @@ void outofband_action(boolean_t lnfetch_or_start)
 			iott_flush(io_std_device.in);
 		if (lnfetch_or_start)
 		{
-			restart_pc =  frame_pointer->mpc;
-			restart_ctxt = frame_pointer->ctxt;
+			frame_pointer->restart_pc = frame_pointer->mpc;
+			frame_pointer->restart_ctxt = frame_pointer->ctxt;
 		}
 		switch(outofband)
 		{
@@ -51,6 +53,9 @@ void outofband_action(boolean_t lnfetch_or_start)
 			case (ctrap):
 				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(3) ERR_CTRAP, 1, ctrap_action_is);
 				break;
+			case (sighup):
+				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_TERMHANGUP);
+				break;
 			case (tptimeout):
 				/*
 				 * Currently following is nothing but an rts_error.
