@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2010 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2019 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -19,7 +20,7 @@
 #include "copy.h"
 #include "min_max.h"
 
-#ifdef UNICODE_SUPPORTED
+#ifdef UTF8_SUPPORTED
 #include "gtm_utf8.h"	/* needed for UTF8_MBNEXT macro */
 #endif
 
@@ -29,12 +30,66 @@ GBLREF	boolean_t	gtm_utf8_mode;
 LITREF	char		ctypetab[NUM_CHARS];
 LITREF	uint4		typemask[PATENTS];
 
+error_def(ERR_COMMAORRPAREXP);
+error_def(ERR_PATCLASS);
+error_def(ERR_PATCODE);
+error_def(ERR_PATLIT);
+error_def(ERR_PATMAXLEN);
+error_def(ERR_PATUPPERLIM);
+
 typedef struct
 {
 	unsigned char	*next;
 	ptstr		altpat;
 } alternation;
 
+#define	TERMINATE_DFA(PATMASKPTR, OUTCHAR, DFA_FIXED_LEN, LV_PTR, LEAF_NUM, EXP_PTR, FSTCHAR,			\
+			TOPCHAR, MIN, MAX, SIZE, TOTAL_MIN, TOTAL_MAX, COUNT, LASTPATPTR, LAST_INFINITE,	\
+			MIN_DFA, ALTMIN, ALTMAX, INSTR, INCHAR, DFA)						\
+{														\
+	gtm_uint64_t		bound;										\
+	int			cursize, i;									\
+														\
+	PATMASKPTR = OUTCHAR;											\
+	/* If this is a fixed pattern, generate code to invoke do_patfixed as that takes up less space in	\
+	 * the object code and has same runtime performance as the DFA. Else go through DFA.			\
+	 */													\
+	if (!DFA_FIXED_LEN)											\
+	{													\
+		assert(OUTCHAR <= TOPCHAR);									\
+		cursize = dfa_calc(LV_PTR, LEAF_NUM, EXP_PTR, &FSTCHAR, &OUTCHAR);				\
+	} else													\
+		cursize = -1;											\
+	if (cursize >= 0)											\
+	{													\
+		assert(OUTCHAR <= TOPCHAR);									\
+		MIN[COUNT] = MIN_DFA;										\
+		MAX[COUNT] = PAT_MAX_REPEAT;									\
+		SIZE[COUNT] = cursize;										\
+		TOTAL_MIN += BOUND_MULTIPLY(MIN[COUNT], SIZE[COUNT], bound);					\
+		if (TOTAL_MIN > PAT_MAX_REPEAT)									\
+			TOTAL_MIN = PAT_MAX_REPEAT;								\
+		TOTAL_MAX += BOUND_MULTIPLY(MAX[COUNT], SIZE[COUNT], bound);					\
+		if (TOTAL_MAX > PAT_MAX_REPEAT)									\
+			TOTAL_MAX = PAT_MAX_REPEAT;								\
+		LASTPATPTR = PATMASKPTR;									\
+		LAST_INFINITE = TRUE;										\
+		COUNT++;											\
+	} else													\
+	{													\
+		OUTCHAR = PATMASKPTR;										\
+		if (!pat_unwind(&COUNT, LV_PTR, LEAF_NUM, &TOTAL_MIN, &TOTAL_MAX,				\
+			&MIN[0], &MAX[0], &SIZE[0], ALTMIN, ALTMAX,						\
+			&LAST_INFINITE, &FSTCHAR, &OUTCHAR, &LASTPATPTR))					\
+		{												\
+			INSTR->addr = (char *)INCHAR;								\
+			return ERR_PATMAXLEN;									\
+		}												\
+		assert(OUTCHAR <= TOPCHAR);									\
+	}													\
+	DFA = FALSE;												\
+}
+
 /*  This procedure is part of the MUMPS compiler.  The function of this procedure is to parse a pattern specification
  *  and compile it into a data structure that will be used by the run-time engine to actually attempt to match the pattern.
  *
@@ -80,7 +135,8 @@ typedef struct
 int patstr(mstr *instr, ptstr *obj, unsigned char **relay)
 {
 	pat_strlit		strlit;
-	boolean_t		infinite, split_atom, done, dfa, fixed_len, prev_fixed_len;
+	boolean_t		dfa, done, infinite;
+	boolean_t		dfa_fixed_len, fixed_len, prev_fixed_len, split_atom, start_dfa;
 	int4			lower_bound, upper_bound, alloclen;
 	gtm_uint64_t		bound;
 	unsigned char		curchar, symbol, *inchar, *in_top, *buffptr;
@@ -111,7 +167,7 @@ int patstr(mstr *instr, ptstr *obj, unsigned char **relay)
 	boolean_t		last_infinite;
 	boolean_t		done_free;
 	unsigned char		*let_go;
-	uint4			*fstchar, *outchar, *lastpatptr;
+	uint4			*fstchar, *lastpatptr, *outchar, *topchar;
 	int			any_alt = FALSE;
 	int			altcount, altsimplify;
 	int			low_in, high_in, size_in, jump;
@@ -119,11 +175,6 @@ int patstr(mstr *instr, ptstr *obj, unsigned char **relay)
 						 * byte in inchar (to be stored in curchar). Therefore from this point onwards,
 						 * "curchar" should never be used in this function. This is also asserted below.
 						 */
-	error_def(ERR_PATCODE);
-	error_def(ERR_PATUPPERLIM);
-	error_def(ERR_PATCLASS);
-	error_def(ERR_PATLIT);
-	error_def(ERR_PATMAXLEN);
 
 	if (0 == instr->len)		/* empty pattern string. Cant do much */
 	{
@@ -140,6 +191,7 @@ int patstr(mstr *instr, ptstr *obj, unsigned char **relay)
 	for (allmask = 0, chidx = 'A'; chidx <= 'X'; chidx++)
 		allmask |= mapbit[chidx - 'A'];
 	outchar = &obj->buff[PAT_MASK_BEGIN_OFFSET]; /* Note: offset is actually PAT_MASK_BEGIN_OFFSET * SIZEOF (uint4) bytes */
+	topchar = &obj->buff[MAX_PATOBJ_LENGTH];
 	last_leaf_mask = *outchar = 0;
 	patmaskptr = lastpatptr = outchar;
 	infinite = last_infinite = FALSE;
@@ -160,7 +212,11 @@ int patstr(mstr *instr, ptstr *obj, unsigned char **relay)
 		prev_fixed_len = fixed_len;
 		if ((NULL != relay) && !saw_delimiter)
 		{
-			assert(!topseen);
+			if (topseen)
+			{
+				instr->addr = (char *)(in_top + 1);
+				return ERR_COMMAORRPAREXP;
+			}
 			if ((',' == curchar) || (')' == curchar))
 			{
 				*relay = (inchar - 1);
@@ -169,7 +225,11 @@ int patstr(mstr *instr, ptstr *obj, unsigned char **relay)
 		}
 		saw_delimiter = 0;
 		if (!altactive || altend)
-		{
+		{	/* Use DFA for patterns by default (fast at runtime even if time/space consuming at compile time).
+			 * If at the end of the parse we find that the input pattern is of fixed length we will decide
+			 * to compile it so (invoke do_patfixed instead of do_pattern which in turn invokes the DFA).
+			 */
+			start_dfa = TRUE;
 			instr->addr = (char*)inchar;
 			if (!topseen && (TK_PERIOD == ctypetab[curchar]))
 			{
@@ -196,31 +256,10 @@ int patstr(mstr *instr, ptstr *obj, unsigned char **relay)
 			} else
 			{
 				if (dfa)
-				{
-					patmaskptr = outchar;
-					cursize = dfa_calc(lv_ptr, leaf_num, exp_ptr, &fstchar, &outchar);
-					if (cursize >= 0)
-					{
-						min[count] = min_dfa;
-						max[count] = PAT_MAX_REPEAT;
-						size[count] = cursize;
-						total_min = MIN((total_min + (min[count] * size[count])), PAT_MAX_REPEAT);
-						total_max = MIN((total_max + (max[count] * size[count])), PAT_MAX_REPEAT);
-						lastpatptr = patmaskptr;
-						last_infinite = TRUE;
-						count++;
-					} else
-					{
-						outchar = patmaskptr;
-						if (!pat_unwind(&count, lv_ptr, leaf_num, &total_min, &total_max,
-							&min[0], &max[0], &size[0], altmin, altmax,
-							&last_infinite, &fstchar, &outchar, &lastpatptr))
-						{
-							instr->addr = (char *)inchar;
-							assert(FALSE);
-							return ERR_PATMAXLEN;
-						}
-					}
+				{	/* Note: Below macro can do a "return ERR_PATMAXLEN" if there is not enough space */
+					TERMINATE_DFA(patmaskptr, outchar, dfa_fixed_len, lv_ptr, leaf_num, exp_ptr, fstchar,
+						topchar, min, max, size, total_min, total_max, count, lastpatptr, last_infinite,
+						min_dfa, altmin, altmax, instr, inchar, dfa);
 				}
 				if (outchar == &obj->buff[PAT_MASK_BEGIN_OFFSET])
 				{
@@ -229,6 +268,12 @@ int patstr(mstr *instr, ptstr *obj, unsigned char **relay)
 				}
 				patmaskptr = &obj->buff[0];
 				*patmaskptr++ = fixed_len;
+				assert(outchar <= topchar);
+				if ((outchar + 3 + ((fixed_len ? 2 : 3) * count)) > topchar)
+				{
+					instr->addr = (char *)inchar;
+					return ERR_PATMAXLEN;
+				}
 				*patmaskptr = (uint4)(outchar - patmaskptr); /* unit is SIZEOF(uint4) */
 				*outchar++ = count;
 				*outchar++ = total_min;
@@ -240,6 +285,7 @@ int patstr(mstr *instr, ptstr *obj, unsigned char **relay)
 						*outchar++ = max[seqcnt];
 				for (seqcnt = 0; seqcnt < count; seqcnt++)
 					*outchar++ = size[seqcnt];
+				assert(outchar <= topchar);
 				obj->len = (int4)(outchar - &obj->buff[0]);
 				assert(!topseen || (inchar == in_top));
 				assert(inchar <= in_top);
@@ -268,6 +314,10 @@ int patstr(mstr *instr, ptstr *obj, unsigned char **relay)
 						 * 	fixed part from the indefinite part.
 						 */
 						split_atom = TRUE;
+						if ((count >= (MAX_PATTERN_ATOMS - 1)) ||
+								(atom_map >= (MAX_PATTERN_ATOMS -2)))
+			                         	return ERR_PATMAXLEN;
+
 					} else
 					{
 						infinite = TRUE;
@@ -301,10 +351,7 @@ int patstr(mstr *instr, ptstr *obj, unsigned char **relay)
 			}
 			instr->addr = (char *)inchar;
 			if (count >= MAX_PATTERN_ATOMS)
-			{
-				assert(FALSE);
 				return ERR_PATMAXLEN;
-			}
 		}
 		if (!altend)
 		{
@@ -341,7 +388,7 @@ int patstr(mstr *instr, ptstr *obj, unsigned char **relay)
 					}
 					if (!gtm_utf8_mode)
 						bytelen = 1;
-					UNICODE_ONLY(
+#					ifdef UTF8_SUPPORTED
 					else
 					{
 						if (!UTF8_VALID(inchar, in_top, bytelen))
@@ -351,14 +398,13 @@ int patstr(mstr *instr, ptstr *obj, unsigned char **relay)
 						}
 						assert(1 <= bytelen);
 					}
-					)
+#					endif
 					if (!IS_ASCII(curchar))
 						strlit.flags |= PATM_STRLIT_NONASCII;
 					strlit.bytelen += bytelen;
 					if (strlit.bytelen >= alloclen)
 					{
 						instr->addr = (char *)inchar;
-						assert(FALSE);
 						return ERR_PATMAXLEN;
 					}
 					do
@@ -380,17 +426,13 @@ int patstr(mstr *instr, ptstr *obj, unsigned char **relay)
 			} else if (!topseen && ('(' == curchar))
 			{	/* start of 'alternation' */
 				if (dfa)
-				{
-					if (!pat_unwind(&count, lv_ptr, leaf_num, &total_min, &total_max,
-						&min[0], &max[0], &size[0], altmin, altmax,
-						&last_infinite, &fstchar, &outchar, &lastpatptr))
-					{
-						instr->addr = (char *)inchar;
-						assert(FALSE);
-						return ERR_PATMAXLEN;
-					}
-					dfa = FALSE;
+				{	/* Finish ongoing DFA and start compiling alternation afresh at a nested depth */
+					/* Note: Below macro can do a "return ERR_PATMAXLEN" if there is not enough space */
+					TERMINATE_DFA(patmaskptr, outchar, dfa_fixed_len, lv_ptr, leaf_num, exp_ptr, fstchar,
+						topchar, min, max, size, total_min, total_max, count, lastpatptr, last_infinite,
+						min_dfa, altmin, altmax, instr, inchar, dfa);
 				}
+				start_dfa = FALSE;	/* reset start_dfa for entire duration of alternation */
 				if (inchar >= in_top)
 				{
 					assert(inchar == in_top);
@@ -503,6 +545,11 @@ int patstr(mstr *instr, ptstr *obj, unsigned char **relay)
 				} while (TRUE);
 				if (0 == pattern_mask)
 				{
+					if (any_alt)
+					{
+						instr->addr = alttail.addr + 1;
+						return ERR_PATCODE;
+					}
 					instr->addr = topseen ? (char *)inchar + 1 : (char *)inchar;
 					return ERR_PATCLASS;
 				}
@@ -522,30 +569,37 @@ int patstr(mstr *instr, ptstr *obj, unsigned char **relay)
 			 * are no character cells to hold the mask and flag bits that the DFA code needs. Also strings with
 			 * non-ASCII UTF-8 byte sequences are currently not processed through the DFA logic.
 			 */
-			if (infinite && !any_alt && !dfa
+			assert(outchar <= topchar);
+			if (start_dfa && !any_alt && !dfa
 				&& (!(pattern_mask & PATM_STRLIT) || (strlit.charlen && !(strlit.flags & PATM_STRLIT_NONASCII)))
-				&& ((outchar - &obj->buff[0]) <= (MAX_PATTERN_LENGTH / 2)))
+					&& ((outchar - &obj->buff[0]) <= (MAX_PATTERN_LENGTH / 2)))
 			{
 				dfa = TRUE;
 				last_leaf_mask = 0;
 				leaf_num = 0;
 				sym_num = 0;
 				min_dfa = 0;
+				/* Maintain this just like "fixed_len" but starting from current atom
+				 * whereas "fixed_len" starts from beginning of input pattern string.
+				 */
+				dfa_fixed_len = !infinite;
 				atom_map = count;
 				memset(expand.num_e, 0, SIZEOF(expand.num_e));
 			}
+			start_dfa = FALSE;
 			if (!dfa)
 			{
-				if (count >= MAX_PATTERN_ATOMS ||
-					!add_atom(&count, pattern_mask, &strlit, infinite,
+				assert(outchar <= topchar);
+				if ((MAX_PATTERN_ATOMS <= count)
+					|| !add_atom(&count, pattern_mask, &strlit, infinite,
 						&min[count], &max[count], &size[count],
 						&total_min, &total_max, lower_bound, upper_bound, altmin, altmax,
 						&last_infinite, &fstchar, &outchar, &lastpatptr))
 				{
 					instr->addr = (char *)inchar;
-					assert(FALSE);
 					return ERR_PATMAXLEN;
 				}
+				assert(outchar <= topchar);
 				if (pattern_mask & PATM_ALT)
 				{	/* If the alternation contains only one alternative (altcount == 1) AND
 					 * that alternative contains only one pattern atom, AND that atom is not an
@@ -582,7 +636,8 @@ int patstr(mstr *instr, ptstr *obj, unsigned char **relay)
 					 *    			20 = 5 + 5 + 5 + 5
 					 */
 					altsimplify = ((1 == altcount) && cur_alt) ? TRUE : FALSE;
-					if (altsimplify && (cur_alt->altpat.buff[PAT_MASK_BEGIN_OFFSET] & (PATM_ALT | PATM_DFA)))
+					if (altsimplify && ((cur_alt->altpat.buff[PAT_MASK_BEGIN_OFFSET] & PATM_ALT)
+								|| (PATM_DFA == cur_alt->altpat.buff[PAT_MASK_BEGIN_OFFSET])))
 						altsimplify = FALSE;
 					if (altsimplify)
 					{
@@ -611,14 +666,18 @@ int patstr(mstr *instr, ptstr *obj, unsigned char **relay)
 						lower_bound = min[count - 1];
 						if (!cur_alt->altpat.buff[0])
 							fixed_len = FALSE;
+						max[count - 1] = BOUND_MULTIPLY(high_in, upper_bound, bound);
 						if (!fixed_len)
-						{
-							max[count - 1] = BOUND_MULTIPLY(high_in, upper_bound, bound);
 							upper_bound = max[count - 1];
-						}
 						outchar--;
+						if ((outchar + (jump - PAT_MASK_BEGIN_OFFSET + 1)) > topchar)
+						{
+							instr->addr = (char *)inchar;
+							return ERR_PATMAXLEN;
+						}
 						for (seq = PAT_MASK_BEGIN_OFFSET; seq <= jump; seq++)
 							*outchar++ = cur_alt->altpat.buff[seq];
+						assert(outchar <= topchar);
 						pattern_mask = cur_alt->altpat.buff[PAT_MASK_BEGIN_OFFSET];
 						if (pattern_mask & PATM_STRLIT)
 						{
@@ -632,16 +691,28 @@ int patstr(mstr *instr, ptstr *obj, unsigned char **relay)
 					} else
 					{
 						fixed_len = FALSE;
+						if ((outchar + 3) > topchar)
+						{
+							instr->addr = (char *)inchar;
+							return ERR_PATMAXLEN;
+						}
 						*outchar++ = lower_bound;
 						*outchar++ = upper_bound;
 						for (cur_alt = &init_alt; cur_alt; )
 						{
+							assert(outchar < topchar);
+							if ((outchar + 2 + cur_alt->altpat.len) > topchar)
+							{
+								instr->addr = (char *)inchar;
+								return ERR_PATMAXLEN;
+							}
 							*outchar++ = cur_alt->altpat.len;
 							for (seq = 0; seq < cur_alt->altpat.len; seq++)
 								*outchar++ = cur_alt->altpat.buff[seq];
 							cur_alt = (alternation *)cur_alt->next;
 						}
 						*outchar++ = 0;
+						assert(outchar <= topchar);
 						done_free = TRUE;
 					}
 				}
@@ -654,193 +725,149 @@ int patstr(mstr *instr, ptstr *obj, unsigned char **relay)
 					charpos *= strlit.bytelen;
 					leafcnt = MAX(charpos, leafcnt);
 				}
-				if ((lower_bound > MAX_DFA_REP)
-					|| ((pattern_mask & PATM_STRLIT) && (strlit.flags & PATM_STRLIT_NONASCII))
-					|| (!infinite && lower_bound != upper_bound)
+				if (((pattern_mask & PATM_STRLIT) && (strlit.flags & PATM_STRLIT_NONASCII))
+					|| (!infinite && ((lower_bound != upper_bound) || !lower_bound))
 					|| ((leaf_num + leafcnt) >= (MAX_SYM - 1))
 					|| (charpos > MAX_DFA_STRLEN))
 				{
-					patmaskptr = outchar;
-					cursize = dfa_calc(lv_ptr, leaf_num, exp_ptr, &fstchar, &outchar);
-					if (cursize >= 0)
-					{
-						min[count] = min_dfa;
-						max[count] = PAT_MAX_REPEAT;
-						size[count] = cursize;
-						total_min += BOUND_MULTIPLY(min[count], size[count], bound);
-						if (total_min > PAT_MAX_REPEAT)
-							total_min = PAT_MAX_REPEAT;
-						total_max += BOUND_MULTIPLY(max[count], size[count], bound);
-						if (total_max > PAT_MAX_REPEAT)
-							total_max = PAT_MAX_REPEAT;
-						lastpatptr = patmaskptr;
-						last_infinite = TRUE;
-						count++;
-					} else
-					{
-						outchar = patmaskptr;
-						if (!pat_unwind(&count, lv_ptr, leaf_num, &total_min, &total_max,
-							&min[0], &max[0], &size[0], altmin, altmax,
-							&last_infinite, &fstchar, &outchar, &lastpatptr))
-						{
-							instr->addr = (char *)inchar;
-							assert(FALSE);
-							return ERR_PATMAXLEN;
-						}
-					}
-					dfa = FALSE;
+					/* Note: Below macro can do a "return ERR_PATMAXLEN" if there is not enough space */
+					TERMINATE_DFA(patmaskptr, outchar, dfa_fixed_len, lv_ptr, leaf_num, exp_ptr, fstchar,
+						topchar, min, max, size, total_min, total_max, count, lastpatptr, last_infinite,
+						min_dfa, altmin, altmax, instr, inchar, dfa);
+					start_dfa = ((MAX_DFA_STRLEN >= charpos) /* Try another DFA for remainder if possible */
+							&& (infinite || (lower_bound == upper_bound) && lower_bound) && leaf_num);
 					done = FALSE;
 					continue;
+				}
+				curr_min_dfa = min_dfa;
+				curr_leaf_num = leaf_num;
+				if (pattern_mask & PATM_STRLIT)
+				{
+					memset(&exp_temp[0], 0, SIZEOF(exp_temp));
+					if (atom_map >= MAX_PATTERN_ATOMS)
+						return ERR_PATMAXLEN;
+					min[atom_map] = lower_bound;
+					max[atom_map] = upper_bound;
+					size[atom_map] = strlit.bytelen;
+					atom_map++;
+					min_dfa += lower_bound * strlit.bytelen;
+					cursize = MAX(lower_bound, 1);
+					for (seqcnt = 0; seqcnt < cursize; seqcnt++)
+					{
+						for (charpos = 0; charpos < strlit.bytelen; charpos++)
+						{
+							symbol = strlit.buff[charpos];
+							/* It is ok to use typemask[] below because we are guaranteed
+							 * that "symbol" is a 1-byte valid ASCII character. Assert that.
+							 */
+							assert(!(strlit.flags & PATM_STRLIT_NONASCII) && IS_ASCII(symbol));
+							bitpos = patmaskseq(typemask[symbol]);
+							if (expand.num_e[bitpos] + exp_temp[bitpos] == 0)
+								exp_temp[bitpos]++;
+							for (leafcnt = 1;
+								(leafcnt < (expand.num_e[bitpos] + exp_temp[bitpos]))
+									&& (expand.meta_c[bitpos][leafcnt] != symbol);
+								leafcnt++)
+								;
+							if (leafcnt == expand.num_e[bitpos] + exp_temp[bitpos])
+								exp_temp[bitpos]++;
+							expand.meta_c[bitpos][leafcnt] = symbol;
+							if (!infinite)
+							{
+								leaves.letter[leaf_num][0] = symbol;
+								leaves.letter[leaf_num][1] = -1;
+								leaves.nullable[leaf_num++] = FALSE;
+							} else
+								leaves.letter[leaf_num][charpos] = symbol;
+						}
+						if (infinite)
+						{
+							leaves.letter[leaf_num][charpos] = -1;
+							leaves.nullable[leaf_num++] = infinite;
+						}
+					}
+					last_leaf_mask = PATM_STRLIT;
+					last_infinite = infinite;
+					sym_num = 0;
+					for (leafcnt = 0; leafcnt < leaf_num; leafcnt++)
+					{
+						for (charpos = 0; leaves.letter[leafcnt][charpos] >= 0; charpos++)
+						{
+							if (!(leaves.letter[leafcnt][charpos] & DFABIT))
+								sym_num++;
+							else
+							{
+								bitpos = patmaskseq(leaves.letter[leafcnt][charpos]);
+								sym_num += expand.num_e[bitpos] + exp_temp[bitpos];
+							}
+						}
+					}
 				} else
 				{
-					curr_min_dfa = min_dfa;
-					curr_leaf_num = leaf_num;
-					if (pattern_mask & PATM_STRLIT)
+					if (!(last_leaf_mask & PATM_STRLIT) && infinite && last_infinite)
 					{
-						memset(&exp_temp[0], 0, SIZEOF(exp_temp));
-						min[atom_map] = lower_bound;
-						max[atom_map] = upper_bound;
-						size[atom_map] = strlit.bytelen;
-						atom_map++;
-						min_dfa += lower_bound * strlit.bytelen;
-						cursize = MAX(lower_bound, 1);
-						for (seqcnt = 0; seqcnt < cursize; seqcnt++)
+						y_max = MAX(pattern_mask, last_leaf_mask);
+						if ((last_leaf_mask & pattern_mask) &&
+						   ((last_leaf_mask | pattern_mask) == y_max))
 						{
-							for (charpos = 0; charpos < strlit.bytelen; charpos++)
-							{
-								symbol = strlit.buff[charpos];
-								/* It is ok to use typemask[] below because we are guaranteed
-								 * that "symbol" is a 1-byte valid ASCII character. Assert that.
-								 */
-								assert(!(strlit.flags & PATM_STRLIT_NONASCII) && IS_ASCII(symbol));
-								bitpos = patmaskseq(typemask[symbol]);
-								if (expand.num_e[bitpos] + exp_temp[bitpos] == 0)
-									exp_temp[bitpos]++;
-								for (leafcnt = 1;
-									leafcnt < expand.num_e[bitpos] + exp_temp[bitpos] &&
-										expand.meta_c[bitpos][leafcnt] != symbol;
-									leafcnt++)
-									;
-								if (leafcnt == expand.num_e[bitpos] + exp_temp[bitpos])
-									exp_temp[bitpos]++;
-								expand.meta_c[bitpos][leafcnt] = symbol;
-								if (!infinite)
-								{
-									leaves.letter[leaf_num][0] = symbol;
-									leaves.letter[leaf_num][1] = -1;
-									leaves.nullable[leaf_num++] = FALSE;
-								} else
-									leaves.letter[leaf_num][charpos] = symbol;
-							}
-							if (infinite)
-							{
-								leaves.letter[leaf_num][charpos] = -1;
-								leaves.nullable[leaf_num++] = infinite;
-							}
+							if (last_leaf_mask == y_max)
+								continue;
+							leaf_num--;
+							atom_map--;
 						}
-						last_leaf_mask = PATM_STRLIT;
-						last_infinite = infinite;
-						sym_num = 0;
-						for (leafcnt = 0; leafcnt < leaf_num; leafcnt++)
-						{
-							for (charpos = 0; leaves.letter[leafcnt][charpos] >= 0; charpos++)
-							{
-								if (!(leaves.letter[leafcnt][charpos] & DFABIT))
-									sym_num++;
-								else
-								{
-									bitpos = patmaskseq(leaves.letter[leafcnt][charpos]);
-									sym_num += expand.num_e[bitpos] + exp_temp[bitpos];
-								}
-							}
-						}
-					} else
+					}
+					if (atom_map >= MAX_PATTERN_ATOMS)
+						return ERR_PATMAXLEN;
+					min[atom_map] = lower_bound;
+					max[atom_map] = upper_bound;
+					size[atom_map] = 1;
+					atom_map++;
+					min_dfa += lower_bound;
+					charpos = MAX(lower_bound, 1);
+					last_infinite = infinite;
+					last_leaf_mask = pattern_mask;
+					for (seqcnt = 0; seqcnt < charpos; seqcnt++)
 					{
-						if (!(last_leaf_mask & PATM_STRLIT) && infinite && last_infinite)
+						bitpos = 0;
+						leaves.nullable[leaf_num] = infinite;
+						/* Check all PAT_MAX_BITS bits if there are flags for internationalization,
+						 * otherwise, check only the original PAT_BASIC_CLASSES bits
+						 * (C, L, N, P, U, 0, 1) where
+						 *	0 = PATM_UTF8_ALPHABET, 1 = PATM_UTF8_NONBASIC
+						 */
+						if (PATM_E != pattern_mask)
 						{
-							y_max = MAX(pattern_mask, last_leaf_mask);
-							if ((last_leaf_mask & pattern_mask) &&
-							   ((last_leaf_mask | pattern_mask) == y_max))
+							chidx = (pattern_mask & PATM_I18NFLAGS)
+								? PAT_MAX_BITS : PAT_BASIC_CLASSES;
+						} else
+							chidx = PAT_BASIC_CLASSES;
+						for (bit = 0; bit < chidx; bit++)
+						{
+							mbit = 1 << bit;
+							if ((allmask & mbit) && (pattern_mask & mbit))
 							{
-								if (last_leaf_mask == y_max)
-									continue;
-								leaf_num--;
-								atom_map--;
+								seq = patmaskseq((uint4)mbit);
+								if (expand.num_e[seq] == 0)
+									expand.num_e[seq]++;
+								sym_num += expand.num_e[seq];
+								assert(MAX_DFA_STRLEN >= bitpos);
+								leaves.letter[leaf_num][bitpos++] = DFABIT | mbit;
 							}
 						}
-						min[atom_map] = lower_bound;
-						max[atom_map] = upper_bound;
-						size[atom_map] = 1;
-						atom_map++;
-						min_dfa += lower_bound;
-						charpos = MAX(lower_bound, 1);
-						last_infinite = infinite;
-						last_leaf_mask = pattern_mask;
-						for (seqcnt = 0; seqcnt < charpos; seqcnt++)
-						{
-							bitpos = 0;
-							leaves.nullable[leaf_num] = infinite;
-							/* Check all PAT_MAX_BITS bits if there are flags for internationalization,
-							 * otherwise, check only the original PAT_BASIC_CLASSES bits
-							 * (C, L, N, P, U, 0, 1) where
-							 *	0 = PATM_UTF8_ALPHABET, 1 = PATM_UTF8_NONBASIC
-							 */
-							if (PATM_E != pattern_mask)
-							{
-								chidx = (pattern_mask & PATM_I18NFLAGS)
-									? PAT_MAX_BITS : PAT_BASIC_CLASSES;
-							} else
-								chidx = PAT_BASIC_CLASSES;
-							for (bit = 0; bit < chidx; bit++)
-							{
-								mbit = 1 << bit;
-								if ((allmask & mbit) && (pattern_mask & mbit))
-								{
-									seq = patmaskseq((uint4)mbit);
-									if (expand.num_e[seq] == 0)
-										expand.num_e[seq]++;
-									sym_num += expand.num_e[seq];
-									assert(MAX_DFA_STRLEN >= bitpos);
-									leaves.letter[leaf_num][bitpos++] = DFABIT | mbit;
-								}
-							}
-							assert(MAX_DFA_STRLEN >= bitpos);
-							leaves.letter[leaf_num][bitpos] = -1;
-							leaf_num++;
-						}
+						assert(MAX_DFA_STRLEN >= bitpos);
+						leaves.letter[leaf_num][bitpos] = -1;
+						leaf_num++;
 					}
 				}
 				if (sym_num >= MAX_SYM - 1)
 				{
-					patmaskptr = outchar;
-					cursize = dfa_calc(lv_ptr, curr_leaf_num, exp_ptr, &fstchar, &outchar);
-					if (cursize >= 0)
-					{
-						min[count] = curr_min_dfa;
-						max[count] = PAT_MAX_REPEAT;
-						size[count] = cursize;
-						total_min += BOUND_MULTIPLY(min[count], size[count], bound);
-						if (total_min > PAT_MAX_REPEAT)
-							total_min = PAT_MAX_REPEAT;
-						total_max += BOUND_MULTIPLY(max[count], size[count], bound);
-						if (total_max > PAT_MAX_REPEAT)
-							total_max = PAT_MAX_REPEAT;
-						lastpatptr = patmaskptr;
-						last_infinite = TRUE;
-						count++;
-					} else
-					{
-						outchar = patmaskptr;
-						if (!pat_unwind(&count, lv_ptr, curr_leaf_num, &total_min, &total_max,
-							&min[0], &max[0], &size[0], altmin, altmax,
-							&last_infinite, &fstchar, &outchar, &lastpatptr))
-						{
-							instr->addr = (char *)inchar;
-							assert(FALSE);
-							return ERR_PATMAXLEN;
-						}
-					}
-					dfa = FALSE;
+					leaf_num = curr_leaf_num;
+					min_dfa = curr_min_dfa;
+					/* Note: Below macro can do a "return ERR_PATMAXLEN" if there is not enough space */
+					TERMINATE_DFA(patmaskptr, outchar, dfa_fixed_len, lv_ptr, leaf_num, exp_ptr, fstchar,
+						topchar, min, max, size, total_min, total_max, count, lastpatptr, last_infinite,
+						min_dfa, altmin, altmax, instr, inchar, dfa);
+					start_dfa = curr_leaf_num; /* Try another DFA for remainder if possible */
 					done = FALSE;
 					continue;
 				} else
@@ -857,8 +884,12 @@ int patstr(mstr *instr, ptstr *obj, unsigned char **relay)
 				infinite = TRUE;
 				split_atom = FALSE;
 				done = FALSE;
+				if (dfa)
+					dfa_fixed_len = FALSE;
 			}
 		}
+		if (dfa && infinite)
+			dfa_fixed_len = FALSE;
 		for (cur_alt = &init_alt; cur_alt; )
 		{
 			let_go = (cur_alt != (alternation *)&init_alt) ? (unsigned char *)cur_alt : NULL;
