@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -89,6 +89,8 @@
  */
 
 GBLDEF	int			repl_max_send_buffsize, repl_max_recv_buffsize;
+GBLREF  boolean_t               is_rcvr_server;
+GBLREF  boolean_t               is_src_server;
 #ifdef GTM_TLS
 GBLREF	gtm_tls_ctx_t		*tls_ctx;
 GBLREF	char			dl_err[MAX_ERRSTR_LEN];
@@ -200,6 +202,7 @@ int repl_send(int sock_fd, unsigned char *buff, int *send_len, int timeout GTMTL
 {
 	int		send_size, status, io_ready, save_errno, EMSGSIZE_cnt = 0, EWOULDBLOCK_cnt = 0;
   	ssize_t		bytes_sent;
+	char const	*errptr;
 
 	if (!repl_send_trace_buff)
 		repl_send_trace_buff = malloc(REPL_SEND_TRACE_BUFF_SIZE);
@@ -243,6 +246,10 @@ int repl_send(int sock_fd, unsigned char *buff, int *send_len, int timeout GTMTL
 		{
 			bytes_sent = GTMTLS_ONLY(repl_tls.enabled ? gtm_tls_send(repl_tls.sock, (char *)buff, send_size)
 							: ) send(sock_fd, (char *)buff, send_size, 0);	/* BYPASSOK(send) */
+			if (WBTEST_ENABLED(WBTEST_REPLCOMM_SEND_SRC) && is_src_server)
+			{
+				GTM_WHITE_BOX_TEST(WBTEST_REPLCOMM_SEND_SRC, bytes_sent, -1);
+			}
 			if (0 <= bytes_sent)
 			{
 				assert(0 < bytes_sent);
@@ -264,13 +271,18 @@ int repl_send(int sock_fd, unsigned char *buff, int *send_len, int timeout GTMTL
 			if (-1 == save_errno)
 			{	/* This indicates an error from TLS/SSL layer and not from a system call. */
 				assert(repl_tls.enabled);
-				assert(FALSE);
+				errptr = gtm_tls_get_error();
 				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_TLSIOERROR, 2, LEN_AND_LIT("send"), ERR_TEXT, 2,
-						LEN_AND_STR(gtm_tls_get_error()));
+						LEN_AND_STR(errptr));
 			}
 #			else
 			save_errno = ERRNO;
 #			endif
+			if (WBTEST_ENABLED(WBTEST_REPLCOMM_SEND_SRC) && is_src_server)
+			{
+				repl_log(stderr, TRUE, TRUE,"Changing save errno\n");
+				save_errno = 70;
+			}
 			assert((EMSGSIZE != save_errno) && (EWOULDBLOCK != save_errno));
 			if (EINTR == save_errno)
 				continue;
@@ -300,6 +312,7 @@ int repl_send(int sock_fd, unsigned char *buff, int *send_len, int timeout GTMTL
 				break;
 		}
 		repl_errno = EREPL_SEND;
+		repl_log(stderr, TRUE, TRUE, "Returning err: %d\n",save_errno);
 		return save_errno;
 	} else if (!io_ready)
 		return SS_NORMAL;
@@ -312,6 +325,7 @@ int repl_recv(int sock_fd, unsigned char *buff, int *recv_len, int timeout GTMTL
 {
 	int		status, max_recv_len, io_ready, save_errno;
 	ssize_t		bytes_recvd;
+	char const	*errptr;
 
 	if (!repl_recv_trace_buff)
 		repl_recv_trace_buff = malloc(REPL_RECV_TRACE_BUFF_SIZE);
@@ -345,8 +359,25 @@ int repl_recv(int sock_fd, unsigned char *buff, int *recv_len, int timeout GTMTL
 	{
 		while (TRUE)
 		{
+			if ((WBTEST_ENABLED(WBTEST_REPL_TLS_RECONN)) && is_rcvr_server)
+			{
+				GTM_WHITE_BOX_TEST(WBTEST_REPL_TLS_RECONN, bytes_recvd, -1);
+				if (-1 == bytes_recvd)
+					gtm_wbox_input_test_case_count = 11; /* Do not go to white box again*/
+			}
 			bytes_recvd = GTMTLS_ONLY(repl_tls.enabled ? gtm_tls_recv(repl_tls.sock, (char *)buff, max_recv_len)
 							 : ) recv(sock_fd, (char *)buff, max_recv_len, 0);	/* BYPASSOK */
+			if (((WBTEST_ENABLED(WBTEST_FETCHCOMM_ERR)) || (WBTEST_ENABLED(WBTEST_FETCHCOMM_HISTINFO)))
+				 && !(is_rcvr_server || is_src_server))
+			{	/* Induce the test only in mupip rollback process*/
+				repl_log(stderr, TRUE, TRUE, "Inducing error\n");
+				repl_log(stdout, TRUE, TRUE,"gtm_wbox_input_test_case_count is : %d\n",
+								gtm_wbox_input_test_case_count);
+				GTM_WHITE_BOX_TEST(WBTEST_FETCHCOMM_ERR, bytes_recvd, -1);
+				GTM_WHITE_BOX_TEST(WBTEST_FETCHCOMM_HISTINFO, bytes_recvd, -1);
+				LONG_SLEEP(5);
+				repl_log(stdout, TRUE, TRUE, "bytes_recvd: %d\n",bytes_recvd);
+			}
 			if (0 < bytes_recvd)
 			{
 				*recv_len = (int)bytes_recvd;
@@ -370,19 +401,23 @@ int repl_recv(int sock_fd, unsigned char *buff, int *recv_len, int timeout GTMTL
 			}
 			/* handle error */
 			save_errno = repl_tls.enabled ? gtm_tls_errno() : ERRNO;
+			if ((WBTEST_ENABLED(WBTEST_REPL_TLS_RECONN)) &&
+				(ECONNRESET == save_errno))
+				repl_log(stderr, TRUE, TRUE, "WBTEST_REPL_TLS_RECONN induced\n");
 			if (-1 == save_errno)
 			{
 				assert(repl_tls.enabled);
-				assert(FALSE);
+				errptr = gtm_tls_get_error();
 				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_TLSIOERROR, 2, LEN_AND_LIT("recv"), ERR_TEXT, 2,
-						LEN_AND_STR(gtm_tls_get_error()));
+						LEN_AND_STR(errptr));
 			}
 #			else
 			save_errno = ERRNO;
 #			endif
 			if (0 == bytes_recvd)
 				save_errno = ECONNRESET;
-			if (EINTR == save_errno)
+			if ((EINTR == save_errno) && !((WBTEST_ENABLED(WBTEST_FETCHCOMM_ERR)) ||
+					(WBTEST_ENABLED(WBTEST_FETCHCOMM_HISTINFO))))
 				continue;
 			else if (ETIMEDOUT == save_errno)
 			{
@@ -401,8 +436,25 @@ int repl_recv(int sock_fd, unsigned char *buff, int *recv_len, int timeout GTMTL
 			break;
 		}
 		repl_errno = EREPL_RECV;
+		if ((WBTEST_ENABLED(WBTEST_REPLCOMM_ERR) && is_rcvr_server) ||
+                 (WBTEST_ENABLED(WBTEST_REPLCOMM_ERR_SRC) && is_src_server))
+		{
+			repl_log(stderr, TRUE, TRUE, "Changing save_errno\n");
+			save_errno = 113;
+		}
+		if (((WBTEST_ENABLED(WBTEST_FETCHCOMM_ERR)) || (WBTEST_ENABLED(WBTEST_FETCHCOMM_HISTINFO)))
+								&& !(is_rcvr_server || is_src_server))
+		{
+			repl_log(stdout, TRUE, TRUE, "Changing save_errno\n");
+			GTM_WHITE_BOX_TEST(WBTEST_FETCHCOMM_ERR, save_errno, ECONNRESET);
+			if (WBTEST_ENABLED(WBTEST_FETCHCOMM_HISTINFO) && (-1 == bytes_recvd))
+			{
+				save_errno = ECONNRESET;
+				gtm_wbox_input_test_case_count = 9; /* Do not go to white box again*/
+			}
+		}
 		return save_errno;
-	} else if (!io_ready)
+	} else if (!io_ready)	/* Select call timedout with no errors */
 		return SS_NORMAL;
 	save_errno = ERRNO;
 	repl_errno = EREPL_SELECT;
@@ -474,17 +526,18 @@ int set_recv_sock_buff_size(int sockfd, int buflen)
 	return set_sock_buff_size(sockfd, buflen, SO_RCVBUF);
 }
 
-void repl_log_conn_info(int sock_fd, FILE *log_fp)
+void repl_log_conn_info(int sock_fd, FILE *log_fp,boolean_t debug)
 {
 	struct sockaddr_storage	local, remote;
 	struct sockaddr		*local_sa_ptr, *remote_sa_ptr;
 	GTM_SOCKLEN_TYPE	len;
 	int			save_errno;
-	char			*errptr, local_ip[SA_MAXLEN], remote_ip[SA_MAXLEN];
+	char			*errptr, local_ip[SA_MAXLEN], remote_ip[SA_MAXLEN], *debpr;
 	char			port_buffer[NI_MAXSERV];
 	char			local_port_buffer[NI_MAXSERV], remote_port_buffer[NI_MAXSERV];
 	int			errcode;
 
+	debug ? (debpr = "Debug ") : (debpr = "");
 	len = SIZEOF(local);
 	local_sa_ptr = (struct sockaddr *)&local;
 	remote_sa_ptr = (struct sockaddr *)&remote;
@@ -524,8 +577,8 @@ void repl_log_conn_info(int sock_fd, FILE *log_fp)
 		strcpy(remote_port_buffer, "*UNKNOWN*");
 		strcpy(remote_ip, "*UNKNOWN*");
 	}
-	repl_log(log_fp, TRUE, TRUE, "Connection information:: Local: %s:%s Remote: %s:%s\n", local_ip, local_port_buffer,
-			remote_ip, remote_port_buffer);
+	repl_log(log_fp, TRUE, TRUE, "%sConnection information:: Local: %s:%s Remote: %s:%s\n", debpr,
+				local_ip, local_port_buffer, remote_ip, remote_port_buffer);
 	return;
 }
 
