@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -43,7 +43,6 @@
 #include "stp_parms.h"
 #include "stringpool.h"
 #include "buddy_list.h"		/* needed for tp.h */
-#include "hashtab_int4.h"	/* needed for tp.h */
 #include "tp.h"
 #include "tp_frame.h"
 #include "mlkdef.h"
@@ -82,7 +81,7 @@ GBLREF int			mlmax;
 GBLREF int 			mvar_index;
 GBLREF hash_table_objcode 	cache_table;
 GBLREF unsigned char		*msp, *stackbase, *stacktop, *stackwarn;
-GBLREF int			stp_array_size;
+GBLREF uint4			stp_array_size;
 GBLREF io_log_name		*io_root_log_name;
 GBLREF lvzwrite_datablk		*lvzwrite_block;
 GBLREF mliteral			literal_chain;
@@ -717,8 +716,8 @@ void stp_gcol(size_t space_asked)	/* BYPASSOK */
 #		ifdef GTM_TRIGGER
 		MVAL_STPG_ADD(&dollar_ztwormhole);
 #		endif
-		MVAL_STPG_ADD(TADR(last_fnquery_return_varname));
-		for (index = 0; index < TREF(last_fnquery_return_subcnt); index++);
+		MVAL_STPG_ADD(&(TREF(last_fnquery_return_varname)));
+		for (index = 0; index < TREF(last_fnquery_return_subcnt); index++)
 			MVAL_STPG_ADD(&TAREF1(last_fnquery_return_sub, index));
 		for (mvs = mv_chain; mvs < (mv_stent *)stackbase; mvs = (mv_stent *)((char *)mvs + mvs->mv_st_next))
 		{
@@ -788,7 +787,6 @@ void stp_gcol(size_t space_asked)	/* BYPASSOK */
 				case MVST_STCK:
 				case MVST_STCK_SP:
 				case MVST_PVAL:
-				case MVST_RSTRTPC:
 				case MVST_STORIG:
 				case MVST_ZINTCMD:
 					continue;
@@ -984,10 +982,12 @@ void stp_gcol(size_t space_asked)	/* BYPASSOK */
 			expansion_failed = FALSE;	/* will be set to TRUE by condition handler if can't get memory */
 			assert((stp_incr + stringpool.top - stringpool.base) >= (space_needed + blklen));
 			DBGSTPGCOL((stderr, "incr_factor=%i stp_incr=%i space_needed=%i\n", *incr_factor, stp_incr, space_needed));
-			if ((TREF(gtm_strpllimwarned)) /* previously warned */
-				&& (0 < TREF(gtm_strpllim)) /* watching a stp limit */
-				&& ((stp_incr + stringpool.top - stringpool.base) > TREF(gtm_strpllim))) /* expanding larger */
-					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_STPOFLOW);
+			if ((stringpool.strpllimwarned) /* previously warned */
+				&& ((stp_incr + stringpool.top - stringpool.base) > stringpool.strpllim)) /* expanding larger */
+			{
+				assert(0 < stringpool.strpllim);	/* must have been watching stp limit */
+				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_STPOFLOW);
+			}
 			expand_stp((ssize_t)(stp_incr + stringpool.top - stringpool.base));
 #			ifdef DEBUG
 			/* If expansion failed and stp_gcol_ch did an UNWIND and we were already in exit handling code,
@@ -1099,12 +1099,12 @@ void stp_gcol(size_t space_asked)	/* BYPASSOK */
 		lvmon_pull_values(2);				/* Pull values of monitored vars into index 2 */
 		lvmon_compare_value_slots(1, 2);		/* Make sure they are the same */
 	}
-#	endif	/* !STP_MOVE */
-	if ((0 < TREF(gtm_strpllim)) /* monitoring stp limit */
-		&& ((stringpool.top - stringpool.base) > TREF(gtm_strpllim))) /* past the stp limit */
+	if ((0 < stringpool.strpllim && !stringpool.strpllimwarned) /* monitoring stp limit and no prior warned */
+		&& ((stringpool.top - stringpool.base) > stringpool.strpllim)) /* past the stp limit */
 	{
-		TREF(gtm_strpllimwarned) =  TRUE;
+		stringpool.strpllimwarned =  TRUE;
 		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_STPCRIT);
 	}
+	#	endif	/* !STP_MOVE */
 	return;
 }
