@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -12,9 +12,11 @@
 
 typedef struct
 {
-	unsigned char *base, *free, *top, *lasttop, *invokestpgcollevel;
-	unsigned int gcols; /* some optimizations need to know if the stringpool garbage collected  */
-	unsigned char prvprt;
+	unsigned char	*base, *free, *top, *lasttop, *invokestpgcollevel;
+	unsigned int	gcols;		/* some optimizations need to know if the stringpool garbage collected  */
+	unsigned int	strpllim;	/* non-zero value is user specified guard on expansion */
+	boolean_t	strpllimwarned;	/* if limit is in place, has been exceeded, and not recovered from */
+	unsigned char	prvprt;		/* stores memory protections used in guarding stp space */
 } spdesc;
 
 void	stp_expand_array(void);
@@ -31,9 +33,9 @@ boolean_t	is_stp_space_available(ssize_t space_needed);
 #endif
 
 #ifdef DEBUG
-#define	IS_STP_SPACE_AVAILABLE(SPC)	is_stp_space_available((ssize_t)SPC)
+#define	IS_STP_SPACE_AVAILABLE(SPC)	is_stp_space_available((ssize_t)(SPC))
 #else
-#define	IS_STP_SPACE_AVAILABLE(SPC)	IS_STP_SPACE_AVAILABLE_PRO((ssize_t)SPC)
+#define	IS_STP_SPACE_AVAILABLE(SPC)	IS_STP_SPACE_AVAILABLE_PRO((ssize_t)(SPC))
 #endif
 
 GBLREF	spdesc		stringpool;
@@ -72,7 +74,7 @@ GBLREF	boolean_t	stringpool_unexpandable;
 #define	ADD_TO_STPARRAY(PTR, PTRARRAY, PTRARRAYCUR, PTRARRAYTOP, TYPE)					\
 {													\
 	GBLREF mstr	**stp_array;									\
-	GBLREF int	stp_array_size;									\
+	GBLREF uint4	stp_array_size;									\
 													\
 	if (NULL == PTRARRAY)										\
 	{												\
