@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2019 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -35,7 +35,6 @@
 #include "interlock.h"
 #include "jnl.h"
 #include "buddy_list.h"		/* needed for tp.h */
-#include "hashtab_int4.h"	/* needed for tp.h and cws_insert.h */
 #include "tp.h"
 #include "gdsbgtr.h"
 #include "repl_msg.h"
@@ -151,6 +150,7 @@ error_def(ERR_JNLFLUSH);
 error_def(ERR_JRTNULLFAIL);
 error_def(ERR_NOTREPLICATED);
 error_def(ERR_TEXT);
+error_def(ERR_TNTOOLARGE);
 
 #define BLOCK_FLUSHING(x) (csa->hdr->clustered && x->flushing && !CCP_SEGMENT_STATE(cs_addrs->nl,CCST_MASK_HAVE_DIRTY_BUFFERS))
 
@@ -217,7 +217,7 @@ trans_num t_end(srch_hist *hist1, srch_hist *hist2, trans_num ctn)
 	boolean_t		supplementary = FALSE;	/* this variable is initialized ONLY if "replication" is TRUE. */
 	seq_num			strm_seqno, next_strm_seqno;
 	sm_uc_ptr_t		blk_ptr, backup_blk_ptr;
-	int			blkid;
+	block_id		blkid;
 	boolean_t		is_mm;
 	boolean_t		read_before_image; /* TRUE if before-image journaling or online backup in progress
 						    * This is used to read before-images of blocks whose cs->mode is gds_t_create */
@@ -245,12 +245,12 @@ trans_num t_end(srch_hist *hist1, srch_hist *hist2, trans_num ctn)
 #	ifdef GTM_TRIGGER
 	uint4			cycle;
 #	endif
-	snapshot_context_ptr_t  lcl_ss_ctx;
-	th_index_ptr_t     	cti;
+	snapshot_context_ptr_t	lcl_ss_ctx;
+	th_index_ptr_t		cti;
 	jbuf_rsrv_struct_t	*jrs;
 	jrec_rsrv_elem_t	*first_jre, *jre, *jre_top;
-        int4			event_type, param_val;
-        void (*set_fn)(int4 param);
+	int4			event_type, param_val;
+	void (*set_fn)(int4 param);
 	DCL_THREADGBL_ACCESS;
 
 	SETUP_THREADGBL_ACCESS;
@@ -313,7 +313,7 @@ trans_num t_end(srch_hist *hist1, srch_hist *hist2, trans_num ctn)
 	assert((inctn_invalid_op == inctn_opcode) || mu_reorg_upgrd_dwngrd_in_prog || mu_reorg_encrypt_in_prog || update_trans);
 	assert(!need_kip_incr || update_trans || TREF(in_gvcst_redo_root_search));
 	cti = csa->ti;
-	if (cnl->wc_blocked || (is_mm && (csa->total_blks != cti->total_blks)))
+	if ((WC_BLOCK_RECOVER == cnl->wc_blocked) || (is_mm && (csa->total_blks != cti->total_blks)))
 	{	/* If blocked, or we have MM and file has been extended, force repair */
 		status = cdb_sc_helpedout;	/* force retry with special status so philanthropy isn't punished */
 		assert((CDB_STAGNATE > t_tries) || !is_mm || (csa->total_blks == cti->total_blks));
@@ -389,7 +389,7 @@ trans_num t_end(srch_hist *hist1, srch_hist *hist2, trans_num ctn)
 			 */
 			assert(!csa->now_crit);
 			DEBUG_ONLY(tmp_jnlpool = jnlpool;)
-			grab_crit(reg);
+			grab_crit(reg, WS_45);
 			rel_crit(reg);
 			assert(tmp_jnlpool == jnlpool);
 		}
@@ -401,7 +401,7 @@ trans_num t_end(srch_hist *hist1, srch_hist *hist2, trans_num ctn)
 			DEBUG_ONLY(tmp_jnlpool = jnlpool;)
 			was_crit = csa->now_crit;
 			if (!was_crit)
-				grab_crit(reg);
+				grab_crit(reg, WS_46);
 			status = cdb_sc_gvtrootmod2;
 			if (MISMATCH_ONLN_RLBK_CYCLES(csa, cnl))
 			{
@@ -434,6 +434,7 @@ trans_num t_end(srch_hist *hist1, srch_hist *hist2, trans_num ctn)
 		assert(tmp_jnlpool == jnlpool);
 		CWS_RESET;
 		assert(!csa->now_crit || csa->hold_onto_crit); /* shouldn't hold crit unless asked to */
+		cw_set[0].recompute_list_head = cw_set[0].recompute_list_tail =  NULL;
 		t_tries = 0;	/* commit was successful so reset t_tries */
 		INCR_GVSTATS_COUNTER(csa, cnl, n_nontp_readonly, 1);
 		INCR_GVSTATS_COUNTER(csa, cnl, n_nontp_blkread, n_blks_validated);
@@ -486,8 +487,7 @@ trans_num t_end(srch_hist *hist1, srch_hist *hist2, trans_num ctn)
 		 * to FALSE.
 		 */
 		SS_INIT_IF_NEEDED(csa, cnl);
-	} else
-		SS_RELEASE_IF_NEEDED(csa, cnl);
+	}
 	if (0 != cw_depth)
 	{	/* Caution : since csa->backup_in_prog and read_before_image are initialized below
 	 	 * only if (cw_depth), these variables should be used below only within an if (cw_depth).
@@ -519,6 +519,7 @@ trans_num t_end(srch_hist *hist1, srch_hist *hist2, trans_num ctn)
 						GET_CDB_SC_CODE(cs->blk, status);	/* code is set in status */
 						if (is_mm && (cdb_sc_gbloflow == status))
 						{
+							cw_set[0].recompute_list_head = cw_set[0].recompute_list_tail = NULL;
 							send_msg_csa(CSA_ARG(csa) VARLSTCNT(4) ERR_GBLOFLOW, 2,
 								DB_LEN_STR(csa->region));
 							if (save_jnlpool != jnlpool)
@@ -640,20 +641,21 @@ trans_num t_end(srch_hist *hist1, srch_hist *hist2, trans_num ctn)
 	block_saved = FALSE;
 	ESTABLISH_NOUNWIND(t_ch);	/* avoid hefty setjmp call, which is ok since we never unwind t_ch */
 	assert(!csa->hold_onto_crit || csa->now_crit);
+	CHECK_TN(csa, csd, cti->curr_tn);	/* macro might issue rts_error TNTOOLARGE */
 	if (!csa->now_crit)
 	{
 		/* Get more space if needed. This is done outside crit so that any necessary IO has a chance of occurring
 		 * outside crit. The available space must be double-checked inside crit.
 		 */
 		DEBUG_ONLY(tmp_jnlpool = jnlpool;)
-		if (!is_mm && !WCS_GET_SPACE(reg, cw_set_depth + 1, NULL))
+		if (!is_mm && !WCS_GET_SPACE(reg, cw_set_depth + 1, NULL, csa))
 			/* only reason we currently know why wcs_get_space could fail */
 			assert(csa->nl->wc_blocked || gtm_white_box_test_case_enabled);
 		assert(tmp_jnlpool == jnlpool);
 		for (;;)
 		{
 			DEBUG_ONLY(tmp_jnlpool = jnlpool;)
-			grab_crit(reg);	/* Step CMT01 (see secshr_db_clnup.c for CMTxx step descriptions) */
+			grab_crit(reg, WS_47);	/* Step CMT01 (see secshr_db_clnup.c for CMTxx step descriptions) */
 			if (!FROZEN_HARD(csa))
 			{
 				assert(tmp_jnlpool == jnlpool);
@@ -813,11 +815,11 @@ trans_num t_end(srch_hist *hist1, srch_hist *hist2, trans_num ctn)
 	}
 	/* in crit, ensure cache-space is available. the out-of-crit check done above might not have been enough */
 	DEBUG_ONLY(tmp_jnlpool = jnlpool;)
-	if (!is_mm && !WCS_GET_SPACE(reg, cw_set_depth + 1, NULL))
+	if (!is_mm && !WCS_GET_SPACE(reg, cw_set_depth + 1, NULL, csa))
 	{
 		/* only reason we currently know why wcs_get_space could fail */
 		assert(csa->nl->wc_blocked || gtm_white_box_test_case_enabled);
-		SET_TRACEABLE_VAR(cnl->wc_blocked, TRUE);
+		SET_TRACEABLE_VAR(cnl->wc_blocked, WC_BLOCK_RECOVER);
 		BG_TRACE_PRO_ANY(csa, wc_blocked_t_end_hist);
 		SET_CACHE_FAIL_STATUS(status, csd);
 		assert(tmp_jnlpool == jnlpool);
@@ -900,7 +902,7 @@ trans_num t_end(srch_hist *hist1, srch_hist *hist2, trans_num ctn)
 						if (cr->blk != bt->blk)
 						{
 							assert(FALSE);
-							SET_TRACEABLE_VAR(cnl->wc_blocked, TRUE);
+							SET_TRACEABLE_VAR(cnl->wc_blocked, WC_BLOCK_RECOVER);
 							BG_TRACE_PRO_ANY(csa, wc_blocked_t_end_crbtmismatch1);
 							status = cdb_sc_crbtmismatch;
 							goto failed;
@@ -926,15 +928,19 @@ trans_num t_end(srch_hist *hist1, srch_hist *hist2, trans_num ctn)
 						 * that update only one data block can benefit from this optimization.
 						 * But that has to be carefully thought out.
 						 */
-						if (!IS_DOLLAR_INCREMENT
-							|| t1->level || (1 != cw_set_depth) || (t1->blk_num != cw_set[0].blk))
+						if (t1->level || (1 != cw_set_depth) || (t1->blk_num != cw_set[0].blk)
+							|| !(IS_DOLLAR_INCREMENT || (t1->blk_target
+								&& t1->blk_target->noisolation && cw_set[0].recompute_list_head)))
 						{
 							status = cdb_sc_blkmod;
 							NONTP_TRACE_HIST_MOD(t1, t_blkmod_t_end3);
 							goto failed;
 						} else
 						{
-							status = gvincr_recompute_upd_array(t1, cw_set, cr);
+							assert((t1->blk_target->noisolation && cw_set[0].recompute_list_head)
+								? !IS_DOLLAR_INCREMENT : IS_DOLLAR_INCREMENT);	/* one at a time */
+							status = IS_DOLLAR_INCREMENT ? gvincr_recompute_upd_array(t1, cw_set, cr)
+											: t_recompute_upd_array(t1, cw_set, cr);
 							if (cdb_sc_normal != status)
 							{
 								status = cdb_sc_blkmod;
@@ -946,7 +952,7 @@ trans_num t_end(srch_hist *hist1, srch_hist *hist2, trans_num ctn)
 				}
 				if ((cache_rec_ptr_t)CR_NOTVALID == cr)
 				{
-					SET_TRACEABLE_VAR(cnl->wc_blocked, TRUE);
+					SET_TRACEABLE_VAR(cnl->wc_blocked, WC_BLOCK_RECOVER);
 					BG_TRACE_PRO_ANY(csa, wc_blocked_t_end_hist);
 					SET_CACHE_FAIL_STATUS(status, csd);
 					goto failed;
@@ -957,7 +963,7 @@ trans_num t_end(srch_hist *hist1, srch_hist *hist2, trans_num ctn)
 					if ((NULL != cr) && (NULL != bt) && (cr->blk != bt->blk))
 					{
 						assert(FALSE);
-						SET_TRACEABLE_VAR(cnl->wc_blocked, TRUE);
+						SET_TRACEABLE_VAR(cnl->wc_blocked, WC_BLOCK_RECOVER);
 						BG_TRACE_PRO_ANY(csa, wc_blocked_t_end_crbtmismatch2);
 						status = cdb_sc_crbtmismatch;
 						goto failed;
@@ -1072,7 +1078,7 @@ trans_num t_end(srch_hist *hist1, srch_hist *hist2, trans_num ctn)
 					if (cr->blk != bt->blk)
 					{
 						assert(FALSE);
-						SET_TRACEABLE_VAR(cnl->wc_blocked, TRUE);
+						SET_TRACEABLE_VAR(cnl->wc_blocked, WC_BLOCK_RECOVER);
 						BG_TRACE_PRO_ANY(csa, wc_blocked_t_end_crbtmismatch3);
 						status = cdb_sc_crbtmismatch;
 						goto failed;
@@ -1081,7 +1087,7 @@ trans_num t_end(srch_hist *hist1, srch_hist *hist2, trans_num ctn)
 			}
 			if ((cache_rec_ptr_t)CR_NOTVALID == cr)
 			{
-				SET_TRACEABLE_VAR(cnl->wc_blocked, TRUE);
+				SET_TRACEABLE_VAR(cnl->wc_blocked, WC_BLOCK_RECOVER);
 				BG_TRACE_PRO_ANY(csa, wc_blocked_t_end_bitmap_nullbt);
 				SET_CACHE_FAIL_STATUS(status, csd);
 				goto failed;
@@ -1135,7 +1141,7 @@ trans_num t_end(srch_hist *hist1, srch_hist *hist2, trans_num ctn)
 				cr = db_csh_get(cs->blk);
 				if ((cache_rec_ptr_t)CR_NOTVALID == cr)
 				{
-					SET_TRACEABLE_VAR(cnl->wc_blocked, TRUE);
+					SET_TRACEABLE_VAR(cnl->wc_blocked, WC_BLOCK_RECOVER);
 					BG_TRACE_PRO_ANY(csa, wc_blocked_t_end_jnl_cwset);
 					SET_CACHE_FAIL_STATUS(status, csd);
 					goto failed;
@@ -1227,7 +1233,6 @@ trans_num t_end(srch_hist *hist1, srch_hist *hist2, trans_num ctn)
 			|| (ERR_GVKILLFAIL == t_err) && gv_play_duplicate_kills);	/* exception case (b) */
 	assert(cw_set_depth < CDB_CW_SET_SIZE);
 	ASSERT_CURR_TN_EQUALS_EARLY_TN(csa, dbtn);
-	CHECK_TN(csa, csd, dbtn);	/* can issue rts_error TNTOOLARGE */
 	if (JNL_ENABLED(csa))
 	{	/* Since we got the system time (jgbl.gbl_jrec_time) outside of crit, it is possible that
 		 * journal records were written concurrently to this file with a timestamp that is future
@@ -1309,7 +1314,7 @@ trans_num t_end(srch_hist *hist1, srch_hist *hist2, trans_num ctn)
 			if (!wcs_flu(WCSFLU_FLUSH_HDR | WCSFLU_WRITE_EPOCH | WCSFLU_IN_COMMIT | WCSFLU_SPEEDUP_NOBEFORE))
 			{
 				SET_WCS_FLU_FAIL_STATUS(status, csd);
-				SET_TRACEABLE_VAR(cnl->wc_blocked, TRUE);
+				SET_TRACEABLE_VAR(cnl->wc_blocked, WC_BLOCK_RECOVER);
 				BG_TRACE_PRO_ANY(csa, wc_blocked_t_end_jnl_wcsflu);
 				goto failed;
 			}
@@ -1369,7 +1374,7 @@ trans_num t_end(srch_hist *hist1, srch_hist *hist2, trans_num ctn)
 			SHM_WRITE_MEMORY_BARRIER;
 		}
 	}
-	assert(TN_NOT_SPECIFIED > MAX_TN_V6); /* Ensure TN_NOT_SPECIFIED isn't a valid TN number */
+	assert(TN_NOT_SPECIFIED > MAX_TN_ANY); /* Ensure TN_NOT_SPECIFIED isn't a valid TN number */
 	blktn = (TN_NOT_SPECIFIED == ctn) ? dbtn : ctn;
 	cti->early_tn = dbtn + 1;	/* Step CMT04 */
 	csa->t_commit_crit = T_COMMIT_CRIT_PHASE0;	/* phase0 : write journal records. Step CMT05 */
@@ -1776,11 +1781,6 @@ trans_num t_end(srch_hist *hist1, srch_hist *hist2, trans_num ctn)
 	ASSERT_CR_ARRAY_IS_UNPINNED(csd, cr_array, cr_array_index);
 	cr_array_index = 0;
 	csa->t_commit_crit = FALSE;	/* Step CMT19 */
-	/* Phase 2 commits are completed. See if we had done a snapshot init (csa->snapshot_in_prog == TRUE). If so,
-	 * try releasing the resources obtained while snapshot init.
-	 */
-	if (SNAPSHOTS_IN_PROG(csa))
-		SS_RELEASE_IF_NEEDED(csa, cnl);
 
 skip_cr_array:
 	assert(!csa->now_crit || csa->hold_onto_crit);
@@ -1809,6 +1809,7 @@ skip_cr_array:
 	 */
 	update_trans = 0;
 	CWS_RESET;
+	cw_set[0].recompute_list_head = cw_set[0].recompute_list_tail = NULL;
 	/* although we have the same assert at the beginning of "skip_cr_array" label, the below assert ensures that we did not grab
 	 * crit in between (in process_deferred_stale() or backup_buffer_flush() or wcs_timer_start()). The assert at the beginning
 	 * of the loop is needed as well to ensure that if we are done with the commit, we should have released crit
@@ -1872,6 +1873,7 @@ failed_skip_revert:
 	 * for non-final-retry restarts (relies on "cw_stagnate_reinitialized") so do it always to avoid unnecessary "if" check.
 	 */
 	CWS_RESET;
+	cw_set[0].recompute_list_head = cw_set[0].recompute_list_tail = NULL;
 	cw_map_depth = 0;
 	assert(0 == cr_array_index);
 	if (save_jnlpool != jnlpool)
