@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -28,7 +28,7 @@
 #include "interlock.h"
 #include "jnl.h"
 #include "buddy_list.h"		/* needed for tp.h */
-#include "hashtab_int4.h"	/* needed for tp.h and cws_insert.h */
+#include "hashtab_int4.h"
 #include "tp.h"
 #include "gdsbgtr.h"
 #include "sleep_cnt.h"
@@ -56,6 +56,7 @@
 #include "wcs_backoff.h"
 #include "wcs_wt.h"
 #include "wcs_recover.h"
+#include "util.h"
 
 GBLDEF srch_blk_status	*first_tp_srch_status;	/* the first srch_blk_status for this block in this transaction */
 GBLDEF unsigned char	rdfail_detail;	/* t_qread uses a 0 return to indicate a failure (no buffer filled) and the real
@@ -165,6 +166,17 @@ sm_uc_ptr_t t_qread(block_id blk, sm_int_ptr_t cycle, cache_rec_ptr_ptr_t cr_out
 	is_mm = (dba_mm == csd->acc_meth);
 	/* We better hold crit in the final retry (TP & non-TP). Only exception is journal recovery */
 	assert((t_tries < CDB_STAGNATE) || csa->now_crit || mupip_jnl_recover);
+	if (TREF(in_mupip_integ) && TREF(instance_frozen_crit_skipped))
+	{
+		assert(!dollar_tlevel);
+#		ifdef DEBUG
+		if (WBTEST_ENABLED(WBTEST_MUINTEG_TQREAD))
+		{
+			util_out_print("Whitebox test : t_qread returns NULL", TRUE);
+			return (sm_uc_ptr_t)NULL;
+		}
+#		endif
+	}
 	if (dollar_tlevel)
 	{
 		assert(sgm_info_ptr);
@@ -174,6 +186,7 @@ sm_uc_ptr_t t_qread(block_id blk, sm_int_ptr_t cycle, cache_rec_ptr_ptr_t cr_out
 			if (1 == chain1.flag)
 			{
 				assert(sgm_info_ptr->cw_set_depth);
+				assert((SIZEOF(int) * 8) >= CW_INDEX_MAX_BITS);
 				if ((int)chain1.cw_index < sgm_info_ptr->cw_set_depth)
 					tp_get_cw(sgm_info_ptr->first_cw_set, (int)chain1.cw_index, &cse);
 				else
@@ -351,7 +364,7 @@ sm_uc_ptr_t t_qread(block_id blk, sm_int_ptr_t cycle, cache_rec_ptr_ptr_t cr_out
 			 * the cycles while holding crit (hopefully rare case so it is okay to hold crit for a heavyweight call).
 			 */
 			if (!was_crit)
-				grab_crit(gv_cur_region);
+				grab_crit(gv_cur_region, WS_14);
 			/* Now that we have crit, sync them up by copying the new keys inside crit and opening the key handles
 			 * outside crit (a potentially long running operation).
 			 */
@@ -391,6 +404,11 @@ sm_uc_ptr_t t_qread(block_id blk, sm_int_ptr_t cycle, cache_rec_ptr_ptr_t cr_out
 	{
 		if (NULL == (cr = db_csh_get(blk)))
 		{	/* not in memory */
+			if (TREF(in_mupip_integ) && TREF(instance_frozen_crit_skipped))
+			{	/* We don't want to read the block from Disk when retrying the block in case of integ errors*/
+				rel_crit(gv_cur_region);
+				return (sm_uc_ptr_t)NULL;
+			}
 			if (clustered && (NULL != (bt = bt_get(blk))) && (FALSE == bt->flushing))
 				bt = NULL;
 			if (!csa->now_crit)
@@ -418,7 +436,7 @@ sm_uc_ptr_t t_qread(block_id blk, sm_int_ptr_t cycle, cache_rec_ptr_ptr_t cr_out
 				 * above. But in case a MUPIP REORG ENCRYPT concurrently sneaked in between these lines we
 				 * need to resync.
 				 */
-				sync_needed = grab_crit_encr_cycle_sync(gv_cur_region);
+				sync_needed = grab_crit_encr_cycle_sync(gv_cur_region, WS_15);
 				assert(NULL == reorg_encrypt_restart_csa);
 				assert(!sync_needed || (NULL != encr_ptr));
 				if (sync_needed && IS_NOT_SAFE_TO_SYNC_NEW_KEYS(dollar_tlevel, update_trans))
@@ -443,7 +461,7 @@ sm_uc_ptr_t t_qread(block_id blk, sm_int_ptr_t cycle, cache_rec_ptr_ptr_t cr_out
 				{
 					assert(cnl->wc_blocked); /* only reason we currently know wcs_get_space could fail */
 					assert(gtm_white_box_test_case_enabled);
-					SET_TRACEABLE_VAR(cnl->wc_blocked, TRUE);
+					SET_TRACEABLE_VAR(cnl->wc_blocked, WC_BLOCK_RECOVER);
 					BG_TRACE_PRO_ANY(csa, wc_blocked_t_qread_db_csh_getn_invalid_blk);
 					set_wc_blocked = TRUE;
 					break;
@@ -554,7 +572,7 @@ sm_uc_ptr_t t_qread(block_id blk, sm_int_ptr_t cycle, cache_rec_ptr_ptr_t cr_out
 		}
 		if (CR_NOTVALID == (sm_long_t)cr)
 		{
-			SET_TRACEABLE_VAR(cnl->wc_blocked, TRUE);
+			SET_TRACEABLE_VAR(cnl->wc_blocked, WC_BLOCK_RECOVER);
 			BG_TRACE_PRO_ANY(csa, wc_blocked_t_qread_db_csh_get_invalid_blk);
 			set_wc_blocked = TRUE;
 			break;
@@ -673,7 +691,7 @@ sm_uc_ptr_t t_qread(block_id blk, sm_int_ptr_t cycle, cache_rec_ptr_ptr_t cr_out
 							if (!was_crit)
 							{
 								assert(!hold_onto_crit);
-								grab_crit_encr_cycle_sync(gv_cur_region);
+								grab_crit_encr_cycle_sync(gv_cur_region, WS_16);
 								if (blocking_pid == cr->in_tend)
 								{
 									wcs_recover(gv_cur_region);
@@ -712,7 +730,7 @@ sm_uc_ptr_t t_qread(block_id blk, sm_int_ptr_t cycle, cache_rec_ptr_ptr_t cr_out
 				if (!csa->now_crit)
 				{
 					assert(!hold_onto_crit);
-					grab_crit_encr_cycle_sync(gv_cur_region);
+					grab_crit_encr_cycle_sync(gv_cur_region, WS_17);
 				}
 				if (cr->read_in_progress < -1)
 				{	/* outside of design; clear to known state */
@@ -731,7 +749,7 @@ sm_uc_ptr_t t_qread(block_id blk, sm_int_ptr_t cycle, cache_rec_ptr_ptr_t cr_out
 							assert(0 == cr->bt_index);
 							if (cr->bt_index)
 							{
-								SET_TRACEABLE_VAR(cnl->wc_blocked, TRUE);
+								SET_TRACEABLE_VAR(cnl->wc_blocked, WC_BLOCK_RECOVER);
 								BG_TRACE_PRO_ANY(csa, wc_blocked_t_qread_bad_bt_index1);
 								set_wc_blocked = TRUE;
 								break;
@@ -766,7 +784,7 @@ sm_uc_ptr_t t_qread(block_id blk, sm_int_ptr_t cycle, cache_rec_ptr_ptr_t cr_out
 						assert(0 == cr->bt_index);
 						if (cr->bt_index)
 						{
-							SET_TRACEABLE_VAR(cnl->wc_blocked, TRUE);
+							SET_TRACEABLE_VAR(cnl->wc_blocked, WC_BLOCK_RECOVER);
 							BG_TRACE_PRO_ANY(csa, wc_blocked_t_qread_bad_bt_index2);
 							set_wc_blocked = TRUE;
 							break;
@@ -788,7 +806,7 @@ sm_uc_ptr_t t_qread(block_id blk, sm_int_ptr_t cycle, cache_rec_ptr_ptr_t cr_out
 					rdfail_detail = cdb_sc_tqreadnowait;
 					return (sm_uc_ptr_t)NULL;
 				}
-				BG_TRACE_PRO_ANY(csa, t_qread_ripsleep_cnt);
+				INCR_GVSTATS_COUNTER(csa, cnl, n_wait_for_read, 1);
 				if (!sleep_invoked)	/* Count # of blks for which we ended up sleeping on the read */
 					BG_TRACE_PRO_ANY(csa, t_qread_ripsleep_nblks);
 				wcs_sleep(lcnt);
@@ -804,7 +822,7 @@ sm_uc_ptr_t t_qread(block_id blk, sm_int_ptr_t cycle, cache_rec_ptr_ptr_t cr_out
 		 */
 		assertpro((BAD_LUCK_ABOUNDS - was_crit) >= ocnt);
 		if (!csa->now_crit && !hold_onto_crit)
-			grab_crit_encr_cycle_sync(gv_cur_region);
+			grab_crit_encr_cycle_sync(gv_cur_region, WS_18);
 	} while (TRUE);
 	assert(set_wc_blocked && (cnl->wc_blocked || !csa->now_crit));
 	SET_CACHE_FAIL_STATUS(rdfail_detail, csd);
