@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2016 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -25,7 +25,7 @@
 #include "filestruct.h"
 #include "copy.h"
 #include "jnl.h"
-#include "hashtab_int4.h"	/* needed for tp.h */
+#include "hashtab_int4.h"
 #include "buddy_list.h"		/* needed for tp.h */
 #include "tp.h"
 #include "t_write.h"
@@ -100,13 +100,14 @@ cw_set_element *t_write (
 		assert(cw_set_depth < CDB_CW_SET_SIZE);
 		assert(index < (int)cw_set_depth);
 		new_cse = TRUE;
-		tp_cse = NULL;/* don't bother returning tp_cse for non-TP; it's almost never needed and it distiguishes the cases */
+		tp_cse = cse;
 	} else
 	{
 		assert(!index || index < sgm_info_ptr->cw_set_depth);
 		chain = *(off_chain *)&blk;
 		if (chain.flag == 1)
 		{
+			assert((SIZEOF(int) * 8) >= CW_INDEX_MAX_BITS);
 			tp_get_cw(sgm_info_ptr->first_cw_set, (int)chain.cw_index, &cse);
 			blk = cse->blk;
 		} else
@@ -117,7 +118,7 @@ cw_set_element *t_write (
 				tp_srch_status = NULL;
 			cse = tp_srch_status ? tp_srch_status->cse : NULL;
 				/* tp_srch_status->cse always returns latest in the horizontal list */
-	    	}
+		}
 		assert(!cse || !cse->high_tlevel);
 		if (cse == NULL)
 		{
