@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2016 Fidelity National Information	*
+ * Copyright (c) 2001-2019 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -24,7 +24,6 @@
 #include "filestruct.h"
 #include "jnl.h"
 #include "buddy_list.h"		/* needed for tp.h */
-#include "hashtab_int4.h"	/* needed for tp.h */
 #include "tp.h"
 #include "t_write_map.h"
 #include "min_max.h"
@@ -115,6 +114,7 @@ void t_write_map (
 		while (reference_cnt--)
 		{
 			blkid = *updptr;
+			assert(blkid == (int4)blkid);
 			assert((int4)blkid < csa->hdr->bplmap);
 			updptr++;
 		}
