@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2013 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -12,7 +13,9 @@
 #include "mdef.h"
 
 #include <stddef.h>		/* for offsetof macro (used by OFFSETOF macro) */
+#include "gtm_stdio.h"
 
+#include "gtmio.h"
 #include "gtm_string.h"
 #include "gdsroot.h"
 #include "gtm_facility.h"
@@ -25,6 +28,7 @@
 #include "hashtab_mname.h"
 #include "gtmimagename.h"
 #include "dpgbldir.h"
+#include "io.h"
 
 GBLREF	gv_namehead		*gv_target_list;
 GBLREF	gv_namehead		*gv_target;
@@ -61,21 +65,23 @@ gv_namehead *targ_alloc(int keysize, mname_entry *gvent, gd_region *reg)
 		gvt->gvname.hash_code = gvent->hash_code;
 		return gvt;
 	}
-	assert((NULL == reg) || dba_mm == REG_ACC_METH(reg) || (dba_bg == REG_ACC_METH(reg)));
+	assert((NULL == reg) || IS_REG_BG_OR_MM(reg));
 	assert((NULL == reg) || (reg->max_key_size <= MAX_KEY_SZ));
 	/* Ensure there are no additional compiler introduced filler bytes. This is a safety since a few elements in the
 	 * gv_namehead structure are defined using MAX_BT_DEPTH macros and we want to guard against changes to this macro
 	 * that cause unintended changes to the layout/size of the gv_namehead structure.
 	 */
-	assert(OFFSETOF(gv_namehead, filler_8byte_align2[0]) + SIZEOF(gvt->filler_8byte_align2)
+	assert(OFFSETOF(gv_namehead, filler_8byte_align1[0]) + SIZEOF(gvt->filler_8byte_align1)
 			== OFFSETOF(gv_namehead, last_split_blk_num[0]));
 #	ifdef GTM_TRIGGER
 	assert(OFFSETOF(gv_namehead, last_split_blk_num[0]) + SIZEOF(gvt->last_split_blk_num)
 			== OFFSETOF(gv_namehead, gvt_trigger));
-	GTM64_ONLY(assert(OFFSETOF(gv_namehead, filler_8byte_align3) + SIZEOF(gvt->filler_8byte_align3)
-			== OFFSETOF(gv_namehead, clue));)
+	GTM64_ONLY(assert(OFFSETOF(gv_namehead, filler_8byte_align2) + SIZEOF(gvt->filler_8byte_align2)
+					+ SIZEOF(gvt->filler_clue_end_align)
+				== OFFSETOF(gv_namehead, clue));)
 	NON_GTM64_ONLY(assert(OFFSETOF(gv_namehead, trig_mismatch_test_done) + SIZEOF(gvt->trig_mismatch_test_done)
-			== OFFSETOF(gv_namehead, clue));)
+					+ SIZEOF(gvt->filler_clue_end_align)
+				== OFFSETOF(gv_namehead, clue));)
 #	endif
 	csa = ((NULL != reg) && reg->open) ? &FILE_INFO(reg)->s_addrs : NULL;
 	gvt_hashtab_present = FALSE;
@@ -114,9 +120,11 @@ gv_namehead *targ_alloc(int keysize, mname_entry *gvent, gd_region *reg)
 		gvt->gvname.var_name.len = 0;
 		gvt->gvname.hash_code = 0;
 	}
-	assert(0 == (OFFSETOF(gv_namehead, clue) % SIZEOF(gvt->clue)));
+	assert(SIZEOF(gvt->clue) == SIZEOF(gv_key));			/* gv_key_nobase should be the same size as gv_key */
+	assert(OFFSETOF(gv_namehead, clue) == SIZEOF(gv_namehead) - SIZEOF(gvt->clue));		/* check no padding after clue */
 	gvt->first_rec = (gv_key *)((char *)&gvt->clue + SIZEOF(gv_key) + keysize);
 	gvt->last_rec = (gv_key *)((char *)gvt->first_rec + SIZEOF(gv_key) + keysize);
+	gvt->prev_key = NULL;
 	assert((UINTPTR_T)gvt->first_rec % SIZEOF(gvt->first_rec->top) == 0);
 	assert((UINTPTR_T)gvt->last_rec % SIZEOF(gvt->last_rec->top) == 0);
 	assert((UINTPTR_T)gvt->first_rec % SIZEOF(gvt->first_rec->end) == 0);
@@ -211,11 +219,13 @@ void	targ_free(gv_namehead *gvt)
 		 *      a) We are exiting and in the process of freeing up cs_addrs->dir_tree in gv_rundown
 		 *	b) The GT.CM GNP server which could free up a region as part of running down a client's database.
 		 *	c) In VMS, DAL calls could rundown the database. This is tough to check using an assert.
+		 *      d) This is a statsDB which can be closed on an opt-out and reopened on a subsequent opt-in.
 		 * Assert accordingly.
 		 */
 		UNIX_ONLY(assert(IS_GTCM_GNP_SERVER_IMAGE
 				 || (process_exiting && ((gvt == cs_addrs->dir_tree)
-							 GTMTRIG_ONLY(|| (gvt == cs_addrs->hasht_tree))))));
+							 GTMTRIG_ONLY(|| (gvt == cs_addrs->hasht_tree))))
+				 || (IS_STATSDB_REG(gvt->gd_csa->region))));
 		gv_target = NULL;	/* In that case, set gv_target to NULL to ensure freed up memory is never used */
 	}
 	/* assert we never delete a gvt that is actively used in a TP transaction */
@@ -236,5 +246,7 @@ void	targ_free(gv_namehead *gvt)
 		next_gvnh->prev_gvnh = prev_gvnh;
 	if (NULL != gvt->alt_hist)	/* can be NULL for GT.CM GNP or DDP client */
 		free(gvt->alt_hist);
+	if (NULL != gvt->prev_key)
+		free(gvt->prev_key);
 	free(gvt);
 }
