@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -14,11 +14,10 @@
 #define TP_H
 
 #include <sys/types.h>
+#include "hashtab_int4.h"
 
 error_def(ERR_TPNOTACID);
 
-/* HEADER-FILE-DEPENDENCIES : hashtab_int4.h */
-
 #define JNL_LIST_INIT_ALLOC		16		/* initial allocation for si->jnl_list */
 #define	CW_SET_LIST_INIT_ALLOC		64		/* initial allocation for si->cw_set_list */
 #define	TLVL_CW_SET_LIST_INIT_ALLOC     64		/* initial allocation for si->tlvl_cw_set_list */
@@ -342,18 +341,6 @@ typedef struct ua_list_struct
 
 #define TP_MAX_NEST	127
 
-/* Note gv_orig_key is assigned to tp_pointer->orig_key which then tries to dereference the "begin", "end", "prev", "top"
- * 	fields like it were a gv_currkey pointer. Since these members are 2-byte fields, we need atleast 2 byte alignment.
- * We want to be safer and hence give 4-byte alignment by declaring it as an array of integers.
- * Note that the array stores only ONE key since we need to store gv_currkey at the outermost TSTART only since this needs
- *	to be restored only in case of a TRESTART or TROLLBACK where the objective is to restore it to what the state was
- *	at the outermost TSTART.
- */
-typedef struct gv_orig_key_struct
-{
-	gv_key	gv_orig_key[DBKEYALLOC(MAX_KEY_SZ)];
-}gv_orig_key_array;
-
 #define TRANS_RESTART_HIST_ARRAY_SZ	512 /* See comment (6) in gtm_threadgbl_defs.h as to why this is not inside #ifdef DEBUG */
 #ifdef DEBUG
 /* The following structure stores information pertaining to the most recent invocation of t_retry OR tp_restart. Maintain a 512
@@ -393,7 +380,7 @@ typedef struct trans_restart_hist_struct
 	this_restart_hist->t_tries = t_tries;								\
 	this_restart_hist->dollar_tlevel = dollar_tlevel;						\
 	this_restart_hist->retry_code = RETRY_CODE;							\
-	this_restart_hist->call_from = (caddr_t)caller_id();						\
+	this_restart_hist->call_from = (caddr_t)caller_id(0);						\
 	if ((NULL != jnlpool) && (NULL != jnlpool->jnlpool_ctl))						\
 		this_restart_hist->seq_or_tn.jnl_seqno = jnlpool->jnlpool_ctl->jnl_seqno;		\
 	else												\
@@ -425,7 +412,7 @@ typedef struct trans_restart_hist_struct
 				: is_uchar_wcs_code[STATUS - 'A'];			\
 		if (is_wcs_code)							\
 		{									\
-			SET_TRACEABLE_VAR(cnl->wc_blocked, TRUE);			\
+			SET_TRACEABLE_VAR(cnl->wc_blocked, WC_BLOCK_RECOVER);		\
 			BG_TRACE_PRO_ANY(CSA, wc_blocked_wcs_cdb_sc_final_retry);	\
 		}									\
 	}										\
@@ -900,23 +887,23 @@ GBLREF	unsigned int	t_tries;
 	}															\
 }
 
-#define SAVE_REGION_INFO(SAVE_KEY, SAVE_TARGET, SAVE_CUR_REG, SAVE_SI_PTR, SAVE_JNLPOOL)	\
+#define SAVE_REGION_INFO(SAVE_KEY_BUF, SAVE_TARGET, SAVE_CUR_REG, SAVE_SI_PTR, SAVE_JNLPOOL)	\
 MBSTART {											\
 	SAVE_TARGET = gv_target;								\
 	SAVE_CUR_REG = gv_cur_region;								\
 	SAVE_SI_PTR = sgm_info_ptr;								\
 	SAVE_JNLPOOL = jnlpool;									\
 	assert(NULL != gv_currkey);								\
-	assert((SIZEOF(gv_key) + gv_currkey->end) <= SIZEOF(SAVE_KEY));				\
-	memcpy(&SAVE_KEY[0], gv_currkey, SIZEOF(gv_key) + gv_currkey->end);			\
+	assert((SIZEOF(gv_key) + gv_currkey->end + 1) <= SIZEOF(SAVE_KEY_BUF));			\
+	memcpy(SAVE_KEY_BUF.buf, gv_currkey, SIZEOF(gv_key) + gv_currkey->end + 1);		\
 } MBEND
-#define RESTORE_REGION_INFO(SAVE_KEY, SAVE_TARGET, SAVE_CUR_REG, SAVE_SI_PTR, SAVE_JNLPOOL)	\
+#define RESTORE_REGION_INFO(SAVE_KEY_BUF, SAVE_TARGET, SAVE_CUR_REG, SAVE_SI_PTR, SAVE_JNLPOOL)	\
 MBSTART {											\
 	gv_target = SAVE_TARGET;								\
 	sgm_info_ptr = SAVE_SI_PTR;								\
 	/* check no keysize expansion occurred inside gvcst_root_search */			\
-	assert(gv_currkey->top == SAVE_KEY[0].top);						\
-	memcpy(gv_currkey, &SAVE_KEY[0], SIZEOF(gv_key) + SAVE_KEY[0].end);			\
+	assert(gv_currkey->top == SAVE_KEY_BUF.key.top);					\
+	memcpy(gv_currkey, SAVE_KEY_BUF.buf, SIZEOF(gv_key) + SAVE_KEY_BUF.key.end + 1);	\
 	if (NULL != SAVE_CUR_REG)								\
 	{											\
 		TP_CHANGE_REG_IF_NEEDED(SAVE_CUR_REG);						\
