@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2019 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -106,10 +106,6 @@ void trans_code_finish(void)
 
 	frame_pointer->type = proc_act_type;
 	proc_act_type = 0;
-	/* Save/restore restart_pc over dispatch of this error handler */
-	PUSH_MV_STENT(MVST_RSTRTPC);
-	mv_chain->mv_st_cont.mvs_rstrtpc.restart_pc_save = restart_pc;
-	mv_chain->mv_st_cont.mvs_rstrtpc.restart_ctxt_save = restart_ctxt;
 	if (0 != dollar_zyerror.str.len)
 	{
 		dummy.mvtype = MV_STR;
