@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -26,7 +26,6 @@
 #include "gdsbt.h"
 #include "gdsfhead.h"
 #include "filestruct.h"
-#include "hashtab_int4.h"
 #include "gdskill.h"
 #include "jnl.h"
 #include "gdscc.h"
@@ -146,7 +145,14 @@ void unw_mv_ent(mv_stent *mv_st_ent)
 				(TREF(dollar_etrap)).str.len = 0;
 				if ((TREF(save_xfer_root)))
 				{
-				/*If TP timeout or ztimeout, check conditions before popping out */
+					/* A tp timeout or ztimeout was deferred. Now that
+					 * $ETRAP is no longer in effect and we are not in a
+					 * job interrupt, the timeout can no longer be deferred
+					 * and needs to be recognized. If TP timeout or
+					 * ztimeout, check conditions before popping out.
+				 	 * Below part of code should be kept in sync
+					 * with unw_retarg()
+					 */
 					if (((TREF(save_xfer_root))->set_fn == tptimeout_set)
 						|| ((TREF(save_xfer_root))->set_fn == ztimeout_set))
 					{
@@ -158,17 +164,13 @@ void unw_mv_ent(mv_stent *mv_st_ent)
 							POP_XFER_ENTRY(&event_type, &set_fn, &param_val);
 							xfer_set_handlers(event_type, set_fn, param_val, TRUE);
 						}
-					} else
+					} else if (!dollar_zininterrupt)
 					{
 						DBGDFRDEVNT((stderr, "Calling pop_reset_xfer from unw_mv_ent\n"));
 						POP_XFER_ENTRY(&event_type, &set_fn, &param_val);
 						xfer_set_handlers(event_type, set_fn, param_val, TRUE);
 					}
 				}
-/*				if (tp_timeout_deferred UNIX_ONLY( && !dollar_zininterrupt))*/
-					/* A tp timeout was deferred. Now that $ETRAP is no longer in effect and we are not in a
-					 * job interrupt, the timeout can no longer be deferred and needs to be recognized.
-					 */
 			} else if (mv_st_ent->mv_st_cont.mvs_msav.addr == &dollar_zgbldir)
 			{
 				/* Restore GLD if a match is found, otherwise defer setting gd_header */
@@ -524,10 +526,6 @@ void unw_mv_ent(mv_stent *mv_st_ent)
 			}
 #			endif	/* GTM_TRIGGER */
 			return;
-		case MVST_RSTRTPC:
-			restart_pc = mv_st_ent->mv_st_cont.mvs_rstrtpc.restart_pc_save;
-			restart_ctxt = mv_st_ent->mv_st_cont.mvs_rstrtpc.restart_ctxt_save;
-			return;
 		case MVST_MRGZWRSV:
 			merge_args = mv_st_ent->mv_st_cont.mvs_mrgzwrsv.save_merge_args;
 			zwrtacindx = mv_st_ent->mv_st_cont.mvs_mrgzwrsv.save_zwrtacindx;
