@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2005-2016 Fidelity National Information	*
+ * Copyright (c) 2005-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -16,9 +16,6 @@
 
 #include <sys/mman.h>
 #include <errno.h>
-#ifdef VMS
-#include <descrip.h> /* Required for gtmsource.h */
-#endif
 
 #include "gdsroot.h"
 #include "gdskill.h"
@@ -40,19 +37,8 @@
 #include "gtmsource.h"
 #include "repl_shutdcode.h"
 #include "gvcst_protos.h"
+#include "gvt_inline.h"
 
-#ifdef VMS
-#include <ssdef.h>
-#include <fab.h>
-#include <rms.h>
-#include <iodef.h>
-#include <secdef.h>
-#include <psldef.h>
-#include <lckdef.h>
-#include <syidef.h>
-#include <xab.h>
-#include <prtdef.h>
-#endif
 #include "op.h"
 #include "targ_alloc.h"
 #include "dpgbldir.h"
@@ -92,6 +78,14 @@
 
 #define MAX_LCNT 100
 
+#define IS_GOOD_RECORD(REC, RECLEN, READADDRS, LIMITADDRS, PRE_READ, LOCAL_READ) 	\
+	((IS_VALID_RECTYPE(REC))							\
+	    && (MIN_JNLREC_SIZE <= RECLEN)						\
+	    && (ROUND_DOWN2(RECLEN, JNL_REC_START_BNDRY) == RECLEN)			\
+	    && (READADDRS + RECLEN <= LIMITADDRS)					\
+	    && (RECLEN == REC_LEN_FROM_SUFFIX(READADDRS, RECLEN))			\
+	    && (PRE_READ + RECLEN > LOCAL_READ))
+
 GBLREF	void			(*call_on_signal)();
 GBLREF	gv_namehead		*gv_target;
 GBLREF	gv_key			*gv_currkey;
@@ -107,22 +101,19 @@ GBLREF	FILE			*updhelper_log_fp;
 GBLREF	int			num_additional_processors;
 GBLREF  sgmnt_addrs             *cs_addrs;
 GBLREF 	sgmnt_data_ptr_t 	cs_data;
-#ifdef VMS
-GBLREF  uint4                   image_count;
-#endif
-GBLREF boolean_t		disk_blk_read;
+GBLREF	boolean_t		disk_blk_read;
 #ifdef DEBUG
 GBLREF	sgmnt_addrs		*reorg_encrypt_restart_csa;
 #endif
 
-static	uint4			last_pre_read_offset;
+static	gtm_uint64_t		last_pre_read_offset;
 
 error_def(ERR_DBCCERR);
 error_def(ERR_ERRCALL);
 
 int updhelper_reader(void)
 {
-	uint4			pre_read_offset;
+	gtm_uint64_t		pre_read_offset;
 	int			lcnt;
 	boolean_t		continue_reading;
 
@@ -152,7 +143,7 @@ int updhelper_reader(void)
 boolean_t updproc_preread(void)
 {
 	boolean_t		good_record, was_wrapped;
-	uint4			pre_read_offset;
+	gtm_uint64_t		pre_read_offset;
 	int			rec_len, cnt, retries, spins, maxspins, key_len;
 	enum jnl_record_type	rectype;
 	mstr_len_t		val_len;
@@ -178,7 +169,7 @@ boolean_t updproc_preread(void)
 	gvnh_reg_t		*gvnh_reg;
 #	ifdef REPL_DEBUG
 	unsigned char 		buff[MAX_ZWR_KEY_SZ], *end;
-	uint4			write, write_wrap;
+	gtm_uint64_t		lcl_write, write_wrap;
 #	endif
 	DCL_THREADGBL_ACCESS;
 
@@ -202,7 +193,7 @@ boolean_t updproc_preread(void)
 		}
 		if (recvpool_ctl->write == upd_proc_local->read)
 		{
-			REPL_DPRINT4("Wait: write==read::pre_read_offset = %x read = %x write = %x\n",
+			REPL_DPRINT4("Wait: write==read::pre_read_offset = %lx read = %lx write = %lx\n",
 				pre_read_offset, upd_proc_local->read, recvpool_ctl->write);
 			return TRUE;
 		}
@@ -237,12 +228,12 @@ boolean_t updproc_preread(void)
 		}
 #		ifdef REPL_DEBUG
 		write_wrap = recvpool_ctl->write_wrap;
-		write = recvpool_ctl->write;
+		lcl_write = recvpool_ctl->write;
 #		endif
 		if (pre_read_offset >= recvpool_ctl->write_wrap)
 		{
-			REPL_DPRINT4("Wrapped: pre_read_offset = %x write_wrap = %x write = %x\n",
-				pre_read_offset, write_wrap, write);
+			REPL_DPRINT4("Wrapped: pre_read_offset = %lx write_wrap = %lx write = %lx\n",
+				pre_read_offset, write_wrap, lcl_write);
 			pre_read_offset = 0;
 		}
 		if (pre_read_offset == recvpool_ctl->write && (!recvpool_ctl->wrapped))
@@ -275,27 +266,24 @@ boolean_t updproc_preread(void)
 				readaddrs = recvpool.recvdata_base + pre_read_offset;
 				limit_readaddrs = recvpool.recvdata_base +
 					(recvpool_ctl->wrapped ? recvpool_ctl->write_wrap : recvpool_ctl->write);
-				if (limit_readaddrs - MIN_JNLREC_SIZE > readaddrs)
+				if ((limit_readaddrs - MIN_JNLREC_SIZE) >= readaddrs)
 				{
 					rec = (jnl_record *)readaddrs;
 					rec_len = rec->prefix.forwptr;
-					if (IS_VALID_RECTYPE(rec)
-					    && (ROUND_DOWN2(rec_len, JNL_REC_START_BNDRY) == rec_len)
-					    && (readaddrs + rec_len <= limit_readaddrs)
-					    && (rec_len == REC_LEN_FROM_SUFFIX(readaddrs, rec_len))
-		    			    && (pre_read_offset + rec_len > upd_proc_local->read))
+					if (IS_GOOD_RECORD(rec, rec_len, readaddrs, limit_readaddrs,
+								pre_read_offset, upd_proc_local->read))
 						pre_read_offset += rec_len;
 					else
 						break;
 				} else
 					break;
 			}
-			REPL_DPRINT3("First one::pre_read_offset = %x read %x\n", pre_read_offset, upd_proc_local->read);
+			REPL_DPRINT3("First one::pre_read_offset = %lx read %lx\n", pre_read_offset, upd_proc_local->read);
 			upd_helper_ctl->first_done = TRUE;
 		} else
 		{
 			pre_read_offset = upd_helper_ctl->next_read_offset;
-			REPL_DPRINT3("Non-first::pre_read_offset = %x read %x\n", pre_read_offset, upd_proc_local->read);
+			REPL_DPRINT3("Non-first::pre_read_offset = %lx read %lx\n", pre_read_offset, upd_proc_local->read);
 			if (NULL != csa)
 			{
 				REPL_DPRINT2("Enough read::csa->nl->n_pre_read is %x\n", csa->nl->n_pre_read);
@@ -306,35 +294,32 @@ boolean_t updproc_preread(void)
 				}
 			}
 		}
-		good_record = TRUE;
+		good_record = FALSE;
 		readaddrs = recvpool.recvdata_base + pre_read_offset;
 		limit_readaddrs = recvpool.recvdata_base +
 			(recvpool_ctl->wrapped ? recvpool_ctl->write_wrap : recvpool_ctl->write);
-		if (limit_readaddrs - MIN_JNLREC_SIZE > readaddrs)
+		if ((limit_readaddrs - MIN_JNLREC_SIZE) >= readaddrs)
 		{
 			rec = (jnl_record *)readaddrs;
 			rec_len = rec->prefix.forwptr;
-			if (IS_VALID_RECTYPE(rec)
-			    && (ROUND_DOWN2(rec_len, JNL_REC_START_BNDRY) == rec_len)
-			    && (readaddrs + rec_len <= limit_readaddrs)
-			    && (rec_len == REC_LEN_FROM_SUFFIX(readaddrs, rec_len))
-			    && (pre_read_offset + rec_len > upd_proc_local->read))
+			rectype = (enum jnl_record_type)rec->prefix.jrec_type;
+			if (IS_GOOD_RECORD(rec, rec_len, readaddrs, limit_readaddrs, pre_read_offset, upd_proc_local->read))
 			{
 				upd_helper_ctl->next_read_offset = pre_read_offset + rec_len;
+				good_record = TRUE;
 			}
-			else
-				good_record = FALSE;
-		} else
-			good_record = FALSE;
+		}
 		RELEASE_SWAPLOCK(&upd_helper_ctl->pre_read_lock);
 		DEBUG_ONLY(num_scanned++;)
-		if (good_record)
+		if (good_record && (IS_SET_KILL_ZKILL_ZTRIG(rectype)))
 		{
-			rectype = (enum jnl_record_type)rec->prefix.jrec_type;
-			if (IS_SET_KILL_ZKILL_ZTRIG(rectype))
+			good_record = FALSE;	/* The record is good enough to look at, but it's not fully validated */
+			was_wrapped = recvpool_ctl->wrapped;
+			keystr = (limit_readaddrs > ((sm_uc_ptr_t)rec + SIZEOF(struct_jrec_upd))) ?
+					(jnl_string *)&rec->jrec_set_kill.mumps_node : NULL;
+			/* Avoid reading beyond receive pool boundary. Happens when reading at the end and the pool is wrapped */
+			if ((NULL != keystr) && (limit_readaddrs > ((sm_uc_ptr_t)keystr + keystr->length)))
 			{
-				was_wrapped = recvpool_ctl->wrapped;
-				keystr = (jnl_string *)&rec->jrec_set_kill.mumps_node;
 				key_len = keystr->length;	/* local copy of shared recvpool key */
 				if (MAX_KEY_SZ >= key_len)
 				{	/* The receive pool is shared memory and the contents can be overwritten concurrently by the
@@ -417,31 +402,27 @@ boolean_t updproc_preread(void)
 											   MAX_ZWR_KEY_SZ, gv_currkey, TRUE)))
 								end = &buff[MAX_ZWR_KEY_SZ - 1];
 							util_out_print(
-							       "readaddrs = !XJ pre_read_offset = !XL write_wrap = !XL write = !XL",
-								FALSE, readaddrs, pre_read_offset,
-								recvpool_ctl->write_wrap, recvpool_ctl->write);
+							       "readaddrs = !XJ pre_read_offset = !ZQ write_wrap = !ZQ write = !ZQ",
+								FALSE, readaddrs, &pre_read_offset,
+								&recvpool_ctl->write_wrap, &recvpool_ctl->write);
 							util_out_print(
 								" Seqno = 0x!16@XQ Rectype = !SL gv_currkey = !AD status = !SL",
 								TRUE, &recvpool.recvpool_ctl->jnl_seqno,
 								rectype, end - buff, buff, status);
 #							endif
+							good_record = TRUE;
 						} else
-						{
 							REPL_DPRINT1("Unexpected bad record\n");
-							good_record = FALSE;
-						}
-					} else
-						good_record = FALSE;
-				} else
-					good_record = FALSE;
+					}
+				}
 			}
 		}
 		if (!good_record)
 		{
 #			ifdef REPL_DEBUG
-			REPL_DPRINT5("Skipping record: pre_read_offset = %x read = %x write_wrap = %x write = %x\n",
-				pre_read_offset, upd_proc_local->read, write_wrap, write);
-			REPL_DPRINT3("New values: write_wrap = %x write = %x\n",
+			REPL_DPRINT6("Skipping record: pre_read_offset = %lx read = %lx write_wrap = %lx write = %lx reclen = %x\n",
+				pre_read_offset, upd_proc_local->read, write_wrap, lcl_write, rec_len);
+			REPL_DPRINT3("New values: write_wrap = %lx write = %lx\n",
 					recvpool_ctl->write_wrap, recvpool_ctl->write);
 #			endif
 			return TRUE;
