@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+<<<<<<< HEAD
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -94,6 +95,7 @@
 #ifdef DEBUG
 #include "repl_filter.h"	/* needed by an assert in UPD_GV_BIND_NAME_APPROPRIATE macro */
 #endif
+#include "gvt_inline.h"
 
 #define	UPDPROC_WAIT_FOR_READJNLSEQNO	100	/* ms */
 #define UPDPROC_WAIT_FOR_STARTJNLSEQNO	100	/* ms */
@@ -529,7 +531,7 @@ void updproc_actions(gld_dbname_list *gld_db_files)
 {
 	mval			ts_mv, val_mv;
 	jnl_record		*rec;
-	uint4			temp_write, temp_read;
+	gtm_uint64_t		temp_write, temp_read;
 	enum jnl_record_type	rectype;
 	int4			upd_rec_seqno = 0; /* the total no of journal records excluding TCOM records */
 	int4			tupd_num; /* the number of tset/tkill/tzkill records encountered */
@@ -567,7 +569,8 @@ void updproc_actions(gld_dbname_list *gld_db_files)
 	jnl_private_control	*jpc;
 	gld_dbname_list		*curr;
 	gd_region		*save_reg, *dummy_reg;
-	uint4			write_wrap, cntr, last_nullsubs, last_subs, keyend;
+	uint4			cntr, last_nullsubs, last_subs, keyend;
+	gtm_uint64_t		write_wrap;
 #	ifdef GTM_TRIGGER
 	uint4			nodeflags;
 	boolean_t		primary_has_trigdef, secondary_has_trigdef;
@@ -866,7 +869,7 @@ void updproc_actions(gld_dbname_list *gld_db_files)
 					{
 						TP_CHANGE_REG(curr->gd);
 						assert(!gv_cur_region->read_only);
-						grab_crit(gv_cur_region);
+						grab_crit(gv_cur_region, WS_19);
 						if (cs_addrs->onln_rlbk_cycle != cs_addrs->nl->onln_rlbk_cycle)
 							UPDPROC_ONLN_RLBK_CLNUP(gv_cur_region); /* No return */
 						cs_data->strm_reg_seqno[histinfo.strm_index] = strm_seqno;
@@ -1248,7 +1251,7 @@ void updproc_actions(gld_dbname_list *gld_db_files)
 						upd_helper_ctl->first_done = FALSE;
 						upd_helper_ctl->pre_read_offset =
 							((temp_read + rec_len) >= write_wrap) ? 0 : temp_read + rec_len;
-						REPL_DPRINT2("pre_read_offset = %x\n", upd_helper_ctl->pre_read_offset);
+						REPL_DPRINT2("pre_read_offset = %lx\n", upd_helper_ctl->pre_read_offset);
 						csa->n_pre_read_trigger = (int)((csd->n_bts * (float)csd->reserved_for_upd /
 						csd->avg_blks_per_100gbl) * csd->pre_read_trigger_factor / 100.0);
 					} else
@@ -1303,8 +1306,8 @@ void updproc_actions(gld_dbname_list *gld_db_files)
 			assert(0 == have_crit(CRIT_HAVE_ANY_REG));
 			if (dollar_tlevel)
 			{	/* Copy gv_currkey into temp variable before TROLLBACK overwrites the current variable. */
-				gv_failed_key = (gv_key *)malloc(SIZEOF(gv_key) + gv_currkey->end);
-				memcpy(gv_failed_key, gv_currkey, SIZEOF(gv_key) + gv_currkey->end);
+				gv_failed_key = (gv_key *)malloc(SIZEOF(gv_key) + gv_currkey->end + 1);
+				memcpy(gv_failed_key, gv_currkey, SIZEOF(gv_key) + gv_currkey->end + 1);
 				repl_log(updproc_log_fp, TRUE, TRUE,
 					"OP_TROLLBACK IS CALLED -->Bad trans :: dollar_tlevel = %ld\n", dollar_tlevel);
 				OP_TROLLBACK(0);	/* this should also release crit (if any) on all regions in TP */
