@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2019 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -33,6 +33,7 @@ VIEWTAB("FULL_BOOLWARN",	VTP_NULL,			VTK_FULLBOOLWARN,	MV_STR),
 VIEWTAB("GDSCERT",		VTP_NULL | VTP_VALUE,		VTK_GDSCERT,		MV_STR),
 #ifdef TESTPOLLCRIT
 VIEWTAB("GRABCRIT",		VTP_DBREGION,			VTK_GRABCRIT,		MV_STR),
+VIEWTAB("GRABFSYNC",		VTP_DBREGION,			VTK_GRABFSYNC,		MV_STR),
 VIEWTAB("GRABJNLPH2",		VTP_DBREGION,			VTK_GRABJNLPH2,		MV_STR),
 VIEWTAB("GRABJNLPOOLPH2",	VTP_DBREGION,			VTK_GRABJNLPOOLPH2,	MV_STR),
 VIEWTAB("GRABJNLQIO",		VTP_DBREGION,			VTK_GRABJNLQIO,		MV_STR),
@@ -98,6 +99,7 @@ VIEWTAB("RCSIZE",		VTP_NULL,			VTK_RCSIZE,		MV_NM),
 VIEWTAB("REGION",		VTP_DBKEY,			VTK_REGION,		MV_STR),
 #ifdef TESTPOLLCRIT
 VIEWTAB("RELCRIT",		VTP_DBREGION,			VTK_RELCRIT,		MV_STR),
+VIEWTAB("RELFSYNC",		VTP_DBREGION,			VTK_RELFSYNC,		MV_STR),
 VIEWTAB("RELJNLPH2",		VTP_DBREGION,			VTK_RELJNLPH2,		MV_STR),
 VIEWTAB("RELJNLPOOLPH2",	VTP_DBREGION,			VTK_RELJNLPOOLPH2,	MV_STR),
 VIEWTAB("RELJNLQIO",		VTP_DBREGION,			VTK_RELJNLQIO,		MV_STR),
