@@ -1,7 +1,7 @@
 
 /****************************************************************
  *								*
- * Copyright (c) 2005-2019 Fidelity National Information	*
+ * Copyright (c) 2005-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -22,6 +22,7 @@
 #else
 # define REFTYPE GBLREF
 #endif
+#include <stdlib.h> /* For setenv used below */
 REFTYPE	boolean_t	gtm_white_box_test_case_enabled;
 REFTYPE	int		gtm_white_box_test_case_number;
 REFTYPE	int		gtm_white_box_test_case_count;
@@ -99,7 +100,7 @@ typedef enum {
 	WBTEST_HOLD_SEM_BYPASS,			/* 60 : Hold access and FTOK semaphores so that LKE/DSE can bypass it. */
 	WBTEST_UTIL_OUT_BUFFER_PROTECTION,	/* 61 : Start a timer that would mess with util_out buffers by frequently
 						 *	printing long messages via util_out_print */
-	WBTEST_SET_WC_BLOCKED,			/* 62 : Set the wc_blocked when searching the tree to start wcs_recover process*/
+	WBTEST_WSSTATS_PAUSE,			/* 62 : Pause in GTM-8863 stat testing for sync (Reused WBTEST_SET_WC_BLOCKED)*/
 	WBTEST_REORG_DEBUG,			/* 63 : mupip reorg Will print GTMPOOLLIMIT value */
 	WBTEST_WCS_FLU_IOERR,			/* 64 : Force an I/O error (other than ENOSPC) when wcs_wtstart is invoked from
 						 *      wcs_flu */
@@ -120,7 +121,7 @@ typedef enum {
 	WBTEST_ANTIFREEZE_GVORDERFAIL,		/* 77 :  */
 	WBTEST_ANTIFREEZE_GVQUERYFAIL,		/* 78 :  */
 	WBTEST_ANTIFREEZE_GVQUERYGETFAIL,	/* 79 :  */
-	WBTEST_ANTIFREEZE_GVZTRIGFAIL,		/* 80 :  */
+	WBTEST_UNUSED_80,			/* 80 :  was WBTEST_ANTIFREEZE_GVZTRIGFAIL */
 	WBTEST_ANTIFREEZE_OUTOFSPACE,		/* 81 :  */
 	/* End ANTIFREEZE related white box test cases */
 	WBTEST_SIGTSTP_IN_JNL_OUTPUT_SP,	/* 82 : Send SIGTSTP to self if wcs_timers is 0 */
@@ -199,21 +200,47 @@ typedef enum {
 	WBTEST_LOCK_HASH_OFLOW,			/* 152 : Allow hash neighborhood overflow without complaint */
 	WBTEST_ZTIMEOUT_TRACE,			/* 153 : Trace the setting/deferring of ztimeout when trigerred */
 	WBTEST_ZTIME_DEFER_CRIT,		/* 154 : Defer ZTIMEOUT when CRIT is held */
-	WBTEST_LOW_MEMORY			/* 155 : In tests that limit vmemuse, exit gracefully when system calls,
+	WBTEST_LOW_MEMORY,			/* 155 : In tests that limit vmemuse, exit gracefully when system calls,
 						   external libraries, etc.  fail to allocate more memory */
+	WBTEST_JNLPROCSTUCK_FORCE,		/* 156 : Mess with jnl_sub_write_attempt to force a JNLPROCSTUCK message */
+	WBTEST_JNLEXT_PREREMOVED,		/* 157 : During a MUR_JNLEXT_UNLINK return status that the file does not exist.*/
+	WBTEST_REPLCOMM_ERR,			/* 158 : Reconnect in case of replicaton communication errors */
+	WBTEST_REPL_INIT_ERR,			/* 159 : Error in setting up the sockets */
+	WBTEST_REPL_INIT_ERR2,			/* 160 : Error at listen */
+	WBTEST_REPLCOMM_ERR_SRC,                /* 161 : Communication error in recv source server side */
+	WBTEST_REPLCOMM_SEND_SRC,		/* 162 : Communication error in send at source side */
+	WBTEST_SOCKET_KEEPALIVE,		/* 163 : shorten keepalive parameters so test fails fast */
+	WBTEST_FETCHCOMM_ERR,			/* 164 : Communication error during fetchresync rollback */
+	WBTEST_FETCHCOMM_HISTINFO,		/* 165 : Communication error during fetchresync rollback HISTINFO */
+	WBTEST_MUINTEG_TQREAD,			/* 166 : Force t_qread to return NULL. To test integ with frozen instance */
+	WBTEST_FULLBLKWRT_DB,			/* 167 : Full disk blk writes for new blocks */
+	WBTEST_OPENFILE_DB,			/* 168 : Make OPENFILE_DB fail on a RW db */
+	WBTEST_REPL_TLS_RECONN,			/* 169 : Try reconnect in case of transient TLS errors */
+	WBTEST_ZTIM_EDGE,			/* 170 : Timeout when trying to reset the timer */
+	WBTEST_LCKWAKEOVRFLO,			/* 171 : Force LOCK wakeups to almost overflow */
+	WBTEST_FAKE_SS_SHMDT_WINDOW		/* 172 : skip lcl_ss_ctx->shdw_fd = FD_INVALID to create "double" SHMDT */
 	/* Note 1: when adding new white box test cases, please make use of WBTEST_ENABLED and WBTEST_ASSIGN_ONLY (defined below)
 	 * whenever applicable
 	 * Note 2: when adding a new white box test case, see if an existing WBTEST_UNUSED* slot can be leveraged.
 	 */
 } wbtest_code_t;
 
-#ifdef DEBUG
+#if defined (DEBUG) && !defined (STATIC_ANALYSIS)
 /* Make sure to setenv gtm_white_box_test_case_count if you are going to use GTM_WHITE_BOX_TEST */
 #define GTM_WHITE_BOX_TEST(input_test_case_num, lhs, rhs)						\
 {													\
+	char			count_num[20];								\
 	if (gtm_white_box_test_case_enabled && (gtm_white_box_test_case_number == input_test_case_num))	\
 	{												\
 		gtm_wbox_input_test_case_count++;							\
+		/* To catch the gtm_wbox_input_test_case_count before it is
+		 * reset below and convey correctly to the TLS plugin
+		 */ \
+		if (WBTEST_REPL_TLS_RECONN == input_test_case_num)					\
+		{											\
+			snprintf(count_num, SIZEOF(count_num), "%d", gtm_wbox_input_test_case_count);			\
+			setenv("wbox_count", count_num, 1);						\
+		}											\
 		if (gtm_white_box_test_case_count == gtm_wbox_input_test_case_count)			\
 		{											\
 			lhs = rhs;									\
@@ -233,7 +260,7 @@ typedef enum {
 #define WBTEST_ONLY(WBTEST_NUMBER, ...)
 #endif
 
-#ifdef DEBUG
+#if defined (DEBUG) && !defined (STATIC_ANALYSIS)
 #define WBTEST_ENABLED(WBTEST_NUMBER)	(gtm_white_box_test_case_enabled && (WBTEST_NUMBER == gtm_white_box_test_case_number))
 #define ENABLE_WBTEST_ABANDONEDKILL									\
 {													\
@@ -260,9 +287,14 @@ typedef enum {
 	}												\
 }
 #else
+#define WB_PHASE2_COMMIT_ERR	FALSE
 #define WBTEST_ENABLED(WBTEST_NUMBER)	FALSE
 #define ENABLE_WBTEST_ABANDONEDKILL
+#ifdef DEBUG
+#define WB_COMMIT_ERR_ENABLED		FALSE
+#else
 #define WB_COMMIT_ERR_ENABLED
+#endif
 #define WBTEST_ASSIGN_ONLY(WBTEST_NUMBER, LHS, RHS)
 #endif
 
