@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2019 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -35,7 +35,6 @@
 #include "zshow.h"
 #include "buddy_list.h"		/* needed for tp.h */
 #include "iottdef.h"		/* needed for tp.h */
-#include "hashtab_int4.h"	/* needed for tp.h */
 #include "tp.h"
 #include "op.h"
 #include "mvalconv.h"
@@ -105,6 +104,7 @@ static readonly char zio_text[] = "$ZIO";
 static readonly char zjob_text[] = "$ZJOB";
 static readonly char zlevel_text[] = "$ZLEVEL";
 static readonly char zmaxtptime_text[] = "$ZMAXTPTIME";
+static readonly char zmlkhash_text[] = "$ZMLKHASH";
 static readonly char zmode_text[] = "$ZMODE";
 static readonly char zpatnumeric_text[] = "$ZPATNUMERIC";
 static readonly char zproc_text[] = "$ZPROCESS";
@@ -188,6 +188,7 @@ GBLREF int4		gtm_trigger_depth;
 GBLREF spdesc		stringpool;
 GBLREF mstr		dollar_zpin;
 GBLREF mstr		dollar_zpout;
+GBLREF mlk_subhash_val_t	mlk_last_hash;
 
 LITREF mval		literal_zero, literal_one, literal_null;
 LITREF char		gtm_release_name[];
@@ -289,7 +290,7 @@ void zshow_svn(zshow_out *output, int one_sv)
 				break;
 		/* CAUTION: fall through */
 		case SV_HOROLOG:
-			op_horolog(&var);
+			op_zhorolog(&var, FALSE);
 			ZS_VAR_EQU(&x, horolog_text);
 			mval_write(output, &var, TRUE);
 			if (SV_ALL != one_sv)
@@ -541,7 +542,7 @@ void zshow_svn(zshow_out *output, int one_sv)
 				break;
 		/* CAUTION: fall through */
 		case SV_ZHOROLOG:
-			op_zhorolog(&var);
+			op_zhorolog(&var, TRUE);
 			ZS_VAR_EQU(&x, zhorolog_text);
 			mval_write(output, &var, TRUE);
 			if (SV_ALL != one_sv)
@@ -612,6 +613,12 @@ void zshow_svn(zshow_out *output, int one_sv)
 			mval_write(output, &var, TRUE);
 			if (SV_ALL != one_sv)
 				break;
+		case SV_ZMLKHASH:
+			i2usmval(&var, mlk_last_hash);
+			ZS_VAR_EQU(&x, zmlkhash_text);
+			mval_write(output, &var, TRUE);
+			if (SV_ALL != one_sv)
+				break;
 		/* CAUTION: fall through */
 		case SV_ZMODE:
 			ZS_VAR_EQU(&x, zmode_text);
@@ -727,7 +734,7 @@ void zshow_svn(zshow_out *output, int one_sv)
 				break;
 		/* CAUTION: fall through */
 		case SV_ZSTRPLLIM:
-			count = TREF(gtm_strpllim);
+			count = stringpool.strpllim;
 			MV_FORCE_MVAL(&var, count);
 			ZS_VAR_EQU(&x, zstrpllim_text);
 			mval_write(output, &var, TRUE);
