@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2017 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -64,7 +64,7 @@ void gtcmd_rundown(connection_struct *cnx, bool clean_exit)
 		{
 			was_crit = cs_addrs->now_crit;
 			if (!was_crit)
-				grab_crit(gv_cur_region);
+				grab_crit(gv_cur_region, NOT_APPLICABLE);
 			if (JNL_ENABLED(cs_data))
 			{
 				jpc->pini_addr = ptr->pini_addr;
