@@ -1,7 +1,8 @@
 #!/bin/sh
 #################################################################
 #								#
-#	Copyright 2009, 2013 Fidelity Information Services, Inc #
+# Copyright (c) 2009-2018 Fidelity National Information		#
+# Services, Inc. and/or its subsidiaries. All rights reserved.	#
 #								#
 #	This source code contains the intellectual property	#
 #	of its copyright holder(s), and is made available	#
@@ -36,14 +37,18 @@ check_files()
 	for each_file in $srch_files
 	do
 		flag=0
-		for each_path in $srch_path
+		list=`echo $each_file | tr '|' ' '`
+		for file in $list
 		do
-			for each_ext in $srch_ext
+			for each_path in $srch_path
 			do
-				if [ -f $each_path/$each_file$each_ext ]; then flag=1 break ; fi
+				for each_ext in $srch_ext
+				do
+					if [ -f $each_path/$file$each_ext ]; then flag=1 break ; fi
+				done
+				# The below takes care of files whose extensions are already determined by the caller
+				if [ -f $each_path/$file ]; then flag=1 break; fi
 			done
-			# The below takes care of files whose extensions are already determined by the caller
-			if [ -f $each_path/$each_file ]; then flag=1 break; fi
 		done
 		if [ $flag -eq 0 ]; then
 			missing="$missing $each_file"
@@ -78,7 +83,7 @@ send_mail()
 hostos=`uname -s`
 hostname=`uname -n | awk -F. '{print $1}'`
 machtype=`uname -m`
-server_list="$cms_tools/server_list"
+server_list="$btc_tools/server_list.csh"
 this_host_noencrypt="FALSE"
 if [ -f $server_list ]; then
 	eval `/usr/local/bin/tcsh -efc "
@@ -102,14 +107,15 @@ fi
 lib_search_path="/usr/local/lib64 /usr/local/lib /usr/lib64 /usr/lib /lib64 /lib /usr/local/ssl/lib /usr/lib/x86_64-linux-gnu"
 lib_search_path="$lib_search_path /usr/lib/i386-linux-gnu /lib/x86_64-linux-gnu /lib/i386-linux-gnu /opt/openssl/0.9.8/lib/hpux64"
 include_search_path="/usr/include /usr/local/include /usr/local/include/gpgme /usr/local/ssl/include /opt/openssl/0.9.8/include"
+include_search_path="$include_search_path /usr/include/i386-linux-gnu /usr/include/x86_64-linux-gnu"
 bin_search_path="/usr/bin /usr/local/bin /bin"
 
 mandate_headers="gpgme.h gpg-error.h"
 mandate_libs="libgpg-error libgpgme"
-mandate_bins="gpg"
-gcrypt_headers="gcrypt.h gcrypt-module.h"
+mandate_bins="gpg|gpg2"
+gcrypt_headers="gcrypt.h"
 gcrypt_libs="libgcrypt"
-openssl_headers="openssl/evp.h openssl/sha.h openssl/blowfish.h openssl/ssl.h openssl/err.h"
+openssl_headers="openssl/evp.h openssl/sha.h openssl/ssl.h openssl/err.h"
 openssl_libs="libcrypto libssl"
 lib_ext=".so"
 if [ "AIX" = "$hostos" ]; then
