@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2013 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2019 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -12,25 +13,22 @@
 #include "mdef.h"
 
 #include <errno.h>
-#include <signal.h>
+#include "gtm_signal.h"
 #ifdef GTM_PTHREAD
-#  include <pthread.h>
+#   include "gtm_pthread.h"
 #endif
-
 #include "ctrlc_handler.h"
 #include "std_dev_outbndset.h"
+#include "outofband.h"		/* for CTRLC and CTRLD */
 
 void ctrlc_handler(int sig)
 {
 	int4     ob_char;
 	int	 save_errno;
 
-	if (SIGINT == sig)
-	{
-		FORWARD_SIG_TO_MAIN_THREAD_IF_NEEDED(sig);
-		save_errno = errno;
-		ob_char = 3;
-		std_dev_outbndset(ob_char);
-		errno = save_errno;
-	}
+	FORWARD_SIG_TO_MAIN_THREAD_IF_NEEDED(sig);
+	save_errno = errno;
+	ob_char = (SIGINT == sig) ? CTRLC : CTRLD;	/* borrowing CTRLD for SIGHUP */
+	std_dev_outbndset(ob_char);
+	errno = save_errno;
 }
