@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2019 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -37,28 +37,31 @@
 #include "dm_read.h"
 #include "gtm_tputs.h"
 #include "op.h"
+#include "send_msg.h"
+#include "svnames.h"
+#include "util.h"
 #ifdef UTF8_SUPPORTED
 #include "gtm_icu_api.h"
 #include "gtm_utf8.h"
 #endif
 
-GBLREF bool		prin_in_dev_failure;
-GBLREF boolean_t	dollar_zininterrupt, gtm_utf8_mode;
-GBLREF char		*CLR_EOL, *CURSOR_DOWN, *CURSOR_LEFT, *CURSOR_RIGHT, *CURSOR_UP;
-GBLREF char		*KEY_BACKSPACE, *KEY_DC, *KEY_DOWN, *KEY_INSERT, *KEY_LEFT, *KEY_RIGHT, *KEY_UP;
-GBLREF char		*KEYPAD_LOCAL, *KEYPAD_XMIT;
-GBLREF int		AUTO_RIGHT_MARGIN, EAT_NEWLINE_GLITCH;
-GBLDEF int		comline_index, recall_num;
-GBLREF io_pair 		io_curr_device, io_std_device;
-GBLREF io_desc		*active_device;
-GBLREF mstr		*comline_base;
-GBLREF mv_stent		*mv_chain;
-GBLREF stack_frame	*frame_pointer;
-GBLREF spdesc 		stringpool;
-GBLREF unsigned char	*msp, *stackbase, *stacktop, *stackwarn;
-GBLREF volatile int4	outofband;
-GBLREF	boolean_t	dmterm_default;
-GBLREF	volatile boolean_t	timer_in_handler;
+GBLREF boolean_t		dmterm_default, dollar_zininterrupt, gtm_utf8_mode, prin_in_dev_failure,
+				prin_out_dev_failure;
+GBLREF char			*CLR_EOL, *CURSOR_DOWN, *CURSOR_LEFT, *CURSOR_RIGHT, *CURSOR_UP;
+GBLREF char			*KEY_BACKSPACE, *KEY_DC, *KEY_DOWN, *KEY_INSERT, *KEY_LEFT, *KEY_RIGHT, *KEY_UP;
+GBLREF char			*KEYPAD_LOCAL, *KEYPAD_XMIT;
+GBLREF int			AUTO_RIGHT_MARGIN, EAT_NEWLINE_GLITCH;
+GBLDEF int			comline_index, recall_num;
+GBLREF io_desc			*active_device;
+GBLREF io_pair			io_curr_device, io_std_device;
+GBLREF mstr			*comline_base;
+GBLREF mval			dollar_zstatus;
+GBLREF mv_stent			*mv_chain;
+GBLREF spdesc			stringpool;
+GBLREF stack_frame		*frame_pointer;
+GBLREF unsigned char		*msp, *stackbase, *stacktop, *stackwarn;
+GBLREF volatile int4		outofband;
+GBLREF volatile boolean_t	timer_in_handler;
 
 LITREF unsigned char	lower_to_upper_table[];
 #ifdef UTF8_SUPPORTED
@@ -85,6 +88,7 @@ static	unsigned char	recall_error_msg[][MAX_ERR_MSG_LEN] =
 };
 
 error_def(ERR_IOEOF);
+error_def(ERR_NOPRINCIO);
 error_def(ERR_STACKCRIT);
 error_def(ERR_STACKOFLOW);
 error_def(ERR_ZINTDIRECT);
@@ -378,13 +382,7 @@ void	dm_read (mval *v)
 				continue;
 		} else if (0 == status)
 		{	/* select() says there's something to read, but read() found zero characters; assume connection dropped. */
-			if (io_curr_device.in == io_std_device.in)
-			{
-				if (!prin_in_dev_failure)
-					prin_in_dev_failure = TRUE;
-				else
-					EXIT(errno);
-			}
+			ISSUE_NOPRINCIO_IF_NEEDED(io_ptr, FALSE, FALSE);		/* FALSE, FALSE: READ tt not socket */
 			tt_ptr->discard_lf = FALSE;
 			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_IOEOF);
 		}
