@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2011-2016 Fidelity National Information	*
+ * Copyright (c) 2011-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -18,24 +18,18 @@
 #include <errno.h>
 #include "gtm_c_stack_trace_semop.h"
 #ifdef DEBUG
-#ifdef SUNOS
-#define SEMVALMAX 65535
-#else
 #define SEMVALMAX 32767
 #endif
-#endif
-
+/* maintain in parallel with do_semop.c */
 #define SEMOP(SEMID, SOPS, NSOPS, RC, TO_WAIT)								\
 {													\
 	int numsems; 											\
 													\
 	for (numsems = NSOPS - 1; numsems >= 0; --numsems)						\
-	{												\
 		CHECK_SEMVAL_GRT_SEMOP(SEMID, SOPS[numsems].sem_num, SOPS[numsems].sem_op);		\
-	}												\
 	if (FORCED_WAIT == TO_WAIT)									\
 	{												\
-		RC = try_semop_get_c_stack(SEMID, SOPS, NSOPS);						\
+		RC = try_semop_get_c_stack(SEMID, SOPS, NSOPS);		/* try with patience */		\
 		if (0 != RC)										\
 		{											\
 			errno = RC;									\
@@ -44,10 +38,8 @@
 	} else												\
 	{												\
 		assert(NO_WAIT == TO_WAIT);								\
-		do											\
-		{											\
-			RC = semop(SEMID, SOPS, NSOPS);							\
-		} while (-1 == RC && EINTR == errno);							\
+		while (-1 == (RC = semop(SEMID, SOPS, NSOPS)) && ((EINTR == errno)))			\
+			;										\
 	}												\
 }
 #endif
