@@ -1,6 +1,6 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;								;
-; Copyright (c) 2006-2017 Fidelity National Information		;
+; Copyright (c) 2006-2020 Fidelity National Information		;
 ; Services, Inc. and/or its subsidiaries. All rights reserved.	;
 ;								;
 ;	This source code contains the intellectual property	;
@@ -33,7 +33,18 @@ LOAD
 	i gldfmt>6 s reghasv550fields=TRUE
 	s reghasv600fields=FALSE
 	i gldfmt>7 s reghasv600fields=TRUE
-	s v631=0
+	s v6312=0
+	if (label="GTCGBDUNX014")!(label="GTCGBDUNX114") set label=hdrlab,v6312=1,update=1  ;autoconvert
+	if (v6312=1) new SIZEOF do v6312init
+	s v6312=0
+	s v6311=0
+	if (label="GTCGBDUNX013")!(label="GTCGBDUNX113") set label=hdrlab,v6311=1,update=1  ;autoconvert
+	if (v6311=1) new SIZEOF do v6312init
+	s v6311=0
+	s v6310=0
+	if (label="GTCGBDUNX012")!(label="GTCGBDUNX112") set label=hdrlab,v6310=1,update=1  ;autoconvert
+	if (v6310=1) new SIZEOF do v6310init
+	set v631=0
 	i (label="GTCGBDUNX011")!(label="GTCGBDUNX111") s label=hdrlab,v631=1,update=1  ;autoconvert
 	i (v631=1) n SIZEOF d v631init
 	s v63a=0
@@ -198,7 +209,9 @@ LOAD
 	. f s="BLOCK_SIZE","BUCKET_SIZE","DEFER" d tmpseg(am,s)
 	. i (gldfmt>9) d tmpseg(am,"DEFER_ALLOCATE")
 	. i (seghasencrflag=TRUE) d tmpseg(am,"ENCRYPTION_FLAG")
-	. f s="EXTENSION_COUNT","FILE_TYPE","GLOBAL_BUFFER_COUNT","LOCK_SPACE" d tmpseg(am,s)
+	. f s="EXTENSION_COUNT","FILE_TYPE" d tmpseg(am,s)
+	. i (gldfmt>13) d tmpseg(am,"FULLBLKWRT")
+	. f s="GLOBAL_BUFFER_COUNT","LOCK_SPACE" d tmpseg(am,s)
 	. i (gldfmt>8) d tmpseg(am,"MUTEX_SLOTS")
 	. i 'v30 d tmpseg(am,"RESERVED_BYTES")					;autoconvert, can be condensed someday
 	. d tmpseg(am,"WINDOW_SIZE")
@@ -326,7 +339,8 @@ region:
 	. s regs(s,"EPOCH_INTERVAL")=300
 	. s regs(s,"SYNC_IO")=0
 	. s regs(s,"YIELD_LIMIT")=8
-	s regs(s,"BUFFER_SIZE")=$$bin2num($ze(rec,rel,rel+1)),rel=rel+2
+	if (gldfmt>14) s regs(s,"BUFFER_SIZE")=$$bin2num($zextract(rec,rel,rel+3)),rel=rel+4
+	if (gldfmt<=14) s regs(s,"BUFFER_SIZE")=$$bin2num($ze(rec,rel,rel+1)),rel=rel+2
 	i regs(s,"BUFFER_SIZE")<minreg("BUFFER_SIZE")  s regs(s,"BUFFER_SIZE")=minreg("BUFFER_SIZE"),update=1
 	s regs(s,"BEFORE_IMAGE")=$$bin2num($ze(rec,rel)),rel=rel+1
 	i $ze(rec,rel,rel+3)'=$tr($j("",4)," ",ZERO) zm gdeerr("INPINTEG")				; 4 chars
@@ -341,11 +355,14 @@ region:
 	i (reghasv600fields=TRUE)  d
 	. s regs(s,"INST_FREEZE_ON_ERROR")=$$bin2num($ze(rec,rel)),rel=rel+1
 	. s regs(s,"QDBRUNDOWN")=$$bin2num($ze(rec,rel)),rel=rel+1
-	e  d
+	else  do
 	. s regs(s,"INST_FREEZE_ON_ERROR")=0
 	. s regs(s,"QDBRUNDOWN")=0
 	s l=$$bin2num($ze(rec,rel)),rel=rel+1 ;jnl_file_len
 	s regs(s,"FILE_NAME")=$ze(rec,rel,rel+l-1),rel=rel+SIZEOF("file_spec")
+	if (gldfmt>14) do
+	. if $$bin2num($zextract(rec,rel,rel+1))'=0 zm gdeerr("INPINTEG")
+	. set rel=rel+2                   								; align
 	i $ze(rec,rel,rel+7)'=$tr($j("",8)," ",ZERO) zm gdeerr("INPINTEG")				; reserved
 	s rel=rel+8											; reserved
 	i (gldfmt>8) s rel=rel+4 ; "gd_region.is_spanned" not needed by GDE
@@ -375,10 +392,18 @@ segment:
 	s l=$$bin2num($ze(rec,rel,rel+1)),rel=rel+2
 	s segs(s,"FILE_NAME")=$ze(rec,rel,rel+l-1),rel=rel+SIZEOF("file_spec")
 	s segs(s,"BLOCK_SIZE")=$$bin2num($ze(rec,rel,rel+1)),rel=rel+2
-	s segs(s,"EXTENSION_COUNT")=$$bin2num($ze(rec,rel,rel+1)),rel=rel+2
+	s segs(s,"FULLBLKWRT")=0
+	if (gldfmt>12) do
+	. if gldfmt=13 do
+	. . if $$bin2num($zextract(rec,rel,rel+1))'=0 zm gdeerr("INPINTEG")
+	. . else  set rel=rel+2
+	. else  set segs(s,"FULLBLKWRT")=$$bin2num($zextract(rec,rel,rel+1)),rel=rel+2
+	. set segs(s,"EXTENSION_COUNT")=$$bin2num($zextract(rec,rel,rel+3)),rel=rel+4
+	if (gldfmt<=12) set segs(s,"EXTENSION_COUNT")=$$bin2num($zextract(rec,rel,rel+1)),rel=rel+2
 	s segs(s,"ALLOCATION")=$$bin2num($ze(rec,rel,rel+3)),rel=rel+4
 	i $ze(rec,rel,rel+ptrsize-1)'=$tr($j("",ptrsize)," ",ZERO) zm gdeerr("INPINTEG")	; reserved for clb
-	s rel=rel+$s((gtm64=TRUE):12,1:4)							; padding
+	if (gldfmt>12) set rel=rel+$select((gtm64=TRUE):8,1:4)							; padding
+	else  set rel=rel+$select((gtm64=TRUE):12,1:4)
 	i $ze(rec,rel,rel+3)'=".DAT" zm gdeerr("INPINTEG")
 	s rel=rel+4
 	s segs(s,"DEFER")=$$bin2num($ze(rec,rel))
@@ -413,7 +438,7 @@ segment:
 	e  do
 	. s segs(s,"ASYNCIO")=defseg("ASYNCIO")
 	. if (seghasencrflag=TRUE)&(gtm64=TRUE) s rel=rel+4 ; Padding bytes for 64 bit platforms
-	i (gldfmt>8) s rel=rel+16	; reserved for runtime fillers
+	if (gldfmt>8) set rel=rel+16     	 ; reserved for runtime fillers
 	s abs=abs+SIZEOF("gd_segment")-v30
 	q
 gblname:(i)
@@ -766,7 +791,7 @@ v63ainit:
 	s SIZEOF("dsk_blk")=512
 	s SIZEOF("max_str")=1048576
 	s SIZEOF("reg_jnl_deq")=4
-	d Init^GDEINITSZ
+	d gvstats^GDEINIT
 	s MAXNAMLN=SIZEOF("mident")-1,MAXREGLN=32,MAXSEGLN=32	; maximum name length allowed is 31 characters
 	s PARNAMLN=31,PARREGLN=31,PARSEGLN=31
 	q
@@ -796,7 +821,69 @@ v631init:
 	s SIZEOF("dsk_blk")=512
 	s SIZEOF("max_str")=1048576
 	s SIZEOF("reg_jnl_deq")=4
-	d Init^GDEINITSZ
+	d gvstats^GDEINIT
 	s MAXNAMLN=SIZEOF("mident")-1,MAXREGLN=32,MAXSEGLN=32	; maximum name length allowed is 31 characters
 	s PARNAMLN=31,PARREGLN=31,PARSEGLN=31
 	q
+v6310init:
+	if (olabel="GTCGBDUNX012") do
+	. s SIZEOF("am_offset")=332
+	. s SIZEOF("file_spec")=256
+	. s SIZEOF("gd_contents")=80
+	. s SIZEOF("gd_header")=16
+	. s SIZEOF("gd_map")=16
+	. s SIZEOF("gd_region")=412
+	. s SIZEOF("gd_region_padding")=0
+	. s SIZEOF("gd_segment")=368
+	else  do
+	. s SIZEOF("am_offset")=340
+	. s SIZEOF("file_spec")=256
+	. s SIZEOF("gd_contents")=120
+	. s SIZEOF("gd_header")=16
+	. s SIZEOF("gd_map")=24
+	. s SIZEOF("gd_region")=424
+	. s SIZEOF("gd_region_padding")=4
+	. s SIZEOF("gd_segment")=384
+	s SIZEOF("blk_hdr")=16
+        s SIZEOF("dsk_blk")=512
+        s SIZEOF("gd_gblname")=40
+        s SIZEOF("gd_inst_info")=SIZEOF("file_spec")    ; --> size of the "gd_inst_info" structure (defined in gdsfhead.h)
+        s SIZEOF("max_str")=1048576
+        s SIZEOF("mident")=32
+        s SIZEOF("reg_jnl_deq")=4
+        s SIZEOF("rec_hdr")=4
+	d gvstats^GDEINIT
+	s MAXNAMLN=SIZEOF("mident")-1,MAXREGLN=32,MAXSEGLN=32   ; maximum name length allowed is 31 characters
+        s PARNAMLN=31,PARREGLN=31,PARSEGLN=31
+        q
+v6312init:
+	if ((olabel="GTCGBDUNX014")!(olabel="GTCGBDUNX013")) do
+        . s SIZEOF("am_offset")=336
+        . s SIZEOF("file_spec")=256
+        . s SIZEOF("gd_contents")=80
+        . s SIZEOF("gd_header")=16
+        . s SIZEOF("gd_map")=16
+        . s SIZEOF("gd_region")=412
+        . s SIZEOF("gd_region_padding")=0
+        . s SIZEOF("gd_segment")=372
+        else  do
+        . s SIZEOF("am_offset")=340
+        . s SIZEOF("file_spec")=256
+        . s SIZEOF("gd_contents")=120
+        . s SIZEOF("gd_header")=16
+        . s SIZEOF("gd_map")=24
+        . s SIZEOF("gd_region")=424
+        . s SIZEOF("gd_region_padding")=4
+        . s SIZEOF("gd_segment")=384
+        s SIZEOF("blk_hdr")=16
+        s SIZEOF("dsk_blk")=512
+        s SIZEOF("gd_gblname")=40
+        s SIZEOF("gd_inst_info")=SIZEOF("file_spec")    ; --> size of the "gd_inst_info" structure (defined in gdsfhead.h)
+        s SIZEOF("max_str")=1048576
+        s SIZEOF("mident")=32
+        s SIZEOF("reg_jnl_deq")=4
+        s SIZEOF("rec_hdr")=4
+        d gvstats^GDEINIT
+        s MAXNAMLN=SIZEOF("mident")-1,MAXREGLN=32,MAXSEGLN=32   ; maximum name length allowed is 31 characters
+        s PARNAMLN=31,PARREGLN=31,PARSEGLN=31
+	q
