@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -89,7 +89,7 @@
 GBLREF	VSIG_ATOMIC_T		forced_exit;
 GBLREF	boolean_t		mupip_jnl_recover;
 GBLREF	boolean_t		is_src_server, is_updproc;
-GBLREF	gd_region		*gv_cur_region;
+GBLREF	gd_region		*ftok_sem_reg, *gv_cur_region;
 GBLREF	sgmnt_addrs		*cs_addrs;
 GBLREF	sgmnt_data_ptr_t	cs_data;
 GBLREF	uint4			process_id;
@@ -193,22 +193,17 @@ int4 gds_rundown(boolean_t cleanup_udi)
 		gvusr_rundown();
 		return EXIT_NRM;
 	}
-	/* If this is a read-only database, simply return */
-	if (cs_data->read_only)
-	{
-		if (0 != sem_rmid(udi->semid))
-			rts_error_csa(CSA_ARG(csa) VARLSTCNT(8) ERR_DBFILERR, 2, DB_LEN_STR(reg),
-				      ERR_TEXT, 2, RTS_ERROR_TEXT("Unable to remove semaphore"), errno);
-		udi->sem_deleted = TRUE;		/* Note that we deleted the semaphore */
-		udi->grabbed_access_sem = FALSE;
-		udi->counter_acc_incremented = FALSE;
-		return EXIT_NRM;
-	}
 	/* If this region has a corresponding statsdb region that is open, close that first. This is needed to ensure
 	 * that the statsdb can safely be deleted at basedb rundown time if we happen to be the last one to rundown the basedb.
 	 */
 	is_statsDB = IS_STATSDB_REG(reg);
-	if (!is_statsDB)
+	if (is_statsDB)
+	{	/* This is a statsdb. Fix reg->read_only & csa->read_write based on csa->orig_read_write.
+		* This is so it reflects real permissions this process has on the statsdb.
+		*/
+		reg->read_only = !csa->orig_read_write;
+		csa->read_write = csa->orig_read_write;		/* Maintain read_only/read_write in parallel */
+	} else
 	{	/* Note that even if the baseDB has RDBF_NOSTATS set, we could have opened the statsDB region
 		 * (for example, if statsDB has read-only permissions, we would have opened it and found it is
 		 * read-only when we tried to add the ^%YGS node and would have disabled stats in the baseDB
@@ -218,20 +213,35 @@ int4 gds_rundown(boolean_t cleanup_udi)
 		BASEDBREG_TO_STATSDBREG(reg, statsDBreg);
 		if (statsDBreg->open)
 		{
-			gv_cur_region = statsDBreg;	/* Switch "gv_cur_region" to do rundown of statsDB */
-			tp_change_reg();
-			gds_rundown(cleanup_udi); /* Ignore errors in statsdb rundown. Continue with baseDB rundown. */
-			gv_cur_region = reg;	/* Restore "gv_cur_region" back to continue rundown of baseDB */
-			tp_change_reg();
-			/* Now that statsdb has been rundown, reset basedb stats back to private memory in case it was
+			/* Now we're running down statsdb, reset basedb stats back to private memory in case it was
 			 * pointing to statsdb shared/mapped memory. Note that the following reset of the stats
 			 * pointer back to the internal stats buffer located is sgmnt_data is normally taken care
 			 * of by the statsdb unlink processing in gvcst_remove_statsDB_linkage() but we keep this
 			 * reset here also to be sure it gets done in case of a statsDB rundown issue.
 			 */
 			csa->gvstats_rec_p = &csa->gvstats_rec;
+			gv_cur_region = statsDBreg;	/* Switch "gv_cur_region" to do rundown of statsDB */
+			tp_change_reg();
+			gds_rundown(cleanup_udi); /* Ignore errors in statsdb rundown. Continue with baseDB rundown. */
+			gv_cur_region = reg;	/* Restore "gv_cur_region" back to continue rundown of baseDB */
+			tp_change_reg();
 		}
 	}
+	/* If this is a read-only database, simply return */
+	if (csd->read_only)
+	{
+		if (INVALID_SEMID != udi->semid)
+			semctl(udi->semid, 0, IPC_RMID);
+		udi->sem_deleted = TRUE;		/* Note that we deleted the semaphore */
+		udi->grabbed_access_sem = FALSE;
+		udi->counter_acc_incremented = FALSE;
+		assert(FALSE == udi->grabbed_ftok_sem);
+		if (INVALID_SEMID != udi->ftok_semid)
+			semctl(udi->ftok_semid, 0, IPC_RMID);
+		udi->counter_ftok_incremented = FALSE;
+		udi->semid = udi->ftok_semid = INVALID_SEMID;
+		return EXIT_NRM;
+	}
 	csa->regcnt--;
 	if (csa->regcnt)
 	{	/* There is at least one more region pointing to the same db file as this region.
@@ -240,13 +250,6 @@ int4 gds_rundown(boolean_t cleanup_udi)
 		reg->open = FALSE;
 		return EXIT_NRM;
 	}
-	if (is_statsDB)
-	{	/* This is a statsdb. Fix reg->read_only & csa->read_write based on csa->orig_read_write.
-		 * This is so it reflects real permissions this process has on the statsdb.
-		 */
-		reg->read_only = !csa->orig_read_write;
-		csa->read_write = csa->orig_read_write;		/* Maintain read_only/read_write in parallel */
-	}
 	/* If the process has standalone access, it has udi->grabbed_access_sem set to TRUE at this point. Note that down in a local
 	 * variable as the udi->grabbed_access_sem is set to TRUE even for non-standalone access below and hence we can't rely on
 	 * that later to determine if the process had standalone access or not when it entered this function.  We need to guarantee
@@ -282,6 +285,8 @@ int4 gds_rundown(boolean_t cleanup_udi)
 	CANCEL_DB_TIMERS(reg, csa, canceled_dbsync_timer);
 	we_are_last_user = FALSE;
 	inst_is_frozen = IS_REPL_INST_FROZEN && REPL_ALLOWED(csa->hdr);
+	if (FREEZE_LATCH_HELD(csa))
+		rel_latch(&cnl->freeze_latch);
 	if (!csa->persistent_freeze)
 		region_freeze(reg, FALSE, FALSE, FALSE, FALSE, FALSE);
 	if (!csa->lock_crit_with_db && LOCK_CRIT_HELD(csa))
@@ -357,9 +362,15 @@ int4 gds_rundown(boolean_t cleanup_udi)
 			/* Check # of processes counted on access sem. */
 			if (-1 == (semval = semctl(udi->semid, DB_COUNTER_SEM, GETVAL)))
 			{
+				save_errno = errno;
+				if ((EINVAL == save_errno) && (cleanup_udi))
+				{
+					REVERT;
+					return EXIT_NRM;
+				}
 				assert(FALSE);
-				rts_error_csa(CSA_ARG(csa) VARLSTCNT(12) ERR_CRITSEMFAIL, 2, DB_LEN_STR(reg), ERR_SYSCALL, 5,
-					      RTS_ERROR_TEXT("gds_rundown SEMCTL failed to get semval"), CALLFROM, errno);
+				rts_error_csa(CSA_ARG(csa) VARLSTCNT(12) ERR_CRITSEMFAIL, 2, DB_LEN_STR(reg), ERR_SYSCALL,
+					      5, RTS_ERROR_TEXT("gds_rundown SEMCTL failed to get semval"), CALLFROM, save_errno);
 			}
 			bypassed_access = CAN_BYPASS(semval, csa, inst_is_frozen) || onln_rlbk_pid || csd->file_corrupt;
 			/* Before attempting again in the blocking mode, see if the holding process is an online rollback.
@@ -368,8 +379,12 @@ int4 gds_rundown(boolean_t cleanup_udi)
 			 */
 			holder_pid = semctl(udi->semid, DB_CONTROL_SEM, GETPID);
 			if ((uint4)-1 == holder_pid)
+			{
+				save_errno = errno;
+				assert(FALSE);
 				rts_error_csa(CSA_ARG(csa) VARLSTCNT(12) ERR_CRITSEMFAIL, 2, DB_LEN_STR(reg), ERR_SYSCALL, 5,
-					      RTS_ERROR_TEXT("gds_rundown SEMCTL failed to get holder_pid"), CALLFROM, errno);
+					      RTS_ERROR_TEXT("gds_rundown SEMCTL failed to get holder_pid"), CALLFROM, save_errno);
+			}
 			if (!bypassed_access)
 			{	/* We couldn't get it in one shot-- see if we already have it */
 				if (holder_pid == process_id)
@@ -512,8 +527,8 @@ int4 gds_rundown(boolean_t cleanup_udi)
 			 */
 			assert(mupip_jnl_recover && !jgbl.onlnrlbk && !safe_mode);
 			if (!was_crit)
-				grab_crit(reg);
-			SET_TRACEABLE_VAR(cnl->wc_blocked, TRUE);
+				grab_crit(reg, WS_40);
+			SET_TRACEABLE_VAR(cnl->wc_blocked, WC_BLOCK_RECOVER);
 			BG_TRACE_PRO_ANY(csa, wcb_gds_rundown1);
                         send_msg_csa(CSA_ARG(csa) VARLSTCNT(8) ERR_WCBLOCKED, 6, LEN_AND_LIT("wcb_gds_rundown1"),
 				     process_id, &csa->ti->curr_tn, DB_LEN_STR(reg));
@@ -534,7 +549,7 @@ int4 gds_rundown(boolean_t cleanup_udi)
 				MM_DBFILEXT_REMAP_IF_NEEDED(csa, reg);
 				cnl->remove_shm = TRUE;
 			}
-			if (cnl->wc_blocked && jgbl.onlnrlbk)
+			if (WC_BLOCK_RECOVER == cnl->wc_blocked && jgbl.onlnrlbk)
 			{	/* if the last update done by online rollback was not committed in the normal code-path but
 				 * was completed by secshr_db_clnup, wc_blocked will be set to TRUE. But, since online
 				 * rollback never invokes grab_crit (since csa->hold_onto_crit is set to TRUE), wcs_recover
@@ -570,7 +585,7 @@ int4 gds_rundown(boolean_t cleanup_udi)
 				/* Since "wcs_flu" failed, set wc_blocked to TRUE if not already set. */
 				if (!cnl->wc_blocked)
 				{
-					SET_TRACEABLE_VAR(cnl->wc_blocked, TRUE);
+					SET_TRACEABLE_VAR(cnl->wc_blocked, WC_BLOCK_RECOVER);
 					BG_TRACE_PRO_ANY(csa, wcb_gds_rundown2);
 					send_msg_csa(CSA_ARG(csa) VARLSTCNT(8) ERR_WCBLOCKED, 6,
 							LEN_AND_LIT("wcb_gds_rundown2"), process_id, &csa->ti->curr_tn,
@@ -639,7 +654,7 @@ int4 gds_rundown(boolean_t cleanup_udi)
 				 */
 				do_jnlwait = FALSE;
 				if (!was_crit)
-					grab_crit(reg);
+					grab_crit(reg, WS_41);
 				if (JNL_ENABLED(csd))
 				{
 					SET_GBL_JREC_TIME; /* jnl_ensure_open/jnl_write_pini/pfin/jnl_file_close all need it */
@@ -713,7 +728,7 @@ int4 gds_rundown(boolean_t cleanup_udi)
 		if (we_are_last_writer && !FROZEN_CHILLED(csa))
 		{	/* Flush the fileheader last and harden the file to disk */
 			if (!was_crit)
-				grab_crit(reg);			/* To satisfy crit requirement in fileheader_sync() */
+				grab_crit(reg, WS_42);			/* To satisfy crit requirement in fileheader_sync() */
 			memset(csd->machine_name, 0, MAX_MCNAMELEN); /* clear the machine_name field */
 			if (we_are_last_user && !CHILLED_AUTORELEASE(csa))
 			{
@@ -917,7 +932,7 @@ int4 gds_rundown(boolean_t cleanup_udi)
 			if (!have_standalone_access)
 			{
 				if (0 != sem_rmid(udi->semid))
-					rts_error_csa(CSA_ARG(csa) VARLSTCNT(8) ERR_DBFILERR, 2, DB_LEN_STR(reg),
+					rts_error_csa(CSA_ARG(csa) VARLSTCNT(9) ERR_DBFILERR, 2, DB_LEN_STR(reg),
 						      ERR_TEXT, 2, RTS_ERROR_TEXT("Unable to remove semaphore"), errno);
 				udi->sem_deleted = TRUE;		/* Note that we deleted the semaphore */
 				udi->grabbed_access_sem = FALSE;
@@ -1048,6 +1063,8 @@ int4 gds_rundown(boolean_t cleanup_udi)
 		assert(seg->file_cntl->file_info);
 		free(seg->file_cntl->file_info);
 		free(seg->file_cntl);
+		assert(NULL == ftok_sem_reg);
+		ftok_sem_reg = NULL;		/* always assigning this is less burden than an if mirroring the above assert */
 		seg->file_cntl = NULL;
 	}
 	return EXIT_NRM;
