@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2014 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2019 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -14,7 +15,7 @@
  * If we are nesting our handlers in an improper way, this routine will
  * not return but will immediately invoke core/termination processing.
  *
- * Returns if some condition makes it inadvisable to exit now else invokes the system exit() system call.
+ * Returns if some condition makes it inadvisable to exit now else invokes the system EXIT() system call.
  * For GTMSECSHR it unconditionally returns to gtmsecshr_signal_handler() which later invokes gtmsecshr_exit().
  */
 
@@ -22,15 +23,12 @@
 
 #include "gtm_string.h"
 #include "gtm_unistd.h"
-#include "gtm_stdlib.h"		/* for exit() */
+#include "gtm_stdlib.h"		/* for EXIT() */
 #include "gtm_inet.h"
-
-#include <signal.h>
-#ifdef GTM_PTHREAD
-#  include <pthread.h>
-#endif
-
+#include "gtm_signal.h"
 #include "gtm_stdio.h"
+
+#include "gtm_multi_thread.h"
 #include "error.h"
 #include "gtmsiginfo.h"
 #include "gtmimagename.h"
@@ -41,9 +39,15 @@
 #include "gtmio.h"
 #include "have_crit.h"
 #include "util.h"
+// For gd_region
+#include "gdsroot.h"
+#include "gdsbt.h"
+#include "gdsblk.h"
+#include "gdsfhead.h"
 
-#define	DEFER_EXIT_PROCESSING	((EXIT_PENDING_TOLERANT >= exit_state) &&	\
-	(exit_handler_active || !OK_TO_INTERRUPT))
+#define	DEFER_EXIT_PROCESSING	((EXIT_PENDING_TOLERANT >= exit_state)				\
+					&& (exit_handler_active || multi_thread_in_use		\
+						|| multi_proc_in_use || !OK_TO_INTERRUPT))
 
 /* Combine send_msg and gtm_putmsg into one macro to conserve space. */
 #define SEND_AND_PUT_MSG(...)					\
@@ -74,6 +78,12 @@ GBLREF	void			(*call_on_signal)();
 GBLREF	boolean_t		gtm_quiet_halt;
 GBLREF	volatile int4           gtmMallocDepth;         /* Recursion indicator */
 GBLREF	volatile boolean_t	timer_active;
+GBLREF	sigset_t		block_sigsent;
+GBLREF	gd_region		*gv_cur_region;
+GBLREF	boolean_t		blocksig_initialized;
+#ifdef DEBUG
+GBLREF	boolean_t		in_nondeferrable_signal_handler;
+#endif
 
 LITREF	gtmImageName		gtmImageNames[];
 
@@ -85,15 +95,51 @@ error_def(ERR_KILLBYSIGSINFO2);
 error_def(ERR_KILLBYSIGSINFO3);
 error_def(ERR_KILLBYSIGUINFO);
 error_def(ERR_KRNLKILL);
+error_def(ERR_STATSDBMEMERR);
+
+static inline void check_for_statsdb_memerr()
+{
+	DCL_THREADGBL_ACCESS;
+
+	SETUP_THREADGBL_ACCESS;
+
+	if (TREF(gvcst_statsDB_open_ch_active))
+	{	/* Case where we've tried to create a stats db block
+		 * Issue an rts error and let the statsDB condition handler
+		 * do the clean up
+		 */
+		TREF(statsdb_memerr) = TRUE;
+		send_msg_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_STATSDBMEMERR, 2, gv_cur_region->dyn.addr->fname_len,
+				gv_cur_region->dyn.addr->fname);
+		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_STATSDBMEMERR, 2, gv_cur_region->dyn.addr->fname_len,
+				gv_cur_region->dyn.addr->fname);
+	}
+}
 
 void generic_signal_handler(int sig, siginfo_t *info, void *context)
 {
 	gtm_sigcontext_t	*context_ptr;
 	void			(*signal_routine)();
+#	ifdef DEBUG
+	boolean_t		save_in_nondeferrable_signal_handler;
+#	endif
 	DCL_THREADGBL_ACCESS;
 
 	SETUP_THREADGBL_ACCESS;
+	assert(!thread_block_sigsent || blocksig_initialized);
+	/* If "thread_block_sigsent" is TRUE, it means the threads do not want the master thread to honor external signals
+	 * anymore until the threads complete. Achieve that effect by returning right away from the signal handler.
+	 */
+	if (thread_block_sigsent && sigismember(&block_sigsent, sig))
+		return;
 	FORWARD_SIG_TO_MAIN_THREAD_IF_NEEDED(sig);
+#	ifdef DEBUG
+	/* Note that it is possible "in_nondeferrable_signal_handler" is non-zero if we first went into timer_handler
+	 * and then came here due to a nested signal (e.g. SIG-15). So save current value of global and restore it at
+	 * end of this function even though we will most often not return to the caller (process will exit mostly).
+	 */
+	save_in_nondeferrable_signal_handler = in_nondeferrable_signal_handler;
+#	endif
 	/* Save parameter value in global variables for easy access in core */
 	dont_want_core = FALSE;		/* (re)set in case we recurse */
 	created_core = FALSE;		/* we can deal with a second core if needbe */
@@ -122,12 +168,13 @@ void generic_signal_handler(int sig, siginfo_t *info, void *context)
 			case SIGSEGV:
 			case SIGBUS:
 			case SIGILL:
+				DEBUG_ONLY(in_nondeferrable_signal_handler = IN_GENERIC_SIGNAL_HANDLER;)
 				if (core_in_progress)
 				{
 					if (exit_handler_active)
-						_exit(sig);
+						UNDERSCORE_EXIT(sig);
 					else
-						exit(sig);
+						EXIT(sig);
 				}
 				++core_in_progress;
 				DUMP_CORE;
@@ -155,6 +202,7 @@ void generic_signal_handler(int sig, siginfo_t *info, void *context)
 						SEND_AND_PUT_MSG(VARLSTCNT(1) forced_exit_err);
 					return;
 				}
+				DEBUG_ONLY(in_nondeferrable_signal_handler = IN_GENERIC_SIGNAL_HANDLER;)
 				exit_state = EXIT_IMMED;
 				SET_PROCESS_EXITING_TRUE; 	/* Set this BEFORE cancelling timers as wcs_phase2_commit_wait
 								 * relies on this.
@@ -163,6 +211,7 @@ void generic_signal_handler(int sig, siginfo_t *info, void *context)
 					SEND_AND_PUT_MSG(VARLSTCNT(1) forced_exit_err);
 			} else
 			{	/* Special case for gtmsecshr - no deferral just exit */
+				DEBUG_ONLY(in_nondeferrable_signal_handler = IN_GENERIC_SIGNAL_HANDLER;)
 				forced_exit_err = ERR_GTMSECSHRSHUTDN;
 				if (OK_TO_SEND_MSG)
 					send_msg_csa(CSA_ARG(NULL) VARLSTCNT(1) forced_exit_err);
@@ -201,6 +250,7 @@ void generic_signal_handler(int sig, siginfo_t *info, void *context)
 				assert(!IS_GTMSECSHR_IMAGE);
 				return;
 			}
+			DEBUG_ONLY(in_nondeferrable_signal_handler = IN_GENERIC_SIGNAL_HANDLER;)
 			exit_state = EXIT_IMMED;
 			SET_PROCESS_EXITING_TRUE;
 			switch(signal_info.infotype)
@@ -238,6 +288,7 @@ void generic_signal_handler(int sig, siginfo_t *info, void *context)
 				assert(!IS_GTMSECSHR_IMAGE);
 				return;
 			}
+			DEBUG_ONLY(in_nondeferrable_signal_handler = IN_GENERIC_SIGNAL_HANDLER;)
 			exit_state = EXIT_IMMED;
 			SET_PROCESS_EXITING_TRUE;
 			SEND_AND_PUT_MSG(VARLSTCNT(1) forced_exit_err);
@@ -249,6 +300,7 @@ void generic_signal_handler(int sig, siginfo_t *info, void *context)
 			switch(signal_info.infotype)
 			{
 				case GTMSIGINFO_NONE:
+					DEBUG_ONLY(in_nondeferrable_signal_handler = IN_GENERIC_SIGNAL_HANDLER;)
 					exit_state = EXIT_IMMED;
 					SET_PROCESS_EXITING_TRUE;
 					SEND_AND_PUT_MSG(VARLSTCNT(6) ERR_KILLBYSIG, 4, GTMIMAGENAMETXT(image_type),
@@ -267,30 +319,42 @@ void generic_signal_handler(int sig, siginfo_t *info, void *context)
 						gtm_fork_n_core();	/* Generate "virgin" core while we can */
 						return;
 					}
+					DEBUG_ONLY(in_nondeferrable_signal_handler = IN_GENERIC_SIGNAL_HANDLER;)
 					exit_state = EXIT_IMMED;
 					SET_PROCESS_EXITING_TRUE;
 					SEND_AND_PUT_MSG(VARLSTCNT(8) ERR_KILLBYSIGUINFO, 6, GTMIMAGENAMETXT(image_type),
 						process_id, sig, signal_info.send_pid, signal_info.send_uid);
 					break;
 				case GTMSIGINFO_ILOC + GTMSIGINFO_BADR:
+					check_for_statsdb_memerr();
+					DEBUG_ONLY(in_nondeferrable_signal_handler = IN_GENERIC_SIGNAL_HANDLER;)
 					exit_state = EXIT_IMMED;
 					SET_PROCESS_EXITING_TRUE;
+					/* SIGABRT is usually delivered when memory corruption is detected by glibc
+					 * e.g.  *** glibc detected *** mupip: double free or corruption (!prev): 0x0983f180 ***
+					 * We want to detect that right when it happens so assert fail in that case.
+					 */
+					assert(SIGABRT != sig);
 					SEND_AND_PUT_MSG(VARLSTCNT(8) ERR_KILLBYSIGSINFO1, 6, GTMIMAGENAMETXT(image_type),
 						process_id, sig, signal_info.int_iadr, signal_info.bad_vadr);
 					break;
 				case GTMSIGINFO_ILOC:
+					DEBUG_ONLY(in_nondeferrable_signal_handler = IN_GENERIC_SIGNAL_HANDLER;)
 					exit_state = EXIT_IMMED;
 					SET_PROCESS_EXITING_TRUE;
 					SEND_AND_PUT_MSG(VARLSTCNT(7) ERR_KILLBYSIGSINFO2, 5, GTMIMAGENAMETXT(image_type),
 						process_id, sig, signal_info.int_iadr);
 					break;
 				case GTMSIGINFO_BADR:
+					check_for_statsdb_memerr();
+					DEBUG_ONLY(in_nondeferrable_signal_handler = IN_GENERIC_SIGNAL_HANDLER;)
 					exit_state = EXIT_IMMED;
 					SET_PROCESS_EXITING_TRUE;
 					SEND_AND_PUT_MSG(VARLSTCNT(7) ERR_KILLBYSIGSINFO3, 5, GTMIMAGENAMETXT(image_type),
 						process_id, sig, signal_info.bad_vadr);
 					break;
 				default:
+					DEBUG_ONLY(in_nondeferrable_signal_handler = IN_GENERIC_SIGNAL_HANDLER;)
 					exit_state = EXIT_IMMED;
 					SET_PROCESS_EXITING_TRUE;
 					assertpro(FALSE && signal_info.infotype);;	/* show signal_info if there's a failure */
@@ -304,6 +368,7 @@ void generic_signal_handler(int sig, siginfo_t *info, void *context)
 	/* Stop the timers but do not cancel them. This allows the timer structures to appear in the core where gtmpcat can
 	 * extract them allowing us to see what was going on.
 	 */
+	assert(in_nondeferrable_signal_handler);
 	if (timer_active)
 		sys_canc_timer();
 	FFLUSH(stdout);
@@ -326,7 +391,7 @@ void generic_signal_handler(int sig, siginfo_t *info, void *context)
 		if (!dont_want_core && IS_MCODE_RUNNING && (NULL != (signal_routine = RFPTR(create_fatal_error_zshow_dmp_fptr))))
 		{	/* note assignment of signal_routine above */
 			SFPTR(create_fatal_error_zshow_dmp_fptr, NULL);
-			(*signal_routine)(exi_condition, FALSE);
+			(*signal_routine)(exi_condition);
 		}
 		/* Some mupip functions define an entry point to drive on signals. Make sure to do this AFTER we create the
 		 * dump file above as it may detach things (like the recvpool) we need to create the above dump.
@@ -340,7 +405,10 @@ void generic_signal_handler(int sig, siginfo_t *info, void *context)
 	if (!IS_GTMSECSHR_IMAGE)
 	{
 		assert((EXIT_IMMED <= exit_state) || !exit_handler_active);
-		exit(-exi_condition);
+		EXIT(-exi_condition);
 	} else
+	{
+		DEBUG_ONLY(in_nondeferrable_signal_handler = save_in_nondeferrable_signal_handler;)
 		return;
+	}
 }
