@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -60,7 +60,7 @@ GBLREF stack_frame	*frame_pointer;
 GBLREF tp_frame		*tp_pointer;
 
 LITDEF char		litconst_space = ' ';
-LITDEF mval		literal_null;
+LITREF mval		literal_null;
 
 error_def(ERR_TXTSRCFMT);
 error_def(ERR_SYSCALL);
@@ -372,7 +372,7 @@ STATICFNDEF boolean_t fill_src_tbl_via_mfile(routine_source **src_tbl_result, rh
 	src_tbl->srcbuff = (0 < srcsize) ? malloc(srcsize + 1) : NULL;
 	base = src_tbl->srclines;
 	srcptr = src_tbl->srcbuff;
-	DEBUG_ONLY(srcptr_max = srcptr + srcsize);
+	srcptr_max = srcptr + srcsize;
 	src_tbl->srcrecs = srcrecs;
 	eof_seen = FALSE;
 	for (current = base + 1, top = base + srcrecs ; current < top ; current++)
@@ -403,13 +403,13 @@ STATICFNDEF boolean_t fill_src_tbl_via_mfile(routine_source **src_tbl_result, rh
 				size = (int)STRLEN(buff);
 				prev_srcptr = srcptr;
 				srcptr += size;
-				if ((NULL == src_tbl->srcbuff) || (srcptr > (src_tbl->srcbuff + srcsize)))
+				if ((NULL == src_tbl->srcbuff) || (srcptr > srcptr_max))
 				{	/* source file has been concurrently overwritten (and extended or truncated) */
 					srcstat |= CHECKSUMFAIL;
 					eof_seen = TRUE;
 					size = 0;
 				} else
-				{
+				{	/* Read size fits in the destination buffer */
 					memcpy(prev_srcptr, buff, size);
 					/* Strip trailing '\n' if any (if at least one byte was read in) */
 					if (size && ('\n' == buff[size - 1]))
