@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2012 Fidelity Information Services, Inc	*
+ * Copyright (c) 2012-2016 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -13,9 +14,12 @@
 
 #include "gt_timer.h"
 #include "gt_timers_add_safe_hndlrs.h"
-#include "heartbeat_timer.h"
 #include "semwt2long_handler.h"
 #include "secshr_client.h"
+#include "jnl_file_close_timer.h"
+#ifdef DEBUG
+#include "fake_enospc.h"
+#endif
 
 /* This optional routine adds entries to the safe_handlers[] array. It is separate because while most executables need
  * these timers listed, there is one executable (gtmsecshr) that decidedly does not - gtmsecshr. If these routines are
@@ -25,5 +29,9 @@
 
 void gt_timers_add_safe_hndlrs(void)
 {
-	add_safe_timer_handler(3, semwt2long_handler, client_timer_handler, heartbeat_timer);
+	add_safe_timer_handler(4, semwt2long_handler, client_timer_handler, simple_timeout_timer,
+				jnl_file_close_timer);
+#	ifdef DEBUG
+	add_safe_timer_handler(2, fake_enospc, handle_deferred_syslog);
+#	endif
 }
