@@ -1,6 +1,7 @@
 /****************************************************************
  *                                                              *
- *      Copyright 2011 Fidelity Information Services, Inc       *
+ * Copyright (c) 2011-2020 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *                                                              *
  *      This source code contains the intellectual property     *
  *      of its copyright holder(s), and is made available       *
@@ -9,8 +10,6 @@
  *                                                              *
  ****************************************************************/
 
-#define STRUCT_STAT_HAS_ST_GEN		defined(__osf__) || defined(_AIX)
-
 #include "mdef.h"
 #include "gtm_ipc.h"
 #include "mmrhash.h"
@@ -34,9 +33,6 @@ gtm_ftok(const char *path, int id)
 
     MurmurHash3_x86_32(&statbuf.st_dev, sizeof statbuf.st_dev, key, &key);
     MurmurHash3_x86_32(&statbuf.st_ino, sizeof statbuf.st_ino, key, &key);
-#if STRUCT_STAT_HAS_ST_GEN
-    MurmurHash3_x86_32(&statbuf.st_gen, sizeof statbuf.st_gen, key, &key);
-#endif
 
     /* substitute the id for the top 8 bits of the hash */
     key &= 0x00ffffff;
