@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -23,7 +23,6 @@
 #define	GTM_BLKUPGRADE_FLAG		"$gtm_blkupgrade_flag"
 #define	GTM_DBFILEXT_SYSLOG_DISABLE	"$gtm_dbfilext_syslog_disable"
 #define	GTM_ENV_XLATE			"$gtm_env_translate"
-#define	GTM_FULLBLOCKWRITES		"$gtm_fullblockwrites"
 #define	GTM_GDSCERT			"$gtm_gdscert"
 #define	GTM_GVDUPSETNOOP		"$gtm_gvdupsetnoop"
 #define	GTM_GBLDIR			"$gtmgbldir"
@@ -73,10 +72,12 @@
 #define	DISABLE_ALIGN_STRINGS		"$gtm_disable_alignstr"
 #define	GTM_MAX_SOCKETS			"$gtm_max_sockets"
 #define	GTM_MEMORY_RESERVE		"$gtm_memory_reserve"
+#define	GTM_NOFFLF			"$gtm_nofflf"
 #define	GTM_NOUNDEF			"$gtm_noundef"
 #define	GTM_PRINCIPAL			"$gtm_principal"
 #define	GTM_PROMPT			"$gtm_prompt"
 #define	GTM_SIDE_EFFECT			"$gtm_side_effects"
+#define	GTM_SOCKET_KEEPALIVE_IDLE	"$gtm_socket_keepalive_idle"
 #define	SYSID				"$gtm_sysid"
 #define	GTM_MPROF_TESTING		"$gtm_trace_gbl_name"
 #define	GTM_TRACE_GROUPS		"$gtm_trace_groups"
@@ -153,4 +154,5 @@
 #define GTM_UTFCGR_STRINGS		"$gtm_utfcgr_strings"
 #define GTM_UTFCGR_STRING_GROUPS	"$gtm_utfcgr_string_groups"
 #define GTM_STRPLLIM			"$gtm_string_pool_limit"
+#define GTM_HUPENABLE			"$gtm_hupenable"
 
