@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2019 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -36,7 +36,6 @@
 #include "filestruct.h"
 #include "jnl.h"
 #include "buddy_list.h"
-#include "hashtab_int4.h"	/* needed for tp.h */
 #include "tp.h"
 #include "interlock.h"
 #include "repl_msg.h"
@@ -90,6 +89,7 @@
 #include "jobsp.h" /* For gcall.h */
 #include "gcall.h" /* For ojchildparms() */
 #include "common_startup_init.h"
+#include "trans_numeric.h"
 #ifdef UTF8_SUPPORTED
 #include "utfcgr.h"
 #endif
@@ -142,7 +142,9 @@ void gtm_startup(struct startup_vector *svec)
 	 * while in UNIX, it's all done with environment variables
 	 * hence, various references to data copied from *svec could profitably be referenced directly
 	 */
+	boolean_t	is_defined;
 	char		*temp;
+	int4		temp_gtm_strpllim;
 	mstr		log_name;
 	stack_frame 	*frame_pointer_lcl;
 	static char 	other_mode_buf[] = "OTHER";
@@ -175,6 +177,7 @@ void gtm_startup(struct startup_vector *svec)
 	else if (svec->user_strpl_size > STP_MAXINITSIZE)
 		svec->user_strpl_size = STP_MAXINITSIZE;
 	stp_init(svec->user_strpl_size);
+	assertpro(stringpool.base);
 	rts_stringpool = stringpool;
 	(TREF(tpnotacidtime)).mvtype = MV_NM | MV_INT;	/* gtm_env_init set up a numeric value, now there's a stp: string it */
 	MV_FORCE_STRD(&(TREF(tpnotacidtime)));
@@ -203,6 +206,12 @@ void gtm_startup(struct startup_vector *svec)
 	log_name.len = STR_LIT_LEN(DISABLE_ALIGN_STRINGS);
 	/* mstr_native_align = logical_truth_value(&log_name, FALSE, NULL) ? FALSE : TRUE; */
 	mstr_native_align = FALSE; /* TODO: remove this line and uncomment the above line */
+	/* See if $gtm_string_pool_limit is set */
+	log_name.addr = GTM_STRPLLIM;
+	log_name.len = SIZEOF(GTM_STRPLLIM) - 1;
+	temp_gtm_strpllim = trans_numeric(&log_name, &is_defined, TRUE);
+	if (0 < temp_gtm_strpllim)
+		stringpool.strpllim = (temp_gtm_strpllim < STP_GCOL_TRIGGER_FLOOR ? STP_GCOL_TRIGGER_FLOOR : 0) + temp_gtm_strpllim;
 	getjobname();
 	getzprocess();
 	getzmode();
