@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2013-2018 Fidelity National Information	*
+ * Copyright (c) 2013-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -35,12 +35,23 @@
 
 #include "gtm_tls_interface.h"
 #include "gtm_tls_impl.h"
+#ifdef DEBUG
+#include <stdlib.h>
+#endif
 
 GBLDEF	int			tls_errno;
 GBLDEF	gtmtls_passwd_list_t	*gtmtls_passwd_listhead;
 
 STATICDEF	config_t	gtm_tls_cfg;
 STATICDEF DH			*dh512, *dh1024;	/* Diffie-Hellman structures for Ephemeral Diffie-Hellman key exchange. */
+#ifdef DEBUG
+STATICDEF	char		*wbox_enable = NULL, *wbox_tls_check = NULL,
+				*wbox_count = NULL, *wbox_test_count = NULL;
+STATICDEF	int		wbox_count_val = 0, wbox_test_count_val = 0, wbox_enable_val = 0, wbox_tls_check_val = 0;
+#define	TRUE	1
+#define	FALSE	0
+#define	WBTEST_REPL_TLS_RECONN 169
+#endif
 
 #define MAX_CONFIG_LOOKUP_PATHLEN	64
 
@@ -53,6 +64,9 @@ STATICDEF DH			*dh512, *dh1024;	/* Diffie-Hellman structures for Ephemeral Diffi
 #ifndef TLS1_2_VERSION
 #define	TLS1_2_VERSION	0x0303
 #endif
+#ifndef TLS1_3_VERSION
+#define	TLS1_3_VERSION	0x0304
+#endif
 
 /* Below template translates to: Arrange ciphers in increasing order of strength after excluding the following:
  * ADH: Anonymous Diffie-Hellman Key Exchange (Since we want both encryption and authentication and ADH provides only former).
@@ -67,6 +81,7 @@ STATICDEF DH			*dh512, *dh1024;	/* Diffie-Hellman structures for Ephemeral Diffi
 #define OPTIONENDSTR ":"
 #define OPTIONNOT '!'
 #define DEFINE_SSL_OP(OP_DEF)   { #OP_DEF , OP_DEF }
+
 struct gtm_ssl_options
 {
 	const char	*opt_str;
@@ -174,6 +189,11 @@ STATICDEF struct gtm_ssl_options gtm_ssl_options_list[] =
 #ifdef	SSL_OP_NO_TLSv1_2
 	DEFINE_SSL_OP(SSL_OP_NO_TLSv1_2),
 #endif
+/*	Special case for NO_TLSv1_3 so defined for pre OpenSSL 1.1.1 */
+#ifndef	SSL_OP_NO_TLSv1_3
+#	define SSL_OP_NO_TLSv1_3	0x0
+#endif
+	DEFINE_SSL_OP(SSL_OP_NO_TLSv1_3),
 #ifdef	SSL_OP_NO_TLSv1_1
 	DEFINE_SSL_OP(SSL_OP_NO_TLSv1_1),
 #endif
@@ -210,6 +230,7 @@ STATICDEF struct gtm_ssl_options gtm_ssl_options_list[] =
 
 STATICFNDEF char *parse_SSL_options(struct gtm_ssl_options *opt_table, size_t opt_table_size, const char *options, long *current,
 					long *clear);
+
 STATICFNDEF char *parse_SSL_options(struct gtm_ssl_options *opt_table, size_t opt_table_size, const char *options, long *current,
 					long *clear)
 {
@@ -261,7 +282,6 @@ STATICFNDEF char *parse_SSL_options(struct gtm_ssl_options *opt_table, size_t op
 	*current = bitmask;
 	return NULL;
 }
-
 #ifdef DEBUG_SSL
 #define SSL_DPRINT(FP, ...)		{fprintf(FP, __VA_ARGS__); fflush(FP);}	/* BYPASSOK -- cannot use FFLUSH. */
 #else
@@ -291,13 +311,30 @@ STATICFNDEF int format_ASN1_TIME(ASN1_TIME *tm, char *buf, int maxlen)
 
 STATICFNDEF int ssl_error(gtm_tls_socket_t *tls_sock, int err, long verify_result)
 {
-	int		error_code;
+	int		ssl_error_code, reason_code, err_lib;
+	unsigned long	error_code;
 	char		*errptr, *end;
 	SSL		*ssl;
+#ifdef DEBUG
+	int		is_wb = FALSE;
+#endif
 
 	ssl = tls_sock->ssl;
-	error_code = SSL_get_error(ssl, err); /* generic error code */
-	switch (error_code)
+	ssl_error_code = SSL_get_error(ssl, err); /* generic error code */
+#ifdef DEBUG
+	/* Change the error code to induce error in case of
+	 * WBTEST_REPL_TLS_RECONN white box.
+	 */
+	if ((1 == wbox_enable_val) && (WBTEST_REPL_TLS_RECONN == wbox_tls_check_val)
+		&& (wbox_test_count_val == wbox_count_val))
+	{
+		SSL_DPRINT(stderr, "Prev code: %d setting code to: %d\n",
+				ssl_error_code,SSL_ERROR_SSL);
+		is_wb = TRUE;
+		ssl_error_code = SSL_ERROR_SSL;
+	}
+#endif
+	switch (ssl_error_code)
 	{
 		case SSL_ERROR_ZERO_RETURN:
 			/* SSL/TLS connection has been closed gracefully. The underlying TCP/IP connection is not yet closed. The
@@ -308,8 +345,9 @@ STATICFNDEF int ssl_error(gtm_tls_socket_t *tls_sock, int err, long verify_resul
 
 		case SSL_ERROR_SYSCALL:
 			tls_errno = errno;
-			if (0 == tls_errno)
+			if (0 == tls_errno)   /* If no error at underlying socket, consisder a connecton reset */
 				tls_errno = ECONNRESET;
+			tls_sock->flags |= GTMTMLS_OP_NOSHUTDOWN;
 			break;
 
 		case SSL_ERROR_WANT_WRITE:
@@ -333,10 +371,40 @@ STATICFNDEF int ssl_error(gtm_tls_socket_t *tls_sock, int err, long verify_resul
 				UPDATE_ERROR_STRING("certificate verification error: %s",
 					X509_verify_cert_error_string(verify_result));
 				return -1;
-			} else if (SSL_ERROR_NONE == error_code)
+			} else if (SSL_ERROR_NONE == ssl_error_code)
 			{	/* we are ignoring verify result and no other error */
 				tls_errno = 0;
-				return 0;
+				/* Return the orginal return value from the prev SSL call,
+				 * since SSL_ERROR_NONE is only returned when ret > 0
+				 */
+				return err;
+			} else if (SSL_ERROR_SSL  == ssl_error_code)
+			{
+				SSL_DPRINT(stderr, "Resetting the err code\n");
+				/* Check the first error in the queue */
+				error_code = ERR_peek_error();
+#ifdef DEBUG
+				assert(error_code || ((1 == wbox_enable_val) && (WBTEST_REPL_TLS_RECONN
+					== wbox_tls_check_val)));
+#endif
+				err_lib = ERR_GET_LIB(error_code);
+				reason_code = ERR_GET_REASON(error_code);
+#ifdef DEBUG
+				if (is_wb)
+				{
+					reason_code = SSL_R_DECRYPTION_FAILED_OR_BAD_RECORD_MAC;
+					err_lib = ERR_LIB_SSL;
+				}
+#endif
+				/*	Change the returned error only for replication
+				 *	and if the error comes from SSL library
+				 */
+				if (!(tls_sock->flags & GTMTLS_OP_SOCKET_DEV) && !(tls_sock->flags & GTMTLS_OP_DM_AUDIT)
+					&& (ERR_LIB_SSL == err_lib) &&
+					((SSL_R_DECRYPTION_FAILED_OR_BAD_RECORD_MAC == reason_code) ||
+					(SSL_R_SSLV3_ALERT_BAD_RECORD_MAC == reason_code)))
+					tls_errno = ECONNRESET;
+				tls_sock->flags |= GTMTMLS_OP_NOSHUTDOWN;
 			}
 			do
 			{
@@ -363,7 +431,7 @@ STATICFNDEF int ssl_error(gtm_tls_socket_t *tls_sock, int err, long verify_resul
 
 		default:
 			tls_errno = -1;
-			UPDATE_ERROR_STRING("Unknown error: %d returned by `SSL_get_error'", error_code);
+			UPDATE_ERROR_STRING("Unknown error: %d returned by `SSL_get_error'", ssl_error_code);
 			assert(FALSE);
 			break;
 	}
@@ -404,7 +472,7 @@ STATICFNDEF int new_session_callback(SSL *ssl, SSL_SESSION *session)
 	socket = SSL_get_app_data(ssl);
 	assert(NULL != socket);
 	/* Free up the old session. */
-	SSL_DPRINT(stdout, "new_session_callback: references=%d\n", session->references);
+	SSL_DPRINT(stderr, "new_session_callback: references=%d\n", session->references);
 	if (socket->session)
 		SSL_SESSION_free(socket->session);
 	/* Add the new session to the `socket' structure. */
@@ -524,6 +592,8 @@ gtm_tls_ctx_t *gtm_tls_init(int version, int flags)
 		return NULL;
 	}
 	SSL_CTX_set_options(ctx, DEPRECATED_SSLTLS_PROTOCOLS);
+	if (0 != SSL_OP_NO_TLSv1_3)
+		SSL_CTX_set_options(ctx, SSL_OP_NO_TLSv1_3);	/* until changes made to handle TLSv1.3 ciphers */
 	/* Read the configuration file for more configuration parameters. */
 	cfg = &gtm_tls_cfg;
 	config_init(cfg);
@@ -1289,8 +1359,11 @@ gtm_tls_socket_t *gtm_tls_socket(gtm_tls_ctx_t *tls_ctx, gtm_tls_socket_t *prev_
 			session_id_len = session_id_len / 2;		/* bytes */
 		} else
 		{
-			strcpy((char *)session_id_string, id);		/* default to tlsid */
 			session_id_len = STRLEN(id);
+			if (SSL_MAX_SSL_SESSION_ID_LENGTH <= session_id_len)
+				session_id_len = SSL_MAX_SSL_SESSION_ID_LENGTH - 1;
+			memcpy((char *)session_id_string, id, session_id_len);		/* default to tlsid */
+			session_id_string[session_id_len] = '\0';
 		}
 		if (0 >= SSL_set_session_id_context(ssl, (const unsigned char *)session_id_string, (unsigned int)session_id_len))
 		{
@@ -1379,15 +1452,15 @@ int gtm_tls_connect(gtm_tls_socket_t *socket)
 	DBG_VERIFY_SOCK_IS_BLOCKING(GET_SOCKFD(socket->ssl));
 	if (NULL != socket->session)
 	{	/* Old session available. Reuse it. */
-		SSL_DPRINT(stdout, "gtm_tls_connect(1): references=%d\n", ((SSL_SESSION *)(socket->session))->references);
+		SSL_DPRINT(stderr, "gtm_tls_connect(1): references=%d\n", ((SSL_SESSION *)(socket->session))->references);
 		if (0 >= (rv = SSL_set_session(socket->ssl, socket->session)))
 			return ssl_error(socket, rv, X509_V_OK);
-		SSL_DPRINT(stdout, "gtm_tls_connect(2): references=%d\n", ((SSL_SESSION *)(socket->session))->references);
+		SSL_DPRINT(stderr, "gtm_tls_connect(2): references=%d\n", ((SSL_SESSION *)(socket->session))->references);
 	}
 	if (0 < (rv = SSL_connect(socket->ssl)))
 	{
 		if (NULL != socket->session)
-			SSL_DPRINT(stdout, "gtm_tls_connect(3): references=%d\n", ((SSL_SESSION *)(socket->session))->references);
+			SSL_DPRINT(stderr, "gtm_tls_connect(3): references=%d\n", ((SSL_SESSION *)(socket->session))->references);
 		verify_result = SSL_get_verify_result(socket->ssl);
 		if (X509_V_OK == verify_result)
 			return 0;
@@ -1676,11 +1749,17 @@ int gtm_tls_get_conn_info(gtm_tls_socket_t *socket, gtm_tls_conn_info *conn_info
 				case TLS1_2_VERSION:
 					ssl_version_ptr = "TLSv1.2";
 					break;
+				case TLS1_3_VERSION:
+					ssl_version_ptr = "TLSv1.3";
+					break;
 				default:
-					assert(FALSE && ssl_version);
+					ssl_version_ptr = NULL;
+					SNPRINTF(conn_info->protocol, SIZEOF(conn_info->protocol), "unknown protocol 0x%X",
+							ssl_version);
 					break;
 			}
-			strncpy(conn_info->protocol, ssl_version_ptr, MAX_ALGORITHM_LEN);
+			if (NULL != ssl_version_ptr)
+				strncpy(conn_info->protocol, ssl_version_ptr, MAX_ALGORITHM_LEN);
 			/* SSL-Session Cipher Algorithm */
 			cipher = SSL_get_current_cipher(ssl);
 			SNPRINTF(conn_info->session_algo, SIZEOF(conn_info->session_algo), "%s", SSL_CIPHER_get_name(cipher));
@@ -1798,9 +1877,20 @@ int gtm_tls_recv(gtm_tls_socket_t * socket, char *buf, int recv_len)
 {
 	int		rv;
 	long		verify_result;
-
 	DBG_VERIFY_SOCK_IS_BLOCKING(GET_SOCKFD(socket->ssl));
-	if (0 < (rv = SSL_read(socket->ssl, buf, recv_len)))
+	rv = SSL_read(socket->ssl, buf, recv_len);
+#ifdef DEBUG
+	/* Emulate an error condition in case of WBTEST_REPL_TLS_RECONN white box*/
+	if ((NULL != (wbox_enable = getenv("gtm_white_box_test_case_enable"))) &&
+		(NULL != (wbox_tls_check = getenv("gtm_white_box_test_case_number"))) &&
+		(NULL != (wbox_count = getenv("wbox_count"))) &&
+		(NULL != (wbox_test_count= getenv("gtm_white_box_test_case_count"))) &&
+		(1 == (wbox_enable_val = atoi(wbox_enable))) && (WBTEST_REPL_TLS_RECONN ==
+			(wbox_tls_check_val = atoi(wbox_tls_check))) && (wbox_test_count_val = atoi(wbox_test_count))
+			&& (wbox_count_val = atoi(wbox_count)) && (wbox_test_count_val == wbox_count_val))
+			rv = -1;
+#endif
+	if (0 < rv)
 	{
 		assert(SSL_ERROR_NONE == SSL_get_error(socket->ssl, rv));
 		verify_result = SSL_get_verify_result(socket->ssl);
@@ -1818,9 +1908,11 @@ int gtm_tls_cachedbytes(gtm_tls_socket_t *socket)
 
 void gtm_tls_socket_close(gtm_tls_socket_t *socket)
 {
-	tls_errno = 0;
 	if ((NULL == socket) || (NULL == socket->ssl))
+	{
+		tls_errno = 0;
 		return;
+	}
 	DBG_VERIFY_SOCK_IS_BLOCKING(GET_SOCKFD(socket->ssl));
 	/* Invoke SSL_shutdown to close the SSL/TLS connection. Although the protocol (and the OpenSSL library) supports
 	 * bidirectional shutdown (which waits for the peer's "close notify" alert as well), we intend to only send the
@@ -1828,7 +1920,12 @@ void gtm_tls_socket_close(gtm_tls_socket_t *socket)
 	 * this function and we don't want to consume additional time waiting for a "close notify" acknowledge signal from the
 	 * other side.
 	 */
-	SSL_shutdown(socket->ssl);
+	if (!(GTMTMLS_OP_NOSHUTDOWN & socket->flags))
+	{
+		tls_errno = 0;
+		SSL_shutdown(socket->ssl);
+	}
+	tls_errno = 0;
 	SSL_free(socket->ssl);
 	socket->ssl = NULL;
 }
@@ -1845,7 +1942,7 @@ void gtm_tls_session_close(gtm_tls_socket_t **socket)
 		gtm_tls_socket_close(sock);
 	if (NULL != (session = sock->session))
 	{
-		SSL_DPRINT(stdout, "gtm_tls_session_close: references=%d\n", session->references);
+		SSL_DPRINT(stderr, "gtm_tls_session_close: references=%d\n", session->references);
 		SSL_SESSION_free(session);
 	}
 	sock->session = NULL;
