@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2006-2018 Fidelity National Information	*
+ * Copyright (c) 2006-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -807,6 +807,9 @@ int	trim_U16_line_term(UChar *buffer, int len);
 #define	UTF8_LEN_STRICT(ptr, len)			\
 	utf8_len_strict((unsigned char *)(ptr), (len))
 
+#define	UTF8_LEN_SILENT(ptr, len)			\
+	utf8_len_silent((unsigned char *)(ptr), (len))
+
 /* This macro is needed to to ensure all utf line terminators are considered non-printable. As of this
  * writing, ICU's u_isprint returns TRUE for LS/PS (Line/Paragraph separator; codepoints 0x2028, 0x2029)
  * and this causes problems in extracting and loading data which contains these codepoints (in UTF8 mode).
@@ -845,13 +848,15 @@ typedef enum
 {
 	err_rts,		/* Use rts_error() */
 	err_stx,		/* Use stx_error() */
-	err_dec			/* Use dec_error() */
+	err_dec,		/* Use dec_error() */
+	err_ignore		/* Get length as best possible without complaint */
 } utf8_err_type;
 
 GBLREF		boolean_t       utf8_patnumeric;
 int		utf8_len(mstr* str);
 int		utf8_len_dec(mstr* str);
 int		utf8_len_stx(mstr* str);
+int		utf8_len_silent(mstr* str);
 int		utf8_len_strict(unsigned char* ptr, int len);
 STATICFNDCL int utf8_len_real(utf8_err_type err_type, mstr* str);
 int		gtm_wcwidth(wint_t code);
