@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2009-2016 Fidelity National Information	*
+ * Copyright (c) 2009-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -55,7 +55,7 @@ typedef	void * gtmcrypt_key_t;
  *
  * Returns:	0 if encryption was initialized successfully; -1 otherwise.
  */
-gtm_status_t	gtmcrypt_init(gtm_int_t flags);
+extern gtm_status_t	gtmcrypt_init(gtm_int_t flags);
 
 /*
  * Return the error string. Use this function to provide the current error status. The function is normally invoked following a
@@ -64,7 +64,7 @@ gtm_status_t	gtmcrypt_init(gtm_int_t flags);
  *
  * Returns:	The error string constructed so far.
  */
-gtm_char_t	*gtmcrypt_strerror(void);
+extern gtm_char_t	*gtmcrypt_strerror(void);
 
 /*
  * Find the key by hash and database path and set up database encryption and decryption state objects, if not created yet. Use this
@@ -86,8 +86,8 @@ gtm_char_t	*gtmcrypt_strerror(void);
  * Returns:	0 if the routine found the key, and either found existing database encryption and decryption state objects or
  * 		initialized them; -1 otherwise.
  */
-gtm_status_t	gtmcrypt_init_db_cipher_context_by_hash(gtmcrypt_key_t *handle, gtm_string_t key_hash,
-				gtm_string_t db_path, gtm_string_t iv);
+extern gtm_status_t	gtmcrypt_init_db_cipher_context_by_hash(gtmcrypt_key_t *handle, gtm_string_t key_hash,
+								gtm_string_t db_path, gtm_string_t iv);
 
 /*
  * Find the key by its name and set up device encryption or decryption state object. Use this function to locate a particular key by
@@ -106,8 +106,8 @@ gtm_status_t	gtmcrypt_init_db_cipher_context_by_hash(gtmcrypt_key_t *handle, gtm
  * Returns:	0 if the routine found the key, and either found existing database encryption and decryption state objects or
  *		initialized them; -1 otherwise.
  */
-gtm_status_t	gtmcrypt_init_device_cipher_context_by_keyname(gtmcrypt_key_t *handle, gtm_string_t key_name,
-				gtm_string_t iv, gtm_int_t operation);
+extern gtm_status_t	gtmcrypt_init_device_cipher_context_by_keyname(gtmcrypt_key_t *handle, gtm_string_t key_name,
+									gtm_string_t iv, gtm_int_t operation);
 
 /*
  * Find the key by the path of the database it corresponds to as well as its own path, and obtain its hash. Use this function to
@@ -122,7 +122,8 @@ gtm_status_t	gtmcrypt_init_device_cipher_context_by_keyname(gtmcrypt_key_t *hand
  *
  * Returns:	0 if the routine found the key and copied its hash to the specified location; -1 otherwise.
  */
-gtm_status_t	gtmcrypt_obtain_db_key_hash_by_keyname(gtm_string_t db_path, gtm_string_t key_path, gtm_string_t *hash_dest);
+extern gtm_status_t	gtmcrypt_obtain_db_key_hash_by_keyname(gtm_string_t db_path, gtm_string_t key_path,
+								gtm_string_t *hash_dest);
 
 /*
  * Release the specified encryption or decryption state object, also releasing the decryption state if database encryption state is
@@ -132,7 +133,7 @@ gtm_status_t	gtmcrypt_obtain_db_key_hash_by_keyname(gtm_string_t db_path, gtm_st
  *
  * Returns:	0 if the operation succeeded; -1 otherwise.
  */
-gtm_status_t	gtmcrypt_release_cipher_context(gtmcrypt_key_t handle);
+extern gtm_status_t	gtmcrypt_release_cipher_context(gtmcrypt_key_t handle);
 
 /*
  * Perform encryption or decryption of the provided data based on the specified encryption / decryption state. If the target buffer
@@ -153,8 +154,8 @@ gtm_status_t	gtmcrypt_release_cipher_context(gtmcrypt_key_t handle);
  *
  * Returns:	0 if the operation succeeded; -1 otherwise.
  */
-gtm_status_t	gtmcrypt_encrypt_decrypt(gtmcrypt_key_t handle, gtm_char_t *src_block, gtm_int_t src_block_len,
-					  gtm_char_t *dest_block, gtm_int_t operation, gtm_int_t iv_mode, gtm_string_t iv);
+extern gtm_status_t	gtmcrypt_encrypt_decrypt(gtmcrypt_key_t handle, gtm_char_t *src_block, gtm_int_t src_block_len,
+						  gtm_char_t *dest_block, gtm_int_t operation, gtm_int_t iv_mode, gtm_string_t iv);
 
 /*
  * Compare the keys associated with two encryption or decryption state objects.
@@ -164,13 +165,13 @@ gtm_status_t	gtmcrypt_encrypt_decrypt(gtmcrypt_key_t handle, gtm_char_t *src_blo
  *
  * Returns:	1 if both encryption or decryption state objects use the same key; 0 otherwise.
  */
-gtm_int_t	gtmcrypt_same_key(gtmcrypt_key_t handle1, gtmcrypt_key_t handle2);
+extern gtm_int_t	gtmcrypt_same_key(gtmcrypt_key_t handle1, gtmcrypt_key_t handle2);
 
 /*
  * Disable encryption and discard any sensitive data in memory.
  *
  * Returns:	0 if the operation succeeded; -1 otherwise.
  */
-gtm_status_t	gtmcrypt_close(void);
+extern gtm_status_t	gtmcrypt_close(void);
 
 #endif
