@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2008, 2013 Fidelity Information Services, Inc.*
+ * Copyright (c) 2008-2020 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -44,6 +45,11 @@
 #include "memcoherency.h"
 #include "replgbl.h"
 #include "gtmsource.h"
+#ifdef GTM_TLS
+#include "gtm_repl.h"
+#endif
+#include "jnl.h"
+#include "repl_filter.h"
 
 GBLREF	repl_msg_ptr_t		gtmrecv_msgp;
 GBLREF	int			gtmrecv_max_repl_msglen;
@@ -60,9 +66,8 @@ GBLREF	uint4			log_interval;
 GBLREF	volatile time_t		gtmrecv_now;
 GBLREF	boolean_t		gtmrecv_send_cmp2uncmp;
 GBLREF	repl_conn_info_t	*remote_side;
-GBLREF	jnlpool_addrs		jnlpool;
-GBLREF	jnlpool_ctl_ptr_t	jnlpool_ctl;
-GBLREF	jnlpool_addrs		jnlpool;
+GBLREF	jnlpool_addrs_ptr_t	jnlpool;
+GBLREF	int			gtmrecv_filter;
 
 error_def(ERR_RECVPOOLSETUP);
 error_def(ERR_REPLCOMM);
@@ -102,14 +107,13 @@ int gtmrecv_poll_actions1(int *pending_data_len, int *buff_unprocessed, unsigned
 	unsigned char		*msg_ptr;				/* needed for REPL_{SEND,RECV}_LOOP */
 	int			tosend_len, sent_len, sent_this_iter;	/* needed for REPL_SEND_LOOP */
 	int			torecv_len, recvd_len, recvd_this_iter;	/* needed for REPL_RECV_LOOP */
-	int			status;					/* needed for REPL_{SEND,RECV}_LOOP */
+	int			status, poll_dir;			/* needed for REPL_{SEND,RECV}_LOOP */
 	int			temp_len, pending_msg_size;
 	int			upd_start_status, upd_start_attempts;
 	int			buffered_data_len;
 	int			upd_exit_status;
 	seq_num			temp_send_seqno;
 	boolean_t		bad_trans_detected = FALSE, onln_rlbk_flg_set = FALSE;
-	uint4			jnl_status;
 	recvpool_ctl_ptr_t	recvpool_ctl;
 	upd_proc_local_ptr_t	upd_proc_local;
 	gtmrecv_local_ptr_t	gtmrecv_local;
@@ -123,12 +127,19 @@ int gtmrecv_poll_actions1(int *pending_data_len, int *buff_unprocessed, unsigned
 	upd_proc_local = recvpool.upd_proc_local;
 	gtmrecv_local = recvpool.gtmrecv_local;
 	upd_helper_ctl = recvpool.upd_helper_ctl;
-	jnl_status = 0;
 	if (SHUTDOWN == gtmrecv_local->shutdown)
 	{
 		repl_log(gtmrecv_log_fp, TRUE, TRUE, "Shutdown signalled\n");
 		gtmrecv_end(); /* Won't return */
 	}
+#	ifdef GTM_TLS
+	/* If we sent a REPL_RENEG_ACK, then we cannot afford to send anymore asynchronous messages (like XOFF_ACK_ME) until we
+	 * receive a REPL_RENEG_COMPLETE from the source server. This ensures that while the source server attempts to do a SSL/TLS
+	 * renegotiation, it doesn't have any application data (like XOFF_ACK_ME) sitting in the pipe.
+	 */
+	if (REPLTLS_WAITING_FOR_RENEG_COMPLETE == repl_tls.renegotiate_state)
+		return STOP_POLL;
+#	endif
 	/* Reset report_cnt and next_report_at to 1 when a new upd proc is forked */
 	if ((1 == report_cnt) || (report_cnt == next_report_at))
 	{
@@ -152,8 +163,8 @@ int gtmrecv_poll_actions1(int *pending_data_len, int *buff_unprocessed, unsigned
 			if (1 == report_cnt)
 			{
 				send_xoff = TRUE;
-				QWASSIGN(recvpool_ctl->old_jnl_seqno, recvpool_ctl->jnl_seqno);
-				QWASSIGNDW(recvpool_ctl->jnl_seqno, 0);
+				recvpool_ctl->old_jnl_seqno = recvpool_ctl->jnl_seqno;
+				recvpool_ctl->jnl_seqno = 0;
 				/* Even though we have identified that the update process is NOT alive, a waitpid on the update
 				 * process PID is necessary so that the system doesn't leave any zombie process lying around.
 				 * This is possible since any child process that dies without the parent doing a waitpid on it
@@ -193,8 +204,7 @@ int gtmrecv_poll_actions1(int *pending_data_len, int *buff_unprocessed, unsigned
 		 */
 		onln_rlbk_flg_set = TRUE;
 		send_xoff = TRUE;
-	}
-	else if (!send_cmp2uncmp && gtmrecv_send_cmp2uncmp)
+	} else if (!send_cmp2uncmp && gtmrecv_send_cmp2uncmp)
 	{
 		send_xoff = TRUE;
 		send_seqno = recvpool_ctl->jnl_seqno;
@@ -424,7 +434,7 @@ int gtmrecv_poll_actions1(int *pending_data_len, int *buff_unprocessed, unsigned
 			bad_trans_msg.len  = GTM_BYTESWAP_32(MIN_REPL_MSGLEN);
 			bad_trans_msg.start_seqno = GTM_BYTESWAP_64(send_seqno);
 		}
-		REPL_SEND_LOOP(gtmrecv_sock_fd, &bad_trans_msg, bad_trans_msg.len, REPL_POLL_NOWAIT)
+		REPL_SEND_LOOP(gtmrecv_sock_fd, &bad_trans_msg, MIN_REPL_MSGLEN, REPL_POLL_NOWAIT)
 			; /* Empty Body */
 		if (SS_NORMAL == status)
 		{
@@ -483,7 +493,7 @@ int gtmrecv_poll_actions1(int *pending_data_len, int *buff_unprocessed, unsigned
 		/* Reset last_rcvd_histinfo, last_valid_histinfo etc. as they reflect context from unprocessed data
 		 * in the receive pool and those are no longer valid because we have drained the receive pool.
 		 */
-		GTMRECV_CLEAR_CACHED_HISTINFO(recvpool.recvpool_ctl, jnlpool, jnlpool_ctl, INSERT_STRM_HISTINFO_FALSE);
+		GTMRECV_CLEAR_CACHED_HISTINFO(recvpool.recvpool_ctl, jnlpool, INSERT_STRM_HISTINFO_FALSE);
 		if (UPDPROC_START == upd_proc_local->start_upd)
 		{
 			/* Attempt starting the update process */
@@ -528,10 +538,10 @@ int gtmrecv_poll_actions1(int *pending_data_len, int *buff_unprocessed, unsigned
 			gtmrecv_wait_for_jnl_seqno = TRUE;/* set this to TRUE to break out and go back to a fresh "do_main_loop" */
 			if (onln_rlbk_flg_set)
 			{
-				assert(NULL != jnlpool_ctl);
+				assert(jnlpool && (NULL != jnlpool->jnlpool_ctl));
 				repl_log(gtmrecv_log_fp, TRUE, TRUE, "Closing connection due to ONLINE ROLLBACK\n");
  				repl_log(gtmrecv_log_fp, TRUE, TRUE, "REPL INFO - Current Jnlpool Seqno : %llu\n",
- 						jnlpool_ctl->jnl_seqno);
+ 						jnlpool->jnlpool_ctl->jnl_seqno);
 				repl_log(gtmrecv_log_fp, TRUE, TRUE, "REPL INFO - Current Receive Pool Seqno : %llu\n",
 						recvpool_ctl->jnl_seqno);
 				repl_close(&gtmrecv_sock_fd);
@@ -544,9 +554,9 @@ int gtmrecv_poll_actions1(int *pending_data_len, int *buff_unprocessed, unsigned
 				 * just syncing the journal pool cycles as the databases are not opened. But, to be safe, grab
 				 * the lock and sync the cycles.
 				 */
-				grab_lock(jnlpool.jnlpool_dummy_reg, TRUE, GRAB_LOCK_ONLY);
+				grab_lock(jnlpool->jnlpool_dummy_reg, TRUE, GRAB_LOCK_ONLY);
 				SYNC_ONLN_RLBK_CYCLES;
-				rel_lock(jnlpool.jnlpool_dummy_reg);
+				rel_lock(jnlpool->jnlpool_dummy_reg);
 				return_status = STOP_POLL;
 				recvpool_ctl->jnl_seqno = 0;
 			} else
@@ -575,9 +585,12 @@ int gtmrecv_poll_actions1(int *pending_data_len, int *buff_unprocessed, unsigned
 			repl_log_fd2fp(&gtmrecv_log_fp, gtmrecv_log_fd);
 			repl_log(gtmrecv_log_fp, TRUE, TRUE, "Change log to %s successful\n",gtmrecv_local->log_file);
 		}
-		upd_proc_local->changelog = gtmrecv_local->changelog; /* Pass changelog request to the update process */
 		/* NOTE: update process and receiver each ignore any setting specific to the other (REPLIC_CHANGE_UPD_LOGINTERVAL,
 		 * REPLIC_CHANGE_LOGINTERVAL) */
+		if (REPLIC_CHANGE_LOGINTERVAL == gtmrecv_local->changelog)
+			upd_proc_local->changelog = 0;
+		else
+			upd_proc_local->changelog = gtmrecv_local->changelog; /* Pass changelog request to the update process */
 		gtmrecv_local->changelog = 0;
 	}
 	if (0 == *pending_data_len && !gtmrecv_logstats && gtmrecv_local->statslog)
@@ -590,6 +603,12 @@ int gtmrecv_poll_actions1(int *pending_data_len, int *buff_unprocessed, unsigned
 		/* Force all data out to the file before closing the file */
 		repl_log(gtmrecv_log_fp, TRUE, TRUE, "End statistics logging\n");
 	}
+	if ((0 == *pending_data_len) && (gtmrecv_filter &  EXTERNAL_FILTER) && ('\0' == gtmrecv_local->filter_cmd[0]))
+	{
+		repl_log(gtmrecv_log_fp, TRUE, TRUE, "Stopping filter\n");
+		repl_stop_filter();
+		gtmrecv_filter &= ~EXTERNAL_FILTER;
+	}
 	if (0 == *pending_data_len)
 	{
 		if (upd_helper_ctl->start_helpers)
