@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2014 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -29,13 +30,13 @@
 #include "gtm_stat.h"
 #include "gtm_socket.h"
 #include <sys/param.h>
-#include <signal.h>
 #if !defined(_AIX) && !defined(__linux__) && !defined(__hpux) && !defined(__CYGWIN__) && !defined(__MVS__)
 # include <siginfo.h>
 #endif
 #include "gtm_syslog.h"
 #include "gtm_limits.h"
 #include "gtm_stdlib.h"
+#include "gtm_signal.h"
 #include "gtm_sem.h"
 #include "gtm_string.h"
 #include "gtm_un.h"
@@ -50,7 +51,6 @@
 #if defined(__MVS__)
 # include "gtm_zos_io.h"
 #endif
-
 #include "cli.h"
 #include "error.h"
 #include "gtm_logicals.h"
@@ -76,13 +76,15 @@
 #include "have_crit.h"
 #include "sig_init.h"
 #include "gtmio.h"
-#include "file_head_read.h"
-#include "file_head_write.h"
 #include "common_startup_init.h"
 #include "gtm_threadgbl_init.h"
 #include "hashtab.h"
 #include "fork_init.h"
-#ifdef UNICODE_SUPPORTED
+#include "jnl.h"
+#include "repl_msg.h"			/* needed by gtmsource.h */
+#include "gtmsource.h"			/* for anticipatory_freeze.h */
+#include "anticipatory_freeze.h"
+#ifdef UTF8_SUPPORTED
 # include "gtm_icu_api.h"
 # include "gtm_utf8.h"
 #endif
@@ -91,8 +93,6 @@
 #define execname 	"gtmsecshr"	/* this is what this executable is supposed to be called. A different name is verboten */
 #define intent_open	"for open"	/* FLUSH_DB_IPCS_INFO types */
 #define intent_close	"for close"
-#define TZLOCATOR	"TZ="
-#define NEWLINE		0x0a
 
 GBLDEF CLI_ENTRY	*cmd_ary = NULL; /* GTMSECSHR does not have any command tables so initialize command array to NULL */
 
@@ -214,7 +214,6 @@ int main(int argc, char_ptr_t argv[])
 	int			save_errno;
 	int			recv_complete, send_complete;
 	int			num_chars_recd, num_chars_sent, rundir_len;
-	int4			msec_timeout;			/* timeout in milliseconds */
 	TID			timer_id;
 	GTM_SOCKLEN_TYPE	client_addr_len;
 	char			*recv_ptr, *send_ptr, *rundir;
@@ -225,13 +224,14 @@ int main(int argc, char_ptr_t argv[])
 	DCL_THREADGBL_ACCESS;
 
 	GTM_THREADGBL_INIT;
+	assert(MAXPOSINT4 >= GTMSECSHR_MESG_TIMEOUT);
 	common_startup_init(GTMSECSHR_IMAGE); 	/* Side-effect : Sets skip_dbtriggers = TRUE if platorm lacks trigger support */
 	err_init(gtmsecshr_cond_hndlr);
 	gtmsecshr_init(argv, &rundir, &rundir_len);
 	timer_id = (TID)main;
 	while (TRUE)
 	{
-		input_timeval.tv_sec  = MAX_TIMEOUT_VALUE;	/* Restart timeout each interation for platforms that save
+		input_timeval.tv_sec = MAX_TIMEOUT_VALUE;	/* Restart timeout each interation for platforms that save
 								 * unexpired time when select exits.
 								 */
 		input_timeval.tv_usec = 0;
@@ -239,7 +239,7 @@ int main(int argc, char_ptr_t argv[])
 		FD_ZERO(&wait_on_fd);
 		FD_SET(gtmsecshr_sockfd, &wait_on_fd);
 		gtmsecshr_timer_popped = FALSE;
-		SELECT(gtmsecshr_sockfd+1, (void *)&wait_on_fd, NULL, NULL, &input_timeval, selstat);
+		SELECT(gtmsecshr_sockfd + 1, (void *)&wait_on_fd, NULL, NULL, &input_timeval, selstat);
 		if (0 > selstat)
 			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_GTMSECSHR, 1, process_id, ERR_GTMSECSHRSCKSEL, 0, errno);
 		else if (0 == selstat)
@@ -249,9 +249,8 @@ int main(int argc, char_ptr_t argv[])
 		}
 		recv_ptr = (char *)&mesg;
 		client_addr_len = SIZEOF(struct sockaddr_un);
-		msec_timeout = timeout2msec(GTMSECSHR_MESG_TIMEOUT);
-		DBGGSSHR((LOGFLAGS, "gtmsecshr: Select rc = %d  message timeout = %d\n", selstat, msec_timeout));
-		start_timer(timer_id, msec_timeout, gtmsecshr_timer_handler, 0, NULL);
+		DBGGSSHR((LOGFLAGS, "gtmsecshr: Select rc = %d  message timeout = %d\n", selstat, GTMSECSHR_MESG_TIMEOUT));
+		start_timer(timer_id, GTMSECSHR_MESG_TIMEOUT, gtmsecshr_timer_handler, 0, NULL);
 		recv_complete = FALSE;
 		do
 		{	/* Note RECVFROM does not loop on EINTR return codes so must be handled */
@@ -274,8 +273,7 @@ int main(int argc, char_ptr_t argv[])
 		if (INVALID_COMMAND != mesg.code)
 		{	/* Reply if code not overridden to mean no acknowledgement required */
 			send_ptr = (char *)&mesg;
-			msec_timeout = timeout2msec(GTMSECSHR_MESG_TIMEOUT);
-			start_timer(timer_id, msec_timeout, gtmsecshr_timer_handler, 0, NULL);
+			start_timer(timer_id, GTMSECSHR_MESG_TIMEOUT, gtmsecshr_timer_handler, 0, NULL);
 			send_complete = FALSE;
 			do
 			{
@@ -293,7 +291,7 @@ int main(int argc, char_ptr_t argv[])
 			cancel_timer(timer_id);
 		} else
 			DBGGSSHR((LOGFLAGS, "gtmsecshr: SENDTO reply skipped due to mesg.code = INVALID_COMMAND\n"));
-		assert('a' > 'F' && 'a' > '9');
+		assert(('a' > 'F') && ('a' > '9'));
 		if ('a' <= client_addr.sun_path[strlen(client_addr.sun_path) - 1])
 			clean_client_sockets(client_addr.sun_path);
 	}
@@ -312,15 +310,10 @@ void gtmsecshr_init(char_ptr_t argv[], char **rundir, int *rundir_len)
 	gtmsecshr_mesg	mesg;
 	struct stat	stat_buf;
 	char		*gtm_tmp_ptr;
-	int		rc;
+	int		status;
 	int		lib_gid;
 	struct stat	dist_stat_buff;
-#	ifdef _AIX
-	FILE		*envfile;
-	int		recnum, reclen;
-	char		*fgets_rc, *newtz;
-	boolean_t	tzfnd;
-#	endif
+	intrpt_state_t	prev_intrpt_state;
 	DCL_THREADGBL_ACCESS;
 
 	SETUP_THREADGBL_ACCESS;
@@ -394,80 +387,9 @@ void gtmsecshr_init(char_ptr_t argv[], char **rundir, int *rundir_len)
 			RTS_ERROR_LITERAL("Server"), process_id, ERR_GTMSECSHRBADDIR);
 		gtmsecshr_exit(BADGTMDISTDIR, FALSE);
 	}
-#	ifdef _AIX
-	/* Note time is handled very oddly on AIX. If one undefines the TZ environment variable (such as our wrapper does to
-	 * prevent reporting time in the timezone of the process that started gtmsecshr, process time reverts to GMT. So to
-	 * prevent that, lookup the default timezone in /etc/environment and use it to prevent whacky time reporting by
-	 * gtmsecshr in the operator log. Secure file read method is to switch dir first, then do relative open.
-	 */
-	if (-1 == CHDIR("/etc"))	/* Note chdir is changed again below so this is only temporary */
-	{
-		save_errno = errno;
-		send_msg_csa(CSA_ARG(NULL) VARLSTCNT(10) ERR_GTMSECSHRSTART, 3, RTS_ERROR_LITERAL("Server"), process_id, ERR_TEXT,
-			2, RTS_ERROR_LITERAL("Error during chdir to /etc - TZ cannot be determined"), save_errno);
-	} else
-	{
-		envfile = fopen("environment", "r");
-		if (NULL == envfile)
-		{
-			save_errno = errno;
-			send_msg_csa(CSA_ARG(NULL) VARLSTCNT(10) ERR_GTMSECSHRSTART, 3,
-				RTS_ERROR_LITERAL("Server"), process_id, ERR_TEXT, 2,
-				RTS_ERROR_LITERAL("Failed to read /etc/environment - TZ cannot be determined"), save_errno);
-		} else
-		{	/* /etc/environments is open, locate TZ= record */
-			tzfnd = FALSE;
-			for (recnum = 0; ; recnum++)
-			{	/* Reuse realpathbef as it is not otherwise needed any longer */
-				FGETS(realpathbef, GTM_PATH_MAX, envfile, fgets_rc);
-				if (NULL == fgets_rc)
-					break;
-				if (STRLEN(TZLOCATOR) >= (reclen = STRLEN(realpathbef)))
-					continue;
-				if (0 == memcmp(TZLOCATOR, realpathbef, STRLEN(TZLOCATOR)))
-				{
-					tzfnd = TRUE;
-					break;
-				}
-			}
-			if (!tzfnd)
-			{	/* We didn't find the TZ record - report it depending if we just hit EOF or something more
-				 * severe.
-				 */
-				if (!feof(envfile))
-				{
-					save_errno = errno;
-					send_msg_csa(CSA_ARG(NULL) VARLSTCNT(10) ERR_GTMSECSHRSTART, 3,
-						RTS_ERROR_LITERAL("Server"), process_id, ERR_TEXT, 2,
-						RTS_ERROR_LITERAL("Error reading /etc/environment - TZ cannot be determined"),
-						save_errno);
-				} else
-					/* Have EOF - didn't find TZ */
-					send_msg_csa(CSA_ARG(NULL) VARLSTCNT(9) ERR_GTMSECSHRSTART, 3,
-						RTS_ERROR_LITERAL("Server"), process_id, ERR_TEXT, 2,
-						RTS_ERROR_LITERAL("TZ cannot be determined"));
-			} else
-			{	/* TZ record acquired - isolate TZ - malloc space to put it once determine exact length */
-				if (NEWLINE == realpathbef[reclen - 1])
-					realpathbef[reclen-- - 1] = '\0';	/* Overwrite nl with null and decr length */
-				newtz = malloc(reclen);				/* putenv arg must be new - no stack vars */
-				strcpy(newtz, realpathbef);
-				rc = putenv(newtz);
-				if (0 != rc)
-				{
-					save_errno = errno;
-					send_msg_csa(CSA_ARG(NULL) VARLSTCNT(10) ERR_GTMSECSHRSTART, 3,
-						RTS_ERROR_LITERAL("Server"), process_id, ERR_TEXT, 2,
-						RTS_ERROR_LITERAL("TZ reset with putenv() failed"), save_errno);
-				}
-			}
-			fclose(envfile);
-		}
-	}
-#	endif /* _AIX */
 	/*
-      **** With invocation validated, begin our priviledge escalation ****
-      */
+	 **** With invocation validated, begin our priviledge escalation ****
+	 */
 	if (-1 == setuid(ROOTUID))
 	{
 		save_errno = errno;
@@ -479,9 +401,9 @@ void gtmsecshr_init(char_ptr_t argv[], char **rundir, int *rundir_len)
 	 * the logging capability disappears on some systems too - On others, it takes the executable name instead.
 	 * Either one causes our tests to fail.
 	 */
-	DEFER_INTERRUPTS(INTRPT_IN_LOG_FUNCTION);
+	DEFER_INTERRUPTS(INTRPT_IN_LOG_FUNCTION, prev_intrpt_state);
 	CLOSELOG();
-	ENABLE_INTERRUPTS(INTRPT_IN_LOG_FUNCTION);
+	ENABLE_INTERRUPTS(INTRPT_IN_LOG_FUNCTION, prev_intrpt_state);
 	first_syslog = TRUE;
 	FORK(pid);
 	if (0 > pid)
@@ -489,10 +411,10 @@ void gtmsecshr_init(char_ptr_t argv[], char **rundir, int *rundir_len)
 		save_errno = errno;
 		send_msg_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_GTMSECSHRSTART, 3,
 			RTS_ERROR_LITERAL("Server"), process_id, ERR_GTMSECSHRFORKF, 0, save_errno);
-		exit(GNDCHLDFORKFLD);
+		EXIT(GNDCHLDFORKFLD);
 	} else if (0 < pid)
 		/* This is the original process - it dies quietly (no exit handler of any sort) to isolate us */
-		_exit(EXIT_SUCCESS);
+		UNDERSCORE_EXIT(EXIT_SUCCESS);
 	/****** We are now in the (isolated) child process ******/
 	getjobnum();
 	pid = getsid(process_id);
@@ -507,9 +429,9 @@ void gtmsecshr_init(char_ptr_t argv[], char **rundir, int *rundir_len)
 	/* Close standard IO devices - we don't need/want them as all IO goes to operator log. Else we would have IO devices
 	 * tied to whatever process started gtmsecshr up which we definitely don't want.
 	 */
-	CLOSEFILE(0, rc);	/* No stdin */
-	CLOSEFILE(1, rc);	/* No stdout */
-	CLOSEFILE(2, rc);	/* No stderr */
+	CLOSEFILE(0, status);	/* No stdin */
+	CLOSEFILE(1, status);	/* No stdout */
+	CLOSEFILE(2, status);	/* No stderr */
 	/* Init signal handling which works slightly different than in other utilities - gtmsecshr has its own handler which
 	 * *calls* generic_signal_handler (which always returns for gtmsecshr) - we then drive our normal exit handling.
 	 */
@@ -519,18 +441,19 @@ void gtmsecshr_init(char_ptr_t argv[], char **rundir, int *rundir_len)
 	{	/* Close the file only if we have it open. This is to avoid a CLOSEFAIL error in case of
 		 * trying to close an invalid file descriptor.
 		 */
-		CLOSEFILE_IF_OPEN(file_des, rc);
+		CLOSEFILE_IF_OPEN(file_des, status);
 	}
 	if (-1 == CHDIR(P_tmpdir))	/* Switch to temporary directory as CWD */
 	{
 		save_errno = errno;
-		send_msg_csa(CSA_ARG(NULL) VARLSTCNT(10) ERR_GTMSECSHRSTART, 3,
+		send_msg_csa(CSA_ARG(NULL) VARLSTCNT(10) MAKE_MSG_SEVERE(ERR_GTMSECSHRSTART), 3,
 			RTS_ERROR_LITERAL("Server"), process_id, ERR_GTMSECSHRCHDIRF, 2, LEN_AND_STR(P_tmpdir), save_errno);
-		exit(UNABLETOCHDIR);
+		EXIT(UNABLETOCHDIR);
 	}
 	umask(0);
 	if (0 != gtmsecshr_pathname_init(SERVER, *rundir, *rundir_len))
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(5) ERR_GTMSECSHRSOCKET, 3, RTS_ERROR_LITERAL("Server path"), process_id);
+		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(5) MAKE_MSG_SEVERE(ERR_GTMSECSHRSOCKET), 3, RTS_ERROR_LITERAL("Server path"),
+			process_id);
 	if (-1 == (secshr_sem = semget(gtmsecshr_key, FTOK_SEM_PER_ID, RWDALL | IPC_NOWAIT)))
 	{
 		secshr_sem = INVALID_SEMID;
@@ -551,7 +474,7 @@ void gtmsecshr_init(char_ptr_t argv[], char **rundir, int *rundir_len)
 	SEMOP(secshr_sem, sop, 2, semop_res, NO_WAIT);
 	if (0 > semop_res)
 	{
-		send_msg_csa(CSA_ARG(NULL) VARLSTCNT(10) MAKE_MSG_SEVERE(ERR_GTMSECSHRSTART), 3,
+		send_msg_csa(CSA_ARG(NULL) VARLSTCNT(10) MAKE_MSG_WARNING(ERR_GTMSECSHRSTART), 3,
 			RTS_ERROR_LITERAL("Server"), process_id, ERR_TEXT, 2,
 			RTS_ERROR_LITERAL("server already running"), errno);
 		/* If gtm_tmp is not defined, show default path */
@@ -563,7 +486,8 @@ void gtmsecshr_init(char_ptr_t argv[], char **rundir, int *rundir_len)
 		gtmsecshr_exit(SEMAPHORETAKEN, FALSE);
 	}
 	if (0 != gtmsecshr_sock_init(SERVER))
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(5) ERR_GTMSECSHRSOCKET, 3, RTS_ERROR_LITERAL("Server"), process_id);
+		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(5) MAKE_MSG_SEVERE(ERR_GTMSECSHRSOCKET), 3, RTS_ERROR_LITERAL("Server"),
+			process_id);
 	if (-1 == Stat(gtmsecshr_sock_name.sun_path, &stat_buf))
 		send_msg_csa(CSA_ARG(NULL) VARLSTCNT(10) MAKE_MSG_WARNING(ERR_GTMSECSHRSTART), 3,
 			RTS_ERROR_LITERAL("Server"), process_id, ERR_TEXT, 2,
@@ -623,7 +547,7 @@ void gtmsecshr_exit(int exit_code, boolean_t dump)
 			send_msg_csa(CSA_ARG(NULL) VARLSTCNT(3) ERR_GTMSECSHRREMSEMFAIL, 1, errno);
 	}
 	/* Note shutdown message taken care of by generic_signal_handler */
-	exit(exit_code);
+	EXIT(exit_code);
 }
 
 void gtmsecshr_timer_handler(void)
@@ -645,14 +569,16 @@ void gtmsecshr_signal_handler(int sig, siginfo_t *info, void *context)
 
 void service_request(gtmsecshr_mesg *buf, int msglen, char *rundir, int rundir_len)
 {
-	int			fn_len, index, basind, save_errno, save_code;
+	int			flags, fn_len, index, basind, save_errno, save_code;
 	int			stat_res, fd;
 	char			*basnam, *fn;
 	struct shmid_ds		temp_shmctl_buf;
 	struct stat		statbuf;
-	sgmnt_data		header;
+	sgmnt_data		header, *csd;
 	endian32_struct		check_endian;
-	char			*intent;
+	char			*intent, *buff;
+	boolean_t		fd_opened_with_o_direct;
+	uint4			fsb_size;
 	ZOS_ONLY(int		realfiletag;)
 	DCL_THREADGBL_ACCESS;
 
@@ -858,7 +784,7 @@ void service_request(gtmsecshr_mesg *buf, int msglen, char *rundir, int rundir_l
 			fn = buf->mesg.db_ipcs.fn;
 			fn_len = buf->mesg.db_ipcs.fn_len;
 			if ((GTM_PATH_MAX < fn_len) || ('\0' != *(fn + fn_len))
-			    || (fn_len != (msglen - GTM_MESG_HDR_SIZE - offsetof(ipcs_mesg, fn[0]) - 1)))
+				|| (fn_len != (msglen - GTM_MESG_HDR_SIZE - offsetof(ipcs_mesg, fn[0]) - 1)))
 			{
 				buf->code = EINVAL;
 				send_msg_csa(CSA_ARG(NULL) VARLSTCNT(12) ERR_GTMSECSHRSRVFID, 6,
@@ -867,7 +793,14 @@ void service_request(gtmsecshr_mesg *buf, int msglen, char *rundir, int rundir_l
 				break;
 			}
 			/* First open and read-in the fileheader */
-			OPENFILE(fn, O_RDWR, fd);
+			flags = O_RDWR;
+			if (buf->mesg.db_ipcs.open_fd_with_o_direct)
+			{
+				fd_opened_with_o_direct = TRUE;
+				flags = flags | O_DIRECT_FLAGS;
+			} else
+				fd_opened_with_o_direct = FALSE;
+			OPENFILE(fn, flags, fd); /* udi not available so OPENFILE_DB not used */
 			if (FD_INVALID == fd)
 			{
 				save_errno = buf->code = errno;
@@ -909,7 +842,14 @@ void service_request(gtmsecshr_mesg *buf, int msglen, char *rundir, int rundir_l
 				CLOSEFILE_RESET(fd, save_errno);	/* resets "fd" to FD_INVALID */
 				break;
 			}
-			LSEEKREAD(fd, 0, &header, SIZEOF(header), save_errno);
+			if (fd_opened_with_o_direct)
+			{
+				fsb_size = get_fs_block_size(fd);
+				DIO_BUFF_EXPAND_IF_NEEDED_NO_UDI(SGMNT_HDR_LEN, fsb_size, &(TREF(dio_buff)));
+				csd = (sgmnt_data *)(TREF(dio_buff)).aligned;
+			} else
+				csd = &header;
+			DB_LSEEKREAD(((unix_db_info *)NULL), fd, 0, csd, SGMNT_HDR_LEN, save_errno);
 			if (0 != save_errno)
 			{
 				buf->code = save_errno;
@@ -919,7 +859,7 @@ void service_request(gtmsecshr_mesg *buf, int msglen, char *rundir, int rundir_l
 				CLOSEFILE_RESET(fd, save_errno);	/* resets "fd" to FD_INVALID */
 				break;
 			}
-			if (0 != memcmp(header.label, GDS_LABEL, GDS_LABEL_SZ - 1))	/* Verify is GT.M database file */
+			if (0 != memcmp(csd->label, GDS_LABEL, GDS_LABEL_SZ - 1))	/* Verify is GT.M database file */
 			{
 				buf->code = ERR_DBNOTGDS;
 				send_msg_csa(CSA_ARG(NULL) VARLSTCNT(12) ERR_GTMSECSHRSRVFID, 6,
@@ -929,7 +869,7 @@ void service_request(gtmsecshr_mesg *buf, int msglen, char *rundir, int rundir_l
 				break;
 			}
 			/* It would be easier to use the CHECK_DB_ENDIAN macro here but we'd prefer it didn't raise rts_error */
-			check_endian.word32 = header.minor_dbver;
+			check_endian.word32 = csd->minor_dbver;
 			if (!check_endian.shorts.ENDIANCHECKTHIS)
 			{
 				buf->code = ERR_DBENDIAN;
@@ -995,15 +935,12 @@ void service_request(gtmsecshr_mesg *buf, int msglen, char *rundir, int rundir_l
 				}
 			}
 			/* Update file header fields */
-			header.semid = buf->mesg.db_ipcs.semid;
-			header.shmid = buf->mesg.db_ipcs.shmid;
-			header.gt_sem_ctime.ctime = buf->mesg.db_ipcs.gt_sem_ctime;
-			header.gt_shm_ctime.ctime = buf->mesg.db_ipcs.gt_shm_ctime;
-			/* And flush the changes back out. Note this service code is only used by read-only processes so
-			 * we don't need any anticipatory freeze insertion which anyway pulls in the world so LSEEKWRITE does
-			 * what we need.
-			 */
-			LSEEKWRITE(fd, 0, &header, SIZEOF(header), save_errno);
+			csd->semid = buf->mesg.db_ipcs.semid;
+			csd->shmid = buf->mesg.db_ipcs.shmid;
+			csd->gt_sem_ctime.ctime = buf->mesg.db_ipcs.gt_sem_ctime;
+			csd->gt_shm_ctime.ctime = buf->mesg.db_ipcs.gt_shm_ctime;
+			/* And flush the changes back out. */
+			GTMSECSHR_DB_LSEEKWRITE(((unix_db_info *)NULL), fd, 0, csd, SGMNT_HDR_LEN, save_errno);
 			if (0 != save_errno)
 			{
 				buf->code = save_errno;
@@ -1059,7 +996,7 @@ int validate_receiver(gtmsecshr_mesg *buf, char *rundir, int rundir_len, int sav
 	ppptr = (char*)i2asc((uchar_ptr_t)ppptr, buf->mesg.id);
 	ppptr_save = ppptr;			/* Save where adding cmdline so can replace if need to move to check #2 */
 	memcpy(ppptr, PROCPATH_CMDLSUFFIX, SIZEOF(PROCPATH_CMDLSUFFIX));	/* Copy includes terminating null of literal */
-	procstrm = Fopen(procpath, "r");
+	Fopen(procstrm, procpath, "r");
 	if (NULL == procstrm)
 	{
 		save_errno = errno;
@@ -1082,7 +1019,7 @@ int validate_receiver(gtmsecshr_mesg *buf, char *rundir, int rundir_len, int sav
 	FCLOSE(procstrm, clrv);
 	if (-1 == clrv)
 		/* Not a functional issue so just warn about it in op-log */
-		send_msg_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_SYSCALL, 5, LEN_AND_LIT("fclose()"), CALLFROM, errno);
+		send_msg_csa(CSA_ARG(NULL) VARLSTCNT(8) MAKE_MSG_WARNING(ERR_SYSCALL), 5, LEN_AND_LIT("fclose()"), CALLFROM, errno);
 	lnln = STRLEN(cmdbuf);
 	/* Look from the end backwards to find the last '/' to isolate the directory */
 	for (cptr = cmdbuf + lnln - 1; (cptr >= cmdbuf) && ('/' != *cptr); cptr--)
@@ -1102,7 +1039,7 @@ int validate_receiver(gtmsecshr_mesg *buf, char *rundir, int rundir_len, int sav
 	}
 	/* Check #1 failed - attempt check #2 - read /proc/<pid>/maps to see if libgtmshr is there */
 	memcpy(ppptr_save, PROCPATH_MAPSSUFFIX, SIZEOF(PROCPATH_MAPSSUFFIX));	/* Copy includes terminating null of literal */
-	procstrm = Fopen(procpath, "r");
+	Fopen(procstrm, procpath, "r");
 	if (NULL == procstrm)
 	{
 		save_errno = errno;
@@ -1116,7 +1053,7 @@ int validate_receiver(gtmsecshr_mesg *buf, char *rundir, int rundir_len, int sav
 	FCLOSE(procstrm, clrv);
 	if (-1 == clrv)
 		/* Not a functional issue so just warn about it in op-log */
-		send_msg_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_SYSCALL, 5, LEN_AND_LIT("fclose()"), CALLFROM, errno);
+		send_msg_csa(CSA_ARG(NULL) VARLSTCNT(8) MAKE_MSG_WARNING(ERR_SYSCALL), 5, LEN_AND_LIT("fclose()"), CALLFROM, errno);
 #	endif
 	return 0;
 }
