@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2006-2018 Fidelity National Information	*
+ * Copyright (c) 2006-2019 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -619,24 +619,25 @@ void gtmsource_recv_ctl(void)
 				repl_log(gtmsource_log_fp, TRUE, TRUE,
 					"Connection reset while attempting to receive from secondary."
 					" Status = %d ; %s\n", status, STRERROR(status));
+			} else
+			{
+				repl_log(gtmsource_log_fp, TRUE, TRUE,
+					"Error receiving Control message from Receiver. Error in recv."
+					" Status = %d ; %s\n", status, STRERROR(status));
+				repl_log_conn_info(gtmsource_sock_fd, gtmsource_log_fp, TRUE);
+				SEND_SYSMSG_REPLCOMM(LEN_AND_LIT("Error receiving Control message from Receiver. Error in recv."));
+			}
 				repl_close(&gtmsource_sock_fd);
 				SHORT_SLEEP(GTMSOURCE_WAIT_FOR_RECEIVER_CLOSE_CONN);
 				gtmsource_state = gtmsource_local->gtmsource_state
 					= GTMSOURCE_WAITING_FOR_CONNECTION;
 				return;
-			} else
-			{
-				SNPRINTF(err_string, SIZEOF(err_string),
-						"Error receiving Control message from Receiver. Error in recv : %s",
-						STRERROR(status));
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_REPLCOMM, 0, ERR_TEXT, 2,
-						 LEN_AND_STR(err_string));
-			}
 		} else if (EREPL_SELECT == repl_errno)
 		{
 			SNPRINTF(err_string, SIZEOF(err_string),
 					"Error receiving Control message from Receiver. Error in select : %s",
 					STRERROR(status));
+			SEND_SYSMSG_REPLCOMM(LEN_AND_STR(err_string));
 			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_REPLCOMM, 0, ERR_TEXT, 2,
 					 LEN_AND_STR(err_string));
 		}
@@ -687,6 +688,7 @@ int gtmsource_process(void)
 	gtmsource_state_t		gtmsource_state_sav;
 	uint4				phase2_commit_index1;
 	int				phase2_commit_wait_cnt;
+	boolean_t			close_retry = FALSE;
 	DEBUG_ONLY(uchar_ptr_t		save_inbuff;)
 	DEBUG_ONLY(uchar_ptr_t		save_outbuff;)
 	DCL_THREADGBL_ACCESS;
@@ -847,15 +849,35 @@ int gtmsource_process(void)
 					repl_log(gtmsource_log_fp, TRUE, TRUE,
 					       "Connection reset while sending XOFF_ACK due to possible update process shutdown. "
 					       "Status = %d ; %s\n", status, STRERROR(status));
+						close_retry = TRUE;
+				} else
+#				ifdef _AIX
+				if (ENETUNREACH == status)
+#				else
+				if (ECOMM == status) /*Communication error in send */
+#				endif
+				{
+					repl_log(gtmsource_log_fp, TRUE, TRUE,"Error sending XOFF_ACK_ME message."
+						" Error in send : %d, %s\n", status, STRERROR(status));
+					repl_log_conn_info(gtmsource_sock_fd, gtmsource_log_fp, TRUE);
+					close_retry = TRUE;
+				}
+				if (close_retry)
+				{
 					repl_close(&gtmsource_sock_fd);
 					SHORT_SLEEP(GTMSOURCE_WAIT_FOR_RECEIVER_CLOSE_CONN);
 					gtmsource_state = gtmsource_local->gtmsource_state = GTMSOURCE_WAITING_FOR_CONNECTION;
+					close_retry = FALSE;
 					continue;
-				}
-				SNPRINTF(err_string, SIZEOF(err_string), "Error sending XOFF_ACK_ME message. Error in send : %s",
-						STRERROR(status));
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_REPLCOMM, 0, ERR_TEXT, 2,
+				} else
+				{
+					SNPRINTF(err_string, SIZEOF(err_string),
+						"Error sending XOFF_ACK_ME message. Error in send : %s\n",
+							STRERROR(status));
+					SEND_SYSMSG_REPLCOMM(LEN_AND_STR(err_string));
+					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_REPLCOMM, 0, ERR_TEXT, 2,
 						LEN_AND_STR(err_string));
+				}
 			} else if (EREPL_SELECT == repl_errno)
 			{
 				SNPRINTF(err_string, SIZEOF(err_string), "Error receiving RESTART SEQNO/sending XOFF_ACK_ME.  "
@@ -1695,30 +1717,61 @@ int gtmsource_process(void)
 						gtmsource_flush_fh(post_read_seqno);
 						if (GTMSOURCE_HANDLE_ONLN_RLBK == gtmsource_state)
 							break; /* the outerloop will continue */
-						if (REPL_CONN_RESET(status) && EREPL_SEND == repl_errno)
-						{
-							repl_log(gtmsource_log_fp, TRUE, TRUE,
-								"Connection reset while sending seqno data from "
-								INT8_FMT" "INT8_FMTX" to "INT8_FMT" "INT8_FMTX
-								". Status = %d ; %s\n", pre_read_seqno, pre_read_seqno,
-								post_read_seqno, post_read_seqno, status, STRERROR(status));
-							repl_close(&gtmsource_sock_fd);
-							SHORT_SLEEP(GTMSOURCE_WAIT_FOR_RECEIVER_CLOSE_CONN);
-							gtmsource_state = gtmsource_local->gtmsource_state
-								= GTMSOURCE_WAITING_FOR_CONNECTION;
-							break;
-						}
 						if (EREPL_SEND == repl_errno)
 						{
-							SNPRINTF(err_string, SIZEOF(err_string),
-								"Error sending DATA. Error in send : %s", STRERROR(status));
-							rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_REPLCOMM, 0, ERR_TEXT, 2,
-								  LEN_AND_STR(err_string));
+							if (REPL_CONN_RESET(status))
+							{
+								repl_log(gtmsource_log_fp, TRUE, TRUE,
+									"Connection reset while sending seqno data from "
+									INT8_FMT" "INT8_FMTX" to "INT8_FMT" "INT8_FMTX
+									". Status = %d ; %s\n", pre_read_seqno,
+									pre_read_seqno, post_read_seqno, post_read_seqno,
+									 status, STRERROR(status));
+								close_retry = TRUE;
+							} else
+# 							ifdef _AIX
+							if (ENETUNREACH == status)
+#							else
+							if (ECOMM == status) /*Communication error in send */
+#							endif
+							{
+								repl_log(gtmsource_log_fp, TRUE, TRUE,
+									"Error sending DATA"
+									" from "INT8_FMT" "INT8_FMTX" to "INT8_FMT" "
+									INT8_FMTX" . Error in send: %s\n",
+									pre_read_seqno, pre_read_seqno, post_read_seqno,
+									post_read_seqno, STRERROR(status));
+								repl_log_conn_info(gtmsource_sock_fd, gtmsource_log_fp,
+											TRUE);
+								SNPRINTF(err_string, SIZEOF(err_string),
+                                                                	"Error sending DATA. Error in send : %s",
+										STRERROR(status));
+								close_retry = TRUE;
+							}
+							if (close_retry)
+							{
+								repl_close(&gtmsource_sock_fd);
+								SHORT_SLEEP(GTMSOURCE_WAIT_FOR_RECEIVER_CLOSE_CONN);
+								gtmsource_state = gtmsource_local->gtmsource_state
+									= GTMSOURCE_WAITING_FOR_CONNECTION;
+								close_retry = FALSE;
+								break;
+							} else
+							{
+								SNPRINTF(err_string, SIZEOF(err_string),
+                                                                	"Error sending DATA. Error in send : %s",
+										STRERROR(status));
+								SEND_SYSMSG_REPLCOMM(LEN_AND_STR(err_string));
+								rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_REPLCOMM, 0,
+									 ERR_TEXT, 2, LEN_AND_STR(err_string));
+							}
 						}
 						if (EREPL_SELECT == repl_errno)
 						{
 							SNPRINTF(err_string, SIZEOF(err_string),
-								"Error sending DATA. Error in select : %s", STRERROR(status));
+								"Error sending DATA. Error in select : %s",
+											STRERROR(status));
+							SEND_SYSMSG_REPLCOMM(LEN_AND_STR(err_string));
 							rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_REPLCOMM, 0, ERR_TEXT, 2,
 								  LEN_AND_STR(err_string));
 						}
