@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -357,7 +357,6 @@ MBSTART {											\
 	gv_cur_region = SAVE_REG;			\
 }
 
-GBLREF	int4			gtm_fullblockwrites;	/* Do full (not partial) database block writes */
 GBLREF	boolean_t		is_src_server;
 GBLREF  boolean_t               mupip_jnl_recover;
 GBLREF	gd_region		*gv_cur_region, *db_init_region;
@@ -383,10 +382,12 @@ LITREF  int4                    gtm_release_name_len;
 
 OS_PAGE_SIZE_DECLARE
 
+error_def(ERR_DBFILNOFULLWRT);
 error_def(ERR_BADDBVER);
 error_def(ERR_CRITSEMFAIL);
 error_def(ERR_DBBLKSIZEALIGN);
 error_def(ERR_DBCREINCOMP);
+error_def(ERR_DBFILERDONLY);
 error_def(ERR_DBFILERR);
 error_def(ERR_DBFLCORRP);
 error_def(ERR_DBGLDMISMATCH);
@@ -494,7 +495,15 @@ gd_region *dbfilopn(gd_region *reg)
 	{
 		if (!open_read_only)
 		{
+#ifdef DEBUG
+			if (WBTEST_ENABLED(WBTEST_OPENFILE_DB))
+			{
+				udi->fd = FD_INVALID;
+				errno = gtm_white_box_test_case_count ? gtm_white_box_test_case_count : EPERM;
+			} else
+#endif
 			OPENFILE_DB(fnptr, O_RDWR, udi, seg);
+			save_errno = errno;
 			/* When opening an auto-create type of reservedDB database file, it is possible
 			 * that it does not (yet) exist. In that case, make a call out to create it. Note that
 			 * a statsdb (which is also an autodb) gets created in "gvcst_init" so skip that here.
@@ -592,6 +601,13 @@ gd_region *dbfilopn(gd_region *reg)
 			reg->read_only = TRUE;		/* maintain csa->read_write simultaneously */
 			csa->read_write = FALSE;	/* maintain reg->read_only simultaneously */
 			csa->orig_read_write = FALSE;
+			if (!open_read_only && !((EPERM == save_errno) || (EACCES == save_errno)))
+			{
+				if (!IS_GTM_IMAGE)
+					gtm_putmsg_csa(CSA_ARG(csa) VARLSTCNT(6) ERR_DBFILERDONLY, 3,
+							DB_LEN_STR(reg), (int)0, save_errno);
+				send_msg_csa(CSA_ARG(csa) VARLSTCNT(6) ERR_DBFILERDONLY, 3, DB_LEN_STR(reg), (int)0, save_errno);
+			}
 		}
 		if (!reg->owning_gd->is_dummy_gbldir && (pool_init || !jnlpool_init_needed || !CUSTOM_ERRORS_AVAILABLE))
 			break;
@@ -1146,6 +1162,7 @@ int db_init(gd_region *reg, boolean_t ok_to_bypass)
 	 */
 	reg->dyn.addr->acc_meth = tsd->acc_meth;
 	reg->dyn.addr->read_only = tsd->read_only;
+	reg->dyn.addr->full_blkwrt = tsd->write_fullblk;
 	COPY_AIO_SETTINGS(reg->dyn.addr, tsd);	/* copy "asyncio" from tsd to reg->dyn.addr */
 	new_shm_ipc = udi->shm_created;
 	if (new_shm_ipc && !tsd->read_only)
@@ -1375,7 +1392,7 @@ int db_init(gd_region *reg, boolean_t ok_to_bypass)
 #			include "tab_db_csh_acct_rec.h"
 #			undef TAB_DB_CSH_ACCT_REC
 		}
-		cnl->wc_blocked = FALSE; 	/* Since we are creating shared memory, reset wc_blocked to FALSE */
+		SET_TRACEABLE_VAR(cnl->wc_blocked, WC_UNBLOCK);	/* Since we are creating shared memory, reset wc_blocked to 0 */
 		gvstats_rec_csd2cnl(csa);	/* should be called before "db_auto_upgrade" */
 		reg->dyn.addr->ext_blk_count = csd->extension_size;
 		mlk_shr_init((sm_uc_ptr_t)csa->mlkctl, csd->lock_space_size, csa, (FALSE == read_only));
@@ -1711,7 +1728,7 @@ int db_init(gd_region *reg, boolean_t ok_to_bypass)
 		udi->counter_ftok_incremented = FALSE;
 		assert(-1 != status);	/* since we hold the access control lock, we do not expect any errors */
 	}
-	if (gtm_fullblockwrites)
+	if (csd->write_fullblk)
 	{	/* We have been asked to do FULL BLOCK WRITES for this database. On *NIX, attempt to get the filesystem
 		 * blocksize from statvfs. This allows a full write of a blockwithout the OS having to fetch the old
 		 * block for a read/update operation. We will round the IOs to the next filesystem blocksize if the
@@ -1728,8 +1745,21 @@ int db_init(gd_region *reg, boolean_t ok_to_bypass)
 		 */
 		fbwsize = get_fs_block_size(udi->fd);
 		dblksize = csd->blk_size;
-		if (0 != fbwsize && (0 == dblksize % fbwsize) && (0 == (BLK_ZERO_OFF(csd->start_vbn)) % fbwsize))
-			csa->do_fullblockwrites = gtm_fullblockwrites;		/* This region is fullblockwrite enabled */
+		if (0 == fbwsize || (0 != dblksize % fbwsize) || (0 != (BLK_ZERO_OFF(csd->start_vbn)) % fbwsize))
+		{
+			if (!IS_STATSDB_REGNAME(reg))
+			{
+				if (!fbwsize)
+					gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(7) ERR_DBFILNOFULLWRT, 5,
+					LEN_AND_LIT("Could not get native filesize"),
+					LEN_AND_LIT("File size extracted: "), fbwsize);
+				else if (0 != dblksize % fbwsize)
+					gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(7) ERR_DBFILNOFULLWRT, 5,
+					LEN_AND_LIT("Database block size not a multiple of file system block size\n"),
+					LEN_AND_LIT("DB blocks size: "), dblksize);
+			}
+			csd->write_fullblk = 0;		/* This region is not fullblockwrite enabled */
+		}
 		/* Report this length in DSE even if not enabled */
 		csa->fullblockwrite_len = fbwsize;		/* Length for rounding fullblockwrite */
 	}
