@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2019 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -31,16 +31,17 @@
 #include "gtm_stdlib.h"
 #include "min_max.h"
 #include "arit.h"
-#ifdef UTF8_SUPPORTED
-#include "gtm_conv.h"
-#include "gtm_utf8.h"
-#endif
 #include "gtmcrypt.h"
 #include "error.h"
 #include "send_msg.h"
 #include "get_fs_block_size.h"
 #include "op.h"
+#include "util.h"
 #include "indir_enum.h"
+#ifdef UTF8_SUPPORTED
+#include "gtm_conv.h"
+#include "gtm_utf8.h"
+#endif
 
 /* Only want to do fstat() once on this file, not on every use. */
 #define FSTAT_CHECK(GETMODE)										\
@@ -176,7 +177,7 @@ error_def(ERR_SDSEEKERR);
 
 void	iorm_use(io_desc *iod, mval *pp)
 {
-	boolean_t	seen_wrap, fstat_done, get_mode_done, outdevparam;
+	boolean_t	seen_wrap, fstat_done, get_mode_done, outdevparam, prin_in_dev_failure;
 	boolean_t	input_key_not_empty, output_key_not_empty, input_key_entry_present, output_key_entry_present;
 	boolean_t	init_input_encryption, init_output_encryption, reset_input_encryption, reset_output_encryption;
 	boolean_t	seek_specified, ichset_specified, ochset_specified, chset_allowed;
@@ -343,7 +344,11 @@ void	iorm_use(io_desc *iod, mval *pp)
 				if (0 != statbuf.st_size)
 					iod->dollar.zeof = FALSE;
 				else
+				{
 					iod->dollar.zeof = TRUE;
+					if (io_std_device.in == iod)
+						prin_in_dev_failure = FALSE;
+				}
 				iod->dollar.y = 0;
 				iod->dollar.x = 0;
 				rm_ptr->lastop = RM_NOOP;
@@ -887,6 +892,12 @@ void	iorm_use(io_desc *iod, mval *pp)
 			}
 			rm_ptr->fsblock_buffer_size = fwrite_buffer_size;
 			break;
+		case iop_fflf:				/* Turn off GTM-9136 NL suppression regardless of env gtm_fflf setting */
+			iod->fflf = TRUE;
+			break;
+		case iop_nofflf:			/* Turn on GTM-9136 NL suppression regardless of env gtm_fflf setting */
+			iod->fflf = FALSE;
+			break;
 		default:
 			break;
 		}
