@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2019 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -143,10 +143,7 @@ error_def(ERR_CRYPTBADWRTPOS);
  * NOPRINCIO error.
  */
 #define ISSUE_NOPRINCIO_IF_NEEDED_RM(VAR, CMP_SIGN, IOD)						\
-{													\
-	GBLREF io_pair		io_std_device;								\
-	GBLREF bool		prin_out_dev_failure;							\
-													\
+MBSTART {												\
 	int			write_status;								\
 													\
 	if (0 CMP_SIGN VAR)										\
@@ -155,25 +152,16 @@ error_def(ERR_CRYPTBADWRTPOS);
 			prin_out_dev_failure = FALSE;							\
 	} else												\
 	{												\
+		ISSUE_NOPRINCIO_IF_NEEDED(iod, TRUE, FALSE); /* TRUE, FALSE: WRITE, not socket */	\
 		write_status = (-1 == VAR) ? errno : VAR;						\
-		if (IOD == io_std_device.out)								\
-		{											\
-			if (!prin_out_dev_failure)							\
-				prin_out_dev_failure = TRUE;						\
-			else										\
-			{										\
-				send_msg_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_NOPRINCIO);			\
-				stop_image_no_core();							\
-			}										\
-		}											\
 		if (EAGAIN == write_status)								\
-			SET_DOLLARDEVICE_ERRSTR(IOD, ONE_COMMA_UNAVAILABLE);		\
+			SET_DOLLARDEVICE_ERRSTR(IOD, ONE_COMMA_UNAVAILABLE);				\
 		else											\
-			SET_DOLLARDEVICE_ONECOMMA_STRERROR(IOD, write_status);		\
+			SET_DOLLARDEVICE_ONECOMMA_STRERROR(IOD, write_status);				\
 		IOD->dollar.za = 9;									\
 		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) write_status);					\
 	}												\
-}
+} MBEND
 
 /* Operations for this device type */
 #define RM_NOOP		0
