@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2014-2017 Fidelity National Information	*
+ * Copyright (c) 2014-2019 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -280,10 +280,9 @@ void iosocket_pass_local(io_desc *iod, pid_t pid, int4 msec_timeout, int argcnt,
 			cancel_timer(timer_id);
 		iod->dollar.za = 9;
 		errptr = PROTOCOL_ERROR;
-		SET_DOLLARDEVICE_ONECOMMA_ERRSTR(iod, errptr);
+		SET_DOLLARDEVICE_ONECOMMA_ERRSTR(iod, errptr, errlen);
 		if (socketptr->ioerror)
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_SOCKPASS, 0, ERR_TEXT, 2, STR_LIT_LEN(PROTOCOL_ERROR),
-				errptr);
+			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_SOCKPASS, 0, ERR_TEXT, 2, errlen, errptr);
 		REVERT_GTMIO_CH(&iod->pair, ch_set);
 		return;
 	}
@@ -435,7 +434,7 @@ void iosocket_accept_local(io_desc *iod, mval *handlesvar, pid_t pid, int4 msec_
 	{
 		if ((NO_M_TIMEOUT != msec_timeout) && !out_of_time)
 			cancel_timer(timer_id);
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(2) ERR_SOCKNOTPASSED, 0);
+		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_SOCKNOTPASSED);
 		return;
 	}
 	assert(rval == iov.iov_len);
@@ -443,10 +442,9 @@ void iosocket_accept_local(io_desc *iod, mval *handlesvar, pid_t pid, int4 msec_
 	{
 		iod->dollar.za = 9;
 		errptr = PROTOCOL_ERROR;
-		SET_DOLLARDEVICE_ONECOMMA_ERRSTR(iod, errptr);
+		SET_DOLLARDEVICE_ONECOMMA_ERRSTR(iod, errptr, errlen);
 		if (socketptr->ioerror)
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_SOCKACCEPT, 0, ERR_TEXT, 2, STR_LIT_LEN(PROTOCOL_ERROR),
-				errptr);
+			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_SOCKACCEPT, 0, ERR_TEXT, 2, errlen, errptr);
 	}
 	cmsg = CMSG_FIRSTHDR(&msg);
 	while((cmsg != NULL) && ((SOL_SOCKET != cmsg->cmsg_level) || (SCM_RIGHTS != cmsg->cmsg_type)))
@@ -549,10 +547,9 @@ void iosocket_accept_local(io_desc *iod, mval *handlesvar, pid_t pid, int4 msec_
 		}
 		iod->dollar.za = 9;
 		errptr = PROTOCOL_ERROR;
-		SET_DOLLARDEVICE_ONECOMMA_ERRSTR(iod, errptr);
+		SET_DOLLARDEVICE_ONECOMMA_ERRSTR(iod, errptr, errlen);
 		if (socketptr->ioerror)
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_SOCKACCEPT, 0, ERR_TEXT, 2, STR_LIT_LEN(PROTOCOL_ERROR),
-				errptr);
+			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_SOCKACCEPT, 0, ERR_TEXT, 2, errlen, errptr);
 		REVERT_GTMIO_CH(&iod->pair, ch_set);
 		return;
 	}
