@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2014 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2019 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -13,12 +14,14 @@
 
 #include <sys/ioctl.h>
 #include <errno.h>
+
 #include "gtm_fcntl.h"
-#include <signal.h>
 #include "gtm_string.h"
 #include "gtm_iconv.h"
 #include "gtm_termios.h"
 #include "gtm_unistd.h"
+#include "gtm_signal.h"	/* for SIGPROCMASK used inside Tcsetattr */
+#include "gtm_stdlib.h"
 
 #include "io_params.h"
 #include "io.h"
@@ -32,10 +35,16 @@
 #include "stringpool.h"
 #include "send_msg.h"
 #include "namelook.h"
+#include "gtm_conv.h"
 #include "error.h"
 #include "gtm_tputs.h"
 #include "gtm_tparm.h"
 #include "outofband.h"
+#include "restrict.h"
+#include "op.h"
+#include "indir_enum.h"
+#include "svnames.h"
+#include "util.h"
 
 LITDEF nametabent filter_names[] =
 {
@@ -51,12 +60,11 @@ LITDEF unsigned char filter_index[27] =
 	,4, 4, 4
 };
 
-GBLREF boolean_t		ctrlc_on, dollar_zininterrupt;
-GBLREF bool			prin_out_dev_failure;
-GBLREF char			*CURSOR_ADDRESS, *CLR_EOL, *CLR_EOS;
-GBLREF io_pair			io_std_device;
-GBLREF io_pair			io_curr_device;
-GBLREF void 			(*ctrlc_handler_ptr)();
+GBLREF boolean_t	ctrlc_on, dollar_zininterrupt, hup_on, prin_in_dev_failure, prin_out_dev_failure;
+GBLREF char		*CURSOR_ADDRESS, *CLR_EOL, *CLR_EOS;
+GBLREF io_pair		io_curr_device, io_std_device;
+GBLREF mval		dollar_zstatus;
+GBLREF void		(*ctrlc_handler_ptr)();
 
 LITREF unsigned char	io_params_size[];
 
@@ -71,7 +79,7 @@ error_def(ERR_ZINTRECURSEIO);
 
 void iott_use(io_desc *iod, mval *pp)
 {
-	boolean_t		flush_input;
+	boolean_t		flush_input, terminator_specified = FALSE;
 	char			dc1, *ttab;
 	d_tt_struct		*temp_ptr, *tt_ptr;
 	int			p_offset, fil_type, save_errno, status;
@@ -80,11 +88,18 @@ void iott_use(io_desc *iod, mval *pp)
 	io_termmask		mask_term;
 	struct sigaction	act;
 	struct termios		t;
+	mstr			chset_mstr;
+	gtm_chset_t		temp_chset, old_ochset, old_ichset;
 	uint4			mask_in;
 	unsigned char		ch, len;
+	boolean_t		ch_set;
+	mval			mv;
+	DCL_THREADGBL_ACCESS;
 
+	SETUP_THREADGBL_ACCESS;
 	p_offset = 0;
 	assert(iod->state == dev_open);
+	ESTABLISH_GTMIO_CH(&iod->pair, ch_set);
 	iott_flush(iod);
 	tt_ptr = (d_tt_struct *)iod->dev_sp;
 	if (*(pp->str.addr + p_offset) != iop_eol)
@@ -102,16 +117,7 @@ void iott_use(io_desc *iod, mval *pp)
 		if (0 != status)
 		{
 			save_errno = errno;
-			if (io_curr_device.out == io_std_device.out)
-			{
-				if (!prin_out_dev_failure)
-					prin_out_dev_failure = TRUE;
-				else
-				{
-					send_msg_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_NOPRINCIO);
-					stop_image_no_core();
-				}
-			}
+			ISSUE_NOPRINCIO_IF_NEEDED(io_curr_device.out, FALSE, FALSE);	/* FALSE, FALSE: READ (sorta), not socket */
 			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_TCGETATTR, 1, tt_ptr->fildes, save_errno);
 		}
 		flush_input = FALSE;
@@ -120,6 +126,8 @@ void iott_use(io_desc *iod, mval *pp)
 		temp_ptr = (d_tt_struct *)d_in->dev_sp;
 		mask_in = temp_ptr->term_ctrl;
 		mask_term = temp_ptr->mask_term;
+		old_ochset = iod->ochset;
+		old_ichset = iod->ichset;
 		while (*(pp->str.addr + p_offset) != iop_eol)
 		{
 			switch (ch = *(pp->str.addr + p_offset++))
@@ -139,7 +147,7 @@ void iott_use(io_desc *iod, mval *pp)
 					tt_ptr->ext_cap &= ~TT_EMPTERM;
 					break;
 				case iop_cenable:
-					if (!ctrlc_on)
+					if (!ctrlc_on && !RESTRICTED(cenable))
 					{	/* if it's already cenable, no need to change */
 						temp_ptr = (d_tt_struct *)io_std_device.in->dev_sp;
 						if (tt_ptr->fildes == temp_ptr->fildes)
@@ -153,7 +161,7 @@ void iott_use(io_desc *iod, mval *pp)
 					}
 					break;
 				case iop_nocenable:
-					if (ctrlc_on)
+					if (ctrlc_on && !RESTRICTED(cenable))
 					{	/* if it's already nocenable, no need to change */
 						temp_ptr = (d_tt_struct *)io_std_device.in->dev_sp;
 						if (tt_ptr->fildes == temp_ptr->fildes)
@@ -234,9 +242,7 @@ void iott_use(io_desc *iod, mval *pp)
 						gtm_tputs(CLR_EOL, 1, outc);
 					break;
 				case iop_exception:
-					iod->error_handler.len = *(pp->str.addr + p_offset);
-					iod->error_handler.addr = (char *)(pp->str.addr + p_offset + 1);
-					s2pool(&iod->error_handler);
+					DEF_EXCEPTION(pp, p_offset, iod);
 					break;
 				case iop_filter:
 					len = *(pp->str.addr + p_offset);
@@ -274,6 +280,34 @@ void iott_use(io_desc *iod, mval *pp)
 				case iop_nohostsync:
 					t.c_iflag &= ~IXOFF;
 					break;
+				case iop_hupenable:
+					if (!hup_on)
+					{	/* if it's already hupenable, no need to change */
+						temp_ptr = (d_tt_struct *)io_std_device.in->dev_sp;
+						if (tt_ptr->fildes == temp_ptr->fildes)
+						{	/* if $PRINCIPAL, enable hup_handler; similar code in term_setup.c */
+							sigemptyset(&act.sa_mask);
+							act.sa_flags = 0;
+							act.sa_handler = ctrlc_handler_ptr;
+							sigaction(SIGHUP, &act, 0);
+							hup_on = TRUE;
+						}
+					}
+					break;
+				case iop_nohupenable:
+					if (hup_on)
+					{	/* if it's already nohupenable, no need to change */
+						temp_ptr = (d_tt_struct *)io_std_device.in->dev_sp;
+						if (tt_ptr->fildes == temp_ptr->fildes)
+						{	/* if $PRINCIPAL, disable the hup_handler */
+							sigemptyset(&act.sa_mask);
+							act.sa_flags = 0;
+							act.sa_handler = SIG_IGN;
+							sigaction(SIGHUP, &act, 0);
+							hup_on = FALSE;
+						}
+					}
+					break;
 				case iop_insert:
 					if (io_curr_device.in == io_std_device.in)
 						tt_ptr->ext_cap &= ~TT_NOINSERT;	/* $PRINCIPAL only */
@@ -307,6 +341,7 @@ void iott_use(io_desc *iod, mval *pp)
 					break;
 				case iop_terminator:
 					memcpy(&mask_term.mask[0], (pp->str.addr + p_offset), SIZEOF(io_termmask));
+					terminator_specified = TRUE;
 					temp_ptr = (d_tt_struct *)d_in->dev_sp;
 					if (mask_term.mask[0] == NUL &&
 						mask_term.mask[1] == NUL &&
@@ -316,17 +351,10 @@ void iott_use(io_desc *iod, mval *pp)
 						mask_term.mask[5] == NUL &&
 						mask_term.mask[6] == NUL &&
 						mask_term.mask[7] == NUL)
-					{
 						temp_ptr->default_mask_term = TRUE;
-						if (CHSET_UTF8 == d_in->ichset)
-						{
-							mask_term.mask[0] = TERM_MSK_UTF8_0;
-							mask_term.mask[4] = TERM_MSK_UTF8_4;
-						} else
-							mask_term.mask[0] = TERM_MSK;
-					} else
+					else
 						temp_ptr->default_mask_term = FALSE;
-						break;
+					break;
 				case iop_noterminator:
 					temp_ptr = (d_tt_struct *)d_in->dev_sp;
 					temp_ptr->default_mask_term = FALSE;
@@ -400,7 +428,7 @@ void iott_use(io_desc *iod, mval *pp)
 				case iop_ipchset:
 					{
 #						ifdef KEEP_zOS_EBCDIC
-						if ( (iconv_t)0 != iod->input_conv_cd )
+						if ((iconv_t)0 != iod->input_conv_cd)
 						{
 							ICONV_CLOSE_CD(iod->input_conv_cd);
 						}
@@ -409,12 +437,20 @@ void iott_use(io_desc *iod, mval *pp)
 							ICONV_OPEN_CD(iod->input_conv_cd,
 								(char *)(pp->str.addr + p_offset + 1), INSIDE_CH_SET);
 #						endif
+						GET_ADDR_AND_LEN(chset_mstr.addr, chset_mstr.len);
+						SET_ENCODING(temp_chset, &chset_mstr);
+						if (!gtm_utf8_mode && IS_UTF_CHSET(temp_chset))
+							break;	/* ignore UTF chsets if not utf8_mode. */
+						if (IS_UTF16_CHSET(temp_chset))		/* UTF16 is not valid for TTY */
+							rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_BADCHSET, 2,
+								 chset_mstr.len, chset_mstr.addr);
+						iod->ichset = temp_chset;
 						break;
 					}
 				case iop_opchset:
 					{
 #						ifdef KEEP_zOS_EBCDIC
-						if ( (iconv_t)0 != iod->output_conv_cd)
+						if ((iconv_t)0 != iod->output_conv_cd)
 						{
 							ICONV_CLOSE_CD(iod->output_conv_cd);
 						}
@@ -423,6 +459,26 @@ void iott_use(io_desc *iod, mval *pp)
 							ICONV_OPEN_CD(iod->output_conv_cd, INSIDE_CH_SET,
 								(char *)(pp->str.addr + p_offset + 1));
 #						endif
+						GET_ADDR_AND_LEN(chset_mstr.addr, chset_mstr.len);
+						SET_ENCODING(temp_chset, &chset_mstr);
+						if (!gtm_utf8_mode && IS_UTF_CHSET(temp_chset))
+							break;	/* ignore UTF chsets if not utf8_mode. */
+						if (IS_UTF16_CHSET(temp_chset))		/* UTF16 is not valid for TTY */
+							rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_BADCHSET, 2,
+								 chset_mstr.len, chset_mstr.addr);
+						iod->ochset = temp_chset;
+						break;
+					}
+				case iop_chset:
+					{
+						GET_ADDR_AND_LEN(chset_mstr.addr, chset_mstr.len);
+						SET_ENCODING(temp_chset, &chset_mstr);
+						if (!gtm_utf8_mode && IS_UTF_CHSET(temp_chset))
+							break;	/* ignore UTF chsets if not utf8_mode. */
+						if (IS_UTF16_CHSET(temp_chset))		/* UTF16 is not valid for TTY */
+							rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_BADCHSET, 2,
+								 chset_mstr.len, chset_mstr.addr);
+						iod->ichset = iod->ochset = temp_chset;
 						break;
 					}
 			}
@@ -433,8 +489,24 @@ void iott_use(io_desc *iod, mval *pp)
 		Tcsetattr(tt_ptr->fildes, TCSANOW, &t, status, save_errno);
 		if (0 != status)
 			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_TCSETATTR, 1, tt_ptr->fildes, save_errno);
-		temp_ptr->term_ctrl = mask_in;
-		memcpy(&temp_ptr->mask_term, &mask_term, SIZEOF(io_termmask));
+		if (tt == d_in->type)
+		{
+			temp_ptr->term_ctrl = mask_in;
+			/* reset the mask to default if chset was changed without specifying new terminators or Default */
+			if ((!terminator_specified && (old_ichset != iod->ichset)) ||
+				(terminator_specified && temp_ptr->default_mask_term))
+			{
+				memset(&mask_term.mask[0], 0, SIZEOF(io_termmask));
+				if (CHSET_M != iod->ichset)
+				{
+					mask_term.mask[0] = TERM_MSK_UTF8_0;
+					mask_term.mask[4] = TERM_MSK_UTF8_4;
+				} else
+					mask_term.mask[0] = TERM_MSK;
+				temp_ptr->default_mask_term = TRUE;
+			}
+			memcpy(&temp_ptr->mask_term, &mask_term, SIZEOF(io_termmask));
+		}
 		if (flush_input)
 		{
 			TCFLUSH(tt_ptr->fildes, TCIFLUSH, status);
@@ -448,6 +520,7 @@ void iott_use(io_desc *iod, mval *pp)
 		tt_ptr->tt_state_save.who_saved = ttwhichinvalid;
 		io_find_mvstent(iod, TRUE);	/* clear mv stack entry */
 	}
+	REVERT_GTMIO_CH(&iod->pair, ch_set);
 	return;
 }
 
