@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2014 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -38,29 +39,28 @@
 
 GBLREF	gd_region		*gv_cur_region;
 GBLREF 	jnl_gbls_t		jgbl;
-GBLREF	jnlpool_addrs		jnlpool;
+GBLREF	jnlpool_addrs_ptr_t	jnlpool;
 GBLREF	sgmnt_addrs		*cs_addrs;
 GBLREF	sgmnt_data_ptr_t	cs_data;
 GBLREF	boolean_t		in_jnl_file_autoswitch;
 
+error_def(ERR_DBFILERR);
+error_def(ERR_DSKSPACEFLOW);
 error_def(ERR_JNLEXTEND);
+error_def(ERR_JNLFILEXTERR);
+error_def(ERR_JNLNOCREATE);
+error_def(ERR_JNLRDERR);
 error_def(ERR_JNLREADEOF);
 error_def(ERR_JNLSPACELOW);
-error_def(ERR_NEWJNLFILECREAT);
-error_def(ERR_DSKSPACEFLOW);
-error_def(ERR_JNLFILEXTERR);
-error_def(ERR_DBFILERR);
-error_def(ERR_NOSPACEEXT);
-error_def(ERR_JNLRDERR);
 error_def(ERR_JNLWRERR);
-error_def(ERR_JNLNOCREATE);
-error_def(ERR_PREMATEOF);
+error_def(ERR_NEWJNLFILECREAT);
+error_def(ERR_NOSPACEEXT);
 
 uint4 jnl_file_extend(jnl_private_control *jpc, uint4 total_jnl_rec_size)
 {
 	file_control		*fc;
 	boolean_t		need_extend;
-	jnl_buffer_ptr_t     	jb;
+	jnl_buffer_ptr_t	jb;
 	jnl_create_info 	jnl_info;
 	jnl_file_header		*header;
 	unsigned char		hdr_buff[REAL_JNL_HDR_LEN + MAX_IO_BLOCK_SIZE];
@@ -73,8 +73,11 @@ uint4 jnl_file_extend(jnl_private_control *jpc, uint4 total_jnl_rec_size)
 	gtm_uint64_t		avail_blocks;
 	uint4			aligned_tot_jrec_size, count;
 	uint4			jnl_fs_block_size, read_write_size;
+	unix_db_info		*udi;
+	jnlpool_addrs_ptr_t	local_jnlpool;	/* needed by INST_FREEZE_ON_NOSPC_ENABLED */
 	DCL_THREADGBL_ACCESS;
 
+	SETUP_THREADGBL_ACCESS;
 	switch(jpc->region->dyn.addr->acc_meth)
 	{
 	case dba_mm:
@@ -82,7 +85,7 @@ uint4 jnl_file_extend(jnl_private_control *jpc, uint4 total_jnl_rec_size)
 		csa = &FILE_INFO(jpc->region)->s_addrs;
 		break;
 	default:
-		assertpro((dba_mm == jpc->region->dyn.addr->acc_meth) || (dba_bg == jpc->region->dyn.addr->acc_meth));
+		assertpro(IS_REG_BG_OR_MM(jpc->region));
 	}
 	csd = csa->hdr;
 	assert(csa == cs_addrs && csd == cs_data);
@@ -91,7 +94,10 @@ uint4 jnl_file_extend(jnl_private_control *jpc, uint4 total_jnl_rec_size)
 	assert(csa->jnl_state == csd->jnl_state);
 	assertpro(JNL_ENABLED(csa) && (NOJNL != jpc->channel) && (!JNL_FILE_SWITCHED(jpc)));
 		/* crit and messing with the journal file - how could it have vanished? */
-	if (!csd->jnl_deq || (csd->jnl_alq + csd->jnl_deq > csd->autoswitchlimit))
+	if (!total_jnl_rec_size)
+	{	/* rec_size=0 has special meaning. Caller (source server for now) wants to force an autoswitch. */
+		new_blocks = 0;
+	} else if (!csd->jnl_deq || (csd->jnl_alq + csd->jnl_deq > csd->autoswitchlimit))
 	{
 		assert(DIVIDE_ROUND_UP(total_jnl_rec_size, DISK_BLOCK_SIZE) <= csd->jnl_alq);
 		assert(csd->jnl_alq == csd->autoswitchlimit);
@@ -103,13 +109,14 @@ uint4 jnl_file_extend(jnl_private_control *jpc, uint4 total_jnl_rec_size)
 	jb = jpc->jnl_buff;
 	assert(0 <= new_blocks);
 	DEBUG_ONLY(count = 0);
-	for (need_extend = (jb->last_eof_written || (0 != new_blocks)); need_extend; )
+	do
 	{
 		DEBUG_ONLY(count++);
-		/* usually we will do the loop just once where we do the file extension.
-		 * rarely we might need to do an autoswitch instead after which again rarely
-		 * 	we might need to do an extension on the new journal to fit in the transaction's	journal requirements.
-		 * therefore we should do this loop a maximum of twice. hence the assert below.
+		/* Usually we will do the loop just once where we do the file extension.
+		 * Rarely we might need to do an autoswitch instead.
+		 * Even more rarely, we might need to do a file extension of the autoswitched journal to fit in the
+		 * transaction's journal requirements (because journal file initial allocation did not fit it).
+		 * Therefore we should do this loop a maximum of twice. Assert it.
 		 */
 		assert(count <= 2);
 		need_extend = FALSE;
@@ -118,7 +125,6 @@ uint4 jnl_file_extend(jnl_private_control *jpc, uint4 total_jnl_rec_size)
 			warn_blocks = (csd->jnl_alq + csd->jnl_deq > csd->autoswitchlimit)
 					? ((csd->jnl_deq > csd->autoswitchlimit) ? csd->jnl_deq : csd->autoswitchlimit)
 					: new_blocks;
-
 			if ((warn_blocks * EXTEND_WARNING_FACTOR) > avail_blocks)
 			{
 				if (new_blocks > avail_blocks)
@@ -128,13 +134,12 @@ uint4 jnl_file_extend(jnl_private_control *jpc, uint4 total_jnl_rec_size)
 					 * we will freeze the instance and wait for space to become available and keep
 					 * retrying the writes. Therefore, we make the NOSPACEEXT a warning in this case.
 					 */
-					SETUP_THREADGBL_ACCESS;
-					if (!INST_FREEZE_ON_NOSPC_ENABLED(csa))
+					if (!INST_FREEZE_ON_NOSPC_ENABLED(csa, local_jnlpool))
 					{
 						send_msg_csa(CSA_ARG(csa) VARLSTCNT(6) ERR_NOSPACEEXT, 4,
 								JNL_LEN_STR(csd), new_blocks, avail_blocks);
-						new_blocks = 0;
-						jpc->status = SS_NORMAL;
+						new_blocks = -1;
+						jpc->status = ERR_NOSPACEEXT;
 						break;
 					} else
 						send_msg_csa(CSA_ARG(csa) VARLSTCNT(6) MAKE_MSG_WARNING(ERR_NOSPACEEXT), 4,
@@ -151,15 +156,15 @@ uint4 jnl_file_extend(jnl_private_control *jpc, uint4 total_jnl_rec_size)
 		if (csd->autoswitchlimit < (jb->filesize + (EXTEND_WARNING_FACTOR * new_blocks)))	/* close to max */
 			send_msg_csa(CSA_ARG(csa) VARLSTCNT(5) ERR_JNLSPACELOW, 3, JNL_LEN_STR(csd),
 					csd->autoswitchlimit - jb->filesize);
-		if (jb->last_eof_written || (csd->autoswitchlimit < new_alq))
-		{	/* Reached max, need to autoswitch */
+		if (jb->last_eof_written || (csd->autoswitchlimit < new_alq) || !new_blocks)
+		{	/* Reached max OR caller wants to switch unconditionally. Need to autoswitch */
 			/* Ensure new journal file can hold the entire current transaction's journal record requirements */
 			assert(csd->autoswitchlimit >= MAX_REQD_JNL_FILE_SIZE(total_jnl_rec_size));
 			memset(&jnl_info, 0, SIZEOF(jnl_info));
 			jnl_info.prev_jnl = &prev_jnl_fn[0];
 			set_jnl_info(gv_cur_region, &jnl_info);
 			assert(JNL_ENABLED(csa) && (NOJNL != jpc->channel) && !(JNL_FILE_SWITCHED(jpc)));
-			jnl_status = jnl_ensure_open();
+			jnl_status = jnl_ensure_open(gv_cur_region, csa);
 			if (0 != jnl_status)
 			{
 				if (SS_NORMAL != jpc->status)
@@ -231,7 +236,14 @@ uint4 jnl_file_extend(jnl_private_control *jpc, uint4 total_jnl_rec_size)
 				send_msg_csa(CSA_ARG(csa) VARLSTCNT(4) ERR_NEWJNLFILECREAT, 2, JNL_LEN_STR(csd));
 				fc = gv_cur_region->dyn.addr->file_cntl;
 				fc->op = FC_WRITE;
-				fc->op_buff = (sm_uc_ptr_t)csd;
+				udi = FC2UDI(fc);
+				if (!udi->fd_opened_with_o_direct)
+					fc->op_buff = (sm_uc_ptr_t)csd;
+				else
+				{
+					memcpy((TREF(dio_buff)).aligned, csd, SGMNT_HDR_LEN);
+					fc->op_buff = (sm_uc_ptr_t)(TREF(dio_buff)).aligned;
+				}
 				fc->op_len = SGMNT_HDR_LEN;
 				fc->op_pos = 1;
 				status = dbfilop(fc);
@@ -239,7 +251,7 @@ uint4 jnl_file_extend(jnl_private_control *jpc, uint4 total_jnl_rec_size)
 					send_msg_csa(CSA_ARG(csa) VARLSTCNT(5) ERR_DBFILERR, 2, DB_LEN_STR(gv_cur_region), status);
 				assert(JNL_ENABLED(csa));
 				/* call jnl_ensure_open instead of jnl_file_open to make sure jpc->pini_addr is set to 0 */
-				jnl_status = jnl_ensure_open();	/* sets jpc->status */
+				jnl_status = jnl_ensure_open(gv_cur_region, csa);	/* sets jpc->status */
 				if (0 != jnl_status)
 				{
 					if (jpc->status)
@@ -296,10 +308,10 @@ uint4 jnl_file_extend(jnl_private_control *jpc, uint4 total_jnl_rec_size)
 			}
 			jb->filesize = new_alq;	/* Actually this is virtual file size blocks */
 		}
-		if (0 >= new_blocks)
+		if (0 > new_blocks)
 			break;
-	}
-	if (0 < new_blocks)
+	} while (need_extend);
+	if (0 <= new_blocks)
 	{
 		INCR_GVSTATS_COUNTER(csa, csa->nl, n_jnl_extends, 1);
 		return EXIT_NRM;
@@ -307,7 +319,7 @@ uint4 jnl_file_extend(jnl_private_control *jpc, uint4 total_jnl_rec_size)
 	if (SS_NORMAL == jpc->status)
 		jpc->status = ERR_JNLREADEOF;
 	jnl_file_lost(jpc, ERR_JNLEXTEND);
-       	return EXIT_ERR;
+	return EXIT_ERR;
 }
 
 CONDITION_HANDLER(jnl_file_autoswitch_ch)
