@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2010 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2015 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -35,7 +36,7 @@
  */
 bool lke_get_answ(char *prompt)
 {
-	char buff[11], *bp = buff;
+	char buff[11], *bp;
 
 	PRINTF("%s", prompt);
 	if (NULL != (bp = util_input(buff, 10, stdin, TRUE)))
