@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -89,7 +89,7 @@ MBSTART {														\
 		 */													\
 		getzposition(&zpos);											\
 		send_msg_csa(CSA_ARG(CSAARG) VARLSTCNT((8 + VARCNT)) ERR_MUCREFILERR, 6, zpos.str.len, zpos.str.addr,	\
-			     DB_LEN_STR(gv_cur_region), REG_LEN_STR(gv_cur_region), ERRORID, __VA_ARGS__);		\
+				DB_LEN_STR(gv_cur_region), REG_LEN_STR(gv_cur_region), ERRORID, __VA_ARGS__);		\
 	}														\
 } MBEND
 #define PUTMSG_ERROR_CSA(CSAARG, VARCNT, ERRORID, ...)									\
@@ -142,6 +142,7 @@ error_def(ERR_NOSPACECRE);
 error_def(ERR_PARNORMAL);
 error_def(ERR_PREALLOCATEFAIL);
 error_def(ERR_RAWDEVUNSUP);
+error_def(ERR_DBFILNOFULLWRT);
 
 /* Condition handler primarily to handle ERR_MEMORY errors by cleaning up the file that we created
  * before passing on control to higher level condition handlers.
@@ -163,7 +164,7 @@ unsigned char mu_cre_file(void)
 	char		path[MAX_FN_LEN + 1], errbuff[OUT_LINE];
 	unsigned char	buff[DISK_BLOCK_SIZE];
 	int		i, lower, upper, norm_vbn;
-        ssize_t         status;
+	ssize_t		status;
 	uint4		raw_dev_size;		/* size of a raw device, in bytes */
 	int4		save_errno;
 	gtm_uint64_t	avail_blocks, blocks_for_create, blocks_for_extension, delta_blocks;
@@ -182,6 +183,8 @@ unsigned char mu_cre_file(void)
 	sgmnt_addrs	*baseDBcsa;
 	struct stat	stat_buf;
 	struct perm_diag_data	pdd;
+	uint4		fbwsize;
+	int4		dblksize;
 	ZOS_ONLY(int	realfiletag;)
 	DCL_THREADGBL_ACCESS;
 
@@ -307,7 +310,7 @@ unsigned char mu_cre_file(void)
 				/* Note: Above macro internally invokes CLEANUP(EXIT_ERR) */
 			if (IS_MUPIP_IMAGE)
 				send_msg_csa(CSA_ARG(cs_addrs) VARLSTCNT(6) ERR_NOSPACECRE, 4, LEN_AND_STR(path),
-					     &blocks_for_create, &avail_blocks);
+						&blocks_for_create, &avail_blocks);
 			REVERT;
 			return EXIT_ERR;
 		}
@@ -318,7 +321,7 @@ unsigned char mu_cre_file(void)
 					&blocks_for_extension, DISK_BLOCK_SIZE, &delta_blocks);
 			if (IS_MUPIP_IMAGE)	/* Is not mupip, msg already went to operator log */
 				send_msg_csa(CSA_ARG(cs_addrs) VARLSTCNT(8) ERR_LOWSPACECRE, 6, LEN_AND_STR(path),
-					     EXTEND_WARNING_FACTOR, &blocks_for_extension, DISK_BLOCK_SIZE, &delta_blocks);
+						EXTEND_WARNING_FACTOR, &blocks_for_extension, DISK_BLOCK_SIZE, &delta_blocks);
 		}
 	}
 	gv_cur_region->dyn.addr->file_cntl = &fc;
@@ -345,6 +348,42 @@ unsigned char mu_cre_file(void)
 		REVERT;
 		return EXIT_ERR;
 	}
+	cs_data->write_fullblk = gv_cur_region->dyn.addr->full_blkwrt;
+	if (cs_data->write_fullblk)
+        {       /* We have been asked to do FULL BLOCK WRITES for this database. On *NIX, attempt to get the filesystem
+                 * blocksize from statvfs. This allows a full write of a blockwithout the OS having to fetch the old
+                 * block for a read/update operation. We will round the IOs to the next filesystem blocksize if the
+                 * following criteria are met:
+                 *
+                 * 1) Database blocksize must be a whole multiple of the filesystem blocksize for the above
+                 *    mentioned reason.
+                 *
+                 * 2) Filesystem blocksize must be a factor of the location of the first data block
+                 *    given by the start_vbn.
+                 *
+                 * The saved length (if the feature is enabled) will be the filesystem blocksize and will be the
+                 * length that a database IO is rounded up to prior to initiation of the IO.
+                 */
+                fbwsize = get_fs_block_size(udi->fd);
+                dblksize = seg->blk_size;
+                if (0 == fbwsize || (0 != dblksize % fbwsize) || (0 != (BLK_ZERO_OFF(cs_data->start_vbn)) % fbwsize))
+		{
+                        cs_data->write_fullblk = 0;         /* This region is not fullblockwrite enabled */
+			if (!IS_STATSDB_REGNAME(gv_cur_region))
+			{
+				if (!fbwsize)
+					gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(7) ERR_DBFILNOFULLWRT, 5,
+					LEN_AND_LIT("Could not get native filesize"),
+					LEN_AND_LIT("File size extracted: "), fbwsize);
+				else if (0 != dblksize % fbwsize)
+					gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(7) ERR_DBFILNOFULLWRT, 5,
+					LEN_AND_LIT("Database block size not a multiple of file system block size\n"),
+					LEN_AND_LIT("DB blocks size: "), dblksize);
+			}
+		}
+                /* Report this length in DSE even if not enabled */
+                cs_addrs->fullblockwrite_len = fbwsize;              /* Length for rounding fullblockwrite */
+        }
 	cs_data->trans_hist.total_blks = gv_cur_region->dyn.addr->allocation;
 	/* There are (bplmap - 1) non-bitmap blocks per bitmap, so add (bplmap - 2) to number of non-bitmap blocks
 	 * and divide by (bplmap - 1) to get total number of bitmaps for expanded database. (must round up in this
