@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -59,7 +59,6 @@
 #include "jnl.h"		/* needed for tp.h */
 #include "gdscc.h"		/* needed for tp.h */
 #include "buddy_list.h"		/* needed for tp.h */
-#include "hashtab_int4.h"	/* needed for tp.h */
 #include "tp.h"
 #include "gtmcrypt.h"
 #include "is_proc_alive.h"
@@ -159,7 +158,7 @@ CONDITION_HANDLER(mu_extract_handler)
 		{
 			reg = opened_regions[i];
 			cnl = (&FILE_INFO(reg)->s_addrs)->nl;
-			grab_crit(reg);
+			grab_crit(reg, WS_85);
 			cnl->mupip_extract_count--;
 			rel_crit(reg);
 		}
@@ -370,7 +369,7 @@ void mu_extract(void)
 				csa = &FILE_INFO(reg)->s_addrs;
 				csd = csa->hdr;
 				cnl = csa->nl;
-				grab_crit(reg);
+				grab_crit(reg, WS_86);
 				pid = cnl->reorg_encrypt_pid;
 				if (pid && is_proc_alive(pid, 0))
 				{
@@ -473,7 +472,7 @@ void mu_extract(void)
 			MEMCPY_LIT(outptr, BIN_HEADER_LABEL);
 			outptr += STR_LIT_LEN(BIN_HEADER_LABEL);
 		}
-		op_horolog(&val);
+		op_zhorolog(&val, FALSE);
 		op_fnzdate(&val, (mval *)&mu_bin_datefmt, &null_str, &null_str, &val);
 		memcpy(outptr, val.str.addr, val.str.len);
 		outptr += val.str.len;
@@ -567,7 +566,7 @@ void mu_extract(void)
 		op_val.str.len = label_len;
 		op_val.str.addr = label_buff;
 		op_write(&op_val);
-		op_horolog(&val);
+		op_zhorolog(&val, FALSE);
 		op_fnzdate(&val, &datefmt, &null_str, &null_str, &val);
 		op_val = val;
 		op_val.mvtype = MV_STR;
