@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -163,7 +163,7 @@ MBSTART {										\
 	}										\
 	if (SEM_INCREMENTED)								\
 	{										\
-		do_semop(udi->semid, DB_CONTROL_SEM, -1, IPC_NOWAIT | SEM_UNDO);	\
+		do_semop(UDI->semid, DB_CONTROL_SEM, -1, IPC_NOWAIT | SEM_UNDO);	\
 		SEM_INCREMENTED = FALSE;						\
 	}										\
 	if (SEM_CREATED)								\
@@ -321,7 +321,7 @@ boolean_t mu_rndwn_file(gd_region *reg, boolean_t standalone)
 	int			csd_size;
 	char                    now_running[MAX_REL_NAME];
 	boolean_t		is_gtm_shm, is_statsdb, rc_cpt_removed, statsDBexists;
-	boolean_t		glob_sec_init, db_shm_in_sync, remove_shmid, ftok_counter_halted;
+	boolean_t		glob_sec_init, db_shm_in_sync, remove_shmid, remove_mlk_shmid, ftok_counter_halted;
 	boolean_t		crypt_warning, do_crypt_init, need_statsdb_rundown;
 	boolean_t		baseDBrundown_status, statsDBrundown_status;
 	sgmnt_data_ptr_t	csd, tsd = NULL;
@@ -358,6 +358,7 @@ boolean_t mu_rndwn_file(gd_region *reg, boolean_t standalone)
 	gd_region		*statsDBreg, *save_ftok_sem_reg;
 	gd_segment		*statsDBseg;
 	jnl_buffer_ptr_t	jbp;
+	int			mlk_shmid;
 #	ifdef DEBUG
 	boolean_t		already_grabbed_ftok_sem = FALSE;
 #	endif
@@ -1112,7 +1113,7 @@ boolean_t mu_rndwn_file(gd_region *reg, boolean_t standalone)
 					send_msg_csa(CSA_ARG(csa) VARLSTCNT(8) ERR_MURNDWNOVRD, 2, DB_LEN_STR(reg),
 						ERR_TEXT, 2,
 						LEN_AND_LIT("Overriding OPEN journal file state in shared memory"));
-				} else
+				} else if (!FROZEN_CHILLED(csa))
 				{	/* Journal file state being still open in shared memory implies a crashed state,
 					 * so error out.
 					 */
@@ -1229,14 +1230,12 @@ boolean_t mu_rndwn_file(gd_region *reg, boolean_t standalone)
 				return FALSE;
 			}
 			if (FROZEN_CHILLED(csa) && !override_present)
-			{	/* If there is an online freeze, we can't do the file writes, so autorelease or give up. */
-				DO_CHILLED_AUTORELEASE(csa, csd);
-				if (FROZEN_CHILLED(csa))
-				{
-					gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_OFRZACTIVE, 2, DB_LEN_STR(reg));
-					MU_RNDWN_FILE_CLNUP(reg, udi, tsd, sem_created, udi->counter_acc_incremented);
-					return FALSE;
-				}
+			{	/* If there is an online freeze, we can't do the file writes.
+				 * Reject any command that requires a write to the frozen database file.
+				 */
+				gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_OFRZACTIVE, 2, DB_LEN_STR(reg));
+				MU_RNDWN_FILE_CLNUP(reg, udi, tsd, sem_created, udi->counter_acc_incremented);
+				return FALSE;
 			}
 			/* If there was an online freeze and it was autoreleased, we don't want to take down the shared memory
 			 * and lose the freeze_online state.
@@ -1254,6 +1253,8 @@ boolean_t mu_rndwn_file(gd_region *reg, boolean_t standalone)
 			cnl->in_crit = 0;
 			csa->now_crit = FALSE;
 			reg->open = TRUE;
+			if (csd->write_fullblk)
+				csa->fullblockwrite_len = get_fs_block_size(udi->fd);
 			if (rc_cpt_removed) /* reset RC values if we've rundown the RC CPT */
 				csd->rc_srv_cnt = csd->dsid = csd->rc_node = 0;
 			/* At this point we are holding standalone access and are about to invoke wcs_recover/wcs_flu. If
@@ -1343,7 +1344,7 @@ boolean_t mu_rndwn_file(gd_region *reg, boolean_t standalone)
 				}
 				if (NULL != jpc)
 				{	/* this swaplock should probably be a mutex */
-					grab_crit(gv_cur_region);
+					grab_crit(gv_cur_region, WS_87);
 					/* If we own it or owner died, clear the fsync lock */
 					if (process_id == jpc->jnl_buff->fsync_in_prog_latch.u.parts.latch_pid)
 					{
@@ -1360,13 +1361,24 @@ boolean_t mu_rndwn_file(gd_region *reg, boolean_t standalone)
 			/* Write master-map (in addition to file header) as wcs_flu done above would not have updated it */
 			csd_size = SIZEOF_FILE_HDR(csd); 	/* SIZEOF(sgmnt_data) + master-map */
 			TREF(donot_write_inctn_in_wcs_recover) = FALSE;
+			/* Remove lock hash shared memory if needed */
+			if (remove_shmid && (INVALID_SHMID != csa->mlkctl->hash_shmid))
+			{
+				remove_mlk_shmid = TRUE;
+				mlk_shmid = csa->mlkctl->hash_shmid;
+			}
+			else
+				remove_mlk_shmid = FALSE;
+
 		} else
 		{	/* In this case, we just want to clear a few fields in the file header but we do NOT want to
 			 * write the master map which we don't have access to at this point so set csd_size accordingly.
 			 */
 			csd = tsd;
 			csd_size = tsd_size;	/* SIZEOF(sgmnt_data) */
+			remove_mlk_shmid = FALSE;	/* We don't have mlkctl to check, so just leave it. */
 		}
+
 		reg->open = FALSE;
 		/* If wcs_flu returned FALSE, it better be because of MUPIP RUNDOWN run with OVERRIDE qualifier. */
 		assert(wcs_flu_success || (override_present && !standalone));
@@ -1451,6 +1463,24 @@ boolean_t mu_rndwn_file(gd_region *reg, boolean_t standalone)
 	/* Remove the shared memory only if it is a GT.M created one. */
 	if (is_gtm_shm)
 	{
+		if (remove_mlk_shmid)
+		{
+			if (0 != shm_rmid(mlk_shmid))
+			{
+				save_errno = errno;
+				if (!SHM_REMOVED(save_errno))
+				{
+					assert(FALSE);
+					RNDWN_ERR("!AD -> Error removing lock shared memory.", reg);
+					/* Fall through and attempt removing main shared memory */
+				}
+			} else
+			{
+				gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(5) ERR_SHMREMOVED, 3, mlk_shmid, DB_LEN_STR(reg));
+				send_msg_csa(CSA_ARG(NULL) VARLSTCNT(5) ERR_SHMREMOVED, 3, mlk_shmid, DB_LEN_STR(reg));
+			}
+
+		}
 		if (remove_shmid) /* Note: remove_shmid is defined only if is_gtm_shm is TRUE */
 		{
 			if (0 != shm_rmid(udi->shmid))
