@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2012-2016 Fidelity National Information	*
+ * Copyright (c) 2012-2019 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -31,6 +31,8 @@
 /* Include prototypes */
 #include "t_end.h"
 #include "t_retry.h"
+#include "collseq.h"
+#include "mu_getkey.h"
 #include "mupip_size.h"
 #include "util.h"
 #include "t_begin.h"
@@ -48,7 +50,6 @@
 #include "gtm_time.h"
 #include "mvalconv.h"
 #include "longset.h"	    /* needed for cws_insert.h */
-#include "hashtab_int4.h"
 #include "cws_insert.h"
 #include "min_max.h"
 #include <math.h>
@@ -66,6 +67,10 @@ GBLREF	int4			process_id;
 GBLREF	sgmnt_addrs		*cs_addrs;
 GBLREF	sgmnt_data_ptr_t	cs_data;
 GBLREF	unsigned int		t_tries;
+GBLREF	boolean_t		mu_key;
+GBLREF	boolean_t		null_coll_key;
+GBLREF	gv_key			*mu_start_key;
+GBLREF	gv_key			*mu_end_key;
 
 #define APPROX_F_MAX		500		/* Approximate upper bound for the number of records per index block in
 						 * a database. The estimated max fanning factor is initially APPROX_F_MAX.
@@ -150,6 +155,8 @@ int4 mu_size_arsample(glist *gl_ptr, uint4 M, int seed)
 	if (!seed)
 		seed = (int4)(time(0) * process_id);
 	srand48(seed);
+	if (MUKEY_NULLSUBS == mu_key)
+		CHECK_COLL_KEY(gl_ptr, null_coll_key);
 	/* do random traversals until M of them are accepted at level 1 */
 	INIT_STATS(rstat);
 	for (k = 1; rstat.N[1] < M; k++)
@@ -163,7 +170,7 @@ int4 mu_size_arsample(glist *gl_ptr, uint4 M, int seed)
 			CLEAR_VECTOR(a);
 			if (cdb_sc_normal != (status = mu_size_rand_traverse(r, a)))			/* WARNING assignment */
 			{
-				assert((CDB_STAGNATE > t_tries) || IS_FINAL_RETRY_CODE(status));
+				assert(UPDATE_CAN_RETRY(t_tries, status));
 				t_retry(status);
 				continue;
 			}
@@ -257,9 +264,17 @@ STATICFNDCL void finalize_stats_ar(stat_t *stat)
 			stat->S[j] /= stat->N[j];
 		}
 	stat->N[0] = stat->n;				/* for arithmetic below */
-	for (j = MAX_BT_DEPTH; (0 <= j) && (stat->M[j] < EPS); j--)
+	/* Note: stat->M[0] should remain zero since we don't maintain it. Also stat->M[1] should be > EPS.
+	 * So "j" is guaranteed to be at least 1 at the end of the for loop. Assert that.
+	 * In "pro" we be safe and add the "(0 < j)" check in the for loop below to prevent "j" from becoming negative.
+	 */
+	assert(0 == stat->M[0]);
+	assert(EPS > 0);
+	assert(EPS < 1);
+	assert(1 <= stat->M[1]);
+	for (j = MAX_BT_DEPTH; (0 < j) && (stat->M[j] < EPS); j--)
 		;
-	assert(0 <= j);					/* stat->M[0] should remain zero since we don't maintain it */
+	assert(0 < j);
 	mu_int_adj[j] = stat->AT = stat->blkerr[j] = stat->error = 0;
 	stat->B = stat->blktot[j] = 1;
 	for (k = j - 1; j > 0; j--, k--)
