@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2012, 2013 Fidelity Information Services, Inc	*
+ * Copyright (c) 2012-2019 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -68,10 +69,9 @@ GBLREF	inctn_opcode_t		inctn_opcode;
 GBLREF	kill_set		*kill_set_tail;
 GBLREF	sgmnt_addrs		*kip_csa;
 GBLREF	boolean_t		mu_reorg_process;
-GBLREF	boolean_t		mu_reorg_upgrd_dwngrd_in_prog;
 GBLREF	boolean_t		need_kip_incr;
 GBLREF	uint4			update_trans;
-GBLREF  gv_key                  *gv_altkey;
+GBLREF	gv_key			*gv_altkey;
 GBLREF	char			*update_array, *update_array_ptr;
 GBLREF	uint4			update_array_size;
 GBLREF	inctn_opcode_t		inctn_opcode;
@@ -85,7 +85,7 @@ error_def(ERR_MUTRUNCNOTBG);
 #define RETRY_SWAP		(0)
 #define ABORT_SWAP		(1)
 
-boolean_t mu_swap_root(glist *gl_ptr, int *root_swap_statistic_ptr)
+void	mu_swap_root(glist *gl_ptr, int *root_swap_statistic_ptr)
 {
 	sgmnt_data_ptr_t	csd;
 	sgmnt_addrs		*csa;
@@ -105,8 +105,10 @@ boolean_t mu_swap_root(glist *gl_ptr, int *root_swap_statistic_ptr)
 
 	SETUP_THREADGBL_ACCESS;
 	assert(mu_reorg_process);
-	gv_target->root = 0;
-	gv_target->clue.end = 0;
+	gv_target = gl_ptr->gvt;
+	gv_target->root = 0;		/* reset root so we recompute it in DO_OP_GVNAME below */
+	gv_target->clue.end = 0;	/* reset clue since reorg action on later globals might have invalidated it */
+	reorg_gv_target->gvname.var_name = gv_target->gvname.var_name;	/* needed by SAVE_ROOTSRCH_ENTRY_STATE */
 	dir_hist_ptr = gv_target->alt_hist;
 	gvt_hist_ptr = &(gv_target->hist);
 	inctn_opcode = inctn_invalid_op;
@@ -117,16 +119,18 @@ boolean_t mu_swap_root(glist *gl_ptr, int *root_swap_statistic_ptr)
 	csd = cs_data;	/* Be careful to keep csd up to date. With MM, cs_data can change, and
 			 * dereferencing an older copy can result in a SIG-11.
 			 */
+	if (gv_cur_region->read_only)
+		return;	/* Cannot proceed for read-only data files */
 	if (0 == gv_target->root)
 	{	/* Global does not exist (online rollback). No problem. */
 		gtm_putmsg_csa(CSA_ARG(csa) VARLSTCNT(4) ERR_GBLNOEXIST, 2, GNAME(gl_ptr).len, GNAME(gl_ptr).addr);
-		return TRUE;
+		return;
 	}
 	if (dba_mm == csd->acc_meth)
 		/* return for now without doing any swapping operation because later mu_truncate
 		 * is going to issue the MUTRUNCNOTBG message.
 		 */
-		return TRUE;
+		return;
 	SET_GV_ALTKEY_TO_GBLNAME_FROM_GV_CURRKEY;		/* set up gv_altkey to be just the gblname */
 	/* ------------ Swap root block of global variable tree --------- */
 	t_begin(ERR_MUREORGFAIL, UPDTRNS_DB_UPDATED_MASK);
@@ -185,7 +189,7 @@ boolean_t mu_swap_root(glist *gl_ptr, int *root_swap_statistic_ptr)
 			if (tn_aborted)
 			{	/* It is not an error if the global (that once existed) doesn't exist anymore (due to ROLLBACK) */
 				gtm_putmsg_csa(CSA_ARG(csa) VARLSTCNT(4) ERR_GBLNOEXIST, 2, GNAME(gl_ptr).len, GNAME(gl_ptr).addr);
-				return TRUE;
+				return;
 			}
 			continue;
 		}
@@ -223,7 +227,7 @@ boolean_t mu_swap_root(glist *gl_ptr, int *root_swap_statistic_ptr)
 			if (level >= dir_hist_ptr->depth)
 			{	/* done */
 				t_abort(gv_cur_region, csa);
-				return TRUE;
+				return;
 			}
 			child_blk_ptr = dir_hist_ptr->h[level].buffaddr;
 			child_blk_id = dir_hist_ptr->h[level].blk_num;
@@ -265,7 +269,7 @@ boolean_t mu_swap_root(glist *gl_ptr, int *root_swap_statistic_ptr)
 			break;
 		}
 	}
-	return TRUE;
+	return;
 }
 
 /* Finds a free block and adds information to update array and cw_set */
@@ -276,10 +280,9 @@ block_id swap_root_or_directory_block(int parent_blk_lvl, int child_blk_lvl, src
 	sgmnt_addrs		*csa;
 	node_local_ptr_t	cnl;
 	srch_blk_status		bmlhist, freeblkhist;
-	block_id		hint_blk_num, free_blk_id, parent_blk_id;
+	block_id		hint_blk_num, free_blk_id, parent_blk_id, total_blks, num_local_maps, master_bit, temp_blk;
 	boolean_t		free_blk_recycled;
-	int4			master_bit, num_local_maps, free_bit, hint_bit, maxbitsthismap;
-	uint4			total_blks;
+	int4			free_bit, hint_bit, maxbitsthismap;
 	int			blk_seg_cnt, blk_size;
 	sm_uc_ptr_t		parent_blk_ptr, bn_ptr, saved_blk;
 	blk_segment		*bs1, *bs_ptr;
@@ -289,7 +292,6 @@ block_id swap_root_or_directory_block(int parent_blk_lvl, int child_blk_lvl, src
 	cw_set_element		*tmpcse;
 	jnl_buffer_ptr_t	jbbp; /* jbbp is non-NULL only if before-image journaling */
 	unsigned short		temp_ushort;
-	unsigned long		temp_long;
 	unsigned char		save_cw_set_depth;
 	DCL_THREADGBL_ACCESS;
 
@@ -308,7 +310,7 @@ block_id swap_root_or_directory_block(int parent_blk_lvl, int child_blk_lvl, src
 		t_abort(gv_cur_region, csa);
 		return ABORT_SWAP;
 	}
-	bmlhist.blk_num = (block_id)master_bit * BLKS_PER_LMAP;
+	bmlhist.blk_num = master_bit * BLKS_PER_LMAP;
 	if (NULL == (bmlhist.buffaddr = t_qread(bmlhist.blk_num, (sm_int_ptr_t)&bmlhist.cycle, &bmlhist.cr)))
 	{
 		assert(t_tries < CDB_STAGNATE);
@@ -316,7 +318,9 @@ block_id swap_root_or_directory_block(int parent_blk_lvl, int child_blk_lvl, src
 		return RETRY_SWAP;
 	}
 	hint_bit = 0;
-	maxbitsthismap = (master_bit != (num_local_maps - 1)) ? BLKS_PER_LMAP : total_blks - bmlhist.blk_num;
+	/* (total_blks - bmlhist.blk_num) can be cast because it should never be larger then BLKS_PER_LMAP */
+	DEBUG_ONLY(if(master_bit == (num_local_maps - 1)) assert(BLKS_PER_LMAP >= (total_blks - bmlhist.blk_num)));
+	maxbitsthismap = (master_bit != (num_local_maps - 1)) ? BLKS_PER_LMAP : (int4)(total_blks - bmlhist.blk_num);
 	free_bit = bm_find_blk(hint_bit, bmlhist.buffaddr + SIZEOF(blk_hdr), maxbitsthismap, &free_blk_recycled);
 	free_blk_id = bmlhist.blk_num + free_bit;
 	if (DIR_ROOT >= free_blk_id)
@@ -344,7 +348,7 @@ block_id swap_root_or_directory_block(int parent_blk_lvl, int child_blk_lvl, src
 	CHECK_AND_RESET_UPDATE_ARRAY;
 	if (free_blk_recycled)
 	{	/* Otherwise, it's a completely free block, in which case no need to read. */
-		freeblkhist.blk_num = (block_id)free_blk_id;
+		freeblkhist.blk_num = free_blk_id;
 		if (NULL == (freeblkhist.buffaddr = t_qread(free_blk_id, (sm_int_ptr_t)&freeblkhist.cycle, &freeblkhist.cr)))
 		{
 			assert(t_tries < CDB_STAGNATE);
@@ -408,7 +412,7 @@ block_id swap_root_or_directory_block(int parent_blk_lvl, int child_blk_lvl, src
 	bpntr_end = curr_offset + hdr_len + SIZEOF(block_id);
 	BLK_SEG(bs_ptr, parent_blk_ptr + SIZEOF(blk_hdr), curr_offset + hdr_len - SIZEOF(blk_hdr));
 	BLK_ADDR(bn_ptr, SIZEOF(block_id), unsigned char);
-	PUT_LONG(bn_ptr, free_blk_id);
+	PUT_BLK_ID(bn_ptr, free_blk_id);
 	BLK_SEG(bs_ptr, bn_ptr, SIZEOF(block_id));
 	BLK_SEG(bs_ptr, parent_blk_ptr + bpntr_end, parent_blk_size - bpntr_end);
 	assert(blk_seg_cnt == parent_blk_size);
@@ -429,8 +433,8 @@ block_id swap_root_or_directory_block(int parent_blk_lvl, int child_blk_lvl, src
 	cw_map_depth = cw_set_depth;
 	cw_set_depth = save_cw_set_depth;
 	update_array_ptr += SIZEOF(block_id);
-	temp_long = 0;
-	PUT_LONG(update_array_ptr, temp_long);
+	temp_blk = 0;
+	PUT_BLK_ID(update_array_ptr, temp_blk);
 	update_array_ptr += SIZEOF(block_id);
 	assert(1 == cw_set[cw_map_depth - 1].reference_cnt);
 	/* 4. Child block gets marked recycled in bitmap. (GVCST_BMP_MARK_FREE) */
