@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2017 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -43,8 +43,9 @@ MBSTART {						\
 		rts_error_csa MSGPARMS;			\
 } MBEND
 
-GBLREF gd_region	*gv_cur_region;
-GBLREF sgmnt_addrs	*cs_addrs;
+GBLREF gd_region		*gv_cur_region;
+GBLREF sgmnt_addrs		*cs_addrs;
+GBLREF sgmnt_data_ptr_t        cs_data;
 
 error_def(ERR_AUTODBCREFAIL);
 error_def(ERR_FILECREERR);
@@ -55,13 +56,14 @@ void mucblkini(void)
 	blk_hdr_ptr_t		bp1, bp2;
 	rec_hdr_ptr_t		rp;
 	unix_db_info		*udi;
-	int4			tmp, bmpsize, status;
+	int4			bmpsize, status;
+	block_id		blk;
 
 	udi = FILE_INFO(gv_cur_region);
 	bp1 = (blk_hdr_ptr_t)malloc(cs_addrs->hdr->blk_size);
 	bp2 = (blk_hdr_ptr_t)malloc(cs_addrs->hdr->blk_size);
 	bmpsize = BM_SIZE(cs_addrs->hdr->bplmap);
-	if (cs_addrs->do_fullblockwrites)
+	if (cs_data->write_fullblk)
 		bmpsize = (int4)ROUND_UP(bmpsize, cs_addrs->fullblockwrite_len);
 	bmp = (uchar_ptr_t)malloc(bmpsize);
 	DB_LSEEKREAD(udi, udi->fd, (off_t)BLK_ZERO_OFF(cs_addrs->hdr->start_vbn), bmp, bmpsize, status);
@@ -85,8 +87,8 @@ void mucblkini(void)
 	rp = (rec_hdr_ptr_t)((uchar_ptr_t)bp1 + SIZEOF(blk_hdr));
 	BSTAR_REC(rp);
 	c = CST_BOK(rp);
-	tmp = DIR_DATA;
-	PUT_LONG(c, tmp);
+	blk = DIR_DATA;
+	PUT_BLK_ID(c, blk);
 	bp1->bver = GDSVCURR;
 	bp1->levl = 1;
 	bp1->bsiz = BSTAR_REC_SIZE + SIZEOF(blk_hdr);
