@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2006-2018 Fidelity National Information	*
+ * Copyright (c) 2006-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -113,7 +113,7 @@ typedef struct
 	int			outdb_fd;		/* FD_INVALID if inplace */
 	boolean_t		inplace;		/* update in place */
 	boolean_t		endian_native;		/* original database */
-	uint4			tot_blks;
+	block_id		tot_blks;
 	int			bsize;			/* GDS block size */
 	int4			startvbn;		/* in DISK_BLOCK_SIZE units */
 	block_id		last_blk_cvt;		/* highest block converted so far not lbm */
@@ -138,7 +138,7 @@ typedef struct
 void		endian_header(sgmnt_data *new, sgmnt_data *old, boolean_t new_is_native);
 int4		endian_process(endian_info *info, sgmnt_data *new_data, sgmnt_data *old_data, boolean_t override_specified);
 void		endian_cvt_blk_hdr(blk_hdr_ptr_t blkhdr, boolean_t new_is_native, boolean_t make_empty);
-void		endian_cvt_blk_recs(endian_info *info, char *new_block, blk_hdr_ptr_t blkhdr, int blknum);
+void		endian_cvt_blk_recs(endian_info *info, char *new_block, blk_hdr_ptr_t blkhdr, block_id blknum);
 char		*endian_read_dbblk(endian_info *info, block_id blk_to_get);
 void		endian_find_key(endian_info *info, end_gv_key *gv_key, char *rec_p, int rec_len, int blk_levl);
 boolean_t	endian_match_key(end_gv_key *gv_key1, int blk_levl, end_gv_key *key2);
@@ -181,7 +181,7 @@ void mupip_endiancvt(void)
 	endian32_struct		endian_check;
 	endian_info		info;
 	int			gtmcrypt_errno;
-	ZOS_ONLY(int 		realfiletag;)
+	ZOS_ONLY(int		realfiletag;)
 
 	if (CLI_PRESENT == (cli_status = cli_present("OUTDB")))
 	{
@@ -438,7 +438,7 @@ void mupip_endiancvt(void)
 	info.is_encrypted = endian_native ? old_data->is_encrypted : GTM_BYTESWAP_32(old_data->is_encrypted);
 	info.encryption_hash_cutoff = endian_native
 		? old_data->encryption_hash_cutoff
-		: GTM_BYTESWAP_32(old_data->encryption_hash_cutoff);
+		: BLK_ID_BYTESWAP(old_data->encryption_hash_cutoff);
 	if (USES_ANY_KEY(&info))
 	{	/* Database is encrypted. Initialize encryption and setup the keys to be used in later encryption/decryption */
 		info.database_fn = &db_name[0];
@@ -493,7 +493,7 @@ void mupip_endiancvt(void)
 	info.db_fd = db_fd;
 	info.inplace = !outdb_specified;
 	info.endian_native = info.dtblk.native = info.dtblk.dtrnative = endian_native;
-	info.tot_blks = info.bsize = info.startvbn = info.last_blk_cvt = 0;
+	info.tot_blks = info.last_blk_cvt = info.bsize = info.startvbn = 0;
 	info.dtblk.buff = info.dtblk.dtrbuff = NULL;
 	info.dtblk.blkid = -1;			/* invalid block number */
 	info.dtblk.count = 0;
@@ -669,8 +669,10 @@ void endian_header(sgmnt_data *new, sgmnt_data *old, boolean_t new_is_native)
 
 	/************* MOSTLY STATIC DATABASE STATE FIELDS **************************/
 	SWAP_SD4(blk_size);
+	assert(SIZEOF(int4) == SIZEOF(old->master_map_len));
 	SWAP_SD4(master_map_len);
 	SWAP_SD4(bplmap);
+	assert(SIZEOF(int4) == SIZEOF(old->start_vbn));
 	SWAP_SD4(start_vbn);
 	assert(SIZEOF(int4) == SIZEOF(old->acc_meth));		/* enum */
 	SWAP_SD4_CAST(acc_meth, enum db_acc_method);
@@ -684,6 +686,7 @@ void endian_header(sgmnt_data *new, sgmnt_data *old, boolean_t new_is_native)
 	SWAP_SD4(extension_size);
 	SWAP_SD4(def_coll);
 	SWAP_SD4(def_coll_ver);
+	SWAP_SD4(write_fullblk);
 	assert(SIZEOF(int4) == SIZEOF(old->std_null_coll));	/* boolean_t */
 	SWAP_SD4(std_null_coll);
 	SWAP_SD4(null_subs);
@@ -797,7 +800,7 @@ void endian_header(sgmnt_data *new, sgmnt_data *old, boolean_t new_is_native)
 	SWAP_SD4(ccp_jnl_before);
 	SWAP_SD4(clustered);
 	SWAP_SD4(unbacked_cache);
-        /* RC server related fields sb zero when not active	*/
+	/* RC server related fields sb zero when not active	*/
 	SWAP_SD4(rc_srv_cnt);
 	SWAP_SD4(dsid);
 	SWAP_SD4(rc_node);
@@ -862,7 +865,9 @@ void endian_header(sgmnt_data *new, sgmnt_data *old, boolean_t new_is_native)
 #	undef TAB_DB_CSH_ACCT_REC
 	/************* GVSTATS_REC RELATED FIELDS ***********/
 #	define TAB_GVSTATS_REC(COUNTER,TEXT1,TEXT2)	SWAP_SD8(gvstats_rec.COUNTER);
+#	define IS_CSD_STATS 1
 #	include "tab_gvstats_rec.h"
+#	undef IS_CSD_STATS
 #	undef TAB_GVSTATS_REC
 	/************* INTERRUPTED RECOVERY RELATED FIELDS continued ****************/
 	for (idx = 0; idx < MAX_SUPPL_STRMS; idx++)
@@ -898,13 +903,13 @@ int4	endian_process(endian_info *info, sgmnt_data *new_data, sgmnt_data *old_dat
 	   This routine based on mubinccpy and dbcertify_scan_phase
 	*/
 	int4		startvbn;
-	int		save_errno, bsize, lbmap_cnt, lbm_status;
+	int		save_errno, bsize, lbm_status;
 	int		buff_native, buff_old, buff_new;
-	int		mm_offset, lm_offset;
+	int		lm_offset;
 	int4		bplmap;
-	uint4		totblks, lbm_done, busy_done, recycled_done, free_done, last_blk_written;
 	off_t		dbptr;
-	block_id	blk_num;
+	block_id	blk_num, totblks, last_blk_written, mm_offset,
+			lbm_done, busy_done, recycled_done, free_done, lbmap_cnt;
 	boolean_t	new_is_native;
 	char		*blk_buff[2], *lbmap_buff[2], *errptr;
 	int		crypt_blk_size, gtmcrypt_errno;
@@ -1376,9 +1381,9 @@ block_id	endian_find_dtblk(endian_info *info, end_gv_key *gv_key)
 				return -1;
 			if (0 != blk_levl && BSTAR_REC_SIZE == rec_len)
 			{	/* down to the next level */
-				GET_ULONG(blk_ptr, rec_p + SIZEOF(rec_hdr));
+				GET_BLK_ID(blk_ptr, rec_p + SIZEOF(rec_hdr));
 				if (!blk_is_native)
-					blk_ptr = GTM_BYTESWAP_32(blk_ptr);
+					blk_ptr = BLK_ID_BYTESWAP(blk_ptr);
 				if (blk_ptr > info->tot_blks)
 					return -1;	/* past end of database */
 				blk_to_get = blk_ptr;
@@ -1390,9 +1395,9 @@ block_id	endian_find_dtblk(endian_info *info, end_gv_key *gv_key)
 				if (0 == blk_levl)
 					return blk_to_get;	/* found dtleaf block we are looking for */
 				ptroffset = found_gv_key.end - EVAL_CMPC((rec_hdr *)rec_p) + 1;
-				GET_ULONG(blk_ptr, (rec_p + SIZEOF(rec_hdr) + ptroffset));
+				GET_BLK_ID(blk_ptr, rec_p + SIZEOF(rec_hdr) + ptroffset);
 				if (!blk_is_native)
-					blk_ptr = GTM_BYTESWAP_32(blk_ptr);
+					blk_ptr = BLK_ID_BYTESWAP(blk_ptr);
 				if (blk_ptr > info->tot_blks)
 					return -1;	/* past end of database */
 				blk_to_get = blk_ptr;
@@ -1409,7 +1414,7 @@ block_id	endian_find_dtblk(endian_info *info, end_gv_key *gv_key)
 	}
 }
 
-void endian_cvt_blk_recs(endian_info *info, char *new_block, blk_hdr_ptr_t blkhdr, int blknum)
+void endian_cvt_blk_recs(endian_info *info, char *new_block, blk_hdr_ptr_t blkhdr, block_id blknum)
 {	/* convert records in new_block, could be data, index, or directory
 	   use converted header fields from blkhdr which is in native format */
 	int		rec1_len, rec1_gvn_len, rec2_cmpc, rec2_len;
@@ -1485,9 +1490,9 @@ void endian_cvt_blk_recs(endian_info *info, char *new_block, blk_hdr_ptr_t blkhd
 		else if (SIZEOF(block_id) <= ((rec1_ptr + rec1_len) - ++key_top) &&
 			 (SIZEOF(block_id) + MAX_SPEC_TYPE_LEN) >= ((rec1_ptr + rec1_len) - key_top))
 		{	/* record value long enough for block_id but not longer than block_id plus collation information */
-			GET_LONG(ptr2blk, key_top);
+			GET_BLK_ID(ptr2blk, key_top);
 			if (new_is_native)
-				ptr2blk = GTM_BYTESWAP_32(ptr2blk);
+				ptr2blk = BLK_ID_BYTESWAP(ptr2blk);
 			if (ptr2blk <= info->tot_blks)
 			{	/* might be a pointer so need to check the hard way */
 				gv_key.key = rec1_ptr + SIZEOF(rec_hdr);
@@ -1520,9 +1525,9 @@ void endian_cvt_blk_recs(endian_info *info, char *new_block, blk_hdr_ptr_t blkhd
 			} else
 				assert((key_top + SIZEOF(block_id) == blk_top) || blk_levl);	/* must be last if not leaf */
 			assert((key_top + SIZEOF(block_id)) <= (rec1_ptr + rec1_len));
-			GET_LONG(ptr2blk, key_top);
-			ptr2blk_swap = GTM_BYTESWAP_32(ptr2blk);
-			PUT_LONG(key_top, ptr2blk_swap);
+			GET_BLK_ID(ptr2blk, key_top);
+			ptr2blk_swap = BLK_ID_BYTESWAP(ptr2blk);
+			PUT_BLK_ID(key_top, ptr2blk_swap);
 #ifdef DEBUG
 			if (new_is_native)
 				ptr2blk = ptr2blk_swap;
