@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2015-2018 Fidelity National Information	*
+ * Copyright (c) 2015-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -17,7 +17,6 @@
 #include "gdsfhead.h"
 #include "filestruct.h"
 #include "jnl.h"
-#include "hashtab_int4.h"
 #include "gdscc.h"
 #include "buddy_list.h"
 #include "tp.h"
@@ -79,7 +78,7 @@ GBLREF tp_region	*grlist;
 GBLREF uint4		process_id;
 GBLREF uint4		update_array_size;		/* for the BLK_INIT/BLK_SEG/BLK_ADDR macros */
 GBLREF uint4		update_trans;
-GBLREF unsigned char    cw_map_depth;
+GBLREF unsigned char	cw_map_depth;
 GBLREF unsigned char	cw_set_depth;
 GBLREF unsigned char	rdfail_detail;
 GBLREF sgmnt_addrs	*reorg_encrypt_restart_csa;
@@ -145,7 +144,7 @@ void mupip_reorg_encrypt(void)
 	char			key[GTM_PATH_MAX], hash[GTMCRYPT_HASH_LEN];
 	char			*db_name, *bml_lcl_buff;
 	int			db_name_len, gtmcrypt_errno, status, reg_status, status1;
-	int			reg_count, i, total_blks, cycle, lcnt, bml_status;
+	int			reg_count, i, cycle, lcnt, bml_status;
 	int4			blk_seg_cnt, blk_size, mapsize;	/* needed for BLK_INIT,BLK_SEG and BLK_FINI macros */
 	unsigned short		key_len;
 	gd_region		*reg;
@@ -158,19 +157,19 @@ void mupip_reorg_encrypt(void)
 	sgmnt_addrs		*csa;
 	sgmnt_data_ptr_t	csd;
 	node_local_ptr_t	cnl;
-	block_id		*blkid_ptr, start_blk, start_bmp, last_bmp, curbmp, curblk;
+	block_id		*blkid_ptr, start_blk, start_bmp, last_bmp, curbmp, curblk, total_blks;
 	trans_num		curr_tn, start_tn, blk_tn;
 	sm_uc_ptr_t		blkBase, bml_sm_buff;	/* shared memory pointer to the bitmap global buffer */
 	cache_rec_ptr_t		cr;
 	blk_segment		*bs1, *bs_ptr;
 	sm_uc_ptr_t		bptr, buff;
 	blk_hdr			new_hdr;
-	unsigned char    	save_cw_set_depth;
+	unsigned char		save_cw_set_depth;
 	uint4			lcl_update_trans, pid, bptr_size;
 	jnl_private_control	*jpc;
 #	ifdef DEBUG
 	uint4			reencryption_count;
-	uint4			initial_blk_cnt;
+	block_id		initial_blk_cnt;
 #	endif
 	unix_db_info		*udi;
 	DCL_THREADGBL_ACCESS;
@@ -297,7 +296,7 @@ void mupip_reorg_encrypt(void)
 				IS_ERROR, gtm_putmsg_csa, (CSA_ARG(csa) VARLSTCNT(4) ERR_DBRDONLY, 2, db_name_len, db_name));
 		}
 		/* ++++++++++++++++++++++++++ IN CRIT ++++++++++++++++++++++++++ */
-		grab_crit(reg);
+		grab_crit(reg, WS_91);
 		if (!csd->fully_upgraded)
 		{
 			CONTINUE_TO_NEXT_REGION(csa, csd, cnl, reg, reg_status, status,
@@ -454,7 +453,7 @@ void mupip_reorg_encrypt(void)
 			assert(!csa->now_crit);
 			bml_sm_buff = t_qread(curbmp, (sm_int_ptr_t)&cycle, &cr); /* bring block into the cache outside of crit */
 			/* ++++++++++++++++++++++++++ IN CRIT ++++++++++++++++++++++++++ */
-			grab_crit_encr_cycle_sync(reg); /* needed so t_qread does not return NULL below */
+			grab_crit_encr_cycle_sync(reg, WS_92); /* needed so t_qread does not return NULL below */
 			/* Safeguard against someone concurrently changing the database file header. It is unsafe to continue. */
 			if (start_tn != csd->encryption_hash2_start_tn)
 			{
@@ -515,7 +514,9 @@ void mupip_reorg_encrypt(void)
 			 *         (Re)encrypt all BUSY and REUSABLE blocks in the current bitmap
 			 * ------------------------------------------------------------------------
 			 */
-			mapsize = (curbmp == last_bmp) ? (total_blks - curbmp) : BLKS_PER_LMAP;
+			/* (total_blks - curbmp) can be cast because it should never be larger then BLKS_PER_LMAP */
+			DEBUG_ONLY(if(curbmp == last_bmp) assert(BLKS_PER_LMAP >= (total_blks - curbmp)));
+			mapsize = (curbmp == last_bmp) ? (int4)(total_blks - curbmp) : BLKS_PER_LMAP;
 			assert(0 != mapsize);
 			for (lcnt = 0; lcnt < mapsize; lcnt++)
 			{
@@ -720,7 +721,7 @@ void mupip_reorg_encrypt(void)
 		if (SS_NORMAL == reg_status)
 		{
 			get_ftok_semaphore(reg, csa);
-			grab_crit(reg);
+			grab_crit(reg, WS_93);
 			/* Wait for all the readers to complete to prevent them from attempting to digest an
 			 * encrypted block or decrypt a block with a wrong key in case MUPIP REORG -ENCRYPT has
 			 * concurrently processed that block.
@@ -764,7 +765,7 @@ void mupip_reorg_encrypt(void)
 		if (!csa->now_crit)
 		{
 			get_ftok_semaphore(reg, csa);
-			grab_crit(reg);
+			grab_crit(reg, WS_94);
 		}
 		assert(csa->now_crit);
 		assert(UNSTARTED == csd->encryption_hash_cutoff || (SS_NORMAL != reg_status));
