@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -42,7 +42,6 @@
 #include "timersp.h"
 #include "gt_timer.h"
 #include "buddy_list.h"
-#include "hashtab_int4.h"	/* needed for tp.h */
 #include "tp.h"
 #include "util.h"
 #include "mupip_set.h"
@@ -113,22 +112,26 @@ MBSTART {									\
 
 int4 mupip_set_file(int db_fn_len, char *db_fn)
 {
-	boolean_t		bypass_partial_recov, got_standalone, need_standalone = FALSE, acc_meth_changing;
+	boolean_t		bypass_partial_recov, flush_buffers = FALSE, got_standalone, need_standalone = FALSE,
+				acc_meth_changing;
 	char			acc_spec[MAX_ACC_METH_LEN + 1], *command = "MUPIP SET VERSION", *errptr, exit_stat, *fn,
 				ver_spec[MAX_DB_VER_LEN + 1];
 	enum db_acc_method	access, access_new;
 	enum db_ver		desired_dbver;
+	uint4                   fbwsize;
+	int4			dblksize;
 	gd_region		*temp_cur_region;
 	int			asyncio_status, defer_allocate_status, defer_status, disk_wait_status, encryptable_status,
 				encryption_complete_status, epoch_taper_status, extn_count_status, fd, fn_len, glbl_buff_status,
 				gtmcrypt_errno, hard_spin_status, inst_freeze_on_error_status, key_size_status, locksharesdbcrit,
 				lock_space_status, mutex_space_status, null_subs_status, qdbrundown_status, rec_size_status,
 				reg_exit_stat, rc, rsrvd_bytes_status, sleep_cnt_status, save_errno, stats_status, status,
-				status1, stdnullcoll_status, trigger_flush_limit_status, wrt_per_flu_status;
+				status1, stdnullcoll_status, trigger_flush_limit_status, wrt_per_flu_status,
+				full_blkwrt_status;
 	int4			defer_time, new_cache_size, new_disk_wait, new_extn_count, new_flush_trigger, new_hard_spin,
 				new_key_size, new_lock_space, new_mutex_space, new_null_subs, new_rec_size, new_sleep_cnt,
 				new_spin_sleep, new_stdnullcoll, new_wrt_per_flu, reserved_bytes, spin_sleep_status,
-				read_only_status;
+				read_only_status, new_full_blkwrt;
 	sgmnt_data_ptr_t	csd, pvt_csd;
 	tp_region		*rptr, single;
 	unsigned short		acc_spec_len = MAX_ACC_METH_LEN, ver_spec_len = MAX_DB_VER_LEN;
@@ -168,7 +171,8 @@ int4 mupip_set_file(int db_fn_len, char *db_fn)
 					    which has not yet been read */
 	if (asyncio_status = cli_present("ASYNCIO"))
 		need_standalone = TRUE;
-	defer_allocate_status = cli_present("DEFER_ALLOCATE");
+	if (defer_allocate_status = cli_present("DEFER_ALLOCATE"))
+		flush_buffers = TRUE;
 	if (encryptable_status = cli_present("ENCRYPTABLE"))
 	{
 		need_standalone = TRUE;
@@ -211,6 +215,25 @@ int4 mupip_set_file(int db_fn_len, char *db_fn)
 			gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_SETQUALPROB, 2, LEN_AND_LIT("EXTENSION COUNT"));
 			exit_stat |= EXIT_ERR;
 		}
+		flush_buffers = TRUE;
+	}
+	if (full_blkwrt_status = cli_present("FULLBLKWRT"))
+	{
+		if (cli_get_int("FULLBLKWRT", &new_full_blkwrt))
+		{
+			/* minimum is 0 & mupip_cmd defines this qualifier to not accept negative values, so no min check */
+			if (new_full_blkwrt > FULL_DATABASE_WRITE)
+			{
+				gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_MUPIPSET2BIG, 4, new_full_blkwrt,
+					LEN_AND_LIT("FULLBLKWRT"), 2);
+				exit_stat |= EXIT_ERR;
+			}
+			need_standalone = TRUE;
+		} else
+		{
+			gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_SETQUALPROB, 2, LEN_AND_LIT("FULLBLKWRT"));
+			exit_stat |= EXIT_ERR;
+		}
 	}
 	if (glbl_buff_status = cli_present("GLOBAL_BUFFERS"))
 	{
@@ -397,6 +420,7 @@ int4 mupip_set_file(int db_fn_len, char *db_fn)
 			desired_dbver = GDSV6;
 		else
 			assertpro(FALSE);		/* CLI should prevent us ever getting here */
+		flush_buffers = TRUE;
 	} else
 		desired_dbver = GDSVLAST;	/* really want to keep version, which has not yet been read */
 	if (disk_wait_status = cli_present("WAIT_DISK"))
@@ -688,6 +712,14 @@ int4 mupip_set_file(int db_fn_len, char *db_fn)
 				DO_CLNUP_AND_SET_EXIT_STAT(exit_stat, reg_exit_stat);
 				continue;
 			}
+			if (full_blkwrt_status)
+			{
+				fbwsize = get_fs_block_size(fd);
+				dblksize = pvt_csd->blk_size;
+				if (0 != fbwsize && (0 == dblksize % fbwsize) &&
+						(0 == (BLK_ZERO_OFF(pvt_csd->start_vbn)) % fbwsize))
+					pvt_csd->write_fullblk = new_full_blkwrt;
+			}
 			csd = pvt_csd;
 		} else /* if (!need_standalone) */
 		{
@@ -717,19 +749,15 @@ int4 mupip_set_file(int db_fn_len, char *db_fn)
 			}
 			csd = cs_data;
 			assert(!cs_addrs->hold_onto_crit); /* this ensures we can safely do unconditional grab_crit and rel_crit */
-			grab_crit(gv_cur_region);
+			grab_crit(gv_cur_region, WS_95);
 			if (FROZEN_CHILLED(cs_addrs))
 			{
-				DO_CHILLED_AUTORELEASE(cs_addrs, cs_data);
-				if (FROZEN_CHILLED(cs_addrs))
-				{
-					gtm_putmsg_csa(CSA_ARG(cs_addrs) VARLSTCNT(4) ERR_OFRZACTIVE, 2,
-								DB_LEN_STR(gv_cur_region));
-					exit_stat |= EXIT_WRN;
-					exit_stat |= gds_rundown(CLEANUP_UDI_TRUE);
-					mu_gv_cur_reg_free();
-					continue;
-				}
+				gtm_putmsg_csa(CSA_ARG(cs_addrs) VARLSTCNT(4) ERR_OFRZACTIVE, 2,
+							DB_LEN_STR(gv_cur_region));
+				exit_stat |= EXIT_WRN;
+				exit_stat |= gds_rundown(CLEANUP_UDI_TRUE);
+				mu_gv_cur_reg_free();
+				continue;
 			}
 		}
 		access_new = (n_dba == access ? csd->acc_meth : access);
@@ -923,6 +951,24 @@ int4 mupip_set_file(int db_fn_len, char *db_fn)
 			if (wrt_per_flu_status)
 				util_out_print("Database file !AD now has writes per flush !UL",
 					TRUE, fn_len, fn, csd->n_wrt_per_flu);
+			if (full_blkwrt_status)
+			{
+				switch(csd->write_fullblk)
+				{
+					case 0:
+						util_out_print("Database file !AD now has full blk writes: !AD",
+							TRUE, fn_len, fn, LEN_AND_LIT("disabled"));
+						break;
+					case 1:
+						util_out_print("Database file !AD now has full blk writes: !AD",
+							TRUE, fn_len, fn, LEN_AND_LIT("file system block writes"));
+						break;
+					case 2:
+						util_out_print("Database file !AD now has full blk writes: !AD",
+							TRUE, fn_len, fn, LEN_AND_LIT("full DB block writes"));
+						break;
+				}
+			}
 			if (got_standalone)
 			{
 				DB_LSEEKWRITE(NULL, ((unix_db_info *)NULL), NULL, fd, 0, pvt_csd, SIZEOF(sgmnt_data), status);
@@ -935,6 +981,7 @@ int4 mupip_set_file(int db_fn_len, char *db_fn)
 					util_out_print("Database file !AD not changed: ", TRUE, fn_len, fn);
 					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_DBRDERR, 2, fn_len, fn);
 				}
+
 				if (defer_status && (dba_mm == pvt_csd->acc_meth))
 					util_out_print("Database file !AD now has defer_time set to !SL",
 							TRUE, fn_len, fn, pvt_csd->defer_time);
@@ -980,7 +1027,10 @@ int4 mupip_set_file(int db_fn_len, char *db_fn)
 							(CLI_PRESENT == stdnullcoll_status) ?
 							"M standard null collation" : "GT.M null collation      ");
 			} else
-				wcs_flu(WCSFLU_FLUSH_HDR);
+				if (flush_buffers)
+					wcs_flu(WCSFLU_FLUSH_HDR);
+				else
+					fileheader_sync(gv_cur_region);
 		} else
 			exit_stat |= reg_exit_stat;
 		if (got_standalone)
