@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2012-2018 Fidelity National Information	*
+ * Copyright (c) 2012-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -33,7 +33,6 @@
 #include "gdscc.h"
 #include "jnl.h"
 #include "buddy_list.h"		/* needed for tp.h */
-#include "hashtab_int4.h"	/* needed for tp.h */
 #include "t_qread.h"
 #include "tp.h"
 #include "mupint.h"
@@ -46,7 +45,9 @@
 #include "gtmmsg.h"
 #include "mu_getlst.h"
 #include "warn_db_sz.h"
+#include "mu_getkey.h"
 
+error_def(ERR_INTEGERRS);
 error_def(ERR_MUNOACTION);
 error_def(ERR_MUNOFINISH);
 error_def(ERR_MUPCLIERR);
@@ -59,9 +60,19 @@ GBLREF bool			mu_ctrlc_occurred;
 GBLREF bool			mu_ctrly_occurred;
 GBLREF int			muint_adj;
 GBLREF int4			mu_int_adj[MAX_BT_DEPTH + 1];
-GBLREF	sgmnt_data_ptr_t	cs_data;
+GBLREF sgmnt_data_ptr_t		cs_data;
 GBLREF tp_region		*grlist;
-GBLREF	unsigned char		rdfail_detail;
+GBLREF unsigned char		rdfail_detail;
+GBLDEF boolean_t mu_subsc	= FALSE;
+GBLDEF boolean_t mu_key		= MUKEY_FALSE;
+GBLDEF int mu_sub_idx_st	= 0;
+GBLDEF int mu_sub_idx_end	= 0;
+GBLDEF boolean_t null_coll_key	= FALSE;
+GBLREF boolean_t		null_coll_key;
+GBLREF gv_key			*mu_start_key;
+GBLREF gv_key			*mu_end_key;
+GBLREF int			mu_start_keyend;
+GBLREF int			mu_end_keyend;
 
 typedef struct {
 	enum {arsample, scan, impsample}	heuristic;
@@ -90,6 +101,9 @@ void mupip_size(void)
 	unsigned short		BUFF_LEN = SIZEOF(buff), n_len;
 	sgmnt_addrs 		*tcsa;
 	char 			*db_file_name;
+	unsigned char           key_buff[2048];
+	unsigned short          keylen;
+	boolean_t		subscr = FALSE;
 	DCL_THREADGBL_ACCESS;
 
 	SETUP_THREADGBL_ACCESS;
@@ -106,13 +120,53 @@ void mupip_size(void)
 		mu_getlst("REGION", SIZEOF(tp_region));
 	}
 	mupip_size_check_error();
+	if (CLI_PRESENT == cli_present("SUBSCRIPT"))
+        {
+		if (NULL == gv_target)
+			gv_target = (gv_namehead *)targ_alloc(DUMMY_GLOBAL_VARIABLE_LEN, NULL, NULL);
+		subscr = TRUE;
+                keylen = SIZEOF(key_buff);
+                if (0 == cli_get_str("SUBSCRIPT", (char *)key_buff, &keylen))
+                        mupip_exit(ERR_MUPCLIERR);
+                if (FALSE == mu_getkey(key_buff, keylen))
+		{
+			error_mupip = TRUE;
+			gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_MUSIZEINVARG, 2, LEN_AND_LIT("SUBSCRIPT"));
+		}
+		mupip_size_check_error();
+		assert(mu_key);
+		/* or else "mu_getkey" call above would have returned FALSE */
+		/* Create the buff with the global names only to be selected by gv_select below */
+		p_end = cli_buff;
+		memcpy(p_end ,"^",1);
+		p_end++;
+		memcpy(p_end , mu_start_key->base, mu_sub_idx_st);
+		p_end += mu_sub_idx_st;
+		if (mu_sub_idx_end)
+		{
+			memcpy(p_end ,":^",2);
+			p_end += 2;
+			memcpy(p_end, mu_end_key->base, mu_sub_idx_end);
+			p_end += mu_sub_idx_end;
+		}
+		*p_end = '\0';
+		n_len = p_end - cli_buff;
+		gv_target->regcnt--;
+		gv_target = NULL;
+        }
 	/* SELECT qualifier */
-	memset(cli_buff, 0, SIZEOF(cli_buff));
-	n_len = SIZEOF(cli_buff);
-	if (CLI_PRESENT != cli_present("SELECT"))
+	if (!subscr)
 	{
-		n_len = 1;
-		cli_buff[0] = '*';
+		memset(cli_buff, 0, SIZEOF(cli_buff));
+		n_len = SIZEOF(cli_buff);
+	}
+	if ((CLI_PRESENT != cli_present("SELECT")))
+	{
+		if (!subscr)
+		{	/* No select or subscript */
+			n_len = 1;
+			cli_buff[0] = '*';
+		}
 	}
 	else if (FALSE == cli_get_str("SELECT", cli_buff, &n_len))
 	{
@@ -236,21 +290,21 @@ STATICDEF void mupip_size_check_error(void)
  /* Performs a random traversal for the sampling methods */
 enum cdb_sc mu_size_rand_traverse(double *r, double *a)
 {
-	sm_uc_ptr_t			pVal, pTop, pRec, pBlkBase;
-	block_id			nBlkId;
-	block_id			valBlk[MAX_RECS_PER_BLK];	/* valBlk[j] := value in j-th record of current block */
-	boolean_t			is_mm;
+	boolean_t			first_key = TRUE;
+	boolean_t			musz_range_done, is_mm;
+	block_id			nBlkId, valBlk[MAX_RECS_PER_BLK]; /* valBlk[j] := value in j-th record of current block */
 	cache_rec_ptr_t			cr;
 	enum cdb_sc			status;
-	int				cycle;
-	int4				random;
-	int4				rCnt;				/* number of entries in valBlk */
+	int				blk_size, buff_length, cycle, key_size, name_len, rec_len;
+	int4				cmp_key, musz_rec, random, rCnt;	/* rCnt : number of entries in valBlk */
 	register gv_namehead		*pTarg;
 	register srch_blk_status	*pCurr;
 	register srch_hist		*pTargHist;
+	sm_uc_ptr_t			pVal, pTop, pRec, pBlkBase;
 	trans_num			tn;
-	unsigned char			nLevl;
-	unsigned short			nRecLen;
+	uchar_ptr_t			key_base, ptr;
+	unsigned char			buff[MAX_KEY_SZ + 1], nLevl;
+	unsigned short			nRecLen, rec_cmpc;
 	DCL_THREADGBL_ACCESS;
 
 	SETUP_THREADGBL_ACCESS;
@@ -281,11 +335,13 @@ enum cdb_sc mu_size_rand_traverse(double *r, double *a)
 	pCurr->cr = cr;
 	for (;;)
 	{
+		musz_range_done = FALSE;
+		musz_rec = 0;
 		assert(pCurr->level == nLevl);
 		pCurr->cse = NULL;
 		pCurr->blk_num = nBlkId;
 		pCurr->buffaddr = pBlkBase;
-		BLK_LOOP(rCnt, pRec, pBlkBase, pTop, nRecLen)
+		BLK_LOOP(rCnt, pRec, pBlkBase, pTop, nRecLen, musz_range_done)
 		{	/* enumerate records in block */
 			GET_AND_CHECK_RECLEN(status, nRecLen, pRec, pTop, nBlkId);
 			if (cdb_sc_normal != status)
@@ -293,12 +349,35 @@ enum cdb_sc mu_size_rand_traverse(double *r, double *a)
 				assert(CDB_STAGNATE > t_tries);
 				return status;
 			}
-			valBlk[rCnt] = nBlkId;
 			CHECK_ADJACENCY(nBlkId, nLevl, a[nLevl]);
+			if (mu_subsc) /*Subscript option chosen */
+			{
+				rec_cmpc = EVAL_CMPC((rec_hdr_ptr_t)pRec);
+				key_base = pRec + SIZEOF(rec_hdr);
+				if ((((rec_hdr *)pRec)->rsiz) != BSTAR_REC_SIZE) /* Did not find the star key */
+				{
+					GET_KEY_CPY_BUFF(key_base, rec_cmpc, ptr, first_key,
+							name_len, key_size,buff, buff_length, rec_len);
+					if (0 == key_size)
+						continue;	/* indication of trouble parsing the block - don't use it */
+					cmp_key = memcmp(buff, mu_start_key->base, mu_start_key->end + 1);
+					if (mu_end_key)
+					{
+						if (0 < memcmp(buff, mu_end_key->base, mu_end_key->end + 1))
+							musz_range_done = TRUE;
+					} else if (0 < cmp_key)
+						musz_range_done = TRUE;
+					if (0 > cmp_key)
+						continue;
+				}
+				valBlk[musz_rec] = nBlkId;
+				musz_rec++;
+			} else
+				valBlk[rCnt] = nBlkId;
 		}
-		r[nLevl] = rCnt;
+		(mu_subsc) ? (r[nLevl] = musz_rec) : (r[nLevl] = rCnt);
 		/* randomly select next block */
-		random = (int4)(rCnt * drand48());
+		random = ((mu_subsc) ? (int4)(musz_rec * drand48()) : (int4)(rCnt * drand48()));
 		random = random & 0x7fffffff; /* to make sure that the sign bit(msb) is off */
 		nBlkId = valBlk[random];
 		if (is_mm && (nBlkId > cs_addrs->total_blks))
