@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -35,7 +35,8 @@
 #include "callintogtmxfer.h"
 #include "min_max.h"
 #include "have_crit.h"
-#include "gtm_malloc.h"		/* for verifyAllocatedStorage */
+#include "gtmdbglvl.h"		/* for gtm_malloc.h */
+#include "gtm_malloc.h"		/* for VERIFY_STORAGE_CHAINS */
 #include "send_msg.h"
 
 /******************************************************************************
@@ -114,6 +115,7 @@ GBLREF pthread_t	gtm_main_thread_id;
 GBLREF boolean_t	gtm_main_thread_id_set;
 #endif
 
+LITREF	mval		literal_null;
 LITREF mval		skiparg;
 
 error_def(ERR_JNI);
@@ -122,6 +124,7 @@ error_def(ERR_TEXT);
 error_def(ERR_UNIMPLOP);
 error_def(ERR_XCVOIDRET);
 error_def(ERR_ZCARGMSMTCH);
+error_def(ERR_ZCCONVERT);
 error_def(ERR_ZCCTENV);
 error_def(ERR_ZCMAXPARAM);
 error_def(ERR_ZCNOPREALLOUTPAR);
@@ -153,6 +156,28 @@ static const int buff_boarder_len = 7;
 static const char *buff_front_boarder = "SMARKER";
 static const char *buff_end_boarder = "EMARKER";
 
+/* If the assigned pointer value is NULL, pass back a literal_null */
+#define	VALIDATE_AND_CONVERT_PTR_TO_TYPE(LMVTYPE, TYPE, CONTAINER, DST, SRC)	\
+MBSTART {									\
+	if (NULL == SRC)							\
+		*DST = literal_null;						\
+	else 									\
+	{									\
+		CONTAINER = *((TYPE *)SRC);					\
+		MV_FORCE_##LMVTYPE(DST, CONTAINER);				\
+	}									\
+} MBEND
+
+/* if FLAG is false, set it to true and syslog it once */
+#define ISSUE_ERR_ONCE(FLAG, ...)		\
+{						\
+	if (!FLAG)				\
+	{					\
+		FLAG = TRUE;			\
+		send_msg_csa(__VA_ARGS__);	\
+	}					\
+}
+
 /* Routine to set up boarders around the external call buffer */
 STATICFNDCL gparam_list *set_up_buffer(char *p_list, int len)
 {
@@ -169,10 +194,13 @@ STATICFNDCL void verify_buffer(char *p_list, int len, char *m_label)
 		send_msg_csa(CSA_ARG(NULL) VARLSTCNT(3) ERR_EXTCALLBOUNDS, 1, m_label);
 		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(3) ERR_EXTCALLBOUNDS, 1, m_label);
 	}
+	VERIFY_STORAGE_CHAINS;
 }
 
 STATICFNDCL void free_return_type(INTPTR_T ret_val, enum gtm_types typ)
 {
+	if (0 == ret_val)	/* Nothing to free regardless of type */
+		return;
 	switch (typ)
 	{
 		case gtm_jstring:
@@ -198,7 +226,8 @@ STATICFNDCL void free_return_type(INTPTR_T ret_val, enum gtm_types typ)
 		case gtm_jbig_decimal:
 			break;
 		case gtm_string_star:
-			free((((gtm_string_t *)ret_val)->address));
+			if (NULL != (((gtm_string_t *)ret_val)->address))
+				free((((gtm_string_t *)ret_val)->address));
 			free(((gtm_string_t *)ret_val));
 			break;
 		case gtm_float_star:
@@ -223,7 +252,8 @@ STATICFNDCL void free_return_type(INTPTR_T ret_val, enum gtm_types typ)
 			free(((gtm_char_t *)ret_val));
 			break;
 		case gtm_char_starstar:
-			free(*((gtm_char_t **)ret_val));
+			if (NULL != *((gtm_char_t **)ret_val))
+				free(*((gtm_char_t **)ret_val));
 			free(((gtm_char_t **)ret_val));
 			break;
 	}
@@ -287,9 +317,9 @@ STATICFNDEF void extarg2mval(void *src, enum gtm_types typ, mval *dst, boolean_t
 				dst->mvtype = MV_STR;
 				if (sp->len > MAX_STRLEN)
 					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_MAXSTRLEN);
-				dst->str.len = (mstr_len_t)sp->len;
 				if ((0 < sp->len) && (NULL != sp->addr))
 				{
+					dst->str.len = (mstr_len_t)sp->len;
 					dst->str.addr = sp->addr;
 					s2pool(&dst->str);
 					/* In case of GTMByteArray or GTMString, the buffer is allocated in xc_gateway.c (since the
@@ -297,7 +327,8 @@ STATICFNDEF void extarg2mval(void *src, enum gtm_types typ, mval *dst, boolean_t
 					 * case of a string the content is immutable). So, if we have determined that the provided
 					 * value buffer is legitimate (non-null and non-empty), we free it on the GT.M side. */
 					free(sp->addr);
-				}
+				} else
+					*dst = literal_null;
 				break;
 			default:
 				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_UNIMPLOP);
@@ -330,12 +361,10 @@ STATICFNDEF void extarg2mval(void *src, enum gtm_types typ, mval *dst, boolean_t
 			MV_FORCE_UMVAL(dst, uns_int_num);
 			break;
 		case gtm_int_star:
-			s_int_num = *((gtm_int_t *)src);
-			MV_FORCE_MVAL(dst, s_int_num);
+			VALIDATE_AND_CONVERT_PTR_TO_TYPE(MVAL, gtm_int_t, s_int_num, dst, src);
 			break;
 		case gtm_uint_star:
-			uns_int_num = *((gtm_uint_t *)src);
-			MV_FORCE_UMVAL(dst, uns_int_num);
+			VALIDATE_AND_CONVERT_PTR_TO_TYPE(UMVAL, gtm_uint_t, uns_int_num, dst, src);
 			break;
 		case gtm_long:
 			s_long_num = (gtm_long_t)src;
@@ -346,53 +375,71 @@ STATICFNDEF void extarg2mval(void *src, enum gtm_types typ, mval *dst, boolean_t
 			MV_FORCE_ULMVAL(dst, uns_long_num);
 			break;
 		case gtm_long_star:
-			s_long_num = *((gtm_long_t *)src);
-			MV_FORCE_LMVAL(dst, s_long_num);
+			VALIDATE_AND_CONVERT_PTR_TO_TYPE(LMVAL, gtm_long_t, s_long_num, dst, src);
 			break;
 		case gtm_ulong_star:
-			uns_long_num = *((gtm_ulong_t *)src);
-			MV_FORCE_ULMVAL(dst, uns_long_num);
+			VALIDATE_AND_CONVERT_PTR_TO_TYPE(ULMVAL, gtm_ulong_t, uns_long_num, dst, src);
 			break;
 		case gtm_string_star:
 			sp = (struct extcall_string *)src;
-			dst->mvtype = MV_STR;
-			if (sp->len > MAX_STRLEN)
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_MAXSTRLEN);
-			if ((0 <= prealloc_size) && (sp->len > prealloc_size) && (sp->addr >= (char *)ext_buff_start)
-				&& (sp->addr < ((char *)ext_buff_start + ext_buff_len)))
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(5) ERR_EXCEEDSPREALLOC, 3, prealloc_size, m_label, sp->len);
-			dst->str.len = (mstr_len_t)sp->len;
-			if ((0 < sp->len) && (NULL != sp->addr))
+			if (NULL == sp) /* If the assigned pointer value is NULL, pass back a literal_null */
+				*dst = literal_null;
+			else
 			{
-				dst->str.addr = sp->addr;
-				s2pool(&dst->str);
+				dst->mvtype = MV_STR;
+				if (sp->len > MAX_STRLEN)
+					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_MAXSTRLEN);
+				if ((0 <= prealloc_size) && (sp->len > prealloc_size)
+						&& (sp->addr >= (char *)ext_buff_start)
+						&& (sp->addr < ((char *)ext_buff_start + ext_buff_len)))
+					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(5) ERR_EXCEEDSPREALLOC, 3,
+							prealloc_size, m_label, sp->len);
+				if ((0 < sp->len) && (NULL != sp->addr))
+				{
+					dst->str.len = (mstr_len_t)sp->len;
+					dst->str.addr = sp->addr;
+					s2pool(&dst->str);
+				} else
+					*dst = literal_null;
 			}
 			break;
 		case gtm_float_star:
-			float2mval(dst, *((float *)src));
+			if (NULL == src) /* If the assigned pointer value is NULL, pass back a literal_null */
+				*dst = literal_null;
+			else
+				float2mval(dst, *((float *)src));
 			break;
 		case gtm_char_star:
 			cp = (char *)src;
-			assert(((INTPTR_T)cp < (INTPTR_T)stringpool.base) || ((INTPTR_T)cp > (INTPTR_T)stringpool.top));
-			dst->mvtype = MV_STR;
-			str_len = STRLEN(cp);
-			if (str_len > MAX_STRLEN)
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_MAXSTRLEN);
-			if ((0 <= prealloc_size) && (str_len > prealloc_size))
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(5) ERR_EXCEEDSPREALLOC, 3, prealloc_size, m_label, str_len);
-			dst->str.len = (mstr_len_t)str_len;
-			dst->str.addr = cp;
-			s2pool(&dst->str);
+			if (NULL == cp) /* If the assigned pointer value is NULL, pass back a literal_null */
+				*dst = literal_null;
+			else
+			{
+				assert(((INTPTR_T)cp < (INTPTR_T)stringpool.base) || ((INTPTR_T)cp > (INTPTR_T)stringpool.top));
+				dst->mvtype = MV_STR;
+				str_len = STRLEN(cp);
+				if (str_len > MAX_STRLEN)
+					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_MAXSTRLEN);
+				if ((0 <= prealloc_size) && (str_len > prealloc_size))
+					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(5) ERR_EXCEEDSPREALLOC,
+									3, prealloc_size, m_label, str_len);
+				dst->str.len = (mstr_len_t)str_len;
+				dst->str.addr = cp;
+				s2pool(&dst->str);
+			}
 			break;
 		case gtm_char_starstar:
-			if (!src)
-				dst->mvtype = 0;
+			if (NULL == src) /* If the assigned pointer value is NULL, pass back a literal_null */
+				*dst = literal_null;
 			else
 				extarg2mval(*((char **)src), gtm_char_star, dst, java, starred, prealloc_size, m_label,
 						ext_buff_start, ext_buff_len);
 			break;
 		case gtm_double_star:
-			double2mval(dst, *((double *)src));
+			if (NULL == src) /* If the assigned pointer value is NULL, pass back a literal_null */
+				*dst = literal_null;
+			else
+				double2mval(dst, *((double *)src));
 			break;
 		default:
 			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_UNIMPLOP);
@@ -406,6 +453,9 @@ STATICFNDEF int extarg_getsize(void *src, enum gtm_types typ, mval *dst, struct
 {
 	char			*cp, **cpp;
 	struct extcall_string	*sp;
+	static bool		issued_ERR_XCRETNULLREF = FALSE;
+	static bool		issued_ERR_ZCCONVERT = FALSE;
+
 	if (!src)
 		switch (typ)
 		{
@@ -423,8 +473,10 @@ STATICFNDEF int extarg_getsize(void *src, enum gtm_types typ, mval *dst, struct
 			case gtm_jdouble:
 				return 0;
 			default:
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_XCRETNULLREF, 2,
-					LEN_AND_STR(entry_ptr->call_name.addr));
+				/* Handle null pointer return types */
+				ISSUE_ERR_ONCE(issued_ERR_XCRETNULLREF, CSA_ARG(NULL) VARLSTCNT(4)
+						ERR_XCRETNULLREF, 2, LEN_AND_STR(entry_ptr->call_name.addr))
+				return 0;
 				break;
 		}
 	switch (typ)
@@ -450,9 +502,12 @@ STATICFNDEF int extarg_getsize(void *src, enum gtm_types typ, mval *dst, struct
 			return 0;
 		case gtm_char_starstar:
 			cpp = (char **)src;
-			if (!(*cpp))
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_XCRETNULLREF, 2,
-					LEN_AND_STR(entry_ptr->call_name.addr));
+			if (NULL == (*cpp))
+			{
+				ISSUE_ERR_ONCE(issued_ERR_XCRETNULLREF, CSA_ARG(NULL) VARLSTCNT(4)
+						ERR_XCRETNULLREF, 2, LEN_AND_STR(entry_ptr->call_name.addr))
+				return 0;
+			}
 			return STRLEN(*cpp);
 		case gtm_char_star:
 			cp = (char *)src;
@@ -461,12 +516,19 @@ STATICFNDEF int extarg_getsize(void *src, enum gtm_types typ, mval *dst, struct
 		case gtm_jbyte_array:
 		case gtm_string_star:
 			sp = (struct extcall_string *)src;
-			if (sp->addr == NULL)
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_XCRETNULLREF, 2,
-					LEN_AND_STR(entry_ptr->call_name.addr));
-			assert(!((0 < sp->len)
-			    && ((INTPTR_T)sp->addr < (INTPTR_T)stringpool.free)
-			    && ((INTPTR_T)sp->addr >= (INTPTR_T)stringpool.base)));
+			if (NULL == sp->addr)
+			{
+				sp->len = 0;
+				ISSUE_ERR_ONCE(issued_ERR_XCRETNULLREF, CSA_ARG(NULL) VARLSTCNT(4)
+						ERR_XCRETNULLREF, 2, LEN_AND_STR(entry_ptr->call_name.addr))
+			} if (0 > sp->len)
+			{	/* Negative string length. syslog and reset to zero */
+				sp->len = 0;
+				ISSUE_ERR_ONCE(issued_ERR_ZCCONVERT, CSA_ARG(NULL) VARLSTCNT(4)
+						ERR_ZCCONVERT, 2, LEN_AND_STR(entry_ptr->call_name.addr))
+			} else
+				assert(!(((INTPTR_T)sp->addr < (INTPTR_T)stringpool.free)
+					&& ((INTPTR_T)sp->addr >= (INTPTR_T)stringpool.base)));
 			return (int)(sp->len);
 			break;
 		default:
@@ -518,6 +580,8 @@ STATICFNDEF void op_fgnjavacal(mval *dst, mval *package, mval *extref, uint4 mas
 	 * allocate more space than we might strictly need for the arguments, return type, and type descriptor.
 	 */
 	types_descr_ptr = (char *)malloc(MAX(SIZEOF(char) * (argcnt + 2), SIZEOF(char *) * 2));
+	/* zero the contents for unmodified output values */
+	memset(types_descr_ptr, 0, MAX(SIZEOF(char) * (argcnt + 2), SIZEOF(char *) * 2));
 	types_descr_dptr = types_descr_ptr;
 	*types_descr_dptr = (char)entry_argcnt;
 	types_descr_dptr++;
@@ -726,16 +790,14 @@ STATICFNDEF void op_fgnjavacal(mval *dst, mval *package, mval *extref, uint4 mas
 	assert((char *)free_space_pointer <= free_string_pointer_start);
 	va_end(var_copy);
 	param_list->n = argcnt + 3;		/* Take care of the three implicit parameters. */
-#ifdef DEBUG
-	verifyAllocatedStorage();		/* GTM-8669 verify that argument placement did not trash allocated memory */
-#endif
+	VERIFY_STORAGE_CHAINS;
 	save_mumps_status = mumps_status; 	/* Save mumps_status as a callin from external call may change it. */
 	assert(INTRPT_OK_TO_INTERRUPT == intrpt_ok_state);	/* Expected for DEFERRED_EXIT_HANDLING_CHECK below */
 	TREF(in_ext_call) = TRUE;
 	status = callg((callgfnptr)entry_ptr->fcn, param_list);
 	TREF(in_ext_call) = FALSE;
-	check_for_timer_pops(!entry_ptr->ext_call_behaviors[SIGSAFE]);
 	verify_buffer((char *)param_list, n, entry_ptr->entry_name.addr);
+	check_for_timer_pops(!entry_ptr->ext_call_behaviors[SIGSAFE]);
 	mumps_status = save_mumps_status;
 	/* The first byte of the type description argument gets set to 0xFF in case error happened in JNI glue code,
 	 * so check for that and act accordingly.
@@ -949,7 +1011,8 @@ void op_fnfgncal(uint4 n_mvals, mval *dst, mval *package, mval *extref, uint4 ma
 	 */
 	/* Note that this is the nominal buffer size; or, explicitly, the size of buffer without the proctection tags*/
 	call_buff_size = 2*n;
-	char param_list_buff[(call_buff_size + 2*buff_boarder_len)];
+	char param_list_buff[(call_buff_size + (2 * buff_boarder_len))];
+	memset(param_list_buff, 0, (call_buff_size + (2 * buff_boarder_len)));	/* zero the contents for unmodified output values */
 	param_list = set_up_buffer(param_list_buff, 2*n);
 	free_space_pointer = (gtm_long_t *)((char *)param_list + SIZEOF(intszofptr_t) + (SIZEOF(void *) * argcnt));
 	free_string_pointer_start = free_string_pointer = (char *)param_list + entry_ptr->parmblk_size;
@@ -1117,13 +1180,14 @@ void op_fnfgncal(uint4 n_mvals, mval *dst, mval *package, mval *extref, uint4 ma
 	assert((char *)free_space_pointer <= free_string_pointer_start);
 	va_end(var);
 	param_list->n = argcnt;
+	VERIFY_STORAGE_CHAINS;
 	save_mumps_status = mumps_status; /* Save mumps_status as a callin from external call may change it */
 	assert(INTRPT_OK_TO_INTERRUPT == intrpt_ok_state);              /* Expected for DEFERRED_EXIT_HANDLING_CHECK below */
 	TREF(in_ext_call) = TRUE;
 	status = callg((callgfnptr)entry_ptr->fcn, param_list);
 	TREF(in_ext_call) = FALSE;
-	check_for_timer_pops(!entry_ptr->ext_call_behaviors[SIGSAFE]);
 	verify_buffer((char *)param_list, (2*n), entry_ptr->entry_name.addr);
+	check_for_timer_pops(!entry_ptr->ext_call_behaviors[SIGSAFE]);
 	mumps_status = save_mumps_status;
 	/* Exit from the residual call-in environment(SFF_CI and base frames) which might
 	 * still exist on M stack when the externally called function in turn called
