@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -42,7 +42,6 @@
 #include "filestruct.h"
 #include "buddy_list.h"		/* needed for tp.h */
 #include "jnl.h"
-#include "hashtab_int4.h"	/* needed for tp.h */
 #include "tp.h"
 #include "send_msg.h"
 #include "gtmmsg.h"		/* for gtm_putmsg() prototype */
@@ -58,6 +57,7 @@
 void	job_timer_handler(void);
 
 GBLDEF	short			jobcnt		= 0;
+GBLDEF	int			job_errno;
 GBLDEF	volatile boolean_t	ojtimeout	= TRUE;
 
 GBLREF	uint4		dollar_trestart;
