@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2010-2017 Fidelity National Information	*
+ * Copyright (c) 2010-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -31,7 +31,6 @@
 #include "jnl.h"
 #include "copy.h"
 #include "buddy_list.h"		/* needed for tp.h */
-#include "hashtab_int4.h"	/* needed for tp.h */
 #include "tp.h"
 #include "repl_msg.h"
 #include "gtmsource.h"
@@ -64,6 +63,7 @@
 #include "gtmimagename.h"
 #include "is_file_identical.h"
 #include "anticipatory_freeze.h"
+#include "gvt_inline.h"
 
 LITREF	mval	literal_null;
 
