@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2014-2015 Fidelity National Information 	*
+ * Copyright (c) 2014-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -100,7 +100,7 @@
 					if (!SHM_REMOVED(save_errno))								\
 						ISSUE_RELINKCTLERR_SYSCALL(&LINKCTL->zro_entry_name, errstr, save_errno);	\
 					else											\
-						ISSUE_REQRLNKCTLRNDWN_SYSCALL(&LINKCTL->zro_entry_name, errstr, save_errno);	\
+						ISSUE_REQRLNKCTLRNDWN_SYSCALL(LINKCTL, errstr, save_errno);			\
 				}												\
 				assert(0 == ((UINTPTR_T)shm_base % 8));								\
 				LINKCTL->rtnobj_shmid[i] = shr_shmid;								\
@@ -119,9 +119,7 @@
 
 #ifdef AUTORELINK_SUPPORTED
 
-error_def(ERR_PERMGENFAIL);
 error_def(ERR_RELINKCTLERR);
-error_def(ERR_REQRLNKCTLRNDWN);
 error_def(ERR_RLNKRECLATCH);
 error_def(ERR_RLNKSHMLATCH);
 error_def(ERR_SYSCALL);
@@ -366,6 +364,7 @@ sm_uc_ptr_t rtnobj_shm_malloc(zro_hist *zhist, int fd, off_t objSize, gtm_uint64
 	int			perm;
 	int			maxvers, curvers;
 	struct shmid_ds		shmstat;
+	struct perm_diag_data	pdd;
 #	ifdef DEBUG
 	DCL_THREADGBL_ACCESS;
 
@@ -382,7 +381,7 @@ sm_uc_ptr_t rtnobj_shm_malloc(zro_hist *zhist, int fd, off_t objSize, gtm_uint64
 	has_relinkctl_lock = FALSE;
 	assert(!linkctl->locked);
 	assert(!linkctl->hdr->file_deleted);
-	if (!grab_latch(&relinkrec->rtnobj_latch, RLNKREC_LATCH_TIMEOUT_SEC))
+	if (!grab_latch(&relinkrec->rtnobj_latch, RLNKREC_LATCH_TIMEOUT_SEC, NOT_APPLICABLE, NULL))
 	{
 		assert(FALSE);
 		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(5)
@@ -396,7 +395,7 @@ sm_uc_ptr_t rtnobj_shm_malloc(zro_hist *zhist, int fd, off_t objSize, gtm_uint64
 			 * for one last time.
 			 */
 			assert(!has_relinkctl_lock);
-			if (!grab_latch(&shm_hdr->relinkctl_latch, RLNKSHM_LATCH_TIMEOUT_SEC))
+			if (!grab_latch(&shm_hdr->relinkctl_latch, RLNKSHM_LATCH_TIMEOUT_SEC, NOT_APPLICABLE, NULL))
 			{
 				assert(FALSE);
 				rel_latch(&relinkrec->rtnobj_latch);
@@ -473,7 +472,7 @@ sm_uc_ptr_t rtnobj_shm_malloc(zro_hist *zhist, int fd, off_t objSize, gtm_uint64
 		 */
 		if (!has_relinkctl_lock)
 		{
-			if (!grab_latch(&shm_hdr->relinkctl_latch, RLNKSHM_LATCH_TIMEOUT_SEC))
+			if (!grab_latch(&shm_hdr->relinkctl_latch, RLNKSHM_LATCH_TIMEOUT_SEC, NOT_APPLICABLE, NULL))
 			{
 				assert(FALSE);
 				rel_latch(&relinkrec->rtnobj_latch);
@@ -557,7 +556,17 @@ sm_uc_ptr_t rtnobj_shm_malloc(zro_hist *zhist, int fd, off_t objSize, gtm_uint64
 				SNPRINTF(errstr, SIZEOF(errstr), "rtnobj stat() of file %s failed", linkctl->zro_entry_name.addr);
 				ISSUE_RELINKCTLERR_SYSCALL(&linkctl->zro_entry_name, errstr, save_errno);
 			}
-			gtm_permissions(&dir_stat_buf, &user_id, &group_id, &perm, PERM_IPC|PERM_EXEC);
+			if (!gtm_permissions(&dir_stat_buf, &user_id, &group_id, &perm, PERM_IPC|PERM_EXEC, &pdd))
+			{
+				rel_latch(&shm_hdr->relinkctl_latch);
+				rel_latch(&relinkrec->rtnobj_latch);
+				shm_rmid(shmid);	/* if error removing shmid we created, just move on */
+				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(10 + PERMGENDIAG_ARG_COUNT)
+						ERR_RELINKCTLERR, 2, RTS_ERROR_MSTR(&linkctl->zro_entry_name),
+						ERR_PERMGENFAIL, 4, RTS_ERROR_STRING("rtnobj"),
+						RTS_ERROR_MSTR(&linkctl->zro_entry_name),
+						PERMGENDIAG_ARGS(pdd));
+			}
 			if (-1 == shmctl(shmid, IPC_STAT, &shmstat))
 			{
 				save_errno = errno;
@@ -881,7 +890,7 @@ void	rtnobj_shm_free(rhdtyp *rhead, boolean_t latch_grabbed)
 	}
 	assert(dbg_shm_index < max_index);
 #	endif
-	if (!latch_grabbed && !grab_latch(&relinkrec->rtnobj_latch, RLNKREC_LATCH_TIMEOUT_SEC))
+	if (!latch_grabbed && !grab_latch(&relinkrec->rtnobj_latch, RLNKREC_LATCH_TIMEOUT_SEC, NOT_APPLICABLE, NULL))
 	{
 		assert(FALSE);
 		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(5)
@@ -938,7 +947,7 @@ void	rtnobj_shm_free(rhdtyp *rhead, boolean_t latch_grabbed)
 	assert(sizeIndex <= maxObjIndex);
 	elemSize = origElemSize = ((gtm_uint64_t)1 << (sizeIndex + MIN_RTNOBJ_SIZE_BITS));
 	/* Now that we need to unload the object from shared memory, get a shared memory lock */
-	if (!grab_latch(&shm_hdr->relinkctl_latch, RLNKSHM_LATCH_TIMEOUT_SEC))
+	if (!grab_latch(&shm_hdr->relinkctl_latch, RLNKSHM_LATCH_TIMEOUT_SEC, NOT_APPLICABLE, NULL))
 	{
 		assert(FALSE);
 		rel_latch(&relinkrec->rtnobj_latch);
