@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2014 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -11,12 +12,13 @@
 
 #include "mdef.h"
 
-#include <stdarg.h>
 #include "gtm_stdio.h"
 
-#include "gtm_putmsg_list.h"
+#include <stdarg.h>
 #include <errno.h>
 
+#include "gtm_multi_thread.h"
+#include "gtm_putmsg_list.h"
 #include "gtmimagename.h"
 #include "error.h"
 #include "util.h"
@@ -28,12 +30,19 @@
 #include "repl_msg.h"
 #include "gtmsource.h"
 #include "anticipatory_freeze.h"
+#include "toktyp.h"
+#include "cgp.h"
 
-GBLREF	int		gtm_errno;
 GBLREF	boolean_t 	created_core;
 GBLREF	boolean_t	dont_want_core;
+GBLREF	boolean_t	run_time;
+GBLREF	char		cg_phase;
 GBLREF	gd_region	*gv_cur_region;
-GBLREF	jnlpool_addrs	jnlpool;
+GBLREF	jnlpool_addrs_ptr_t	jnlpool;
+GBLREF	int		gtm_errno;
+GBLREF	void		(*stx_error_va_fptr)(int in_error, ...);	/* Function pointer for stx_error_va() so this can avoid
+								 	 * pulling stx_error() into gtmsecshr.
+								 	 */
 
 error_def(ERR_ASSERT);
 error_def(ERR_GTMASSERT);
@@ -55,19 +64,21 @@ int rts_error_va(void *csa, int argcnt, va_list var);
  *  =======	zero MUST be specified if there are no parameters.
  * ----------------------------------------------------------------------------------------
  */
-
+/* coverity[+kill] */
 int rts_error(int argcnt, ...)
 {
 	va_list		var;
 	sgmnt_addrs	*csa;
+	jnlpool_addrs_ptr_t	local_jnlpool;	/* needed by PTHREAD_CSA_FROM_GV_CUR_REGION */
 	DCL_THREADGBL_ACCESS;
 
 	SETUP_THREADGBL_ACCESS;
-	csa = CUSTOM_ERRORS_LOADED ? REG2CSA(gv_cur_region) : NULL;
+	PTHREAD_CSA_FROM_GV_CUR_REGION(csa, local_jnlpool);
 	VAR_START(var, argcnt);
 	return rts_error_va(csa, argcnt, var);
 }
 
+/* coverity[+kill] */
 int rts_error_csa(void *csa, int argcnt, ...)
 {
 	va_list		var;
@@ -81,10 +92,11 @@ int rts_error_va(void *csa, int argcnt, va_list var)
 	int 		msgid;
 	va_list		var_dup;
 	const err_ctl	*ctl;
-#	ifdef DEBUG
+	boolean_t	was_holder;
 	DCL_THREADGBL_ACCESS;
 
 	SETUP_THREADGBL_ACCESS;
+#	ifdef DEBUG
 	if (TREF(rts_error_unusable) && !TREF(rts_error_unusable_seen))
 	{
 		TREF(rts_error_unusable_seen) = TRUE;
@@ -95,6 +107,13 @@ int rts_error_va(void *csa, int argcnt, va_list var)
 		assert(FALSE);
 	}
 #	endif
+	PTHREAD_MUTEX_LOCK_IF_NEEDED(was_holder); /* get thread lock in case threads are in use */
+	/* Note that rts_error does not return most of the times (control gets transferred somewhere else) so no point
+	 * unlocking thread mutex at the end of this function. This means that if one thread gets an error, the rest of
+	 * the threads might wait if/when they need to make a PTHREAD_MUTEX_LOCK_IF_NEEDED call. But that is considered
+	 * okay since the process is anyways going to terminate due to the error in one thread. This is at least true
+	 * for the current use of threads in mupip. If/when threads are used in GT.M this will need to be revisited.
+	 */
 	VAR_COPY(var_dup, var);
 	if (-1 == gtm_errno)
 		gtm_errno = errno;
@@ -119,6 +138,12 @@ int rts_error_va(void *csa, int argcnt, va_list var)
 		if (IS_GTMSECSHR_IMAGE)
 			util_out_print(NULL, RESET);
 		SET_ERROR_CONDITION(msgid);	/* sets "error_condition" & "severity" */
+		if (!run_time && (CGP_PARSE == cg_phase) && !DUMP)
+		{
+			(*stx_error_va_fptr)(msgid, var_dup);
+			TREF(director_token) = TK_ERROR;
+			return FALSE;
+		}
 		gtm_putmsg_list(csa, argcnt, var);
 		if (DUMPABLE)
 			created_core = dont_want_core = FALSE;		/* We can create a(nother) core now */
@@ -132,5 +157,5 @@ int rts_error_va(void *csa, int argcnt, va_list var)
 	 * when the severity was error or above. That code had to be removed because of several errors
 	 * that are handled and returned from. An example is EOF errors.  SE 9/2000
 	 */
-	return 0;
+	return FALSE;
 }
