@@ -1,6 +1,6 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;								;
-; Copyright (c) 2010-2018 Fidelity National Information		;
+; Copyright (c) 2010-2019 Fidelity National Information		;
 ; Services, Inc. and/or its subsidiaries. All rights reserved.	;
 ; 								;
 ; 	This source code contains the intellectual property	;
@@ -599,7 +599,7 @@
 	;
 	; Make sure all header files added.
 	;
-	Do CommandToPipe("ls -1 $gtm_inc/*.h",.results)
+	Do CommandToPipe("/bin/ls -1 $gtm_inc/*.h",.results)
 	For i=1:1:results(0) Do
 	. Set incname=$ZPiece(results(i),"/",6)
 	. Quit:(""=incname)					; Sometimes get blank lines at end.
@@ -612,10 +612,11 @@
 	; Run it through the pre-processor to expand pre-processor statements selecting conditional
 	; typedefs, etc. Disable all warnings which may occur in older sources and newer compilers
 	;
-	Do CommandToPipe("gt_cc_"_gtmtyp_" -w -E "_ExtraCcParms_" "_cfile_" > "_cfile_"exp",.results)
+	set cmdToPipe="$gt_cc_"_gtmtyp_" -I$gtm_inc -w -E "_ExtraCcParms_" "_cfile_" > "_cfile_"exp"
+	Do CommandToPipe(cmdToPipe,.results)
 	Do:(0'=results(0))
 	. Use $P
-	. Do DoWrite("Error messages from pre-processor compile: gt_cc_"_gtmtyp_" -E "_ExtraCcParms_" "_cfile_" > "_cfile_"exp")
+	. Do DoWrite("Error messages from pre-processor compile: "_cmdToPipe)
 	. For i=1:1:results(0) Write results(i),!
 	. Do Error("BADPREPCOMP","E","Pre-process compile failed - aborting")
 	;
@@ -901,7 +902,7 @@
 	;
 	; Make sure all header files added.
 	;
-	Do CommandToPipe("ls -1 $gtm_inc/*.h",.results)
+	Do CommandToPipe("/bin/ls -1 $gtm_inc/*.h",.results)
 	For i=1:1:results(0) Do
 	. Set incname=$ZPiece(results(i),"/",6)
 	. If 0=$Data(exclincl(incname)),0=$Data(inclref(incname)) Do
@@ -968,13 +969,13 @@
 	; Need to build and execute this routine now.
 	;
 	Set outfile=$ZPiece(cfile,".",1)
-	Set ccCmd="gt_cc_"_gtmtyp_$Select(("OS390"'=$ZPiece($ZVersion," ",3)):" -O0",1:" -qNOOPT")_" -o "_outfile_".o "
+	Set ccCmd="$gt_cc_"_gtmtyp_$Select(("OS390"'=$ZPiece($ZVersion," ",3)):" -O0",1:" -qNOOPT")_" -I$gtm_inc -o "_outfile_".o "
 	Do CommandToPipe(ccCmd_ExtraCcLdParms_" "_ExtraCcParms_" "_outfile_".c",.compresults)
 	Do:((0'=compresults(0))!(FALSE=$$StatFile(outfile_".o")))
 	. Do dbgzwrite("compresults",$get(lastpipecmd))
 	. Do dbgzwrite("compresults","Results from compile of "_outfile)
 	. Do Error("COMPFAIL","F","Compile of offsets routine failed")
-	Set ldCmd="gt_ld -o "_outfile_" $gt_ld_options_pro -L$gtm_obj $gt_ld_sysrtns ${gt_ld_syslibs:s/-lncurses//} "
+	Set ldCmd="$gt_ld_linker -o "_outfile_" $gt_ld_options_pro -L$gtm_obj $gt_ld_sysrtns ${gt_ld_syslibs:s/-lncurses//} "
 	Do CommandToPipe(ldCmd_ExtraCcLdParms_" "_outfile_".o",.linkresults)
 	Do:(FALSE=$$StatFile(outfile))
 	. Set file=outfile_".link_output.txt"
@@ -1039,9 +1040,9 @@
 	;
 	; Start of last phase - write out the GTM structure definition file
 	;
-	Set sdfile="GTMDefinedTypesInit.m",gdeinitfile="gdeinitsz.m"
-	Open sdfile:New,gdeinitfile:New
-	For f=sdfile,gdeinitfile do Prolog(f)
+	Set sdfile="GTMDefinedTypesInit.m"
+	Open sdfile:New
+	For f=sdfile do Prolog(f)
 	Set lincnt=32,(fldcnt,gvstats,strctcnt)=0
 	Use sdfile
 	;
@@ -1066,7 +1067,7 @@
 	. . Write TAB,"Set gtmtypes(""",type,""",",fldidx,",""len"")=",types(type,"fullexp",fldidx,"fldlen"),!
 	. . Write TAB,"Set gtmtypes(""",type,""",",fldidx,",""type"")=""",types(type,"fullexp",fldidx,"type"),"""",!
 	. . If "gvstats_rec_t"=type,$Increment(gvstats,types(type,"fullexp",fldidx,"fldlen"))
-	. . Set fld=$ZPiece(types(type,"fullexp",fldidx,"fldname"),".",2,99)	; Eliminate structure (type) header 
+	. . Set fld=$ZPiece(types(type,"fullexp",fldidx,"fldname"),".",2,99)	; Eliminate structure (type) header
 	. . 									; isolating just the field name
 	. . Write TAB,"Set gtmtypfldindx(""",type,""",""",fld,""")=",fldidx,!
 	. . Set lincnt=lincnt+5
@@ -1098,12 +1099,6 @@
 	Set endline=" Finished writing GTMDefinedTypesInit.m output file ("_lincnt_" lines written for "_strctcnt
 	Set endline=endline_" structures with "_fldcnt_" fields)"
 	Do DoWrite($ZDate($Horolog,"24:60:SS")_endline)
-	Use gdeinitfile
-	Write TAB,"Set SIZEOF(""gvstats"")=",gvstats,!
-	Write TAB,"Quit",!
-	Close gdeinitfile
-	Use $Principal
-	Do DoWrite($ZDate($Horolog,"24:60:SS")_" Finished writing "_gdeinitfile_" output file with gvstatsize = "_gvstats)
 
 	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 	;
