@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -90,6 +90,7 @@ const static char readonly *secshr_fail_mesg_code[] = {
 	"Remove Shared Memory segment failed",
 	"Remove File failed",
 	"Continue Process failed",
+	"Database Header flush failed",
 };
 /* The below messages match up with gtmsecshr exit codes from gtmsecshr.h. */
 const static char readonly *secshrstart_error_code[] = {
@@ -104,7 +105,7 @@ const static char readonly *secshrstart_error_code[] = {
 	"gtmsecshr startup failed - gtmsecshr unable to chdir to tmp directory",
 	"gtmsecshr startup failed - gtmsecshr unable to determine invocation path",
 	"gtmsecshr startup failed - gtmsecshr not named gtmsecshr",
-	"gtmsecshr startup failed - $gtm_dist not same as startup path"
+	"gtmsecshr startup failed - startup path through $gtm_dist not setup correctly - check path and permissions"
 };
 
 #define MAX_COMM_ATTEMPTS		4	/* 1 to start secshr, 2 maybe slow, 3 maybe really slow, 4 outside max */
@@ -424,18 +425,37 @@ int send_mesg2gtmsecshr(unsigned int code, unsigned int id, char *path, int path
 	}
 	if (MAX_COMM_ATTEMPTS < loop_count)
 	{
-		ret_code = -1;
 		gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(10) ERR_GTMSECSHRSRVF, 4,
 				RTS_ERROR_TEXT("Client"), process_id, loop_count - 1,
 			   	ERR_TEXT, 2, RTS_ERROR_TEXT("Unable to communicate with gtmsecshr"));
+		if (FLUSH_DB_IPCS_INFO >= req_code)
+		{
+			if (ret_code)
+				gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(5) ERR_TEXT, 2,
+						RTS_ERROR_STRING(secshr_fail_mesg_code[req_code]), ret_code);
+			else
+				gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_TEXT, 2,
+						RTS_ERROR_STRING(secshr_fail_mesg_code[req_code]));
+		}
+		ret_code = -1;
 		/* If gtm_tmp is not defined, show default path */
 		if (gtm_tmp_ptr = GETENV("gtm_tmp"))
+		{
+			if (!IS_GTM_IMAGE)
+				gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_GTMSECSHRTMPPATH, 2,
+					RTS_ERROR_TEXT(gtm_tmp_ptr),
+					ERR_TEXT, 2, RTS_ERROR_TEXT("(from $gtm_tmp)"));
 			send_msg_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_GTMSECSHRTMPPATH, 2,
 				RTS_ERROR_TEXT(gtm_tmp_ptr),
 				ERR_TEXT, 2, RTS_ERROR_TEXT("(from $gtm_tmp)"));
-		else
+		} else
+		{
+			if (!IS_GTM_IMAGE)
+				gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(4)
+						ERR_GTMSECSHRTMPPATH, 2, RTS_ERROR_TEXT("/tmp"));
 			send_msg_csa(CSA_ARG(NULL) VARLSTCNT(4)
 					ERR_GTMSECSHRTMPPATH, 2, RTS_ERROR_TEXT("/tmp"));
+		}
 	}
 	if (ONETIMESOCKET == init_ret_code)
 		gtmsecshr_sock_cleanup(CLIENT);
