@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2009-2018 Fidelity National Information	*
+ * Copyright (c) 2009-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -114,11 +114,14 @@ ZOS_ONLY(error_def(ERR_TEXT);)
 	ss_filhdr_ptr->ss_info.ss_shmsize = ss_shm_ptr->ss_info.ss_shmsize;			\
 }
 
-#define GET_CRIT_AND_DECR_INHIBIT_KILLS(REG, CNL)			\
-{									\
-	grab_crit(REG);							\
-	DECR_INHIBIT_KILLS(CNL);					\
-	rel_crit(REG);							\
+#define GET_CRIT_AND_DECR_INHIBIT_KILLS(REG, CNL)				\
+{										\
+	grab_crit(REG, WS_96);							\
+	if (!(TREF(in_mupip_integ) && TREF(instance_frozen_crit_skipped)))	\
+	{									\
+		DECR_INHIBIT_KILLS(CNL);					\
+		rel_crit(REG);							\
+	}									\
 }
 
 #define SS_INIT_SHM(SS_SHMSIZE, SS_SHMADDR, SS_SHMID, RESIZE_NEEDED, RC)				\
@@ -190,10 +193,10 @@ void	ss_initiate_call_on_signal(void)
 	return;
 }
 
-boolean_t	ss_initiate(gd_region *reg, 			/* Region in which snapshot has to be started */
-			    util_snapshot_ptr_t	util_ss_ptr, 	/* Utility specific snapshot structure */
-			    snapshot_context_ptr_t *ss_ctx, 	/* Snapshot context */
-			    boolean_t preserve_snapshot, 	/* Should the snapshots be preserved ? */
+boolean_t	ss_initiate(gd_region *reg,			/* Region in which snapshot has to be started */
+			    util_snapshot_ptr_t util_ss_ptr,	/* Utility specific snapshot structure */
+			    snapshot_context_ptr_t *ss_ctx,	/* Snapshot context */
+			    boolean_t preserve_snapshot,	/* Should the snapshots be preserved ? */
 			    char *calling_utility)		/* Who called ss_initiate */
 {
 	boolean_t		debug_mupip = FALSE;
@@ -205,7 +208,7 @@ boolean_t	ss_initiate(gd_region *reg, 			/* Region in which snapshot has to be s
 	gtm_uint64_t		db_file_size, native_size;
 	int			dsk_addr = 0;
 	int			fclose_res, fstat_res, group_id, perm, pwrite_res, retries;
-	int			save_errno, shdw_fd, ss_shmsize, ss_shm_vbn, status, tmpfd, user_id;
+	int			save_errno, shdw_fd, status, tmpfd, user_id;
 	ZOS_ONLY(int		realfiletag;)
 	long			ss_shmid = INVALID_SHMID;
 	mstr			tempdir_full, tempdir_log, tempdir_trans;
@@ -218,10 +221,13 @@ boolean_t	ss_initiate(gd_region *reg, 			/* Region in which snapshot has to be s
 	snapshot_filhdr_ptr_t	ss_filhdr_ptr;
 	struct perm_diag_data	pdd;
 	struct stat		stat_buf;
-	uint4			crit_counter, fstat_status, prev_ss_shmsize, tempnamprefix_len, tot_blks;
+	uint4			crit_counter, fstat_status, tempnamprefix_len;
+	block_id		tot_blks, ss_shmsize, prev_ss_shmsize, ss_shm_vbn;
 	void			*ss_shmaddr;
 	intrpt_state_t		prev_intrpt_state;
+	DCL_THREADGBL_ACCESS;
 
+	SETUP_THREADGBL_ACCESS;
 	assert(IS_MUPIP_IMAGE);
 	assert(NULL != calling_utility);
 	csa = &FILE_INFO(reg)->s_addrs;
@@ -253,9 +259,9 @@ boolean_t	ss_initiate(gd_region *reg, 			/* Region in which snapshot has to be s
 			ss_release(NULL);
 		else
 		{
-			gtm_putmsg_csa(CSA_ARG(csa) VARLSTCNT(5) ERR_MAXSSREACHED, 3, MAX_SNAPSHOTS, REG_LEN_STR(reg));
 			ss_release_lock(reg);
 			UNFREEZE_REGION_IF_NEEDED(csd, reg);
+			gtm_putmsg_csa(CSA_ARG(csa) VARLSTCNT(5) ERR_MAXSSREACHED, 3, MAX_SNAPSHOTS, REG_LEN_STR(reg));
 			return FALSE;
 		}
 	}
@@ -439,7 +445,7 @@ boolean_t	ss_initiate(gd_region *reg, 			/* Region in which snapshot has to be s
 	assert(NULL != util_ss_ptr);
 	assert(NULL != util_ss_ptr->master_map);
 	assert(NULL != util_ss_ptr->header);
-	grab_crit(reg);
+	grab_crit(reg, WS_97);
 	INCR_INHIBIT_KILLS(cnl);
 	kip_pids_arr_ptr = cnl->kip_pid_array;
 	prev_ss_shmsize = ss_shmsize = 0;
@@ -448,7 +454,7 @@ boolean_t	ss_initiate(gd_region *reg, 			/* Region in which snapshot has to be s
 	{
 		final_retry = (MAX_TRY_FOR_TOT_BLKS == retries);
 		/* On all but the 4th retry (inclusive of retries = -1), release crit */
-		if (!final_retry)
+		if ((!final_retry) && (csa->now_crit))	/* In MUPIP INTEG and instance freeze, it is possible we don't have crit */
 			rel_crit(reg);
 		/*
 		 * Outside crit (on the final retry, we will be holding crit while calculating the below):
@@ -461,7 +467,7 @@ boolean_t	ss_initiate(gd_region *reg, 			/* Region in which snapshot has to be s
 		DEFER_INTERRUPTS(INTRPT_IN_SS_INITIATE, prev_intrpt_state); /* Defer MUPIP STOP till the shared memory is created */
 		tot_blks = csd->trans_hist.total_blks;
 		prev_ss_shmsize = ss_shmsize;
-		ss_shmsize = (int)(ROUND_UP((SNAPSHOT_HDR_SIZE + TOT_BYTES_REQUIRED(tot_blks)), OS_PAGE_SIZE));
+		ss_shmsize = (ROUND_UP((SNAPSHOT_HDR_SIZE + TOT_BYTES_REQUIRED(tot_blks)), OS_PAGE_SIZE));
 		assert((0 == retries) || (0 != prev_ss_shmsize));
 		if (0 == retries)
 		{	/* If this is the first try, then create shared memory anyways. */
@@ -524,13 +530,11 @@ boolean_t	ss_initiate(gd_region *reg, 			/* Region in which snapshot has to be s
 		if (wait_for_zero_kip)
 		{	/* after inhibitting kills wait once for up to a minute; if that's insufficient, it's probably abandoned */
 			/* Release crit before going into the wait loop */
-			rel_crit(reg);
-			if (debug_mupip)
-			{
-				GET_CUR_TIME(time_str);
-				util_out_print("!/MUPIP INFO: !AD : Start kill-in-prog wait for database !AD", TRUE,
-					CTIME_BEFORE_NL, time_str, DB_LEN_STR(reg));
-			}
+			if (csa->now_crit)	/* In MUPIP INTEG and instance freeze, it is possible we don't have crit */
+				rel_crit(reg);
+			GET_CUR_TIME(time_str);
+			util_out_print("!/MUPIP INFO: !AD : Start kill-in-prog wait for database !AD", TRUE,
+				CTIME_BEFORE_NL, time_str, DB_LEN_STR(reg));
 			while (csd->kill_in_prog && (MAX_CRIT_TRY > crit_counter++))
 			{
 				GET_C_STACK_FOR_KIP(kip_pids_arr_ptr, crit_counter, MAX_CRIT_TRY, 1, MAX_KIP_PID_SLOTS);
@@ -543,7 +547,7 @@ boolean_t	ss_initiate(gd_region *reg, 			/* Region in which snapshot has to be s
 					CTIME_BEFORE_NL, time_str, DB_LEN_STR(reg));
 			}
 		}
-		grab_crit(reg);
+		grab_crit(reg, WS_98);
 		/* After we have created the shared memory and before we grab crit, another process can add new blocks to the
 		 * database, in which case csd->trans_hist.total_blks is no longer the value as we noted down above. Check if this
 		 * is the case and if so, retry to obtain a consistent copy of the total number of blocks.
@@ -556,7 +560,7 @@ boolean_t	ss_initiate(gd_region *reg, 			/* Region in which snapshot has to be s
 	 * that occurs henceforth should do a DECR_INHIBIT_KILLS and rel_crit before exiting to let the other processes
 	 * proceed gracefully
 	 */
-	assert(csa->now_crit);
+	assert((TREF(in_mupip_integ) && TREF(instance_frozen_crit_skipped)) || csa->now_crit);
 	assert(NULL != ss_shmaddr);
 	assert(0 == ((long)ss_shmaddr % OS_PAGE_SIZE));
 	assert(0 == ss_shmsize % OS_PAGE_SIZE);
@@ -599,21 +603,26 @@ boolean_t	ss_initiate(gd_region *reg, 			/* Region in which snapshot has to be s
 	/* ===================== STEP 5: Flush the pending updates in the global cache =================== */
 	/* For a readonly database for the current process, we cannot do wcs_flu. We would have waited for the active queues
 	 * to complete in mu_int_reg after doing a freeze. Now that we have crit, unfreeze the region
+	 * We can't do any of it If, however, we don't have crit (instance frozen, and we didn't wait for the freeze to be held off)
 	 */
-	if (reg->read_only)
+	if (!(TREF(in_mupip_integ) && TREF(instance_frozen_crit_skipped)))
 	{
-		region_freeze(reg, FALSE, FALSE, FALSE, FALSE, FALSE);
-	}
-	else if (!wcs_flu(WCSFLU_FLUSH_HDR | WCSFLU_WRITE_EPOCH | WCSFLU_MSYNC_DB)) /* wcs_flu guarantees that all the pending
-								   * phase 2 commits are done with before returning */
-	{
-		assert(process_id != csd->freeze); /* We would not have frozen the region if the database is read-write */
-		gtm_putmsg_csa(CSA_ARG(csa) VARLSTCNT(6) ERR_BUFFLUFAILED, 4, LEN_AND_STR(calling_utility), DB_LEN_STR(reg));
-		GET_CRIT_AND_DECR_INHIBIT_KILLS(reg, cnl);
-		return FALSE;
+		if (reg->read_only)
+		{
+			region_freeze(reg, FALSE, FALSE, FALSE, FALSE, FALSE);
+		}
+		else if (!wcs_flu(WCSFLU_FLUSH_HDR | WCSFLU_WRITE_EPOCH | WCSFLU_MSYNC_DB)) /* wcs_flu guarantees that all the
+								   * pending phase 2 commits are done with before returning */
+		{
+			assert(process_id != csd->freeze); /* We would not have frozen the region if the database is read-write */
+			gtm_putmsg_csa(CSA_ARG(csa) VARLSTCNT(6) ERR_BUFFLUFAILED, 4,
+							LEN_AND_STR(calling_utility), DB_LEN_STR(reg));
+			GET_CRIT_AND_DECR_INHIBIT_KILLS(reg, cnl);
+			return FALSE;
+		}
 	}
 	assert(0 < cnl->num_snapshots_in_effect || (!SNAPSHOTS_IN_PROG(cnl)));
-	assert(csa->now_crit);
+	assert((TREF(in_mupip_integ) && TREF(instance_frozen_crit_skipped)) || csa->now_crit);
 	/* ========== STEP 6: Copy the file header, master map and the native file size into a private structure =========== */
 	memcpy(util_ss_ptr->header, csd, SGMNT_HDR_LEN);
 	memcpy(util_ss_ptr->master_map, MM_ADDR(csd), MASTER_MAP_SIZE(csd));
@@ -661,7 +670,8 @@ boolean_t	ss_initiate(gd_region *reg, 			/* Region in which snapshot has to be s
 	SHM_WRITE_MEMORY_BARRIER;
 	cnl->ss_shmcycle++;	/* indicate that the ss_shmid field of cnl is being reused */
 	lcl_ss_ctx->ss_shmcycle = cnl->ss_shmcycle;
-	rel_crit(reg);
+	if (csa->now_crit)	/* In MUPIP INTEG and instance freeze, it is possible we don't have crit */
+		rel_crit(reg);
 	if ((csa->onln_rlbk_cycle != csa->nl->onln_rlbk_cycle) || csa->nl->onln_rlbk_pid)
 	{	/* A concurrent online rollback happened since we did the gvcst_init. The INTEG is not reliable.
 		 * Cleanup and exit
