@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2010-2018 Fidelity National Information	*
+ * Copyright (c) 2010-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -22,7 +22,6 @@
 #include "gdscc.h"			/* needed for tp.h */
 #include "gdskill.h"			/* needed for tp.h */
 #include "buddy_list.h"			/* needed for tp.h */
-#include "hashtab_int4.h"		/* needed for tp.h */
 #include "filestruct.h"			/* needed for jnl.h */
 #include "jnl.h"			/* needed for tp.h */
 #include "tp.h"
@@ -333,7 +332,7 @@ boolean_t trigger_delete_name(mval *trigger_rec, uint4 *trig_stats)
 	char			*ptr, *trigger_name;
 	char			*name_tail_ptr;
 	char			save_name[MAX_MIDENT_LEN + 1];
-	gv_key			save_currkey[DBKEYALLOC(MAX_KEY_SZ)];
+	gv_key_buf		save_currkey;
 	gd_region		*save_gv_cur_region, *lgtrig_reg;
 	gv_namehead		*save_gv_target;
 	sgm_info		*save_sgm_info_ptr;
