@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2010, 2013 Fidelity Information Services, Inc	*
+ * Copyright (c) 2010-2017 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -24,11 +25,15 @@
 #include "op.h"
 #include "trigger.h"
 #include "trigger_gbl_fill_xecute_buffer.h"
+#include "gtm_trigger_trc.h"
 #include "mvalconv.h"
 #include "memcoherency.h"
 #include "t_retry.h"
 #include "gtmimagename.h"
 #include "filestruct.h"			/* for FILE_INFO, needed by REG2CSA */
+#include "have_crit.h"
+#include "repl_msg.h"
+#include "gtmsource.h"			/* for jnlpool_addrs */
 
 LITREF	mval			literal_ten;
 
@@ -42,10 +47,17 @@ error_def(ERR_TRIGDEFBAD);
 
 GBLREF	sgmnt_data_ptr_t	cs_data;
 GBLREF	sgmnt_addrs		*cs_addrs;
+GBLREF	uint4			dollar_tlevel;
 GBLREF	gd_region		*gv_cur_region;
 GBLREF	gv_key			*gv_currkey;
-GBLREF	uint4			dollar_tlevel;
+#ifdef DEBUG
+GBLREF	boolean_t		is_updproc;
+#endif
+GBLREF	jnlpool_addrs_ptr_t	jnlpool;
 GBLREF	unsigned int		t_tries;
+GBLREF	unsigned char		t_fail_hist[CDB_MAX_TRIES];
+
+LITREF	char			*trigger_subs[];
 
 STATICDEF char			*xecute_buff;
 
@@ -66,7 +78,6 @@ STATICFNDEF CONDITION_HANDLER(trigger_gbl_fill_xecute_buffer_ch)
 char *trigger_gbl_fill_xecute_buffer(char *trigvn, int trigvn_len, mval *trig_index, mval *first_rec, int4 *xecute_len)
 {
 	mval			data_val;
-	boolean_t		have_value;
 	mval			index, key_val, *val_ptr;
 	int4			len, xecute_buff_len;
 	int4			num;
@@ -80,42 +91,52 @@ char *trigger_gbl_fill_xecute_buffer(char *trigvn, int trigvn_len, mval *trig_in
 	DCL_THREADGBL_ACCESS;
 
 	SETUP_THREADGBL_ACCESS;
-	/* assert(0 < dollar_tlevel); Too be added later when it stops breaking MUPIP SELECT & $ZTRIGGER("SELECT"..) */
+	assert(0 < dollar_tlevel); /* Too be added later when it stops breaking MUPIP SELECT & $ZTRIGGER("SELECT"..) */
 	xecute_buff = NULL;
 	ESTABLISH_RET(trigger_gbl_fill_xecute_buffer_ch, NULL);
 	index = *trig_index;
+	DBGTRIGR((stderr, "trigger_gbl_fill_xecute_buffer: entry $tlevel:%d\tindex:%d\t:crit:%d\tkv:%d\n",
+				dollar_tlevel, mval2i(&index), have_crit(CRIT_HAVE_ANY_REG), key_val.str.len));
 	if (NULL != first_rec)
 	{
+		DBGTRIGR((stderr, "trigger_gbl_fill_xecute_buffer: NULL != first_rec\n"));
 		xecute_buff_len = first_rec->str.len;
 		assert(MAX_XECUTE_LEN >= xecute_buff_len);
 		xecute_buff = malloc(xecute_buff_len);
 		memcpy(xecute_buff, first_rec->str.addr, xecute_buff_len);
 	} else
 	{	/* First check for a single record xecute string */
-		BUILD_HASHT_SUB_MSUB_SUB_CURRKEY(trigvn, trigvn_len, index, LITERAL_XECUTE, LITERAL_XECUTE_LEN);
-		if (gvcst_get(&data_val))
+		DBGTRIGR((stderr, "trigger_gbl_fill_xecute_buffer: First check for a single record xecute string\n"));
+		BUILD_HASHT_SUB_MSUB_SUB_CURRKEY(trigvn, trigvn_len, index,
+					trigger_subs[XECUTE_SUB], STRLEN(trigger_subs[XECUTE_SUB]));
+		if (gvcst_get(&key_val))
 		{
-			xecute_buff_len = data_val.str.len;
+			xecute_buff_len = key_val.str.len;
 			assert(MAX_XECUTE_LEN >= xecute_buff_len);
 			xecute_buff = malloc(xecute_buff_len);
-			memcpy(xecute_buff, data_val.str.addr, xecute_buff_len);
+			memcpy(xecute_buff, key_val.str.addr, xecute_buff_len);
 			*xecute_len = xecute_buff_len;
+			DBGTRIGR((stderr, "trigger_gbl_fill_xecute_buffer: Found a single record xecute string of %d bytes\n",
+						xecute_buff_len));
 			REVERT;
 			return xecute_buff;
 		} else
 		{	/* No single line trigger exists. See if multi-line trigger exists. The form is ^#t(gbl,indx,XECUTE,n)
 			 * so can be easily tested for with $DATA().
 			 */
+			DBGTRIGR((stderr, "trigger_gbl_fill_xecute_buffer: No single record xecute string(%d), try for multiline\n",
+						key_val.str.len));
 			op_gvdata(&data_val);
 			if ((literal_ten.m[0] != data_val.m[0]) || (literal_ten.m[1] != data_val.m[1]))
 			{	/* The process' view of the triggers is likely stale. Restart to be safe.
-				 * Triggers can be invoked only by GT.M and Update process. Out of these, we expect only
-				 * GT.M to see restarts due to concurrent trigger changes. Update process is the only
-				 * updater on the secondary so we dont expect it to see any concurrent trigger changes
-				 * Assert accordingly. Note similar asserts occur in t_end.c and tp_tend.c.
+				 * Triggers can be invoked only by GT.M and Update process. We expect to see GT.M processes to
+				 * restart due to concurrent trigger changes. The Update Process should only restart if it is a
+				 * supplementary instance. Assert accordingly. Note similar asserts occur in t_end.c and
+				 * tp_tend.c.
 				 */
-				assert(CDB_STAGNATE > t_tries);
-				assert(IS_GTM_IMAGE);
+				assert(!is_updproc || (jnlpool && jnlpool->repl_inst_filehdr->is_supplementary
+							&& !jnlpool->jnlpool_ctl->upd_disabled));
+				DBGTRIGR((stderr, "trigger_gbl_fill_xecute_buffer: multiline not found, retry\n"));
 				/* Assert that the cycle has changed but in order to properly do the assert, we need a memory
 				 * barrier since cs_data->db_trigger_cycle could be stale in our cache.
 				 */
@@ -124,16 +145,26 @@ char *trigger_gbl_fill_xecute_buffer(char *trigvn, int trigvn_len, mval *trig_in
 				DEBUG_ONLY(gvt_cycle = gv_target->db_trigger_cycle);
 				DEBUG_ONLY(csd_cycle = cs_data->db_trigger_cycle);
 				assert(csd_cycle > gvt_cycle);
-				t_retry(cdb_sc_triggermod);
+				if (UPDATE_CAN_RETRY(t_tries, t_fail_hist[t_tries]))
+					t_retry(cdb_sc_triggermod);
+				assert(WBTEST_HELPOUT_TRIGDEFBAD == gtm_white_box_test_case_number);
+				trgindx = mval2i(&index);
+				SET_PARAM_STRING(util_buff, util_len, trgindx, ",\"XECUTE\"");
+				rts_error_csa(CSA_ARG(REG2CSA(gv_cur_region)) VARLSTCNT(8) ERR_TRIGDEFBAD, 6, trigvn_len, trigvn,
+						trigvn_len, trigvn, util_len, util_buff);
 			}
 		}
 		/* Multi-line triggers exist */
 		num = 0;
 		i2mval(&xecute_index, num);
-		BUILD_HASHT_SUB_MSUB_SUB_MSUB_CURRKEY(trigvn, trigvn_len, index, LITERAL_XECUTE, LITERAL_XECUTE_LEN, xecute_index);
+		BUILD_HASHT_SUB_MSUB_SUB_MSUB_CURRKEY(trigvn, trigvn_len, index,
+					trigger_subs[XECUTE_SUB], STRLEN(trigger_subs[XECUTE_SUB]), xecute_index);
 		if (!gvcst_get(&key_val))
-		{	/* There has to be an XECUTE string */
-			assert(FALSE);
+		{	/* There has to be an XECUTE string or else it is a retry situation (due to concurrent updates) */
+			DBGTRIGR((stderr, "trigger_gbl_fill_xecute_buffer: problem getting multiline record count, retry\n"));
+			if (UPDATE_CAN_RETRY(t_tries, t_fail_hist[t_tries]))
+				t_retry(cdb_sc_triggermod);
+			assert(WBTEST_HELPOUT_TRIGDEFBAD == gtm_white_box_test_case_number);
 			trgindx = mval2i(&index);
 			SET_PARAM_STRING(util_buff, util_len, trgindx, ",\"XECUTE\"");
 			rts_error_csa(CSA_ARG(REG2CSA(gv_cur_region)) VARLSTCNT(8) ERR_TRIGDEFBAD, 6, trigvn_len, trigvn,
@@ -147,14 +178,19 @@ char *trigger_gbl_fill_xecute_buffer(char *trigvn, int trigvn_len, mval *trig_in
 		while (len < xecute_buff_len)
 		{
 			i2mval(&xecute_index, ++num);
-			BUILD_HASHT_SUB_MSUB_SUB_MSUB_CURRKEY(trigvn, trigvn_len, index, LITERAL_XECUTE, LITERAL_XECUTE_LEN,
-				xecute_index);
+			BUILD_HASHT_SUB_MSUB_SUB_MSUB_CURRKEY(trigvn, trigvn_len, index,
+					trigger_subs[XECUTE_SUB], STRLEN(trigger_subs[XECUTE_SUB]), xecute_index);
 			if (!gvcst_get(&key_val))
 				break;
 			if (xecute_buff_len < (len + key_val.str.len))
-			{	/* The DB string total is longer than the length stored at index 0 -- something is wrong */
+			{	/* The DB string total is longer than the length stored at index 0 -- something is wrong.
+				 * Most likely a retry necessary due to concurrent changes.
+				 */
+				DBGTRIGR((stderr, "trigger_gbl_fill_xecute_buffer: problem getting multiline component, retry\n"));
 				free(xecute_buff);
-				assert(FALSE);
+				if (UPDATE_CAN_RETRY(t_tries, t_fail_hist[t_tries]))
+					t_retry(cdb_sc_triggermod);
+				assert(WBTEST_HELPOUT_TRIGDEFBAD == gtm_white_box_test_case_number);
 				SET_PARAM_STRING(util_buff, util_len, num, ",\"XECUTE\"");
 				rts_error_csa(CSA_ARG(REG2CSA(gv_cur_region)) VARLSTCNT(8) ERR_TRIGDEFBAD, 6, trigvn_len, trigvn,
 						trigvn_len, trigvn, util_len, util_buff);
