@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2010-2019 Fidelity National Information	*
+ * Copyright (c) 2010-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -45,7 +45,6 @@
 #include "gdscc.h"			/* needed for tp.h */
 #include "gdskill.h"			/* needed for tp.h */
 #include "buddy_list.h"			/* needed for tp.h */
-#include "hashtab_int4.h"		/* needed for tp.h */
 #include "jnl.h"			/* needed for tp.h */
 #include "tp.h"
 #include "t_retry.h"
@@ -91,12 +90,12 @@ LITREF	char 			*trigger_subs[];
 {													\
 	if ((INTCAST(OUT_PTR - OUT_BUFF) + VAL_LEN) > MAX_BUFF_SIZE)					\
 	{												\
-		util_out_print_gtmio("!AD", NOFLUSH, (unsigned int)(OUT_PTR - OUT_BUFF), OUT_BUFF);	\
+		util_out_print_gtmio("!AD", NOFLUSH_OUT, (unsigned int)(OUT_PTR - OUT_BUFF), OUT_BUFF);	\
 		OUT_PTR = OUT_BUFF;									\
 	}												\
 	if (VAL_LEN > MAX_BUFF_SIZE)									\
 	{												\
-		util_out_print_gtmio("!AD", NOFLUSH, VAL_LEN, VAL);					\
+		util_out_print_gtmio("!AD", NOFLUSH_OUT, VAL_LEN, VAL);					\
 		OUT_PTR = OUT_BUFF;									\
 	} else												\
 	{												\
@@ -438,11 +437,14 @@ STATICFNDEF void write_gbls_or_names(char *gbl_name, uint4 gbl_name_len, boolean
 	/* If ^#t needs to be upgraded, issue error. Cannot read older ^#t format that newer version does not always understand */
 	if (cs_addrs->hdr->hasht_upgrade_needed)
 		rts_error_csa(CSA_ARG(cs_addrs) VARLSTCNT(4) ERR_NEEDTRIGUPGRD, 2, DB_LEN_STR(gv_cur_region));
+	assert((0 < gbl_name_len) && (MAX_MIDENT_LEN >= gbl_name_len));
+	STATIC_ANALYSIS_ONLY(gbl_name_len = MIN(gbl_name_len, MAX_MIDENT_LEN));	/* 4SCA: GV names from DB are <= MAX_MIDENT_LEN */
 	memcpy(save_name, gbl_name, gbl_name_len);
 	wildcard = (NULL != (ptr = memchr(gbl_name, '*', gbl_name_len)));
 	if (wildcard)
 	{
-		gbl_name_len--;
+		STATIC_ANALYSIS_ONLY(if (0 < gbl_name_len))	/* 4SCA: GV names from DB <= MAX_MIDENT_LEN and > 0*/
+			gbl_name_len--;
 		assert(INTCAST(ptr - gbl_name) == gbl_name_len);
 	}
 	memcpy(curr_name, gbl_name, gbl_name_len);
@@ -728,7 +730,7 @@ STATICFNDEF boolean_t trigger_select(char *select_list, uint4 select_list_len)
 	mname_entry		gvname;
 	int			len, len1, badpos;
 	boolean_t		trig_name;
-	gv_key			save_currkey[DBKEYALLOC(MAX_KEY_SZ)];
+	gv_key_buf		save_currkey;
 	gd_region		*save_gv_cur_region;
 	gv_namehead		*save_gv_target;
 	sgm_info		*save_sgm_info_ptr;
