@@ -1,6 +1,6 @@
  /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -57,21 +57,17 @@ GBLDEF	char			facility[MAX_INSTNAME_LEN + 100];
 GBLDEF	boolean_t		util_out_print_vaparm_flush;
 #endif
 
+GBLREF	boolean_t		blocksig_initialized, err_same_as_out, hup_on, is_rcvr_server, is_src_server, is_updproc;
 GBLREF	io_pair			io_std_device;
-GBLREF	boolean_t		blocksig_initialized;
-GBLREF	sigset_t		block_sigsent;
-GBLREF	boolean_t		err_same_as_out;
 GBLREF	gd_addr			*gd_header;
 GBLREF	jnlpool_addrs_ptr_t	jnlpool;
-GBLREF	boolean_t		is_src_server;
-GBLREF	boolean_t		is_rcvr_server;
-GBLREF	boolean_t		is_updproc;
-GBLREF	uint4			is_updhelper;
 GBLREF	recvpool_addrs		recvpool;
-GBLREF	uint4			process_id;
+GBLREF	sigset_t		block_sigsent;
+GBLREF	uint4			is_updhelper, process_id;
 GBLREF	VSIG_ATOMIC_T		forced_exit;
 
 error_def(ERR_REPLINSTACC);
+error_def(ERR_TERMHANGUP);
 error_def(ERR_TEXT);
 
 #define	ZTRIGBUFF_INIT_ALLOC		1024	/* start at 1K */
@@ -189,6 +185,7 @@ caddr_t util_format(caddr_t message, va_list fao, caddr_t buff, ssize_t size, in
 	unsigned int	ch;
 	UINTPTR_T	addr_val;
 	ssize_t		chlen;
+	uint4		stringlength;
 	boolean_t	indirect;
 	qw_num_ptr_t	val_ptr;
 	unsigned char	numa[22];
@@ -324,8 +321,9 @@ caddr_t util_format(caddr_t message, va_list fao, caddr_t buff, ssize_t size, in
 						length = c ? *c++ : 0;
 						break;
 					case 'D':
-					case 'F': /* string with length and addr parameters */
-						GETFAOVALDEF(faocnt, fao, int4, length, 0);
+					case 'F': /* string with length and addr parameters. restrict to 1 MiB length */
+						GETFAOVALDEF(faocnt, fao, uint4, stringlength, 0);
+						length = (MAX_STRLEN > stringlength) ? (int)stringlength : MAX_STRLEN;
 						GETFAOVALDEF(faocnt, fao, caddr_t, c, NULL);
 						break;
 					case 'S':
@@ -343,7 +341,8 @@ caddr_t util_format(caddr_t message, va_list fao, caddr_t buff, ssize_t size, in
 						break;
 					case 'Z': /* null teminated string */
 						GETFAOVALDEF(faocnt, fao, caddr_t, c, NULL);
-						length = c ? STRLEN(c) : 0;
+						stringlength = c ? strlen(c) : 0;
+						length = (MAX_STRLEN > stringlength) ? (int)stringlength : MAX_STRLEN;
 				}
 				/* Since gtmsecshr does not load ICU libraries (since dlopen() with LD_LIBRARY_PATH
 				 * does not work for root setuid executables), avoid calling gtm_wcswidth() and
@@ -782,7 +781,7 @@ void	util_out_print_vaparm(caddr_t message, int flush, va_list var, int faocnt)
 	use_gtmio = ((NULL != io_std_device.out) && (!IS_GTMSECSHR_IMAGE) && err_same_as_out);
 	switch (flush)
 	{
-		case NOFLUSH:
+		case NOFLUSH_OUT:
 			break;
 		case RESET:
 			break;
@@ -869,7 +868,7 @@ void	util_out_print_vaparm(caddr_t message, int flush, va_list var, int faocnt)
 	}
 	switch (flush)
 	{
-		case NOFLUSH:
+		case NOFLUSH_OUT:
 			break;
 		case FLUSH:
 		case RESET:
@@ -929,8 +928,9 @@ void util_cond_flush(void)
 
 	SETUP_THREADGBL_ACCESS;
 	ASSERT_SAFE_TO_UPDATE_THREAD_GBLS;
-	if (TREF(util_outptr) != TREF(util_outbuff_ptr))
-		util_out_print(NULL, FLUSH);
+	assert(((int)ERR_TERMHANGUP != SIGNAL) || ((tt == io_std_device.in->type) && hup_on));
+	if ((TREF(util_outptr) != TREF(util_outbuff_ptr)) && ((int)ERR_TERMHANGUP != SIGNAL))	/* if a PRINCIPAL device terminal */
+		util_out_print(NULL, FLUSH);	/* disappeared, sending stuff to it with a PRN_ERROR just complicates things */
 }
 
 #ifdef DEBUG
