@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2012 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -9,8 +10,8 @@
  *								*
  ****************************************************************/
 
-#ifndef __WCS_CLEAN_DBSYNC_H__
-#define __WCS_CLEAN_DBSYNC_H__
+#ifndef WCS_CLEAN_DBSYNC_H_INCLUDED
+#define WCS_CLEAN_DBSYNC_H_INCLUDED
 
 #include "gt_timer.h"
 #include "have_crit.h"
