@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information	*
+ * Copyright (c) 2001-2020 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -55,7 +55,6 @@
 #include "anticipatory_freeze.h"
 #include "gtmcrypt.h"
 #include "buddy_list.h"		/* needed for tp.h */
-#include "hashtab_int4.h"	/* needed for tp.h */
 #include "tp.h"
 #include "t_retry.h"
 #include "min_max.h"
@@ -202,7 +201,7 @@ int4	wcs_wtstart(gd_region *region, int4 writes, wtstart_cr_list_t *cr_list_ptr,
 	INCR_INTENT_WTSTART(cnl);	/* signal intent to enter wcs_wtstart */
 	/* the above interlocked instruction does the appropriate write memory barrier to publish this change to the world */
 	SHM_READ_MEMORY_BARRIER;	/* need to do this to ensure uptodate value of cnl->wc_blocked is read */
-	if (cnl->wc_blocked)
+	if (WC_BLOCK_RECOVER == cnl->wc_blocked)
 	{
 		WCS_OPS_TRACE(csa, process_id, wcs_ops_wtstart2, 0, 0, 0, 0, 0);
 		DECR_INTENT_WTSTART(cnl);
@@ -282,7 +281,7 @@ int4	wcs_wtstart(gd_region *region, int4 writes, wtstart_cr_list_t *cr_list_ptr,
 			if (cr2flush->dirty && !cr2flush->epid)
 			{ 	/* if it is in the active queue */
 				++fast_lock_count; /* Disable wcs_stale for duration */
-				if (grab_latch(&ahead->latch, WT_LATCH_TIMEOUT_SEC))
+				if (grab_latch(&ahead->latch, WT_LATCH_TIMEOUT_SEC, WS_26, csa))
 				{
 					cr = cr2flush;
 					csr = (cache_state_rec_ptr_t)((sm_uc_ptr_t)cr + SIZEOF(cr->blkque));
@@ -296,7 +295,7 @@ int4	wcs_wtstart(gd_region *region, int4 writes, wtstart_cr_list_t *cr_list_ptr,
 							 */
                                                         assert(WBTEST_ENABLED(WBTEST_CRASH_SHUTDOWN_EXPECTED)
                                                                 || WBTEST_ENABLED(WBTEST_MURUNDOWN_KILLCMT06));
-                                                        SET_TRACEABLE_VAR(cnl->wc_blocked, TRUE);
+							SET_TRACEABLE_VAR(cnl->wc_blocked, WC_BLOCK_RECOVER);
                                                         err_status = ERR_DBCCERR;
                                                         break;
                                                 }
@@ -330,7 +329,7 @@ int4	wcs_wtstart(gd_region *region, int4 writes, wtstart_cr_list_t *cr_list_ptr,
 			if (INTERLOCK_FAIL == (INTPTR_T)csr)
 			{
 				assert(FALSE);
-				SET_TRACEABLE_VAR(cnl->wc_blocked, TRUE);
+				SET_TRACEABLE_VAR(cnl->wc_blocked, WC_BLOCK_RECOVER);
 				BG_TRACE_PRO_ANY(csa, wcb_wtstart_lckfail1);
 				break;
 			}
@@ -446,7 +445,8 @@ int4	wcs_wtstart(gd_region *region, int4 writes, wtstart_cr_list_t *cr_list_ptr,
 			 * Also we are meddling with active queue now so we cannot risk a "wcs_recover" call inside
 			 * "grab_crit_immediate" hence the OK_FOR_WCS_RECOVER_FALSE usage below.
 			 */
-			if (!wtfini_called_once && (was_crit || grab_crit_immediate(region, OK_FOR_WCS_RECOVER_FALSE)))
+			if (!wtfini_called_once &&
+				(was_crit || grab_crit_immediate(region, OK_FOR_WCS_RECOVER_FALSE, NOT_APPLICABLE)))
 			{
 				if (csr->twin)
 				{
@@ -539,12 +539,24 @@ int4	wcs_wtstart(gd_region *region, int4 writes, wtstart_cr_list_t *cr_list_ptr,
 			else
 				assert(IS_GDS_BLK_DOWNGRADE_NEEDED(csr->ondsk_blkver) || (GDSV6 == csr->ondsk_blkver));
 #			endif
-			if (csa->do_fullblockwrites)
+			if (csd->write_fullblk)
 			{	/* See similiar logic in wcs_wtstart.c */
+#			ifdef DEBUG
+				if (WBTEST_ENABLED(WBTEST_FULLBLKWRT_DB) && (3 == csr->blk))
+				{
+					DBGFPF((stdout,"Rounding the write size: %d with %d\n", size, \
+					((FULL_DATABASE_WRITE == csd->write_fullblk && csr->needs_first_write) ? \
+					csd->blk_size : csa->fullblockwrite_len)));
+				}
+#			endif
 				size = (int)ROUND_UP(size,
-						(FULL_DATABASE_WRITE == csa->do_fullblockwrites && csr->needs_first_write)
+						(FULL_DATABASE_WRITE == csd->write_fullblk && csr->needs_first_write)
 						? csd->blk_size : csa->fullblockwrite_len);
 			}
+#                       ifdef DEBUG
+			else if (WBTEST_ENABLED(WBTEST_FULLBLKWRT_DB) && (3 == csr->blk))
+				DBGFPF((stdout, "Not rounding the write size\n"));
+#			endif
 			assert(size <= csd->blk_size);
 			INCR_GVSTATS_COUNTER(csa, cnl, n_dsk_write, 1);
 			save_bp = bp;
@@ -665,6 +677,16 @@ int4	wcs_wtstart(gd_region *region, int4 writes, wtstart_cr_list_t *cr_list_ptr,
 			{	/* Due to csa->in_wtstart protection (at the beginning of this module), we are guaranteed
 				 * that the write below won't be interrupted by another nested wcs_wtstart
 				 */
+#ifdef DEBUG
+				/* Going to do a write below, check the size being written for full blk writes */
+				if (WBTEST_ENABLED(WBTEST_FULLBLKWRT_DB) && (3 == csr->blk))
+				{
+					DBGFPF((stdout, "Region : %s Blk num : %ld ",region->rname, csr->blk));
+					DBGFPF((stdout, "needs_first_write : %d ", csr->needs_first_write));
+					DBGFPF((stdout, "size written: %ld ", size));
+					DBGFPF((stdout, "fullblkwrite_len : %ld\n", csa->fullblockwrite_len));
+				}
+#endif
 				offset = BLK_ZERO_OFF(csd->start_vbn) + (off_t)csr->blk * csd->blk_size;
 				if (!do_asyncio)
 				{
@@ -728,7 +750,7 @@ int4	wcs_wtstart(gd_region *region, int4 writes, wtstart_cr_list_t *cr_list_ptr,
 				{
 					assert(FALSE);
 					csr->epid = 0;
-					SET_TRACEABLE_VAR(cnl->wc_blocked, TRUE);
+					SET_TRACEABLE_VAR(cnl->wc_blocked, WC_BLOCK_RECOVER);
 					BG_TRACE_PRO_ANY(csa, wcb_wtstart_lckfail4);
 					err_status = ERR_DBCCERR;
 					break;
