@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2018 Fidelity National Information 	*
+ * Copyright (c) 2001-2020 Fidelity National Information 	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -85,7 +85,7 @@ const static readonly int error_ansi[] = {
 	   0,	/* GVIS */
 	   0,	/* GVKILLFAIL */
 	   1,	/* GVNAKED */
-	   0,	/* GVNEXTARG */
+	   0,	/* UNUSEDMSG256 */
 	   0,	/* GVORDERFAIL */
 	   0,	/* GVPUTFAIL */
 	   0,	/* PATTABSYNTAX */
@@ -134,8 +134,8 @@ const static readonly int error_ansi[] = {
 	   0,	/* JNLINVALID */
 	   0,	/* MBXWRTONLY */
 	   0,	/* MEMORY */
-	   0,	/* UNUSEDMSG305 */
-	   0,	/* UNUSEDMSG306 */
+	   0,	/* DONOBLOCK */
+	   0,	/* ZATRANSCOL */
 	   0,	/* UNUSEDMSG307 */
 	   0,	/* UNUSEDMSG308 */
 	   0,	/* UNUSEDMSG309 */
@@ -460,7 +460,7 @@ const static readonly int error_ansi[] = {
 	   0,	/* COLLTYPVERSION */
 	   0,	/* LVNULLSUBS */
 	   0,	/* GVREPLERR */
-	   0,	/* UNUSEDMSG631 */
+	   0,	/* DBFILERDONLY */
 	  72,	/* RMWIDTHPOS */
 	  13,	/* OFFSETINV */
 	   0,	/* JOBPARTOOLONG */
@@ -489,7 +489,7 @@ const static readonly int error_ansi[] = {
 	   0,	/* INVBITSTR */
 	   0,	/* INVBITPOS */
 	   0,	/* PARNORMAL */
-	   0,	/* PARBUFSM */
+	   0,	/* FILEPATHTOOLONG */
 	  72,	/* RMWIDTHTOOBIG */
 	   0,	/* PATTABNOTFND */
 	   0,	/* OBJFILERR */
@@ -608,7 +608,7 @@ const static readonly int error_ansi[] = {
 	   0,	/* MEMORYRECURSIVE */
 	   0,	/* FREEZEID */
 	   0,	/* BLKWRITERR */
-	   0,	/* UNUSEDMSG779 */
+	   0,	/* DSEINVALBLKID */
 	   0,	/* PINENTRYERR */
 	   0,	/* BCKUPBUFLUSH */
 	   0,	/* NOFORKCORE */
@@ -1072,8 +1072,8 @@ const static readonly int error_ansi[] = {
 	   0,	/* BKUPTMPFILOPEN */
 	   0,	/* BKUPTMPFILWRITE */
 	   0,	/* VMSMEMORY2 */
-	   0,	/* LOADBGSZ2 */
-	   0,	/* LOADEDSZ2 */
+	   0,	/* UNUSEDMSG1243 */
+	   0,	/* UNUSEDMSG1244 */
 	   0,	/* REPLINSTMISMTCH */
 	   0,	/* REPLINSTREAD */
 	   0,	/* REPLINSTDBMATCH */
@@ -1369,7 +1369,7 @@ const static readonly int error_ansi[] = {
 	   0,	/* GBLNOMAPTOREG */
 	   0,	/* ISSPANGBL */
 	   0,	/* TPNOSUPPORT */
-	   0,	/* UNUSEDMSG1542 */
+	   0,	/* EXITSTATUS */
 	   0,	/* ZATRANSERR */
 	   0,	/* FILTERTIMEDOUT */
 	   0,	/* TLSDLLNOOPEN */
@@ -1511,4 +1511,38 @@ const static readonly int error_ansi[] = {
 	   0,	/* LOADRECCNT */
 	   0,	/* COMMFILTERERR */
 	   0,	/* NOFILTERNEST */
+	   0,	/* MLKHASHTABERR */
+	   0,	/* LOCKCRITOWNER */
+	   0,	/* MLKHASHWRONG */
+	   0,	/* XCRETNULLREF */
+	   0,	/* EXTCALLBOUNDS */
+	   0,	/* EXCEEDSPREALLOC */
+	   0,	/* ZTIMEOUT */
+	   0,	/* ERRWZTIMEOUT */
+	   0,	/* MLKHASHRESIZE */
+	   0,	/* MLKHASHRESIZEFAIL */
+	   0,	/* MLKCLEANED */
+	   0,	/* NOTMNAME */
+	   0,	/* DEVNAMERESERVED */
+	   0,	/* ORLBKREL */
+	   0,	/* ORLBKRESTART */
+	   0,	/* UNIQNAME */
+	   0,	/* APDINITFAIL */
+	   0,	/* APDCONNFAIL */
+	   0,	/* APDLOGFAIL */
+	   0,	/* STATSDBMEMERR */
+	   0,	/* BUFSPCDELAY */
+	   0,	/* AIOQUEUESTUCK */
+	   0,	/* INVGVPATQUAL */
+	   0,	/* NULLPATTERN */
+	   0,	/* MLKREHASH */
+	   0,	/* MUKEEPPERCENT */
+	   0,	/* MUKEEPNODEC */
+	   0,	/* MUKEEPNOTRUNC */
+	   0,	/* MUTRUNCNOSPKEEP */
+	   0,	/* TERMHANGUP */
+	   0,	/* DBFILNOFULLWRT */
+	   0,	/* BADCONNECTPARAM */
+	   0,	/* BADPARAMCOUNT */
+	   0,	/* REPLALERT */
 	};
